%%%-------------------------------------------------------------------
%%% File    : lgi.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : Law Governed Interaction
%%%
%%% Created : 27 Apr 2004 by  <svg@surnet.ru>
%%%-------------------------------------------------------------------
%%
%% $Id$
%%
%%**
%% 
%% @doc Law Governed Interaction
%% @end
%%*
-module(lgi).

-export([behaviour_info/1]).

-export([self_aid/2, aid/2, aid/3, aid/4, connect/3, connect_virtual/3, disconnect/1, disconnect/2]).
-export([deliver/0, deliver/1, deliver/2, forward/0, forward/1, forward/2]).
-export([get@/1, rm@/1, send/3, cast/3, cast/4, set@/2, aid@/0]).
-export([start_link/0, stop/0]).

-import(lgi.agent).

-define(answer_timeout, 5000).

behaviour_info(callbacks) ->
  [{initial_context, 1}, {sent, 2}, {arrived, 2}, {post_condition/4}].

start_link() ->
  agent:start_link().

stop() ->
  agent:stop().

%% Props = {law_module, ModName} | {K, V}
connect(Law, Name, Props) ->
  agent:new(Law, Name, Props).

connect_virtual(Law, Name, Props) ->
  agent:new(Law, Name, undefined, Props).

disconnect(Aid) ->
  agent:delete(Aid).

disconnect(Law, Name) ->
  agent:delete(Law, Name).

cast(From, To, Msg) ->
  cast(From, To, Msg, ?answer_timeout).

cast(From, To, Msg, Timeout) -> 
  agent:cast(From, To, Msg, Timeout).

send(FromName, ToName, Msg) ->
  agent:send(FromName, ToName, Msg).


self_aid(Law, Name) ->
  aid(Law, self(), Name, node()).

aid(Law, Name) ->
  aid(Law, Name, node()).

aid(Law, Name, Node) ->
  aid(Law, undefined, Name, Node).

aid(Law, Pid, Name, Node) ->
  agent:aid(Law, Pid, Name, Node).

aid@() ->
  agent:law_do(aid).

set@(K, V) ->
  agent:law_do({context_set, K, V}).

get@(K) ->
  agent:law_do({context_get, K}).

rm@(K) ->
  agent:law_do({context_delete, K}).

forward() ->
  agent:law_do(forward).

forward(To) ->
  agent:law_do({forward, To}).

forward(To, Msg) ->
  agent:law_do({forward, To, Msg}).

deliver() ->
  agent:law_do(deliver).

deliver(To) ->
  agent:law_do({deliver, To}).

deliver(To, Msg) ->
  agent:law_do({deliver, To, Msg}).
