
-module(mdisp_cb).

-behaviour(mdisp).
-export([init/2, awake/2, terminate/3]).

%% internal exports
-export([f1/2, f2/2]).

-define(write(Fmt, Args), ok=io:format(Fmt, Args)).


%%% A somewhat weird example of the mdisp callback concept.
%%% Run it like this:
%%% 
%%% Eshell V5.0.2.11  (abort with ^G)
%%% 1> mdisp:start_link(mdisp).
%%% {ok,<0.38.0>}
%%% 2> mdisp:new(mdisp, mdisp_cb, {key,a}, {f1,1}).
%%% ok
%%% init({key,a}, {f1,1})
%%% f1(1)
%%% f1 going to sleep (1)                          
%%% 3> mdisp:send(mdisp, {key,a}, this).           
%%% ok
%%% awake({key,a}, {f1,1})
%%% f1(1)
%%% f1 ==> this
%%% f2(2)
%%% 4> mdisp:send(mdisp, {key,a}, this). <=== if called within 5 seconds
%%% ok
%%% f2 ==> this
%%% f1(3)
%%% 5> mdisp:send(mdisp, {key,a}, this).
%%% ok
%%% f1 ==> this
%%% f2(4)
%%% f2 going to sleep (4)            <=== if more than 5 seconds pass
%%% 6> mdisp:send(mdisp, {key,a}, this).
%%% ok
%%% awake({key,a}, {f2,4})
%%% f2(4)
%%% f2 ==> this
%%% f1(5)
%%% f1 going to sleep (5)            <=== if more than 5 seconds pass
%%% mdisp:delete(mdisp, {key, a}).
%%% ok



init(Key, State = {Fun, S}) ->
    ?write("init(~p, ~p)~n", [Key, State]),
    ?MODULE:Fun(Key, S).

awake(Key, State = {Fun, S}) ->
    ?write("awake(~p, ~p)~n", [Key, State]),
    ?MODULE:Fun(Key, S).


terminate(Reason, Key, State) ->
    exit({Reason, {Key, State}}).


f1(Key, S) ->
    ?write("f1(~p)~n", [S]),
    receive
	Msg ->
	    ?write("f1 ==> ~p~n", [Msg]),
	    f2(Key, S+1)
    after 5000 ->
	    ?write("f1 going to sleep (~p)~n", [S]),
	    mdisp:sleep(mdisp, Key, {f1, S})
    end.

f2(Key, S) ->
    ?write("f2(~p)~n", [S]),
    receive
	this ->
	    ?write("f2 ==> ~p~n", [this]),
	    f1(Key, S+1)
    after 5000 ->
	    ?write("f2 going to sleep (~p)~n", [S]),
	    mdisp:sleep(mdisp, Key, {f2, S})
    end.

