%%
%% $Id: attr_type.erl,v 1.1 2003/10/27 22:01:04 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(odbcdb.attr_type).

-export([coerce_type/3]).
-export([to_date/1, to_datetime/1, to_time/1, from_date/1, from_datetime/1, from_time/1]).
-export([to_boolean/1, from_boolean/1, from_boolean/2]).

-import(lists).
-import(io_lib).
-import(regexp).

%% in
coerce_type(in, _, undefined) ->
  undefined;
coerce_type(in, date, Date) ->
  to_date(Date);
coerce_type(in, datetime, DateTime) ->
  to_datetime(DateTime);
coerce_type(in, atom, A) when is_atom(A) ->
  A;
coerce_type(in, atom, Str) when is_list(Str) ->
  list_to_atom(Str);
coerce_type(in, boolean, B) when B == true;
				 B == false ->
  B;
coerce_type(in, boolean, V) ->
  to_boolean(V);
%% sequences in PostgreSQL return text type
coerce_type(in, integer, V) when is_list(V) ->
  list_to_integer(V);
coerce_type(in, time, V) ->
  to_time(V);
coerce_type(in, _, V) ->
  V;
%% out
coerce_type(out, _, V=undefined) ->
  V;
coerce_type(out, date, T) ->
  from_date(T);
coerce_type(out, datetime, T) ->
  from_datetime(T);
coerce_type(out, time, T) ->
  from_time(T);
coerce_type(out, _, V) when is_float(V) ->
  float_to_list(V);
coerce_type(out, _, V) when is_integer(V) ->
  integer_to_list(V);
coerce_type(out, _, V) when is_atom(V) ->
  atom_to_list(V);
coerce_type(out, _, V) when is_list(V) ->
  V;
coerce_type(out, _, T={Y, M, D}) -> % date
  from_date(T);
coerce_type(out, _, T={{Y, M, D}, {H24, MI, S}}) -> % datetime
  from_datetime(T);
coerce_type(out, _, V) ->
  io_lib:write(V).

to_date(Date={Y, M, D}) ->
  Date;
to_date({Date={Y, M, D}, _}) ->
  Date;
to_date(T) when is_list(T) ->
  {ok, [Y, M, D], _} = io_lib:fread("~4d-~2d-~2d", T),
  {Y, M, D}.

from_date({Y, M, D}) when is_integer(Y),
			   is_integer(M),
			   is_integer(D) ->
  lists:flatten(io_lib:format("~4.4.0w-~2.2.0w-~2.2.0w", [Y, M, D]));
from_date({Date, _}) ->
  from_date(Date).


%% Example: 2003-06-10 02:13:57
to_datetime(T={Y, M, D}) ->
  {T, {0, 0, 0}};
to_datetime(T={{Y, M, D}, {H24, MI, S}}) ->
  T;
to_datetime(T) when is_list(T) ->
  case io_lib:fread("~4d-~2d-~2d ~2d:~2d:~2d", T) of
    {ok, [Y, M, D, H24, MI, S], _} ->
      {{Y, M, D}, {H24, MI, S}};
    {more, _Rest, _Chars, [D, M, Y]} ->
      {{Y, M, D}, {0, 0, 0}};
    _Error ->
      exit({badformat, T})
  end.

from_datetime({Y, M, D}) ->
  from_datetime({{Y, M, D}, {0, 0, 0}});
from_datetime({{Y, M, D}, {H24, MI, S}})  when is_integer(Y),
					       is_integer(M),
					       is_integer(D),
					       is_integer(H24),
					       is_integer(MI),
					       is_integer(S) ->
  lists:flatten(io_lib:format(
		  "~4.4.0w-~2.2.0w-~2.2.0w ~2.2.0w:~2.2.0w:~2.2.0w",
		  [Y, M, D, H24, MI, S])).

%% 04:05:06
to_time(T) when is_list(T)->
  case io_lib:fread("~2d:~2d:~2d", T) of
    {ok, [H24, MI, S], _} ->
      {H24, MI, S};
    {more, [], _Chars, [H24, MI]} ->
      {H24, MI, 0};
    _Error ->
      exit({badformat, T})
  end;
to_time(T={_, _, _}) ->
  T.

from_time({H24, MI, S}) ->
  lists:flatten(io_lib:format("~2.2.0w:~2.2.0w:~2.2.0w", [H24, MI, S])).
 
to_boolean(B) when B == true; B == false ->
  B;
to_boolean(1) ->
  true;
to_boolean(0) ->
  false;
to_boolean(True) ->
  V = hd(True),
  case V of
    _ when V == $t;
	   V == $T;
	   V == $Y;
	   V == $y;
	   V == $1 ->
      true;
    _ when V == $f;
	   V == $F;
	   V == $N;
	   V == $n;
	   V == $0 ->
      false
  end.

from_boolean(V) ->
  from_boolean(char, V).

from_boolean(char, true) -> "t";
from_boolean(char, false) -> "f";
from_boolean(integer, true) -> 1;
from_boolean(integer, false) -> 0.
