%%%-------------------------------------------------------------------
%%% File    : odbcdb_schema.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%%
%%% Created :  9 Sep 2003 by  <svg@surnet.ru>
%%%-------------------------------------------------------------------
%%
%% $Id: odbcdb_schema.erl,v 1.3 2003/10/31 18:02:49 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(odbcdb_schema).

-export([install/1, add_properties/1, attributes/1, primary_key/1, keygenerator/1]).
-export([table_property/2, attribute_property/2, attribute_property/3]).
-export([query_property/2]).

%% Internal exports
-export([info/1]).

-import(wslib.listn).
-import(wslib.log).

-include_lib("stdlib/include/ms_transform.hrl").

-record(odbcSysDict, {key, val}).

-define(dict, odbcSysDict).
-define(prop(Key, Val), #odbcSysDict{key=Key, val=Val}).

install(Nodes) ->
  TableDef = [{attributes, record_info(fields, ?dict)},
	      {disc_copies, Nodes}
	     ],
  mnesia:create_table(?dict, TableDef).

attributes(Table) ->
  table_property(Table, attrs).

primary_key(Table) ->
  table_property(Table, primary_key).

keygenerator(Table) ->
  table_property(Table, keygenerator).

attribute_property(Table, Attr, Prop) ->
  attribute_property({Table, Attr}, Prop).

attribute_property(AK={_Table, _Attr}, P=type) ->
  case mnesia:dirty_read(?dict, {attr, AK, P}) of
    [] ->
      undefined;
    [?prop(_, PV)] ->
      PV
  end.

table_property(Table, Prop) when Prop == primary_key;
				 Prop == attrs;
				 Prop == keygenerator;
				 Prop == coerce_attr_type ->
  case mnesia:dirty_read(?dict, {table, Table, Prop}) of
    [] ->
      undefined;
    [?prop(_, PV)] ->
      PV
  end.

query_property(Query, Prop) when Prop == result_types;
				 Prop == coerce_result_type ->
  case mnesia:dirty_read(?dict, {db_query, Query, Prop}) of
    [] ->
      undefined;
    [?prop(_, PV)] ->
      PV
  end.

add_properties(Props) ->
  Props1 =  listn:mk_list(Props),
  AddProps = fun () -> lists:foreach(fun add_property/1, Props1) end,

  case mnesia:transaction(AddProps) of
    {aborted, Error} ->
      {error, Error};
    _ ->
      Tables =
	lists:usort([element(2, K)
		     || {K, _} <- Props1, element(1,K) == table]),
      lists:foreach(
	fun ({{table, T, keygenerator}, KG}) ->
	    PKs = primary_key(T),
	    odbcdb:generate_schema_phrase(keygenerator, T, KG, PKs);
	    (_) ->
	    ok
	end, Props1),
      lists:foreach(
	fun (Table) ->
	    case catch
	      begin
		PKs = primary_key(Table),
		Attrs = attributes(Table),
		AttrTypes = [{A, attribute_property(Table, A, type)} || A <- Attrs],
		odbcdb:generate_schema_phrases(Table, PKs, Attrs),
		{atomic, ok} = add_table_coerce_type_fun(Table, AttrTypes)
	      end of
	      {'EXIT', Reason} ->
		log:write_error("table ~s,~n error:~n~w", [Table, Reason]);
	      _ ->
		ok
	    end
	end, Tables),
      ok
  end.

add_property({K={table, _T, keygenerator}, KG={sequence, SeqName}}) when is_atom(SeqName) ->
  mnesia:write(?prop(K, KG));
add_property({K={table, _T, keygenerator}, KG={sequence, SeqName, AttrName}})
  when is_atom(SeqName), is_atom(AttrName) ->
  mnesia:write(?prop(K, KG)); 
add_property({K={table, T, primary_key}, Pks}) ->
  check_table_name(T),
  lists:foreach(fun (A) -> check_attr_name(T, A) end, Pks),
  case mnesia:read({?dict, {table, T, attrs}}) of
    [] ->
      ok;
    [?prop(_, Attrs)] ->
      Attrs1 = Attrs -- Pks,
      mnesia:write(?prop({table, T, attrs}, Pks ++ Attrs1))
  end,
  mnesia:write(?prop(K, Pks));
add_property({K={table, T, attrs}, Attrs}) ->
  check_table_name(T),
  MS = ets:fun2ms(
	 fun(?prop({attr, {TN, A}, P}, _)) when TN == T ->
	     {attr, {T, A}, P}
	 end),
  lists:foreach(fun (AK) -> mnesia:delete({?dict, AK}) end,
		mnesia:select(?dict, MS)),
  lists:foreach(fun (A) -> add_table_attr(T, A) end, Attrs),
  AttrNames = lists:map(fun ({AN, _}) -> AN;
			    (AN) -> AN
			end, Attrs),
  AttrNames1 =
    case mnesia:read({?dict, {table, T, primary_key}}) of
      [] ->
	AttrNames;
      [?prop(_, Pks)] ->
	Pks ++ (AttrNames -- Pks)
    end,  
  mnesia:write(?prop(K, AttrNames1));
add_property({_K={db_query, QueryName, result_types}, []}) ->
    mnesia:transaction(
    fun () ->
	mnesia:delete({?dict, {db_query, QueryName, result_types}}),
	mnesia:delete({?dict, {db_query, QueryName, coerce_result_type}}),
	ok
    end);
add_property({_K={db_query, QueryName, result_types}, Attrs}) ->
  Fun = mk_coerce_type(Attrs),
  mnesia:transaction(
    fun () ->
	mnesia:write(?prop({db_query, QueryName, result_types}, Attrs)),
	mnesia:write(?prop({db_query, QueryName, coerce_result_type}, Fun)),
	ok
    end).

add_table_attr(Table, {Attr, AttrProps}) ->
  check_attr_name(Table, Attr),
  lists:foreach(fun (P) -> add_attr_property(Table, Attr, P) end, AttrProps),
  Attr;
add_table_attr(Table, Attr) ->
  add_table_attr(Table, {Attr, [{type, undefined}]}).
 
add_attr_property(Table, AttrName, {type, AttrType})
  when AttrType == integer;
       AttrType == float;
       AttrType == string;
       AttrType == atom;
       AttrType == date;
       AttrType == datetime;
       AttrType == time;
       AttrType == boolean ->
  mnesia:write(?prop({attr, {Table, AttrName}, type}, AttrType));
add_attr_property(Table, AttrName, {type, AttrType})
  when AttrType == any;
       AttrType == undefined ->
  mnesia:write(?prop({attr, {Table, AttrName}, type}, undefined));
add_attr_property(Table, AttrName, {type, AttrType}) ->
  exit({error, {bad_attr_property, {Table, AttrName}, AttrType}}).
  
check_table_name(A) when is_atom(A) ->
  true;
check_table_name(A) ->
  exit({error, {bad_table_name, A}}).

check_attr_name(_T, A) when is_atom(A) ->
  true;
check_attr_name(T, A) ->
  exit({error, {bad_attr_name, {T,A}}}).

add_table_coerce_type_fun(Table, AttrTypes) ->
  %% trying to save time a little
  Fun = mk_coerce_type(AttrTypes),
  mnesia:transaction(
    fun () ->
	mnesia:write(?prop({table, Table, coerce_attr_type}, Fun)),
	ok
    end).

mk_coerce_type(AttrTypes) ->
  FunClauses =
    [io_lib:format(
       "(Dir, ~s, V) -> odbcdb.attr_type:coerce_type(Dir, ~s, V);", [A, T])
     || {A, T} <- AttrTypes, T =/= undefined],
  DefaultClause = "(_, _, V) -> V",
  FunStr = lists:flatten(["fun ",FunClauses,DefaultClause," end."]),
  {ok, Tokens, _} = erl_scan:string(FunStr),
  {ok, [Form]} = erl_parse:parse_exprs(Tokens),
  {value, Fun, _} = erl_eval:expr(Form, erl_eval:new_bindings()),
  Fun.

%% delete_attr_properties(Table, Attr) ->
%%   MS = ets:fun2ms(fun(?prop({attr, {Table, Attr}, Prop}, _)) ->
%% 		      {attr, {Table, Attr}, Prop}
%% 		  end),
%%   lists:foreach(fun (AK) -> mnesia:delete({?dict, AK}) end,
%% 		mnesia:select(?dict, MS)).

info(mnesia_tables) ->
  [?dict].
  
