%%%-------------------------------------------------------------------
%%% File    : odbcdb_sup.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%%
%%% Created : 10 Sep 2003 by  <svg@surnet.ru>
%%%-------------------------------------------------------------------
%%
%% $Id: odbcdb_sup.erl,v 1.3 2003/10/31 18:02:49 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(odbcdb_sup).

-behaviour(supervisor).
-behaviour(application).

%%--------------------------------------------------------------------
%% Include files
%%--------------------------------------------------------------------

%%--------------------------------------------------------------------
%% External exports
%%--------------------------------------------------------------------
-export([start/2, stop/1, start_link/2]).

%%--------------------------------------------------------------------
%% Internal exports
%%--------------------------------------------------------------------
-export([init/1]).

-import(wslib.log).
-import(wslib.mnesiadb).

%%--------------------------------------------------------------------
%% Macros
%%--------------------------------------------------------------------
-define(SERVER, ?MODULE).
-define(appname, odbcdb).
-define(default_pool_opts, [{pool_start_len, 1},
			    {pool_max_len, 30},
			    {pool_type, exclusive},
			    {supervisor_spec, {simple_one_for_one, 1,  1000}}
			   ]).
-define(default_connection_opts, [{odbc_opts, []},
				  {odbc_fatal_fun, mk_uodbc_fatal()}
				 ]).

%%--------------------------------------------------------------------
%% Records
%%--------------------------------------------------------------------

%%====================================================================
%% External functions
%%====================================================================
start(_, _) ->
  MnesiaTables = mnesia:system_info(tables),
  MyTables = odbcdb_schema:info(mnesia_tables)++odbcdb:info(mnesia_tables),
  case lists:any(fun (T) -> lists:member(T, MnesiaTables) == false end,
		 MyTables) of
    false ->
      ok;
    true ->
      log:write_error("you must run odbcdb:install/1 before", []),
      exit(odbcdb_dictionary_not_installed)
  end,
  MnesiaTimeout = akv(mnesia_wait_timeout, infinity),
  PoolOpts = akv(pool_opts, []) ++ ?default_pool_opts,
  ConnStr  = akv(connection_string),
  ConnOpts = akv(connection_opts, []) ++ ?default_connection_opts,
  PoolSpec = [{worker_spec,
	       {{odbcdb, start_odbc, [ConnStr, ConnOpts]},
		transient, 5000, worker, [odbcdb]}}
	      |PoolOpts],
  PhraseOpts = [{src, odbcdb:mk_phrase_src(F)} || F <- akv(dict_src, [])],
  ok = mnesiadb:wait_for_tables(MyTables, MnesiaTimeout),
  case start_link(PoolSpec, PhraseOpts) of
    Ret={ok, _} -> 
      Ret;
    Error ->
      Error
  end.

stop(_State) ->
  ok.

%%--------------------------------------------------------------------
%% Function: start_link/0
%% Description: Starts the supervisor
%%--------------------------------------------------------------------
start_link(PoolSpec, PhraseOpts) ->
  supervisor:start_link({local, ?SERVER}, ?MODULE, {PoolSpec, PhraseOpts}).

%%====================================================================
%% Server functions
%%====================================================================
%%--------------------------------------------------------------------
%% Func: init/1
%% Returns: {ok,  {SupFlags,  [ChildSpec]}} |
%%          ignore                          |
%%          {error, Reason}   
%%--------------------------------------------------------------------
init({PoolSpec, PhraseOpts}) ->
  Pool = {connection_pool,
	  {connection_pool, start_link, [{local, odbcdb}, PoolSpec]},
	  permanent,5000,worker,[connection_pool]},
  Phrase = {phrase_dict,
	    {phrase, start_link,
	     [{local, odbcdb_phrase_dict},
	      phrase.mnesia_dict, odbcdb_phrase_dict,
	      [{default_parameter_value, null}|PhraseOpts]
	     ]},
	    permanent,5000,worker, [phrase]},
  {ok,{{one_for_all,0,1000}, [Phrase, Pool]}}.

%%====================================================================
%% Internal functions
%%====================================================================

%%**
%% @spec mk_uodbc_fatal(  ) -> OdbcFatalFun
%%      OdbcFatalFun = (V::term()) -> true | false
%%
%%       
%% @doc Create function <code>odbc_fatal_fun/1</code> to check is ODBC error
%%      fatal or not.
%%
%% @end
%%*

mk_uodbc_fatal() ->
  {ok, Reg} = regexp:parse("No response from the backend"),
  fun (_Error={error, E}) when list(E) ->
      case regexp:first_match(E, Reg) of
	{match, _, _} ->
	  true;
	nomatch ->
	  false
      end;
      (_) ->
      false
  end.
  
akv(K) ->
  case application:get_env(K) of
    {ok, V} ->
      V;
    undefined ->
      undefined
  end.

akv(K, D) ->
  case akv(K) of
    undefined ->
      D;
    V ->
      V
  end.
