%%%-------------------------------------------------------------------
%%% File    : mnesia_dict.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%%
%%% Created : 10 Sep 2003 by  <svg@surnet.ru>
%%%-------------------------------------------------------------------
%%
%% $Id: mnesia_dict.erl,v 1.2 2003/10/27 17:12:49 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(phrase.mnesia_dict).

-behaviour(phrase).

-export([install/2, start_link/2]).
-export([lookup_phrase/2, write_phrase/4, clear_dict/1, terminate/1, init/1]).

-import(mnesia).
-import(phrase).

-record(phrase, {key, phrase, param_names=[]}).
-define(phrase(K, P, PNs), #phrase{key=K, phrase=P, param_names=PNs}).
-define(wait_table_timeout, 60000).

install(TableName, Nodes) ->
  TableDef = [{record_name, phrase},
	      {attributes, record_info(fields, phrase)},
	      {disc_copies, Nodes}
	     ],
  mnesia:create_table(TableName, TableDef).

start_link(Name, Opts) ->
  phrase:strart_link(Name, ?MODULE, Name, Opts).

init(TableName) ->
  mnesia:wait_for_tables([TableName], ?wait_table_timeout),
  {ok, TableName}.

lookup_phrase(Key, T) ->
  case mnesia:dirty_read(T, Key) of
    [?phrase(_, P, PNs)] ->
      {ok, P, PNs, T};
    [] ->
      {undefined, T}
  end.

write_phrase(Key, Phrase, ParamNames, T) ->
  case mnesia:transaction(
	 fun () ->
	     mnesia:write(T, ?phrase(Key, Phrase, ParamNames), write)
	 end) of
    {aborted, Error} ->
      {{error, Error}, T};
    _ ->
      {ok, T}
  end.


clear_dict(T) ->
  case mnesia:clear_table(T) of
    {atomic, ok} ->
      {ok, T};
    {aborted, Error} ->
      {{error, Error}, T}
  end.

terminate(_) ->
  ok.
