%%
%% $Id$
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(property_file.test_mod1).

-behaviour(property_server).
-behaviour(property_file).

-export([init/1, new_config/1, delete_config/1, config_file/1, bootstrap/2, store_entry/2, handle_op/2, config_to_list/1, init_postload/1]).
-export([parse_config/4, store_config/2, remove_config/1]).

-import(lists).
-import(ets).

%% Handler interface

init(TableName) ->
  {ok, TableName}.

init_postload(TableName) ->
  {ok, do_you_need_it, TableName}.

config_file(_TableName) ->
  {ok, "NoName"}.

bootstrap(Config, _TableName) ->
  {ok, [?MODULE, property_file.test_mod2], []}.

store_entry(Entry, TableName) ->
  true = ets:insert(TableName, Entry),
  {ok, TableName}.

new_config(TableName) ->
  ets:new(TableName, [named_table, protected, bag]),
  {ok, TableName}.

delete_config(TableName) when TableName =/= undefined ->
  true = ets:delete(TableName),
  {ok, TableName}.

handle_op({EtsOperation, Args}, TableName) ->
  {ok, apply(ets, EtsOperation, [TableName|Args]), TableName}.

config_to_list(TableName) ->
  {ok, ets:tab2list(TableName)}.

%% 

parse_config(_RP, [], Content, _Seed) -> %skip empty entries
  Content;
parse_config(RP, Attrs, Content, _Seed) ->
  [{fmt_node_path(RP), Attrs}|Content].

store_config(Entry, Entries) ->
  Entry.

remove_config(DB) ->
  exit(test).

fmt_node_path([NodeName]) ->
  NodeName;
fmt_node_path(RevPath) ->
  list_to_tuple(lists:reverse(RevPath)).
