%%%----------------------------------------------------------------------
%%% File    : property_file_test.erl
%%% Author  :  <svg@surnet.ru>
%%% Purpose : 
%%% Created :  2 Sep 2002 by  <svg@surnet.ru>
%%%----------------------------------------------------------------------
%%%
%%% $Id: property_file_test.erl,v 1.1.1.1 2002/10/04 00:00:00 svg Exp $
%%%
%%% $Log: property_file_test.erl,v $
%%% Revision 1.1.1.1  2002/10/04 00:00:00  svg
%%% Imported sources
%%%
%%% Revision 1.2  2002/10/01 22:32:05  svg
%%% *** empty log message ***
%%%
%%% Revision 1.1.1.1  2002/09/04 20:19:07  svg
%%% Imported sources
%%%
%%%

-module(property_file_test).
-author('svg@surnet.ru').

-export([test_all/0, test_all/1]).
-export([test_apache_parse/1, test_clike_parse/1, test_end_line/1]).
-export([test_walk_config/1, test_parse_config/1, test_property_server/1]).

-define(match(_Pat), fun (_Res) ->
                         case _Res of
                           _Pat -> ok;
                           _ -> false
                         end
                     end).
-define(CLIKE_MATCH,
	[{simple_list,["val1","val2","Val3","testvar1"]},
	 {'Quoted Atom',[{193,54,2,1}]},
	 {test_quote_char,["quote string\n\na"]},
	 {ipv4,[{193,54,2,1}]},
	 {netmask1,[{{193,54,2,0},4294967040}]},
	 {netmask2,[{{193,54,2,0},4294967040}]},
	 {prefix,["@prefix@"]},
	 {exec_prefix,["@exec_prefix@"]},
	 {sysconfdir,["@sysconfdir@"]},
	 {localstatedir,["@localstatedir@"]}|_]).

-define(APACHE_MATCH, [{'ServerType',["standalone"]},
		       {'ServerRoot',["/usr/local/psa/apache"]},
		       {'LockFile',["logs/httpd.lock"]},
		       {'PidFile',["logs/httpd.pid"]},
		       {'ScoreBoardFile',["logs/httpd.scoreboard"]},
		       {'ResourceConfig',["conf/srm.conf"]},
		       {'AccessConfig',["conf/access.conf"]},
		       {'ResourceConfig',["/dev/null"]},
		       {'AccessConfig',["/dev/null"]},
		       {'Timeout',[300]},
		       {'KeepAlive',["On"]},
		       {'MaxKeepAliveRequests',[1000]},
		       {'KeepAliveTimeout',[15]},
		       {'MinSpareServers',[5]}|_]).

-define(WALK_CLIKE_MATCH,
	[{{simple_list},["val1","val2","Val3","testvar1"]},
	 {{'Quoted Atom'},[{193,54,2,1}]},
	 {{test_quote_char},["quote string\n\na"]},
	 {{ipv4},[{193,54,2,1}]},
	 {{netmask1},[{{193,54,2,0},4294967040}]},
	 {{netmask2},[{{193,54,2,0},4294967040}]},
	 {{prefix},["@prefix@"]},
	 {{exec_prefix},["@exec_prefix@"]},
	 {{sysconfdir},["@sysconfdir@"]},
	 {{localstatedir},["@localstatedir@"]}|_]).

-define(LOAD_CLIKE_MATCH,
	[{simple_list,["val1","val2","Val3","testvar1"]},
	 {'Quoted Atom',[{193,54,2,1}]},
	 {test_quote_char,["quote string\n\na"]},
	 {ipv4,[{193,54,2,1}]},
	 {netmask1,[{{193,54,2,0},4294967040}]},
	 {netmask2,[{{193,54,2,0},4294967040}]},
	 {prefix,["@prefix@"]},
	 {exec_prefix,["@exec_prefix@"]},
	 {sysconfdir,["@sysconfdir@"]},
	 {localstatedir,["@localstatedir@"]},
	 {sbindir,["@sbindir@"]},
	 {logdir,["@logdir@"]},
	 {raddbdir,["./priv/test"]},
	 {radacctdir,["@radacctdir@"]}|_]).

test_all([quit]) ->
  (catch test_all()),
  halt().

test_all() ->
  lists:foreach(fun(T) ->
                    io:format("~n~s~n",[apply({?MODULE, T}, [info])]),
                    apply({?MODULE, T}, [do])
                end,
                [test_apache_parse, test_clike_parse, test_end_line,
		 test_walk_config, test_parse_config, test_property_server
		]
               ).

test_clike_parse(info) ->
  "*** Parse file ***";
test_clike_parse(do) ->
  FileConf = "./priv/test/clike.conf",
  Tests =
    [{"C-like config",
      [FileConf, clike, [{test_var1, "testvar1"}]],
      ?match(?CLIKE_MATCH)}
    ],
  process_tests(fun (F, M, E) -> property_file:parse(F, M, E) end, Tests).

test_apache_parse(info) ->
  "*** Parse file ***";
test_apache_parse(do) ->
  FileConf = "./priv/test/httpd.conf",
  Tests =
    [{"Apache config",
      [FileConf, apache],
      ?match(?APACHE_MATCH)}
    ],
  process_tests(fun (F, M) -> property_file:parse(F, M) end, Tests).

test_end_line(info) ->
  "*** Test Unix, DOS and Mac end lines ***";
test_end_line(do) ->
  Unix = property_file:parse("./priv/test/clike.conf", clike),
  Dos  = property_file:parse("./priv/test/clike.conf.dos", clike),
  Mac  = property_file:parse("./priv/test/clike.conf.mac", clike),
  Tests =
    [
     {"Unix", [Unix, Unix], true},
     {"DOS", [Dos, Unix], true},
     {"Mac", [Mac, Unix], true}
    ],
  process_tests(fun (List1, List2) -> List1 == List2 end, Tests).

test_walk_config(info) ->
  "*** Walk parsed config ***";
test_walk_config(do) ->
  FileConf = "./priv/test/clike.conf",
  Tests =
    [{"C-like config",
      [FileConf, clike, [{test_var1, "testvar1"}]],
      ?match(?WALK_CLIKE_MATCH)}
    ],
  process_tests(
    fun (F, M, E) ->
	WalkF = fun ([eof], [], Content, Seed) ->
		    Content;
		    (Path, Attrs, Content, Seed) ->
		    [{list_to_tuple(lists:reverse(Path)), Attrs}|Content]
		end,
	ConfigEntries = property_file:parse(F, M, E),
	property_file:walk_config(ConfigEntries, WalkF, [])
    end, Tests).

test_parse_config(info) ->
  "*** Load parsed config ***";
test_parse_config(do) ->
  FileConf = "./priv/test/clike.conf",
  Tests =
    [{"C-like config",
      [FileConf, clike, [{test_var1, "testvar1"}]],
      ?match(?LOAD_CLIKE_MATCH)}
    ],
  process_tests(
    fun (F, M, E) ->
	ConfigEntries = property_file:parse(F, M, E),
	property_file:parse_config(ConfigEntries, [property_file.test_mod1], [])
    end, Tests).

test_property_server(info) ->
  "*** Test property_server ***";
test_property_server(do) ->
  FileConf = "./priv/test/clike.conf",
  Tests =
    [{"Start server",
      [start_link,
       [{local, ptest}, property_file.test_mod1, ptest, []]
      ],
      ?match({ok, _})
     },
     {"Trying to remove config, config isn't loaded",
      [remove, [ptest]],
      {error, noconfig}
     },
     {"Load nonexistent default config",
      [load, [ptest, default]],
      {error,"error to open file NoName:no such file or directory"}
     },
     {"Load empty config",
      [load, [ptest, empty]],
      ok
     },
     {"Trying to load config, config is loaded",
      [load, [ptest, empty]],
      {error, ready}
     },
     {"Remove config, config is loaded",
      [remove, [ptest]],
      ok
     },
     {"Load config file",
      [load, [ptest, {file, "./priv/test/clike.conf"}]],
      ok
     },
     {"List config",
      [list, [ptest]],
      ?match({ok, _})
     },
     {"Parse config file",
      [parse, [ptest, "./priv/test/clike.conf"]],
      ?match({ok, _, ['property_file.test_mod1', 'property_file.test_mod2']})
     },
     {"Stop server",
      [stop, [ptest]],
      ok
     }
    ],
  process_tests(
    fun (F, As) ->
	apply(property_server, F, As)
    end, Tests).

%% Test help funs

process_tests(Tests) ->
  lists:foreach(fun ({Fun, FunTests}) ->
                    process_tests(Fun, FunTests)
                end, Tests).

process_tests(Fun, Tests) ->
  process_tests(Fun, Tests, 1).

%% Test argument is in the form {Msg, Args, TrueResult}
%% Msg        - test purpose
%% Args       - [Arg]
%% TrueResult - true result pattern
process_tests(Fun, [], _) ->
  ok;
process_tests(Fun, [Test|Tail], Num) ->
  {Msg, Args, ResPat} = Test,
  Status = case catch apply(Fun,Args) of
             Res when is_function(ResPat) ->
               case catch ResPat(Res) of
                 ok ->
                   [passed, ok];
                 _ ->
                   [Res, failed]
               end;
             ResPat ->
               [passed, ok];
             Other ->
               [Other, failed]
           end,
  io:format("Test ~2.2w: "++Msg++" -> ~p, ~p~n", [Num|Status]),
  process_tests(Fun, Tail, Num+1).
