%%%-------------------------------------------------------------------
%%% File    : property_supervisor_handler.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%%
%%% Created :  4 Jun 2004 by  <svg@surnet.ru>
%%%-------------------------------------------------------------------
%%
%% $Id$
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(property_supervisor_handler).

-behaviour(property_server).

-export([init/1, new_config/1, delete_config/1, config_file/1, bootstrap/2, store_entry/2, handle_op/2, config_to_list/1, init_postload/1]).

-record(state, {hlr,
		hlr_state,
		mastersup,
		appsup_id, %Id in MasterSup specification
		appsup,    %Pid
		appsup_static_children=[] %we haven't terminate them on reload
	       }).

init({MasterSup, AppSupId, Hlr, HlrArg}) ->
  {ok, HlrState} = Hlr:init(HlrArg),
  {ok, #state{hlr=Hlr, hlr_state=HlrState,
	      mastersup=MasterSup, appsup_id=AppSupId
	     }
  }.

init_postload(S=#state{appsup=AS}) when is_pid(AS) ->
  {ok, AS, S};
init_postload(S=#state{mastersup=MS, appsup_id=AI}) ->
  {value, {_, AppSup, _, _}} =
    lists:keysearch(AI, 1, supervisor:which_children(MS)),
  StaticChildren = [element(1, C) || C <- supervisor:which_children(AppSup)],
  {ok, AppSup, S#state{appsup=AppSup, appsup_static_children=StaticChildren}}.
  
config_file(S=#state{hlr=H, hlr_state=HS}) ->
  H:config_file(HS).

bootstrap(Config, S=#state{hlr=H, hlr_state=HS}) ->
  H:bootstrap(Config, HS).

store_entry(Entry, S=#state{hlr=H, hlr_state=HS}) ->
  {ok, NxtHS} = H:store_entry(Entry, HS),
  {ok, S#state{hlr_state=NxtHS}}.

new_config(S=#state{hlr=H, hlr_state=HS}) ->
  {ok, NxtHS} = H:new_config(HS),
  {ok, S#state{hlr_state=NxtHS}}.

delete_config(S=#state{hlr=H, hlr_state=HS,
		       appsup=AS, appsup_static_children=ASC}) ->
  (catch stop_appsup_children(AS, ASC)),  
  {ok, NxtHS} = H:delete_config(HS),
  {ok, S#state{hlr_state=NxtHS}}.

handle_op(Op, S=#state{hlr=H, hlr_state=HS}) ->
  {ok, Result, NxtHS} = H:handle_op(Op, HS),
  {ok, Result, S#state{hlr_state=NxtHS}}.

config_to_list(S=#state{hlr=H, hlr_state=HS}) ->
  H:config_to_list(HS).

stop_appsup_children(AppSup, Exclude) ->
  lists:foreach(fun (Child) ->
		    Id = element(1, Child),
		    case lists:member(Id, Exclude) of
		      false ->
			supervisor:terminate_child(AppSup, Id),
			supervisor:delete_child(AppSup, Id);
		      true ->
			ok
		    end
		end,
		supervisor:which_children(AppSup)),
  ok.
  
