;;; mmm-stl.el --- MMM STL submode
;;;
;;; $Id: mmm-stl.el,v 1.1 2003/05/14 16:43:20 vsekissov Exp $
;;;

;; Copyright (C) 2003 by Free Software Foundation, Inc.

;; Author:  <svg@surnet.ru>
;; Keywords: languages

;; This file is free software; you can redistribute it and/or modify
;; it under the terms of the GNU General Public License as published by
;; the Free Software Foundation; either version 2, or (at your option)
;; any later version.

;; This file is distributed in the hope that it will be useful,
;; but WITHOUT ANY WARRANTY; without even the implied warranty of
;; MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;; GNU General Public License for more details.

;; You should have received a copy of the GNU General Public License
;; along with GNU Emacs; see the file COPYING.  If not, write to
;; the Free Software Foundation, Inc., 59 Temple Place - Suite 330,
;; Boston, MA 02111-1307, USA.

;;; Commentary:

;;; 
;;; mmm-stl is a MMM submode for editing STL code
;;;

;;{{{

;;; Installation:
;;; Download and install
;;; mmm-mode         http://mmm-mode.sourceforge.net/
;;; css-mode         http://www.emacswiki.org/elisp/css-mode.el
;;; html-helper-mode http://www.gest.unipd.it/~saint/hth.html
;;;
;;; another alternative which you could try
;;; 
;;; psgml-mode http://sourceforge.net/projects/psgml/
;;; psgml-html http://www.cs.utexas.edu/users/suvrit/work/progs/psgml.html
;;; (only for Emacs, Xemacs has it)
;;; 
;;; Put this file somewhere in the emacs load path.
;;; Add to your .emacs:
;;;
;;; (load "mmm-stl")
;;;

;;}}}

;;{{{

;;; Invoking:
;;; If you are in different major mode invoke C-c-%-C-c and type `stl'
;;; C-c-%-s inserts pear of STL tags (*,  ,*)

;;}}}

;;; Code:

;;
(require 'html-helper-mode)

;; configure css-mode

(autoload 'css-mode "css-mode")
(add-to-list 'auto-mode-alist '("\\.css\\'" . css-mode))
(setq cssm-indent-function #'cssm-c-style-indenter)
(setq cssm-indent-level '2)
;;

(require 'mmm-mode)
(setq mmm-global-mode 'maybe)

;; set up an mmm group for fancy html editing
(mmm-add-group
 'stl
 '((stl-code
    :submode erlang-mode
    :face mmm-code-submode-face
    :front "(\\* *,?"
    :include-front t
    :back "\\*)"
    :include-back t
    :insert ((?s stl-code nil @ "(*," @ " " _ " " @ ",*)" @))
    )
   (stl-css-attribute
    :submode css-mode
    :face mmm-declaration-submode-face
    :front "style=\""
    :back "\"")))

;;
(add-to-list 'auto-mode-alist '("\\.stl\\'" . html-helper-mode))
;;
(add-to-list 'mmm-mode-ext-classes-alist '(html-helper-mode nil html-js))
(add-to-list 'mmm-mode-ext-classes-alist '(html-helper-mode nil embedded-css))
(add-to-list 'mmm-mode-ext-classes-alist '(html-helper-mode nil stl))

;;
(defvar mmm-stl-enabled nil)

;;(defadvice erlang-syntax-table-init
;;  (around mmm-stl-syntax-table-init first nil activate)
;;  ad-do-it
;;  (if mmm-stl-enabled
;;      (progn
;;	(modify-syntax-entry ?* "_ 1234" erlang-mode-syntax-table)
;;	(modify-syntax-entry ?, ". 14" erlang-mode-syntax-table)
;;	(modify-syntax-entry ?\( "() 3" erlang-mode-syntax-table)
;;	(modify-syntax-entry ?\) ")() 2" erlang-mode-syntax-table)
;;	(set-syntax-table erlang-mode-syntax-table)
;;	))
;;  )

(defadvice erlang-calculate-indent
  (around mmm-stl-erlang-calculate-indent first nil activate)
  (if mmm-stl-enabled
      (condition-case nil
	  ad-do-it
	(error nil))
    ad-do-it))

(defun mmm-stl-erlang-mode-hook ()
  (make-local-variable 'erlang-mode-syntax-table)
  (make-local-variable 'mmm-stl-enabled)
  (setq mmm-stl-enabled t)
  (erlang-syntax-table-init))

(add-hook 'mmm-html-helper-mode-hook 'mmm-stl-erlang-mode-hook)

;;; mmm-stl.el ends here

