%%%----------------------------------------------------------------------
%%% File    : stl_sup.erl
%%% Author  :  <svg@surnet.ru>
%%% Purpose : 
%%% Created :  3 Oct 2001 by  <svg@surnet.ru>
%%%----------------------------------------------------------------------

-module(stl_sup).
-vsn('$Id: stl_sup.erl,v 1.1.1.1 2003/05/05 20:20:58 vsekissov Exp $.').
-author('svg@surnet.ru').

-behaviour(application).
-behaviour(supervisor).

%% Supervisor exports
-export([start_link/0]).

%% application callbacks
-export([start/2, stop/1, config_change/3]).

%% supervisor callbacks
-export([init/1]).

-import(wslib.getopt).

-include("stl_app.hrl").

%%%----------------------------------------------------------------------
%%% API
%%%----------------------------------------------------------------------
start_link() ->
  supervisor:start_link({local, stl_sup}, stl_sup, []).

%%%----------------------------------------------------------------------
%%% Callback functions from application
%%%----------------------------------------------------------------------

%%----------------------------------------------------------------------
%% Func: start/2
%% Returns: {ok, Pid}        |
%%          {ok, Pid, State} |
%%          {error, Reason}   
%%----------------------------------------------------------------------

start(Type, StartArgs) ->
  case start_link() of
    {ok, SupPid} ->
      StlOpts = 
	case application:get_env(stl, options) of
	  {ok, Opts} ->
	    Opts;
	  Undefined ->
	    []
	end,
      case start_stl(?MODULE, StlOpts) of
	{ok, Pid} ->
	  {ok, SupPid};
	Other ->
	  Other
      end;
    Other ->
      Other
  end.

config_change(Changed, New, Removed) ->
  handle_change(Changed, New, Removed).

%%----------------------------------------------------------------------
%% Func: stop/1
%% Returns: any 
%%----------------------------------------------------------------------
stop(State) ->
  ok.

%%%----------------------------------------------------------------------
%%% Callback functions from supervisor
%%%----------------------------------------------------------------------

%%----------------------------------------------------------------------
%% Func: init/1
%% Returns: {ok,  {SupFlags,  [ChildSpec]}} |
%%          ignore                          |
%%          {error, Reason}   
%%----------------------------------------------------------------------
init([]) ->
  {ok,{?SUPER_SPEC, []}}.

%%%----------------------------------------------------------------------
%%% Internal functions
%%%----------------------------------------------------------------------

handle_change(Changed, New, Removed) ->
  handle_change([Changed ++ New]).

handle_change([]) ->
  ok;

handle_change([{options, Opts}|_]) ->
  kill_stl(),
  start_stl(?MODULE, Opts),
  ok;
handle_change([_|Rest]) ->
  handle_change(Rest).

kill_stl() ->
  supervisor:terminate_child(?MODULE, stl),
  supervisor:delete_child(?MODULE, stl).

start_stl(Sup, Opts) ->
  Dir = 
    case application:get_env(stl, dir) of
      undefined ->
	code:priv_dir(stl) ++ "/stl";
      {ok, V} ->
	string:strip(V, both, 16#22)
    end,
  StlOpts =
    getopt:options(Opts, [{dir, Dir, list}]),
  supervisor:start_child(?MODULE, ?STL_SPEC([{local, stl}, StlOpts])).
