%%% File    : aaa_schema.hrl
%%% Author  :  <svg@surnet.ru>
%%% Purpose : 
%%% Created : 13 Nov 2002 by  <svg@surnet.ru>

%%
%% $Id$
%% 
%%*

-include("mnesiadb.hrl").

-record(time_limit, {day=unlimited, 
		     week=unlimited, 
		     month=unlimited,
		     year=unlimited,
		     total=unlimited
		    }).

-record(access_time, {restday_mask=0,
		      restday_disabled,
		      workday_disabled
		     }).

-record(aaa_global, {id, value}).

-record(aaa_party, {id,
		    party_type_id,
		    name,
		    attrs=[]
		   }).

-record(aaa_user, {id,
		   party_id,
		   password={cleartext, undefined},
		   enabled=false,
		   disabled_datetime,
		   group_member_id,
		   default_service, % permit | deny | undefined
		   attrs=[]
		  }).

-record(aaa_service, {id, % {UserId, Service}
			  % Service = "shell" | "ppp" | "slip" | "exec" | ...
		      party_relationship_id,
		      service_id, % surrogate key, could be duplicated
		      status,            % open | closed
		      default_attr=deny, % permit | deny
		      attrs,             % [msg_arg()]
					 % | [{protocol(), [msg_arg()]}]
					 % | [{Perm, regexp()}]
					 % Perm = permit | deny
		      tariff_id,
		      info=""
		     }).

-record(aaa_service_constraint, {service_id,  % aaa_service:service_id
				 caller_phone,% string()
				 access_time, % #access_time{},
				 time_limit,  % #time_limit{},
				 max_sessions,% integer()
				 nas          % [ip()]
				}).

-record(aaa_nas, {ip,
		  provider_id,
		  info="",
		  attrs=[]
		 }).

%% service element
-record(se, {id,
	     tariff_id,
	     provider_id,
	     provider_host % peer {Ip, Port}
	    }).

%% service consumer
-record(sc, {user_id,
	     subscriber_id,
	     context_id,
	     access_point %  peer {Ip, Port|Phone..etc}
	    }).

%% service session
-record(ss, {id, % {node ++ task_id ++ start_time}
	     service,
	     record_type, % start | stop | interim | start_stop
	     start_time,  % UTC
	     end_time,    % UTC
	     terminate_cause,
	     se,
	     sc,
	     ue
	    }).

%% chargeable event
-record(ce, {charge_time,
	     quantity,
	     amount,
	     discount,
	     tariff_id,
	     customer_relationship_id,
	     measured_unit,
	     chargeable_unit,
	     currency_type
	    }).

%% accounting record
-record(aaa_ar, {account_id,
		 entry_time,
		 ss,
		 ce
		}).

-record(aaa_account, {id, %{party_relationship_id, currency_type}
		      debit,
		      credit,
		      credit_limit=0,
		      status % open|closed
		     }).

-record(aaa_tariff, {id,
		     charge_type,
		     currency_type,
		     info
		    }).

-record(aaa_tariff_ce, {id, % {tariff_id, from_date}
			seq_num,
			chargeable_unit, % duration, etc
			measured_unit,   % seconds, minutes, bytes, ...
			%% fun (SS#ss, Ce#ce) -> {ok, {ProviderId, CustomerId}}
			customer_relationship_fun,
			%% fun (SS#ss, Ce#ce) -> {ok, Quantity}
			quantity_fun,
			%% fun (SS#ss, Ce#ce) -> {ok, {Amount, CurrencyType}}
			amount_fun,
			%% fun (SS#ss, Ce#ce) -> {ok, Discount}
			discount_fun,
			info
		       }).

%% internet access
-record(ue_ia, {duration,
		input_octets,
		output_octets,
		input_bandwidth,
		output_bandwidth,
		transport_proto,  % PPP, TCP, etc
		nas_port_type     % RFC 2865 a:5.41
	       }).

%%-record(aaa_acc_session, {id, %{node, start_time, task_id},
%%			  user_id,    % UserId
%%			  service,    % Service
%%			  start_time, % human readable
%%			  entry_time, % human readable
%%			  nas_peer,   % {Ip, Port}
%%			  user_peer,  % {Ip, Port}
%%			  tariff_id,
%%			  volume,
%%			  phase,      % start | continue | stop
%%			  attrs=[]
%%			 }).
