-ifndef(TACACS_DEF).

-define(TACACS_DEF, 1).

-author('svg@surnet.ru').

-include("aaa_records.hrl").

-ifndef(CONFIG_FILE).
-define(CONFIG_FILE, "tacacs.conf").
-endif.

-define(DEFAULT_PORT, 49).
-define(TCP_TIMEOUT, 180000).
-define(FSM_TIMEOUT, 180000).
-define(SERVER_TIMEOUT, 180000).

-define(TAC_PLUS_VERSION_0, 16#c0). %tacacs version + monor version default
-define(TAC_PLUS_VERSION_1, 16#c1). %tacacs version + minor version one

%Packet type
-define(TAC_PLUS_AUTHEN, 16#01). %Authentication
-define(TAC_PLUS_AUTHOR, 16#02). %Authorization
-define(TAC_PLUS_ACCT,   16#03). %Accounting
%%%Authentication start
%Action
-define(TAC_PLUS_AUTHEN_LOGIN,    16#01).
-define(TAC_PLUS_AUTHEN_CHPASS,   16#02).
-define(TAC_PLUS_AUTHEN_SENDPASS, 16#03). %deprecated
-define(TAC_PLUS_AUTHEN_SENDAUTH, 16#04).
%Privilege level
-define(TAC_PLUS_PRIV_LVL_MAX , 16#0f).
-define(TAC_PLUS_PRIV_LVL_ROOT, 16#0f).
-define(TAC_PLUS_PRIV_LVL_USER, 16#01).
-define(TAC_PLUS_PRIV_LVL_MIN , 16#00).
%Authentication type
-define(TAC_PLUS_AUTHEN_TYPE_ASCII , 16#01).
-define(TAC_PLUS_AUTHEN_TYPE_PAP   , 16#02).
-define(TAC_PLUS_AUTHEN_TYPE_CHAP  , 16#03).
-define(TAC_PLUS_AUTHEN_TYPE_ARAP  , 16#04).
-define(TAC_PLUS_AUTHEN_TYPE_MSCHAP, 16#05).
%Service
-define(TAC_PLUS_AUTHEN_SVC_NONE   , 16#00).
-define(TAC_PLUS_AUTHEN_SVC_LOGIN  , 16#01).
-define(TAC_PLUS_AUTHEN_SVC_ENABLE , 16#02).
-define(TAC_PLUS_AUTHEN_SVC_PPP    , 16#03).
-define(TAC_PLUS_AUTHEN_SVC_ARAP   , 16#04).
-define(TAC_PLUS_AUTHEN_SVC_PT     , 16#05).
-define(TAC_PLUS_AUTHEN_SVC_RCMD   , 16#06).
-define(TAC_PLUS_AUTHEN_SVC_X25    , 16#07).
-define(TAC_PLUS_AUTHEN_SVC_NASI   , 16#08).
-define(TAC_PLUS_AUTHEN_SVC_FWPROXY, 16#09).

%%%Authentication REPLY
%Status
-define(TAC_PLUS_AUTHEN_STATUS_PASS   , 16#01).
-define(TAC_PLUS_AUTHEN_STATUS_FAIL   , 16#02).
-define(TAC_PLUS_AUTHEN_STATUS_GETDATA, 16#03).
-define(TAC_PLUS_AUTHEN_STATUS_GETUSER, 16#04).
-define(TAC_PLUS_AUTHEN_STATUS_GETPASS, 16#05).
-define(TAC_PLUS_AUTHEN_STATUS_RESTART, 16#06).
-define(TAC_PLUS_AUTHEN_STATUS_ERROR  , 16#07).
-define(TAC_PLUS_AUTHEN_STATUS_FOLLOW , 16#21).
%Flags
-define(TAC_PLUS_REPLY_FLAG_NOECHO, 16#01).

%%%Authentication CONTINUE
%%Flags
-define(TAC_PLUS_CONTINUE_FLAG_ABORT, 16#01).
%%%Authorisation
%%authen_method
-define(TAC_PLUS_AUTHEN_METH_NOT_SET    , 16#00).
-define(TAC_PLUS_AUTHEN_METH_NONE       , 16#01).
-define(TAC_PLUS_AUTHEN_METH_KRB5       , 16#02).
-define(TAC_PLUS_AUTHEN_METH_LINE       , 16#03).
-define(TAC_PLUS_AUTHEN_METH_ENABLE     , 16#04).
-define(TAC_PLUS_AUTHEN_METH_LOCAL      , 16#05).
-define(TAC_PLUS_AUTHEN_METH_TACACSPLUS , 16#06).
-define(TAC_PLUS_AUTHEN_METH_GUEST      , 16#08).
-define(TAC_PLUS_AUTHEN_METH_RADIUS     , 16#10).
-define(TAC_PLUS_AUTHEN_METH_KRB4       , 16#11).
-define(TAC_PLUS_AUTHEN_METH_RCMD       , 16#20).
%%reply status
%%This field indicates the authorization status
-define(TAC_PLUS_AUTHOR_STATUS_PASS_ADD  , 16#01).
-define(TAC_PLUS_AUTHOR_STATUS_PASS_REPL , 16#02).
-define(TAC_PLUS_AUTHOR_STATUS_FAIL      , 16#10).
-define(TAC_PLUS_AUTHOR_STATUS_ERROR     , 16#11).
-define(TAC_PLUS_AUTHOR_STATUS_FOLLOW    , 16#21).

%%% Accounting
%% Request flags
-define(TAC_PLUS_ACCT_FLAG_MORE     , 16#01).
-define(TAC_PLUS_ACCT_FLAG_START    , 16#02).
-define(TAC_PLUS_ACCT_FLAG_STOP     , 16#04).
-define(TAC_PLUS_ACCT_FLAG_WATCHDOG , 16#08).
%% Reply status
-define(TAC_PLUS_ACCT_STATUS_SUCCESS , 16#01).
-define(TAC_PLUS_ACCT_STATUS_ERROR   , 16#02).
-define(TAC_PLUS_ACCT_STATUS_FOLLOW  , 16#21).

%%Common for requests user identification field
-record(userid, {user,
		 peer={"undefined", "undefined"} % {UserIP, UserPort}
		}).
-record(serverid, {peer, skey}).
%% Tacacs session
-record(session, {ver=?TAC_PLUS_VERSION_0,
		  type,        % TAC_PLUS_AUTHEN|TAC_PLUS_AUTHOR|TAC_PLUS_ACCT
		  seq_no,
		  unencrypt,
		  single_conn,
		  sess_no,
		  peer         % {ServerIP, ServerPort}
		 }).
%%
-record(error_reply, {}).
-record(authen, {priv_lvl=16#0, type=ascii, service=none, method=tacacs}).
%%authentication
-record(authen_start, {action=16#0,
		       authen=#authen{},
		       user=#userid{},
		       data=""
		      }).
-record(authen_reply,   {status=16#0,
			 flags=16#0,
			 server_msg="",
			 data=""
			}).
-record(authen_continue,{flags=16#0, user_msg="", data=""}).
%%authorisation
-record(author_request,   {authen=#authen{}, user=#userid{}, args=[]}).
-record(author_response,  {status=16#0, server_msg="", data="", args=[]}).
%%accounting
-record(acct_request,   {flags=16#0,
			 authen=#authen{},
			 user=#userid{},
			 args=[]
			}).
-record(acct_reply,   {status=16#0, server_msg="", data=""}).
%%common client request
-record(client_request,  {action, user=#userid{}, pass="",
                          authen=#authen{},
                          flags, user_msg="", data="", args=[]}).

-endif. % TACACS_DEF
