%%% File    : account.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created :  8 Oct 2004 by  <svg@surnet.ru>

%%
%% $Id$
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(aaa.account).
-author('svg@surnet.ru').

-export([get_rec/2, find_rec/2, balance/1, check_balance/2, withdraw/2]).

-import(mnesia).
-import(wslib.mnesiadb).
-import(wslib.props).
-import(lists).
-import(erlang).

-include("aaa_schema.hrl").

check_balance(service, KVs) ->
  [PartyRelationshipId, TariffId] =
    props:gv_map([party_relationship_id, tariff_id], KVs),
  [#aaa_tariff{currency_type=CT}] = mnesia:dirty_read(aaa_tariff, TariffId),
  [#aaa_account{debit=D, credit=C, credit_limit=CL}] =
    mnesia:dirty_read(aaa_account, {PartyRelationshipId, CT}),
  ((C-D) >= CL).

balance(Key={_PartyRelationshipId, _CurrencyType}) ->
  case mnesia:dirty_read(aaa_account, Key) of
    [Account] ->
      balance(Account);
    [] ->
      undefined
  end;
balance(_Acc=#aaa_account{debit=D, credit=C}) ->
  C-D.

withdraw(Ss=#ss{record_type=RT}, CEs) ->
  TrF =
    fun () ->
	EntryTime = erlang:localtime(),
	lists:foldl(
	  fun (Ce=#ce{amount=Amount,
		      discount=Discount,
		      customer_relationship_id=CRId,
		      currency_type=CT
		     },
	       Accs) when Amount >= 0;
			  Discount >= 0 ->
	      AccountId = {CRId, CT},
	      AR = #aaa_ar{account_id=AccountId, 
			   entry_time=EntryTime,
			   ss=Ss#ss.id,
			   ce=Ce
			  },
	      {ok, Account} =
		mnesiadb:fetch(mnesia:read(aaa_account, AccountId, write)),
	      #aaa_account{debit=D} = Account,
	      NxtAccount = Account#aaa_account{debit=D+Amount+Discount},
	      mnesia:write(aaa_ar_session, Ss, write),
	      if (RT == stop) or (RT == start_stop) ->
		  mnesia:write(NxtAccount),
		  mnesia:write(AR);
		 (RT == interim) or (RT == start) ->
		  %% FixMe - interim accounting
		  ok
	      end,
	      case is_balance_changed(Account, NxtAccount) of
		true ->
		  [NxtAccount|Accs];
		false ->
		  Accs
	      end
	  end, [], CEs)
    end,
  case  mnesia:transaction(TrF) of
    {atomic, Accounts} ->
      lists:foreach(fun (A=#aaa_account{id=Id}) ->
			E = {balance_changed, Id, balance(A)},
			account_event(E)
		    end, Accounts),
      ok;
    {aborted, Error} ->
      exit(Error)
  end.

get_rec(PartyRelId, CurrencyType) ->
  {ok, Acc} = find_rec(PartyRelId, CurrencyType),
  Acc.

find_rec(PartyRelId, CurrencyType) ->
  TrF = fun () ->
	    mnesiadb:fetch(mnesia:dirty_read(aaa_account,
					     {PartyRelId, CurrencyType})
			  )
	end,
  mnesia:transaction(TrF).

is_balance_changed(Old=#aaa_account{credit_limit=OldL},
		   New=#aaa_account{credit_limit=NewL}) ->
  case {(balance(Old) < OldL), (balance(New) < NewL)} of
    {false, true} ->
      true;
    _ ->
      false
  end.

account_event(_Event) ->
  ok.
