%%%-------------------------------------------------------------------
%%% File    : etacacs_pam.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%%
%%% Created :  7 Mar 2004 by  <svg@surnet.ru>
%%%-------------------------------------------------------------------
%%
%% $Id$
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(aaa.dialup_law).

-author('svg@surnet.ru').

-include("tacacs.hrl").
-include("aaa_schema.hrl").

-define(SEC_IN_DAY, 86400).
-define(EXPIRE_WARN, 14*?SEC_IN_DAY).

-define(cmd_arg,          "cmd").
-define(service_arg,      "service").
-define(protocol_arg,     "protocol").
-define(cmd_param_arg,    "cmd-arg").
%% !!!FixME!!!
-define(caller_phone_arg, "!!!add_me!!!").
-define(skip_args, [?cmd_arg,
		    ?service_arg,
		    ?protocol_arg,
		    ?cmd_param_arg
		   ]).

-export([parse_config/4]).
-export([initial_context/1, sent/2, arrived/2, post_condition/4]).
%% internal exports
-export([authenticate/1, authorize/1, account/1]).

-import(calendar).
-import(lists).
-import(regexp).
-import(wslib.passwd).
-import(wslib.listn).
-import(wslib.str).
-import(wslib.cal).
-import(wslib.log).
-import(wslib.props).
-import(aaa.db).
-import(lgi).
-import(tac_event).
-import(tac_law).
-import(tac_lib).
-import(tac_session).
-import(taclient).

%%
%%  property_file callbacks
%%

parse_config([N=proxy, aaa_law], [], Content, _) ->
  [Ip, Port, SKey] = props:gv_map([address, {port, 49}, skey], Content),
  {N, #serverid{peer={Ip, Port}, skey=SKey}};
parse_config([N=address, proxy, aaa_law], [Str], [], _) when is_list(Str) ->
  {N, Str};
parse_config([N=skey, proxy, aaa_law], [Str], [], _) when is_list(Str) ->
  {N, Str};
parse_config([N=port, proxy, aaa_law], [Int], [], _) when is_integer(Int) ->
  {N, Int}.
%%
%% Law callbacks
%%
initial_context(Props) ->
  Props.

%%
%% Authentication
%%
sent(_To, {Action, Data}) ->
  Uid=#userid{user=UserId, peer=UserPeer} = element(1, Data),
  #session{peer={NasIp, _}} = element(2, Data),
  case aaa_user(NasIp, UserId) of
    undefined ->
      lgi:set@(aaa_user, Uid),
      {RetStatus, RetArgs, SrvMsg, UserData} =
	proxy(Action, Data, lgi:get@(proxy)),
      reply(RetStatus, RetArgs, SrvMsg, UserData);
    {AaaUserId, AaaUser} ->
      lgi:set@(aaa_user, Uid#userid{user=AaaUserId}),
      NewData = setelement(1, Data, [{peer, UserPeer}|AaaUser]),
      ?MODULE:Action(NewData)
  end.

post_condition(sent, _, {Action, Data}, _) ->
  DoLog = lgi:get@(generate_aaa_event),
  AddArgs = logging_args(Action, Data),
  if DoLog ->
      EventArgs =
	case lgi:get@(law_result) of
	  {Status, Reason} ->
	    [{permit, trans_status(Status)}, {reason, Reason}|AddArgs];
	  Status ->
	    [{permit, trans_status(Status)}|AddArgs]
	end,
      NewUid = lgi:get@(aaa_user),
      NewData = setelement(1, Data, NewUid),
      tac_event:Action(NewData, EventArgs);
     true ->
      ok
  end,
  lgi:rm@(aaa_user),
  lgi:rm@(law_result);
post_condition(_, _, _, _) ->
  lgi:rm@(aaa_user),
  lgi:rm@(law_result),
  ok.

logging_args(authenticate,{_, _, _, login, nopassword}) ->
  [{nopassword, true}];
logging_args(_, _) ->
  [].

trans_status(Ok) when Ok == true; Ok == ok; Ok == replace ->
  true;
trans_status(V) ->
  V.

arrived(_From, _Msg) ->
  lgi:deliver().

authenticate({UserAttrs, _Session, Authen, ReqType, ReqData}) ->
  authenticate(ReqType, UserAttrs, Authen, ReqData).

authenticate(enable, _UserAttrs, _A=#authen{priv_lvl=PrivLvl}, Pass) ->
  {enable, Enable} = db:lookup({enable, PrivLvl}),
  do_authen(Enable, ascii, Pass);
authenticate(login, UserAttrs, A=#authen{type=AuthenType}, Pass) ->
  do_authen(UserAttrs, AuthenType, Pass);
authenticate(chpass, UserAttrs, Authen, {OldPass, NewPass}) ->
  reject("Change password is not implemented").

do_authen(UserAttrs, AuthenType, Pass) ->
  [Id, Cipher, Enabled, DisabledDate] =
    props:gv_map([id, password, enabled, disabled_datetime], UserAttrs),
  (Enabled == true) orelse reject(disabled, "User disabled"),
  authen_check_password(AuthenType, Id, Pass, Cipher),
  authen_check_expires(DisabledDate),
  accept("").

authen_check_expires(undefined) ->
  true;
authen_check_expires(ExpTime) ->
  CurTime = calendar:local_time(),
  Del = 
    calendar:datetime_to_gregorian_seconds(ExpTime) -
    calendar:datetime_to_gregorian_seconds(CurTime),
  if
    Del =< 0 ->
      reject(expired, "Password has expired");
    Del < ?EXPIRE_WARN ->
      accept("Password will expire soon");
    true ->
      true
  end.

authen_check_password(_, User, Pass, {file, _Fname}) ->
  authen_not_implemented(file, User);
authen_check_password(_, User, Pass, skey) ->
  authen_not_implemented(skey, User);
authen_check_password(pap, User, AuthenData, SecretData) ->
  authen_check_password(ascii, User, AuthenData, SecretData);
authen_check_password(AuthenType, _User, AuthenData, SecretData) ->
  passwd:authenticate(AuthenType, AuthenData, SecretData)
    orelse reject(bad_password, "Bad password").

authen_not_implemented(Opt, User) ->
  log:write_error("password option ~p not implemented, user ~p", [Opt, User]),
  reject("").

%%
%% Authorization
%%
authorize({UserAttrs, Session, Authen, NasArgs}) ->
  UserId = props:gv(id, UserAttrs),
  ServiceType = arg_val(?service_arg, NasArgs, ""),
  case ServiceType of
    "shell" ->
      case arg_val(?cmd_arg, NasArgs) of
	undefined ->
	  reject(no_cmd_args, "No command argument specified");
	"" ->
	  authorize_service(UserId, "exec", NasArgs);
	Cmd ->
	  authorize_cmd(UserId, Cmd, NasArgs)
      end;
    _ ->
      authorize_service(UserId, ServiceType, NasArgs)
  end.

%%----------------------------------------------------------------------
%% Return = {Status, ArgList, SrvMsg}
%% Status = ok | false | replace | error
%% ArgList = [#msg_arg{}, ...]
%% SrvMsg = string()
%%----------------------------------------------------------------------
%% FixME - check account
authorize_cmd(UserId, Cmd, NasArgs) ->
  CmdArgStr =
    str:join([V || X=#msg_arg{value=V, name=N} <- NasArgs, N == ?cmd_param_arg],
	     " "),
  case db:lookup_command(UserId, Cmd) of
    undefined ->
      reject(no_exists, "");
    permit ->
      accept("");
    deny ->
      reject(nomatch, "");
    PermRegexps ->
      case match_command(CmdArgStr, PermRegexps) of
	permit ->
	  accept("");
	deny ->
	  reject("")
      end
  end.

authorize_service(UserId, "", NasArgs) ->
  error("No identifiable service in authorization request");
authorize_service(UserId, ServiceType, NasArgs) ->
  case db:lookup_service(UserId, ServiceType) of
    permit ->
      accept("");
    deny ->
      reject("");
    undefined ->
      reject(no_exists, "");
    {service, Service} ->
      authorize_service(UserId, Service, ServiceType, NasArgs)
  end.

authorize_service(UserId, Service, ServiceType, NasArgs) ->
  [State, CallerPhone, AccessTime, TimeLimit, MaxSess, DefAttr, Attrs] =
    props:gv_map([status,
		  caller_phone,
		  access_time,
		  time_limit,
		  max_sessions,
		  default_attr,
		  attrs
		 ], Service),
  author_check_phone(arg_val(?caller_phone_arg, NasArgs), CallerPhone),
  author_check_access_time(AccessTime),
  author_check_balance(Service),
  %% author_check_time_limit(TimeLimit),
  author_check_max_sessions(UserId, MaxSess),
  Args = author_service_args(ServiceType, NasArgs, Attrs),
  {RetStatus, RetArgs} =
    tac_lib:author_check_args(NasArgs, Attrs, DefAttr, ?skip_args),
  lgi:set@(law_result, RetStatus),
  reply(RetStatus, RetArgs, "", "").

author_check_balance(Service) ->
  case account:check_balance(service, Service) of
    false ->
      reject(account_balance, "Account balance is smaller than credit limit");
    true ->
      true
  end.

author_check_max_sessions(_, undefined) -> true;
author_check_max_sessions(UserId, MS) ->
  case (tac_session:total_counter_value(UserId) =< MS) of
    false ->
      reject(max_sessions, "Sessions limit exceeded");
    true ->
      true
  end.

author_check_access_time(undefined) -> true;
author_check_access_time(AT=#access_time{restday_mask=RM,
					 restday_disabled=RD,
					 workday_disabled=WD
					}) ->
  DateTime={_, Time}=erlang:localtime(),
  CheckInterval =
    case cal:week_is_weekday(DateTime, RM) of
      true ->
	WD;
      false ->
	RD
    end,
  author_check_access_time(Time, CheckInterval).

author_check_access_time(Time, undefined) ->
  true;
author_check_access_time(Time, CheckInterval) ->
  true = cal:is_time_between(Time, CheckInterval).

%% we match the end of the phone
author_check_phone(_, Any)  when Any == undefined;
				 Any == any;
				 Any == [] ->
  true;
author_check_phone(Phone, Regexps) ->
  true =
    lists:any(fun (Regexp) ->
		  case regexp:first_match(Phone, ".*"++Regexp++"\$") of
		    {match, _, _} ->
		      true;
		    _ ->
		      false
		  end
	      end, Regexps).

author_service_args("ppp", NasArgs, Attrs) ->
  NasProto = arg_val(?protocol_arg, NasArgs),
  if NasProto == "lcp" ->
      ProtoArgs = props:gv(NasProto, Attrs, []);
     true ->
      case props:gv(NasProto, Attrs) of
	undefined ->
	  reject(nomatch_protocol, "");
	ProtoArgs ->
	  ProtoArgs
      end
  end;
author_service_args("exec", _, Attrs) ->
  Attrs.

match_command(_Cmd, []) ->
  deny;
match_command(Cmd, [{Perm, Regexp}|Rest]) ->
  case regexp:match(Cmd, Regexp) of
    {match, _, _} ->
      Perm;
    Error={error, _} ->
      error("bad regular expression in command pattern");
    _ ->
      match_command(Cmd, Rest)
  end.

arg_val(Key, Args) ->
  arg_val(Key, Args, undefined).

arg_val(Key, Args, Default) ->
  case lists:keysearch(Key, #msg_arg.name, Args) of
    {value, Arg} ->
      Arg#msg_arg.value;
    false ->
      Default
  end.

%%
%% Accounting
%%
account({UserAttrs, Session, Authen, Phase=start, Args}) ->
  UserId = props:gv(id, UserAttrs),
  tac_session:increment_counter(UserId),
  accept("");
account({UserAttrs, Session, Authen, Phase=stop, Args}) ->
  UserId = props:gv(id, UserAttrs),
  tac_session:decrement_counter(UserId),
  accept("");
account(_) ->
  accept("").

%%
%% Proxy
%%

proxy(Action, Data, ServerId) ->
  {LawResult, Ret} =
    case proxy1(Action, Data, ServerId) of
      Status when is_atom(Status) ->
	{Status, {Status, [], "", ""}};
      {error, Error} ->
	{{error, str:to_str(Error)}, {error, [], "unspecified error", ""}};
      Result={Status, _, "", _} ->
	{Status, Result};
      Result={Status, _, SrvMsg, _} ->
	{{Status, SrvMsg}, Result}
    end,
  lgi:set@(law_result, LawResult),
  Ret.

proxy1(authenticate, {Uid, _, Authen, Action, Data}, ServerId) ->
  Data1 = if Data == nopassword -> "";
	     true -> Data
	  end,
  taclient:Action(ServerId, Uid, Authen, Data1, [{debug, lgi:get@(debug)}]);
proxy1(authorize, {Uid, _, Authen, NasArgs}, ServerId) ->
  taclient:authorize(ServerId, Uid, Authen, NasArgs, []);
proxy1(account, {Uid, _, Authen, Phase, Args}, ServerId) ->
  taclient:account(ServerId, Uid, Authen, Phase, Args, []).

%%%

accept(Msg) ->
  lgi:set@(law_result, true),
  tac_law:accept(Msg).

reject(Reason, Msg) ->
  lgi:set@(law_result, {false, Reason}),
  tac_law:reject(Msg).
  
reject(Msg) ->
  lgi:set@(law_result, false),
  tac_law:reject(Msg).

error(Msg) ->
  lgi:set@(law_result, {false, Msg}),
  tac_law:error(Msg).

reply(Status, Args, ServerMsg, UserData) ->
  tac_law:reply(Status, Args, ServerMsg, UserData).

aaa_user(NasIp, UserId) ->
  {nas, Nas} = db:lookup({nas, NasIp}, {nas, []}),
  {user, User} = db:lookup({user, UserId}, {user, []}),
  NasAttrs = props:gv(attrs, Nas, []),
  NasUser = props:gv(aaa_user, NasAttrs),
  case {NasUser, User} of
    {undefined, []} ->
      undefined;
    {undefined, _} ->
      {UserId, User};
    _ ->
      NasUserId =  props:gv(id, NasUser),
      {NasUserId, NasUser ++ props:delete(id, User)}
  end.

serice_type_to_db("ppp")   -> "dialup";
serice_type_to_db("slip")  -> "dialup";
serice_type_to_db("pop3")  -> "email";
serice_type_to_db("cmd")   -> "command";
serice_type_to_db(Other) -> Other.

aaa_status_arg(ok)     -> {permit, true};
aaa_status_arg(true)   -> {permit, true};
aaa_status_arg(error)  -> {permit, false};
aaa_status_arg(false)  -> {permit, false}.
