%%% File    : odbcdb_schema.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created : 14 Apr 2004 by  <svg@surnet.ru>

%%
%% $Id$
%%
%%**
%% 
%% @doc Schema generation for <code>odbcdb</code>
%% @end
%% @see odbcdb
%%*
-module(aaa.odbcdb_schema).

-export([install/0]).

-import(odbcdb_schema).

-define(table_attrs(T, As), {{table, T, attrs}, As}).
-define(pkey(T, PKs), {{table, T, primary_key}, PKs}).
-define(seq(T, SN),   {{table, T, keygenerator}, {sequence, SN}}).
-define(seq1(T, SN, A),   {{table, T, keygenerator}, {sequence, SN, A}}).

install() ->
  odbcdb_schema:add_properties(
    [?table_attrs(role_type, [id, has_table, description, {form, [{type, atom}]}]),
     ?pkey(role_type, [id]),
     
     ?table_attrs(role_type_attr, [role_type_id, {attr_name, [{type, atom}]}]),
     ?pkey(role_type_attr, [role_type_id, attr_name]),

     ?table_attrs(location_role_type, [id, has_table, description, {form, [{type, atom}]}]),
     ?pkey(location_role_type, [id]),

     ?table_attrs(charge_type, [{id, [{type, atom}]},
				 has_table,
				 description,
				 {form, [{type, atom}]}]),
     ?pkey(charge_type, [id]),

     ?table_attrs(currency_type, [{id, [{type, atom}]},
				   has_table,
				   description,
				   {form, [{type, atom}]}]),
     ?pkey(currency_type, [id]),
     
     ?table_attrs(location_role_type_attr, [location_role_type_id, {attr_name, [{type, atom}]}]),
     ?pkey(location_role_type_attr, [location_role_type_id, attr_name]),

     ?table_attrs(contact_mech_role_type, [id, has_table, description, {form, [{type, atom}]}]),
     ?pkey(contact_mech_role_type, [id]),
     
     ?table_attrs(contact_mech_role_type_attr, [contact_mech_role_type_id, {attr_name, [{type, atom}]}]),
     ?pkey(contact_mech_role_type_attr, [contact_mech_role_type_id, attr_name]),

     ?table_attrs(party_type, [id, has_table, description, {form, [{type, atom}]}]),
     ?pkey(party_type, [id]),

     ?table_attrs(party_type_attr, [party_type_id, {attr_name, [{type, atom}]}]),
     ?pkey(party_type_attr, [party_type_id, attr_name]),

     ?table_attrs(party, [{id, [{type, integer}]}, party_type_id, name]),
     ?pkey(party, [id]),
     ?seq(party, party_id_seq),

     ?table_attrs(party_role, [party_id, role_type_id]),
     ?pkey(party_role, [party_id, role_type_id]),

     ?table_attrs(party_relationship_category, [id, description]),
     ?pkey(party_relationship_category, [id]),

     ?table_attrs(party_relationship_type, [id, role_type_id_from_valid, role_type_id_to_valid, description, category_id, {form, [{type, atom}]}]),
     ?pkey(party_relationship_type, [id]),

     ?table_attrs(party_relationship, [{id, [{type, integer}]},
				       party_id_from,
				       role_type_id_from,
				       party_id_to,
				       role_type_id_to,
				       {from_date, [{type, datetime}]},
				       {thru_date, [{type, datetime}]},
				       party_relationship_type_id
				      ]),
     ?pkey(party_relationship, [party_id_from, role_type_id_from, party_id_to, role_type_id_to, from_date]),
     ?seq1(party_relationship, party_relationship_id_seq, id),

     ?table_attrs(party_location, [party_id, postal_address_id, from_date, thru_date, location_role_type_id]),
     ?pkey(party_location, [party_id, location_role_type_id, from_date]),

     ?table_attrs(party_contact_mech, [party_id, contact_means_id, from_date, thru_date, contact_mech_role_type_id]),
     ?pkey(party_contact_mech, [party_id, contact_mech_role_type_id, from_date]),

     ?table_attrs(person, [party_id, last_name, first_name, middle_name, personal_title, gender, comments]),
     ?pkey(person, [party_id]),

     ?table_attrs(security_permission, [id, comment]),
     ?pkey(security_permission, [id]),

     ?table_attrs(security_group, [id, comment]),
     ?pkey(security_group, [id]),

     ?table_attrs(security_group_permission, []),
     ?pkey(security_group_permission, [group_id, permission_id]),

     ?table_attrs(postal_address, [{id, [{type, integer}]}, city, address1, address2, postal_code]),
     ?pkey(postal_address, [id]),
     ?seq(postal_address, postal_address_id_seq),

     ?table_attrs(contact_means_type, [id, description, comment]),
     ?pkey(contact_means_type, [id, description]),

     ?table_attrs(contact_means, [{id, [{type, integer}]}, type_id, name, type_description, country_code, area_code, contact_number, email_address]),
     ?pkey(contact_means, [id]),
     ?seq(contact_means, contact_means_id_seq),

     ?table_attrs(user_login,
            [login_id, password, password_hint, party_id,
             {disabled_datetime, [{type, datetime}]},
             {enabled,       [{type, boolean}]},
             login_attempts
            ]),
     ?pkey(user_login, [login_id]),

     ?table_attrs(user_login_security_group, [login_id, group_id]),
     ?pkey(user_login_security_group, [login_id, group_id]),

     ?table_attrs(service_type, [id, has_table, description, {form, [{type, atom}]}]),
     ?pkey(service_type, [id]),

     ?table_attrs(service_type_attr, [service_type_id, {attr_name, [{type, atom}]}]),
     ?pkey(service_type_attr, [service_type_id, attr_name]),

     ?table_attrs(product_type, [id, has_table, description, {form, [{type, atom}]}]),
     ?pkey(product_type, [id]),

     ?table_attrs(product_type_attr, [product_type_id, {attr_name, [{type, atom}]}]),
     ?pkey(product_type_attr, [product_type_id, attr_name]),

     ?table_attrs(party_relationship_service,
     [{id, [{type, integer}]}, service_type_id,
      party_id_from, role_type_id_from, party_id_to, role_type_id_to, from_date,
      created_date, status_date,
      {status, [{type, atom}]},
      comments, name,
      user_party_id, tariff_id,
      login_id, caller_phone, email_address,
      restday_mask,
      {restday_disabled_time_from, [{type, time}]},
      {restday_disabled_time_to,   [{type, time}]},
      {workday_disabled_time_from, [{type, time}]},
      {workday_disabled_time_to,   [{type, time}]},
      {day_time_limit, [{type, integer}]},
      {month_time_limit, [{type, integer}]}, 
      {year_time_limit, [{type, integer}]}, 
      {total_time_limit, [{type, integer}]} 
     ]),
     ?pkey(party_relationship_service, [id]),
     ?seq(party_relationship_service, party_relationship_service_id_seq),
     
     ?table_attrs(tariff,
     [{id, [{type, integer}]},
      service_type_id,
      {charge_type_id, [{type, atom}]},
      {currency_type_id, [{type, atom}]},
      description
     ]),
     ?pkey(tariff, [id]),
     ?seq(tariff, tariff_id_seq),

     ?table_attrs(party_relationship_account,
     [{id, [{type, integer}]},
      {currency_type_id, [{type, atom}]},
      party_id_from, role_type_id_from, party_id_to, role_type_id_to,
      {from_date, [{type, datetime}]},
      {created_date, [{type, datetime}]},
      {status_date, [{type, datetime}]},
      {status,[{type, atom}]},
      debit,
      credit,
      credit_limit
     ]),
     ?pkey(party_relationship_account, [party_id_from, role_type_id_from, party_id_to, role_type_id_to, currency_type_id]),
     ?seq1(party_relationship_account, party_relationship_account_id_seq, id),

     ?table_attrs(account_transaction,
		  [{id                 , [{type, integer}]},
		   {account_id         , [{type, integer}]},
		   {transaction_type_id, [{type, atom}]},
		   {entry_date         , [{type, datetime}]},
		   {post_date          , [{type, datetime}]},
		   {source_type_id     , [{type, atom}]},
		   {source_id          , [{type, integer}]},
		   {tariff_id          , [{type, integer}]},
		   {quantity           , [{type, integer}]},
		   {amount             , [{type, integer}]},
		   {discount           , [{type, integer}]}
		  ]),
     ?pkey(account_transaction, [id]),
     ?seq(account_transaction, account_transaction_id_seq),

     ?table_attrs(audit_trail_log,
		  [{id,  [{type, integer}]},
		   {audit_trail_seq, [{type, integer}]},
		   table_name,
		   {action_date, [{type, datetime}]},
		   {action, [{type, atom}]}
		  ]),
     ?pkey(audit_trail_log, [id])
    ]).
