%%%----------------------------------------------------------------------
%%% File    : schema.erl
%%% Author  :  <svg@surnet.ru>
%%% Purpose : 
%%% Created : 14 Nov 2002 by  <svg@surnet.ru>
%%%----------------------------------------------------------------------

%%
%% $Id$
%%
%% $Log$
%%
%%**
%% 
%% .* schema
%% 
%%*
-module(aaa.schema).
-author('svg@surnet.ru').

-export([install/1, uninstall/0, add_worker/1, add_worker/2]).
-export([drop_worker/0, drop_worker/1, delete_worker/0, delete_worker/1]).

-import(lists).
-import(mnesia).
-import(wslib.misc).
-import(wslib.mnesiadb).

-import(proplists).
-import(odbcdb.attr_type).

-include("aaa_schema.hrl").
-include("tacacs.hrl").

-define(worker_tables,     [table_sequence,
			    odbcSysDict,
			    odbcdb_phrase_dict,
			    aaa_global,
			    aaa_party,
			    aaa_user,
			    aaa_service,
			    aaa_service_constraint,
			    aaa_session,
			    aaa_group,
			    aaa_service_group,
			    aaa_service_group_constraint,
			    aaa_nas,
			    aaa_account,
			    aaa_tariff,
			    aaa_tariff_ce,
			    aaa_session_counter
			    %% aaa_ar,
			    %% aaa_ar_session,
			   ]).

install(Nodes) ->
  mnesiadb:create_schema(Nodes, database_schema()),
  mnesiadb:create_constraints(database_constraints()),
  mnesiadb:init_sequences(Nodes, []).

add_worker(Master) ->
  add_worker(Master, node()).

drop_worker() ->
  drop_worker(node()).

drop_worker(Worker) ->
  misc:call_node(Worker, mnesia, stop, []),
  ok = mnesia:delete_schema([Worker]),
  misc:call_node(Worker, mnesia, start, []),
  ok.

delete_worker() ->
  delete_worker(node()).

delete_worker(Worker) ->
  TableNames = lists:map(fun ({TN, _}) -> TN;
			     (TN) -> TN
			 end, ?worker_tables),
  mnesiadb:delete_tables([Worker], TableNames).

add_worker(Master, Worker) ->
  TableDefs = ?worker_tables,
  ok = mnesiadb:connect(Worker, Master),
  mnesiadb:add_tables([Worker], TableDefs),
  misc:call_node(Worker, mnesia, set_master_nodes, [[Master]]).

uninstall() ->
  lists:foreach(fun ({Tab, _}) -> mnesia:delete_table(Tab) end,
		database_schema()).

database_schema() ->
  [
   ?table_def(aaa_global, []),
   ?table_def(aaa_party, []),
   ?table_def(aaa_user, []),
   ?table_def(aaa_service, [{index, [service_id]}]),
   ?table_def(aaa_service_constraint, []),
   ?table_def(aaa_nas, []),
   ?table_def(aaa_account, []),
   ?table_def(aaa_tariff, []),
   ?table_def(aaa_tariff_ce, [{type, bag}]),
   ?table_def(aaa_ar, [{type, bag}]),
   %%
   ?table_def1(aaa_session, ss, [{local_content, true}]),
   ?table_def1(aaa_ar_session, ss, []),
   ?table_def1(aaa_group, aaa_user, []),
   ?table_def1(aaa_service_group, aaa_service, []),
   ?table_def1(aaa_service_group_constraint, aaa_service_constraint, [])
  ].

database_constraints() ->
  [
   ?record_attrs(time_limit, [{day, [{type, term},
				     {default, unlimited}
				    ]}, 
			      {week , [{type, term},
				       {default, unlimited}
				      ]}, 
			      {month, [{type, term},
				       {default, unlimited}
				      ]},
			      {year , [{type, term},
				       {default, unlimited}
				      ]},
			      {total, [{type, term},
				       {default, unlimited}
				      ]}
			     ]),
   ?record_attrs(access_time, [{restday_mask, [{type, integer},
					       {default, 0}
					      ]},
			       {restday_disabled, [{type, term}]},
			       {workday_disabled, [{type, term}]}
			      ]),
   ?table_attrs(aaa_party, [{id, [{type, integer}, {required, true}]},
		      {party_type_id, [{type, text}, {required, true}]},
		      {name, [{type, text}, {required, true}]},
		      {attrs, [{type, list}]}
		    ]),
   ?table_attrs(aaa_user, 
		[{id, [{type, term}, {required, true}]},
		 {party_id, [{type, integer}, {required, true}]},
		 {password, [{type, tuple}]},
		 {enabled,  [{type, atom}]},
		 {disabled_datetime, [{type, tuple}]},
		 {group_member_id, [{type, text}]},
		 {attrs, [{type, list}, {default, []}]}
		]),
   ?table_attrs(aaa_service, 
		[{id, [{type, term}, {required, true}]},
		 {service_id, [{type, term}]},
		 {tariff_id, [{type, integer}]},
		 {attrs, [{type, list}, {default, []}]},
		 {info, [{type, text}]}
		      ]),
   ?table_attrs(aaa_group, 
		[{id, [{type, text}, {required, true}]},
		 {party_id, [{type, integer}, {required, true}]},
		 {password, [{type, tuple}]},
		 {enabled,  [{type, atom}]},
		 {disabled_datetime, [{type, tuple}]},
		 {group_member_id, [{type, text}]},
		 {attrs, [{type, list}, {default, []}]}
		]),
   ?table_attrs(aaa_service_group, [{id, [{type, term}, {required, true}]},
				    {service_id, [{type, term}]},
				    {tariff_id, [{type, integer}]},
				    {attrs, [{type, list}, {default, []}]},
				    {info, [{type, text}]}
				   ]),
   ?table_attrs(aaa_service_constraint,
		[{service_id, [{type, term}]},
		 {caller_phone, [{type, list}]},
		 {access_time, [{type, tuple}]},
		 {time_limit, [{type, tuple}]},
		 {max_sessions, [{type, integer}, {default, 1}]},
		 {nas, [{type, list}, {default, []}]}
		]),
   ?table_attrs(aaa_nas,
		[{ip, [{type, term}, {required, true}]},
		 {provider_id, [{type, term}, {required, true}]},
		 {info, [{type, text}]},
		 {attrs, [{type, list}, {default, []}]}
		]),
   ?record_attrs(se, 
		 [{id, [{type, term}, {required, true}]},
		  {tariff_id, [{type, term}]},
		  {provider_id, [{type, term}]},
		  {provider_host, [{type, tuple}]}
		 ]),
   ?record_attrs(sc,
		 [{user_id, [{type, term}, {required, true}]},
		  {subscriber_id, [{type, term}, {required, true}]},
		  {context_id, [{type, term}]},
		  {access_point, [{type, tuple}]}
		 ]),
   ?record_attrs(ce, 
		 [{charge_time, [{type, term}, {required, true}]},
		  {quantity, [{type, number}, {required, true}]},
		  {amount, [{type, number}, {required, true}]},
		  {discount, [{type, number}, {default, 0}]},
		  {tariff_id, [{type, term}, {required, true}]},
		  {supplier_id, [{type, term}, {required, true}]},
		  {customer_id, [{type, term}, {required, true}]},
		  {measured_unit, [{type, atom}, {required, true}]},
		  {chargeable_unit, [{type, atom}, {required, true}]},
		  {currency_type, [{type, text}, {default, unit}]}
		 ]),
   ?record_attrs(ss,
		 [{id, [{type, term}, {required, true}]},
		  {service, [{type, term}, {required, true}]},
		  {record_type, [{type, atom}, {default, start}]},
		  {terminate_cause, [{type, integer}]}, 
		  {start_time, [{type, term}, {required, true}]},
		  {end_time, [{type, term}]},
		  {se, [{type, {record, se}}]},
		  {sc, [{type, {record, sc}}]},
		  {ue, [{type, term}]}
		 ]),
   ?table_attrs(aaa_session,
		 [{id, [{type, term}, {required, true}]},
		  {service, [{type, term}, {required, true}]},
		  {record_type, [{type, atom}, {default, start}]},
		  {terminate_cause, [{type, integer}]}, 
		  {start_time, [{type, term}, {required, true}]},
		  {end_time, [{type, term}]},
		  {se, [{type, {record, se}}]},
		  {sc, [{type, {record, sc}}]},
		  {ue, [{type, term}]}
		 ]),
  ?table_attrs(aaa_ar_session,
		 [{id, [{type, term}, {required, true}]},
		  {service, [{type, term}, {required, true}]},
		  {record_type, [{type, atom}, {default, start}]},
		  {terminate_cause, [{type, integer}]}, 
		  {start_time, [{type, term}, {required, true}]},
		  {end_time, [{type, term}]},
		  {se, [{type, {record, se}}]},
		  {sc, [{type, {record, sc}}]},
		  {ue, [{type, term}]}
		 ]),
   ?table_attrs(aaa_ar,
		[{account_id, [{type, term}, {required, true}]},
		 {entry_time, [{type, term}, {required, true}]},
		 {ss, [{type, term}]},
		 {ce, [{type, term}, {required, true}]}
		]),
   ?table_attrs(aaa_account,
		[{id, [{type, term}, {required, true}]},
		 {debit, [{type, number}, {default, 0}]},
		 {credit, [{type, number}, {default, 0}]},
		 {credit_limit, [{type, number}, {default, 0}]},
		 {status, [{type, atom}]}
		]),
   ?table_attrs(aaa_tariff,
		[{id, [{type, term}, {required, true}]},
		 {charge_type, [{type, text}, {required, true}]},
		 {currency_type, [{type, text}, {required, true}]},
		 {info, [{type, text}, {required, true}]}
		]),
   ?table_attrs(aaa_tariff_ce,		     
		[{id,  [{type, term}, {required, true}]},
		 {seq_num, [{type, number}, {required, true}]},
		 {chargeable_unit, [{type, atom}, {required, true}]},
		 {measured_unit, [{type, atom}, {required, true}]},
		 {customer_relationship_fun,[{type, term}, {required, true}]},
		 {quantity_fun,[{type, term}, {required, true}]},
		 {amount_fun,[{type, term}, {required, true}]},
		 {discount_fun,[{type, term}, {required, true}]},
		 {info,[{type, text}, {required, true}]}
		])
  ].

