%%
%% $Id$
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(aaa.test_data).

-export([gen_enable/0, gen_users/0, gen_groups/0, gen_services/0, gen_service_groups/0, gen_tariff/0, gen_accounts/0]).
-export([gen_all/0]).

-import(lists).
-import(wslib.mnesiadb).
-import(wslib.passwd).

-include("tacacs.hrl").

create_password(Type, PasswdString) ->
  {Type, passwd:cipher(PasswdString, Type)}.

gen_all() ->
  lists:foreach(
    fun (F) -> F() end,
    [fun gen_tariff/0, fun gen_enable/0, fun gen_users/0, fun gen_groups/0,
     fun gen_services/0, fun gen_service_groups/0, fun gen_accounts/0
    ]).

gen_tariff() ->
  DialupTariffId = 4,
  CreditDialupTariffId = 6,
  SeqNum = 1,
  FromDate = {{2004, 08, 01}, {0, 0, 0}},
  tariff:write_obj(DialupTariffId,
		   [{info, "dialup"},
		    {charge_type, prepayment},
		    {currency_type, unit}
		   ]),
  tariff:write_obj(CreditDialupTariffId,
		   [{info, "dialup-credit"},
		    {charge_type, debit},
		    {currency_type, unit}
		   ]),
  Ce = [{chargeable_unit, duration},
	{measured_unit,  seconds},
	{customer_relationship_fun, {aaa.dialup_tariff_ce, customer_relationship}},
	{quantity_fun, {aaa.dialup_tariff_ce, quantity}},
	{amount_fun, {aaa.dialup_tariff_ce, amount}},
	{discount_fun, {aaa.dialup_tariff_ce, discount}},
	{info, "duration tariff"}
       ],
  tariff:write_ce_obj(CreditDialupTariffId, FromDate, SeqNum, Ce),
  tariff:write_ce_obj(DialupTariffId, FromDate, SeqNum, Ce).

gen_enable() ->
  lists:foreach(
    fun (Row) -> ok = db:write({enable, Row}) end,
    [[{id, {enable, 4}},
      {party_id, -1},
      {password, create_password(md5, "md5pass")},
      {enabled,true},
      {attrs,[]}
     ],
     [{id, {enable, 15}},
      {party_id, -1},
      {password, create_password(cleartext, "enablepass")},
      {enabled,true},
      {attrs,[]}
     ]
    ]).

gen_users() ->
  lists:foreach(
    fun (Row) -> ok = db:write({user, Row}) end,
    [[{id, "credit"},
      {enabled, false},
      {password, {cleartext, undefined}},
      {group_member_id, "ppp"}
     ],
     [{id, "agnes"},
      {enabled, true},
      {password, create_password(cleartext, "globalpass")}
     ],
     [{id, "exp1"},
      {enabled, true},
      {disabled_datetime, {{2038, 02, 01}, {0, 0, 0}}}, %Feb 01 2038
      {password, create_password(cleartext, "globalpass")}
     ],
     [{id, "exp2"},
      {enabled, true},
      {disabled_datetime, {{1900, 06,02}, {0, 0, 0}}}, %Jul 02 1900
      {password, create_password(cleartext, "globalpass")}
     ],
     [{id, "exp3"},
      {enabled, true},
      {disabled_datetime, {{2000, 01, 29}, {0, 0, 0}}}, %Jan 29 2000
      {password, create_password(cleartext, "globalpass")}
     ],
     [{id, "joe"},
      {enabled, true},
      {group_member_id, "admin"},
      {password, {md5, "$1$7AmSri03$YTPPmHsq7UqcMYY1gJeKF0"}}
     ],
     [{id, "fred"},
      {enabled, true},
      {default_service, permit},
      {group_member_id, "admin"}
     ],

     [{id, "lol"},
     {enabled, true}
     ],
     [{id, "foo"},
      {enabled, true},
      {password, nopassword}
     ],
     [{id, "pp221320"},
      {enabled, true},
      {password, {cleartext, "clearpass"}},
      {group_member_id, "ppp"}
     ],
     [{id, "pp219548"},
      {enabled, true},
      {password, {md5, "$1$7AmSri03$YTPPmHsq7UqcMYY1gJeKF0"}},
      {group_member_id, "ppp"}
     ],
     [{id, "pp187"},
      {enabled, true},
      {password, {des, "$1$7AmSri03$YTPPmHsq7UqcMYY1gJeKF0"}},
      {group_member_id, "ppp"}
     ],
     [{id, "pp225609"},
      {enabled, true},
      {password, {des, "ukpupmAnCUezY"}},
      {group_member_id, "ppp"}
     ],
     [{id, "pp242552"},
      {enabled, true},
      {password, {cleartext, "clearpass"}},
      {group_member_id, "ppp"}
     ],

     [{id, "pp242553"},
      {enabled, true},
      {password, {cleartext, "clearpass"}},
      {group_member_id, "ppp1"}
     ]
    ]).

gen_accounts() ->
  lists:foreach(
    fun (Row) -> ok = db:write({account, Row}) end,
    [
     [{id, {{2, 1}, unit}},
      {debit, 0},
      {credit, 0},
      {credit_limit, 0}
     ]
    ]).

gen_groups() ->
  lists:foreach(
    fun (Row) -> ok = db:write({group, Row}) end,
    [[{id, "uucp"}],
     [{id, "ppp"}],
     [{id, "ppp1"}],
     [{id, "admin"},
      {password, {cleartext, "foobar"}},
      {group_member_id, "admin1"}
     ],
     [{id, "admin1"}]
    ]).

gen_services() ->
  lists:foreach(
    fun (Row) -> ok = db:write({service, Row}) end,
    [[{id, {"credit", "ppp"}},
      {party_relationship_id, 1},
      {service_id,undefined},
      {tariff_id, 6}
     ],
     [{id,{"credit", "exec"}},
      {party_relationship_id,undefined},
      {service_id,undefined},
      {status,undefined},
      {tariff_id, 6}
     ],
     [{id, {"fred", "shell", "telnet"}},
      {attrs, [{deny, "131.108.13.[0-9]+"},
	       {permit, ".*"}
	      ]}
     ],
     [{id, {"lol", "exec"}},
      {default_attr, permit},
      {attrs, [#msg_arg{name="acl", value="4", mandatory=true}]}
     ]
    ]).

gen_service_groups() ->
  lists:foreach(
    fun (Row) -> ok = db:write({service_group, Row}) end,
    [[{id,{"ppp","ppp"}},
      {party_relationship_id,undefined},
      {service_id,undefined},
      {status,undefined},
      {default_attr,undefined},
      {attrs,[{"lcp",[]},{"ip",[]}]},
      {tariff_id,undefined},
      {info,undefined}
     ],
     [{id,{"ppp","exec"}},
      {party_relationship_id,undefined},
      {service_id,undefined},
      {status,undefined},
      {default_attr,undefined},
      {attrs, [#msg_arg{name="autocmd",value="ppp",mandatory=true},
	       #msg_arg{name="noescape",value="true",mandatory=true}
	      ]
      },
      {tariff_id,undefined},
      {info,undefined}
     ],
     [{id, {"uucp", "exec"}},
      {attrs, [#msg_arg{name="autocmd",value="rlogin 195.54.4.79",mandatory=true}]}
     ],
     [{id, {"ppp1", "exec"}},
      {attrs, [#msg_arg{name="autocmd",value="ppp",mandatory=false}]}
     ],

     [{id, {"admin1", "shell", "rlogin"}},
      {attrs, [{permit, "192.168.200.132"}]}
     ]
    ]).

