%%%-------------------------------------------------------------------
%%% File    : tac_law.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%%
%%% Created : 13 May 2004 by  <svg@surnet.ru>
%%%-------------------------------------------------------------------
%%
%% $Id$
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(tac_law).

-behaviour(property_file).

%% tac_protocol callbacks
-export([init/1, authenticate/2, authorize/2, account/2]).

%% law exports
-export([accept/1, accept/3, reject/1, reject/3, error/1, error/3, reply/4]).

%% property_file callbacks
-export([parse_config/4, store_config/2, postload_config/2, remove_config/1]).

-import(wslib.props).
-import(wslib.log).

-define(law, tacacs).
-define(agent, aaa).

%%
%%  property_file callbacks
%%

parse_config([N=aaa_law], [], Content, _) ->
  NewContent =
    case props:check_required(
	   [{law_module}, {generate_aaa_event, true}],
	   proplists:unfold(
	     proplists:substitute_aliases(
	       [{module, law_module}, {generate_events, generate_aaa_event}],
	       Content))) of
      {ok, C} ->
	C;
      {error, Error} ->
	exit(Error)
    end,
  {aaa_law, NewContent};
parse_config([N=module, aaa_law], [Str], [], _) ->
  Mod =  list_to_atom(Str),
  tac_conf:verify_module(Mod),
  {N, Mod};
parse_config([N=generate_events, aaa_law], Bool, [], _) when Bool == true;
							     Bool == false ->
  {N, Bool}.

store_config(must_never_match, _Entries) ->
  [].

postload_config(Config, _Sup) ->
  application:start(lgi).
%%  Props = tac_conf:gv(Config, aaa_law),
%%  ok = lgi:connect_virtual(?law, ?agent, Props).

remove_config(DB) ->
  lgi:disconnect(?law, ?agent),
  ok.

%%
%% tac_protocol callbacks
%%
init(_) ->
  {ok, []}.

authenticate(Data, _) ->
  apply_law({authenticate, Data}).

authorize(Data, _) ->
  apply_law({authorize, Data}).

account(Data, _) ->
  apply_law({account, Data}).

%%
%% Law exports
%%

accept(Msg) ->
  accept([], Msg, "").

accept(Args, ServerMsg, UserData) ->
  reply(ok, Args, ServerMsg, UserData).

reject(Msg) ->
  reject([], Msg, "").

reject(Args, ServerMsg, UserData) ->
  reply(false, Args, ServerMsg, UserData).

error(Msg) ->
  error([], Msg, "").

error(Args, ServerMsg, UserData) ->
  reply(error, Args, ServerMsg, UserData).

reply(Status, Args, ServerMsg, UserData) ->
  lgi:deliver(self, {lgi:aid@(), {Status, Args, ServerMsg, UserData}}),
  exit(normal).

%%
apply_law(Req={Action, Data}) ->
  Self = self(),
  Props = tacacs:config_gv(aaa_law),
  {ok, AgentId} = lgi:connect(?law, Self, [{debug, get(dbg)}|Props]),
  SelfAid = lgi:self_aid(?law, Self),
  Ret =
    case lgi:cast(SelfAid, SelfAid, Req) of
      false ->
	{error, [], "law error", ""};
      true ->
	receive_reply(AgentId)
    end,
  lgi:disconnect(AgentId),
  Ret.

receive_reply(AgentId) ->
  receive
    {AgentId, Answer} ->
      Answer
  end.

%% apply_law(Req={Action, Data}) ->
%%   Aid = lgi:self_aid(?law, ?agent),
%%   case lgi:cast(Aid, Aid, Req) of
%%     false ->
%%       {error, [], "law timeout", "law timeout"};
%%     true ->
%%       receive_reply()
%%   end.

