%%%----------------------------------------------------------------------
%%% File    : tac_server.erl
%%% Author  :  <svg@surnet.ru>
%%% Purpose : 
%%% Created :  9 Dec 1999 by  <svg@surnet.ru>
%%%	$Id: tac_server.erl,v 1.7 2002/09/04 01:05:53 svg Exp $	
%%%----------------------------------------------------------------------

-module(tac_server).
-author('svg@surnet.ru').

-behaviour(property_file).

-include("tacacs.hrl").

-export([start_link/0, stop/0, debug/2]).

%% property_file exports
-export([parse_config/4, store_config/2, remove_config/1, postload_config/2]).

-import(wslib.props).

-define(server, ?MODULE).

start_link() ->
  start_listener(tac_conf).

stop() ->
  stop_listener(tacacs:config_gv(port)).

debug(Level, Bool) ->
  wsocket_listen:debug(tacacs:config_gv(port), Level, Bool).

start_listener(Config) ->
  [Port, SKey, MaxConn, AaaHandler] =
    [tac_conf:gv(Config, Key)
     || Key <- [port, skey, max_connections, aaa_handler]],
  {ok, AaaHandlerState} = AaaHandler:init(tac_conf),
  TrMod = wsocket_tcp_transport,
  PctFun = tac_packet:mk_packer(server, SKey, []),
  PrtFun =
    tac_protocol:mk_fsm(server,
			[{aaa_handler, {AaaHandler, AaaHandlerState}}]),
  Args = [{transp_opts, [{packet, 0}, {reuseaddr,true}]}, {max, MaxConn}],
  wsocket_listen:start_link(Port, TrMod, PctFun, PrtFun, Args).

stop_listener(Port) ->
  wsocket_listen:stop(Port).

parse_config([N=skey], [Skey], [], _) ->
  {N, Skey};
parse_config([N=port], [0], [], _) ->
  {N, ?DEFAULT_PORT};
parse_config([N=port], [Port], [], _) when is_integer(Port) ->
  {N, Port};
parse_config([N=aaa_handler], [Str], [], _) ->
  Module = list_to_atom(Str),
  tac_conf:verify_module(Module),
  {N, Module};
parse_config([N=max_connections], [Num], [], _) when Num == 0;
						     Num == "infinity"->
  {N, infinity};
parse_config([N=max_connections], [Num], [], _) when is_integer(Num);
						     Num > 0 ->
  {N, Num};
parse_config([eof], [], Content, _) ->
  {PropLs, Rest} =
    proplists:split(Content, [element(1, T) || T <- defaults()]),
  ReqProps = lists:flatten(PropLs),
  case props:check_required(defaults(), ReqProps) of
    {ok, Ps} ->
      Ps;
    {error, Error} ->
      exit(Error)
  end.

store_config(must_never_match, _Entries) ->
  [].

postload_config(Config, Sup) ->
  Tacacs =
    {?server,
     {?server, start_link, []},
     permanent, 5000, worker, [?MODULE]
    },
  {ok, _} = supervisor:start_child(Sup, Tacacs),
  ok.

remove_config(Config) ->
  stop_listener(tac_conf:gv(Config, port)),
  ok.

defaults() ->
  [{port, ?DEFAULT_PORT},
   {max_connections, infinity},
   {skey},
   {aaa_handler}
  ].
