%%%----------------------------------------------------------------------
%%% File    : tacacsd.erl
%%% Author  :  <svg@surnet.ru>
%%% Purpose : 
%%% Created :  9 Dec 1999 by  <svg@surnet.ru>
%%%	$Id: tacacsd.erl,v 1.7 2002/09/04 01:05:53 svg Exp $	
%%%----------------------------------------------------------------------

-module(tacacs).
-author('svg@surnet.ru').

-include("tacacs.hrl").

-export([start/0, stop/0, restart/0, debug/2, is_master/0, where_started/0]).
-export([config_gv/1, config_gv/2, config_gv_all/1, config_match_key/1, config_select/1, config_op/2]).
-export([log_info/1, log_error/1, log_info/2, log_error/2]).
-export([install/0, install/1]).

-import(wslib.log).

start() ->
  application:start(tacacs, permanent).

stop() ->
  application:stop(tacacs).

restart() ->
  stop(),
  start().

%% Levels = Level | [Level]
%% Level = all | transport | protocol | packet 
debug(Levels, Bool) ->
  tac_server:debug(Levels, Bool).

%% tac_conf interface

where_started() ->
  lists:usort([node(P) || P <- tac_conf:all_pids()]).

is_master() ->
  case tac_conf:master_pid() of
    undefined ->
      false;
    Pid ->
      node() == node(Pid)
  end.

config_gv(Key) ->
  tac_conf:gv(tac_conf, Key).

config_gv(Key, Default) ->
  tac_conf:gv(tac_conf, Key, Default).

config_gv_all(Key) ->
  tac_conf:gv_all(tac_conf, Key).

config_match_key(KeyPat) ->
  tac_conf:match_key(tac_conf, KeyPat).

config_select(MatchSpec) ->
  tac_conf:select(tac_conf, MatchSpec).

config_op(OpName, OpArgs) ->
  tac_conf:op(tac_conf, {OpName, OpArgs}).

%% tac_logger interface
log_info(Args) when is_list(Args) ->
  App =
    case application:get_application() of
      {ok, A} ->
	A;
      _ ->
	'?'
    end,
  tac_event:info_event([App|Args]).
  
log_info(Fmt, Args) ->
  log_info([{format, Fmt, Args}]).

log_error(Fmt, Args) ->
  log_error([{format, Fmt, Args}]).

log_error(Args) ->
  App =
    case application:get_application() of
      {ok, A} ->
	A;
      _ ->
	'?'
    end,
  tac_event:error_event([App|Args]).

install() ->
  Nodes = [node()],
  install(Nodes).

install(Nodes) ->
  aaa.schema:install(Nodes),
  tac_session:create_tables(Nodes),
  odbcdb:install(Nodes),
  application:load(odbcdb),
  application:set_env(odbcdb, dict_src, []),% it can depend from schema defs
  odbcdb:start(),
  aaa.odbcdb_schema:install(),
  odbcdb:stop(),
  mnesia:dump_log().
