%%%----------------------------------------------------------------------
%%% File    : tacacs_tests.erl
%%% Author  :  <svg@surnet.ru>
%%% Purpose : 
%%% Created : 28 Jan 2000 by  <svg@surnet.ru>
%%% $Id: tests.erl,v 1.4 2002/09/04 01:05:53 svg Exp $
%%%----------------------------------------------------------------------

-module(tacacs_tests).
-author('svg@surnet.ru').

-export([test_all/0, test_chpass/1, test_login/1, test_enable/1]).
-export([test_author/1, test_account/1]).
-export([chpass/4, login/3, enable/3, authorize/2, account/3]).
-export([test_tcp_listen/1, test_udp_listen/1, test_listen/2]).
-export([wsocket_listen_client/1, tacacs_packet_test/1]).
-export([dbg_info/2]).

-import(wslib.listn).
-import(wslib.misc).

-include("tacacs.hrl").

-define(TACACS_SKEY, "Cisco").

%%-define(dbg_opts, {debug, [transport, protocol, packet]}).
-define(dbg_opts, {debug, []}).

-define(dbg(F, A), dbg_info("line ~w, "++F, [?LINE|A])).
-define(match(_Pat), fun (_Res) ->
                         case _Res of
                           _Pat -> ok;
                           _ -> false
                         end
                     end).

-define(MASTER_NODE_NAME, "test1").
-define(PREPAM_TEST_USER, "PREPAM_TEST_USER").

test_all() ->
  lists:foreach(fun(T) ->
                    io:format("~n~s~n",[apply({?MODULE, T}, [info])]),
                    apply({?MODULE, T}, [do])
                end,
                [test_chpass, test_login, test_enable, test_author, test_account,
                 test_prepam_fun_login
                ]
               ).

process_tests(Tests) ->
  lists:foreach(fun ({Fun, FunTests}) ->
                    process_tests(Fun, FunTests)
                end, Tests).

process_tests(Fun, Tests) ->
  process_tests(Fun, Tests, 1).

%% Test argument is in the form {Msg, Args, TrueResult}
%% Msg        - test purpose
%% Args       - [Arg]
%% TrueResult - true result pattern
process_tests(Fun, [], _) ->
  ok;
process_tests(Fun, [Test|Tail], Num) ->
  {Msg, Args, ResPat} = Test,
  Status = case catch apply(Fun,Args) of
             Res when function(ResPat) ->
               case catch ResPat(Res) of
                 ok ->
                   [passed, ok];
                 _ ->
                   [Res, failed]
               end;
             ResPat ->
               [passed, ok];
             Other ->
               [Other, failed]
           end,
  io:format("Test ~2.2w: "++Msg++" -> ~p, ~p~n", [Num|Status]),
  process_tests(Fun, Tail, Num+1).

test_chpass(info) ->
  "*** Test chpass ***";
test_chpass(do) ->
  Tests =
    [{"chpass, ascii", ["agnes", ascii, "globalpass", "newpass"],
      {false, [],
       "Change password is not implemented",
       ""
      }
     },
     {"chpass, chap", ["agnes", chap, "globalpass", "newpass"],
      {error,
       [],
       "Unsupported authentication type",
       ""
      }
     }
    ],
  process_tests({?MODULE, chpass}, Tests).

test_enable(info) ->
  "*** Test enable ***";
test_enable(do) ->
  Tests =
    [{"enable", ["agnes", 15, "enablepass"], {ok,[],[],[]}},
     {"enable", ["agnes", 15, "globalpass"], {false,[],"Bad password",""}},
     {"enable", ["agnes", 4, "md5pass"], {ok,[],[],[]}},
     {"enable", ["agnes", 4, "globalpass"], {false,[],"Bad password",""}}
    ],
  process_tests({?MODULE, enable}, Tests).

test_login(info) ->
  "*** Test login ***";
test_login(do) ->
  Tests =
    [{"nopassword, ascii", ["foo", ascii, ""], {ok,[],[],[]}}
     ,{"global, ascii", ["agnes", ascii, "globalpass"], {ok,[],[],[]}}
     ,{"global, pap", ["agnes", pap, "globalpass"], {ok,[],[],[]}}
     ,{"global, pap", ["agnes", pap, "global"], {false,[],"Bad password",""}}
     ,{"global, chap", ["agnes", chap, {16#1f, "chapchel", "globalpass"}], {ok,[],[],[]}}
     ,{"not expired, ascii", ["exp1", ascii, "globalpass"], {ok,[],[],[]}}
     ,{"expired, ascii", ["exp2", ascii, "globalpass"], {false,[], "Password has expired", ""}}
     ,{"expired soon, ascii", ["exp3", ascii, "globalpass"], {ok,[],[],[]}}
     ,{"md5 ascii",         ["joe", ascii, "md5pass"], {ok,[],[],[]}}
     ,{"md5 ascii",         ["joe", ascii, "pass"], {false,[],"Bad password",""}}
     ,{"clear text, group, ascii", ["fred", ascii, "foobar"], {ok,[],[],[]}}
     ,{"clear text, group, ascii", ["fred", ascii, "foobarfoo"], {false,[],"Bad password",""}}
     ,{"clear text, ascii", ["pp242552", ascii, "clearpass"], {ok,[],[],[]}}
     ,{"clear text, ascii", ["pp242552", ascii, "clearpa"], {false,[],"Bad password",""}}
     ,{"des, ascii",        ["pp225609", ascii, "despass"], {ok,[],[],[]}}
     ,{"des, ascii",        ["pp225609", ascii, "pass"], {false,[],"Bad password",""}}
     ,{"md5 ascii",         ["pp187", ascii, "md5pass"], {ok,[],[],[]}}
     ,{"md5 ascii",         ["pp187", ascii, "despass"], {false,[],"Bad password",""}}
     ,{"md5 ascii",         ["pp219548", ascii, "md5pass"], {ok,[],[],[]}}
     ,{"clear text, pap",   ["pp242552", pap, "clearpass"], {ok,[],[],[]}}
     ,{"clear text, pap",   ["pp242552", pap, "clearpa"], {false,[],"Bad password",""}}
     ,{"des, pap",          ["pp225609", pap, "despass"], {ok,[],[],[]}}
     ,{"des, pap",          ["pp225609", pap, "pass"], {false,[],"Bad password",""}}
     ,{"md5 pap",           ["pp187", pap, "md5pass"], {ok,[],[],[]}}
     ,{"md5 pap",           ["pp187", pap, "despass"], {false,[],"Bad password",""}}
     ,{"md5 pap",           ["pp219548", pap, "md5pass"], {ok,[],[],[]}}
     ,{"clear text, pap",   ["pp221320", pap, "clearpass"], {ok,[],[],[]}}
     ,{"clear text, chap",  ["pp242552", chap,
                            {16#56, "chapchel", "clearpass"}], {ok,[],[],[]}}
     ,{"clear text, chap",  ["pp242552", chap,
			    {16#56, "chapchel", "pass"}], {false,[],"Bad password",""}}
    ],
  process_tests({?MODULE, login}, Tests).

test_author(info) ->
  "*** Test authorization ***";
test_author(do) ->
  Tests =
    [{"exec, mandatory, not exists",
      ["pp242552",
       [#msg_arg{name="service", value="exec", mandatory=true},
	#msg_arg{name="autocmd", value="slip", mandatory=true}
       ]],
      {false,[],[],[]}
     }
     ,{"exec, mandatory, exists",
       ["pp242552",
	[#msg_arg{name="service", value="exec", mandatory=true},
	 #msg_arg{name="autocmd", value="ppp", mandatory=true}
	]],
       {ok,
	[#msg_arg{name="noescape", value="true", mandatory=true}],
	[], []
       }
      }
     ,{"exec, mandatory, not exists, exists optional",
       ["pp242553",
	[#msg_arg{name="service", value="exec", mandatory=true},
	 #msg_arg{name="autocmd", value="ppp", mandatory=true}
	]],
       {ok,[],[], []}}
     ,{"exec, mandatory, exists optional with other value",
       ["pp242553",
	[#msg_arg{name="service", value="exec", mandatory=true},
	 #msg_arg{name="autocmd", value="slip", mandatory=true}
	]],
       {ok, [],[], []}}
     ,{"exec, optional, exists",
       ["pp242553",
	[#msg_arg{name="service",value= "exec", mandatory=true},
	 #msg_arg{name="autocmd",value="ppp",mandatory=false}
	]],
       {ok,[],[],[]}
      }
     ,{"exec, optional, exists optional with same name",
       ["pp242553",
	[#msg_arg{name= "service", value="exec", mandatory=true},
	 #msg_arg{name= "autocmd", value="slip",mandatory=false}
	]],
       {replace,
	[#msg_arg{name="service",value="exec",mandatory=true},
	 #msg_arg{name="autocmd",value="ppp",mandatory=false}],
	[], []
       }}
     ,{"exec, optional, exists mandatory with same name",
       ["pp242552",
	[#msg_arg{name="service",value= "exec", mandatory=true},
	 #msg_arg{name="autocmd",value= "ppp", mandatory=false}
	]],
       {replace,
	[#msg_arg{name="noescape", value="true", mandatory=true},
	 #msg_arg{name="service",  value="exec", mandatory=true},
	 #msg_arg{name="autocmd",  value="ppp",  mandatory=true}
	],
	[], []
       }
      }
     ,{"exec, default attribute permitted",
       ["lol",
	[#msg_arg{name="service",value="exec", mandatory=true},
	 #msg_arg{name="autocmd",value="ppp", mandatory=false}
	]],
       {ok, [#msg_arg{name="acl",value="4",mandatory=true}],  [], []}
      }
     ,{"cmd",
       ["fred",
	[#msg_arg{name="service",value= "shell", mandatory=true},
	 #msg_arg{name="cmd",value= "telnet", mandatory=true},
	 #msg_arg{name="cmd-arg", value="131.108.13.9", mandatory=true}
	]],
       {false,[],[],[]}
      }
     ,{"cmd",
       ["fred",
	[#msg_arg{name="service",value= "shell", mandatory=true},
	 #msg_arg{name="cmd",value= "telnet", mandatory=true},
	 #msg_arg{name="cmd-arg", value="192.168.100.9", mandatory=true}
	]],
       {ok,[],[],[]}
      }
     ,{"cmd",
       ["fred",
	[#msg_arg{name="service",value= "shell", mandatory=true},
	 #msg_arg{name="cmd",value= "rlogin", mandatory=true},
	 #msg_arg{name="cmd-arg",value= "192.168.200.132", mandatory=true}
	]],
       {ok,[],[],[]}
     }
    ],
  process_tests({?MODULE, authorize}, Tests).

test_account(info) ->
  "*** Test accounting ***";
test_account(do) ->
  Tests =
    [{"start", ["accUser", start,
                [#msg_arg{name="task_id",value= "1239", mandatory=true},
                 #msg_arg{name="start_time",value= "11233400", mandatory=true},
                 #msg_arg{name="reason",value= "test", mandatory=true}
                ]],
      {ok,[],[], []}
     },
     {"continue", ["accUser", continue,
                   [#msg_arg{name="task_id",value= "1239", mandatory=true},
                    #msg_arg{name="start_time",value= "112334546", mandatory=true},
                    #msg_arg{name="reason",value= "test", mandatory=true}
                   ]],
      {ok,[],[], []}
     },
     {"duplicate", ["accUser", duplicate,
                    [#msg_arg{name="task_id",value= "1239", mandatory=true},
                     #msg_arg{name="start_time",value= "112334546", mandatory=true},
                     #msg_arg{name="reason",value= "test", mandatory=true}
                    ]],
      {ok,[],[], []}
     },
     {"stop", ["accUser", stop,
               [#msg_arg{name="task_id",value= "1239", mandatory=true},
                #msg_arg{name="stop_time",value= "112334546", mandatory=true},
                #msg_arg{name="reason",value= "test", mandatory=true}
               ]],
      {ok,[],[], []}
     },
     {"start, tariff exists",
      ["credit", start,
       [#msg_arg{name="task_id", value="1208741"},
	#msg_arg{name="timezone", value="GMT+6"},
	#msg_arg{name="service", value="ppp"},
	#msg_arg{name="start_time", value="1096588624"},
	#msg_arg{name="Framed-Protocol", value="1"},
	#msg_arg{name="nas-connect-info", value="45333/24000 V90/V44/LAPM"},
	#msg_arg{name="parent-interface", value="Serial7/1:18"},
	#msg_arg{name="parent-interface-type", value="1"}
       ]],
      {ok,[],[], []}
     },
     {"continue, tariff exists",
      ["credit", continue,
       [#msg_arg{name="task_id", value="859016"},
	#msg_arg{name="timezone", value="GMT+6"},
	#msg_arg{name="service", value="ppp"},
	#msg_arg{name="start_time", value="1096588606"},
	#msg_arg{name="Framed-Protocol", value="1"},
	#msg_arg{name="protocol", value="ip"},
	#msg_arg{name="addr", value="212.57.172.204"},
	#msg_arg{name="pre-session-time", value="23"},
	#msg_arg{name="nas-tx-speed", value="46667"},
	#msg_arg{name="nas-rx-speed", value="26400"},
	#msg_arg{name="elapsed_time", value="0"},
	#msg_arg{name="bytes_in", value="185"},
	#msg_arg{name="bytes_out", value="118"},
	#msg_arg{name="pre-bytes-in", value="75"},
	#msg_arg{name="pre-bytes-out", value="59"},
	#msg_arg{name="paks_in", value="7"},
	#msg_arg{name="paks_out", value="5"},
	#msg_arg{name="pre-paks-in", value="3"},
	#msg_arg{name="pre-paks-out", value="3"},
	#msg_arg{name="nas-connect-info", value="46667/26400 V90/V42bis/LAPM"},
	#msg_arg{name="parent-interface", value="Serial6/2:10"},
	#msg_arg{name="parent-interface-type", value="1"}
       ]],
      {ok,[],[], []}
     },
     {"stop, tariff exists",
      ["credit", stop,
       [#msg_arg{name="task_id", value="859016"},
	#msg_arg{name="timezone", value="GMT+6"},
	#msg_arg{name="service", value="ppp"},
	#msg_arg{name="start_time", value="1096588606"},
	#msg_arg{name="Framed-Protocol", value="1"},
	#msg_arg{name="protocol", value="ip"},
	#msg_arg{name="addr", value="212.57.172.204"},
	#msg_arg{name="pre-session-time", value="26"},
	#msg_arg{name="nas-tx-speed", value="42667"},
	#msg_arg{name="nas-rx-speed", value="26400"},
	#msg_arg{name="elapsed_time", value="114"},
	#msg_arg{name="bytes_in", value="4375"},
	#msg_arg{name="bytes_out", value="20289"},
	#msg_arg{name="pre-bytes-in", value="75"},
	#msg_arg{name="pre-bytes-out", value="59"},
	#msg_arg{name="paks_in", value="158"},
	#msg_arg{name="paks_out", value="144"},
	#msg_arg{name="pre-paks-in", value="3"},
	#msg_arg{name="pre-paks-out", value="3"},
	#msg_arg{name="disc-cause", value="1"},
	#msg_arg{name="disc-cause-ext", value="28"},
	#msg_arg{name="parent-interface", value="Serial6/2:10"},
	#msg_arg{name="parent-interface-type", value="1"},
	#msg_arg{name="nas-connect-info", value="42667/26400 V90/V42bis/LAPM (45333/26400)"}
       ]],
      {ok,[],[], []}
     }
    ],
  process_tests({?MODULE, account}, Tests).

chpass(UserName, Type, Pass, NewPass) ->
  {Server, User} = default_params(),
  Authen = tac_lib:make_authen(Type, ppp),
  taclient:chpass(Server, User#userid{user=UserName}, Authen, {Pass, NewPass},
		  [?dbg_opts]).

login(UserName, Type, Pass) ->
  {Server, User} = default_params(),
  Authen = tac_lib:make_authen(Type, ppp),
  taclient:login(Server, User#userid{user=UserName}, Authen, Pass,
		 [?dbg_opts]).

enable(UserName, PrivLvl, Pass) ->
  {Server, User} = default_params(),
  Authen = tac_lib:make_authen(pap, ppp, PrivLvl, tacacs),
  taclient:enable(Server, User#userid{user=UserName}, Authen, Pass,
		  [?dbg_opts]).

authorize(UserName, Args) ->
  {Server, User} = default_params(),
  Authen = tac_lib:make_authen(ascii, ppp, 16#5, tacacs),
  taclient:authorize(Server, User#userid{user=UserName}, Authen, Args, [?dbg_opts]).

account(UserName, Action, Args) ->
  {Server, User} = default_params(),
  Authen = tac_lib:make_authen(ascii, ppp, 16#5, tacacs),
  taclient:account(Server, User#userid{user=UserName}, Authen, Action, Args, [?dbg_opts]).

default_params() ->
  {ok, Addr} = inet:gethostname(),
  {tac_lib:make_serverid(Addr, 8888, "Cisco"),
   tac_lib:make_userid("test", Addr, "tty01")
  }.

host_name() ->
  [_, Host] = string:tokens(atom_to_list(node()), "@"),
  Host.

%%
%% wsocket_listen
%%

-define(LISTEN_PORT, 7478).
-define(MAX_CONN, 5).
-define(PACK_HEAD, "Hey,").
-define(BAD_HEAD, "Hey!").
-define(PACK_BODY, "Listen to your mother").
-define(BAD_BODY, "Listen to your father").
-define(WAIT_TIMEOUT, 60000).
-define(RECV_TIMEOUT, 60000).
-define(SKEY, "Cisco").

test_udp_listen(What) ->
  test_listen(upd, What).

test_tcp_listen(What) ->
  test_listen(tcp, What).

test_listen(Tr, info) ->
  io_lib:format("*** Test ~w listen ***", [Tr]);
test_listen(Tr, do) ->
  test_listen(Tr, start),
  test_listen(Tr, tests),
  test_listen(Tr, stop);

test_listen(Tr, start) ->
  Port = ?LISTEN_PORT,
  Max = ?MAX_CONN,
  (catch wsocket_listen:stop(Port)),
  {TrMod, TrOpts} =
    case Tr of
      tcp ->
	{wsocket_tcp_transport,
	 [{transp_opts, [{packet, 0}, {reuseaddr,true}]}]
	};
      udp ->
	{wsocket_udp_transport, 
	 [{transp_opts, [{reuseaddr,true}]}]
	}
    end,
  Tests =
    [
     {fun wsocket_listen_start/5, 
      [{io_lib:format("wsocket_listen, start, ~w", [Tr]),
        [Port, TrMod, fun packer_test/2, fun wsocket_listen_proto/2,
         [{max, Max}|TrOpts]
	], 
        ?match({ok, _})}
      ]}
    ],
  process_tests(Tests);
test_listen(Tr, stop) ->
  Tests =
    [{{wsocket_listen, stop},
      [{io_lib:format("stopping wsocket_listen ~w", [Tr]),
	[?LISTEN_PORT],
	ok}
      ]}
    ],
  process_tests(Tests);
test_listen(Tr, tests) ->
  Tests =
    [
     {fun wsocket_listen_client/1,
      [{io_lib:format("simple ~w client, all in one packet", [Tr]),
        [{Tr, ?PACK_HEAD ?PACK_BODY}],
        ?match(?PACK_HEAD ?PACK_BODY)}
       ,{io_lib:format("simple ~w client, all in one packet", [Tr]),
         [{Tr, ?PACK_HEAD ?PACK_BODY}],
         ?match(?PACK_HEAD ?PACK_BODY)}
       ,{io_lib:format("simple ~w client, bad header", [Tr]),
         [{Tr, ?BAD_HEAD ?PACK_BODY}],
         ?match([])}
       ,{io_lib:format("simple ~w client, bad body", [Tr]),
         [{Tr, ?PACK_HEAD ?BAD_BODY}],
         ?match(?PACK_HEAD ?BAD_BODY)}
       ,{io_lib:format("simple ~w client, delay on packets", [Tr]),
         [{Tr, ?PACK_HEAD ?PACK_BODY, 22, 100}],
         ?match(?PACK_HEAD ?PACK_BODY)}
       ,{io_lib:format("simple ~w client, delay on packets, timeout", [Tr]),
         [{Tr, ?PACK_HEAD ?PACK_BODY, 22, 1000}],
         {error,closed}}
      ]}
     %%
     ,{fun wsocket_listen_mclient/2,
       [{io_lib:format("simple ~w client, 3 sessions", [Tr]),
         [{Tr, ?PACK_HEAD ?PACK_BODY}, 3],
         ?match(?PACK_HEAD ?PACK_BODY)}
        ,{io_lib:format("simple ~w client, 100 sessions", [Tr]),
          [{Tr, ?PACK_HEAD ?PACK_BODY}, 100],
          ?match(?PACK_HEAD ?PACK_BODY)}
       ]}
    ],
  process_tests(Tests).

wsocket_listen_start(Port, TrMod, PctFun, PrtFun, Args) ->
  Ret = wsocket_listen:start_link(Port, TrMod, PctFun, PrtFun, Args),
  Ret.

-record(pkg, {header="", body="", data = <<>>}).

%% packing
packer_test(pack, P=#pkg{header=H, body=B}) ->
  ?dbg("sending packet~nhead: ~p,~nbody: ~p", [H, B]),
  {packet, list_to_binary(H++B)}; 
packer_test(pack, _) ->
  {error, badpacket};
%% unpacking
packer_test(Bin, AccBin) when binary(Bin), binary(AccBin) ->
  packer_header(erlang:concat_binary([AccBin, Bin]));
packer_test(Other1, Other2) ->
  ?dbg("packer_test, badarg: ~p, ~p", [Other1, Other2]),
  {error, badarg}.

packer_header(<<H1, H2, H3, H4, Rest/binary>>) ->
  case [H1, H2, H3, H4] of
    ?PACK_HEAD ->
      Header = [H1, H2, H3, H4],
      ?dbg("header, ready, ~p, get body", [Header]),
      packer_body(Rest, #pkg{header=Header, data = <<>>});
    _ ->
      {error, badhead}
  end;
packer_header(Bin) ->
  ?dbg("header, need more, size ~w", [size(Bin)]),
  {more, fun packer_header/2, #pkg{data=Bin}}.

packer_header(Bin, #pkg{data=AccBin}) ->
  packer_header(erlang:concat_binary([AccBin, Bin])).

packer_body(Bin, P=#pkg{data=AccBin}) ->
  BinBody = erlang:concat_binary([AccBin, Bin]),
  BinBodySz = size(BinBody),
  Pat = ?PACK_BODY,
  PatLen = length(Pat),
  if BinBodySz < PatLen ->
      ?dbg("body, need more, body size ~w", [BinBodySz]),
      {more, fun packer_body/2, P#pkg{data=BinBody}}; 
     BinBodySz > PatLen ->
      {error, badsize};
     true ->
      ?dbg("body, done size ~w", [BinBodySz]),
      {done, P#pkg{body=binary_to_list(BinBody), data= <<>> }, <<>>}
  end.

wsocket_listen_proto(start, _) ->
  {recv, fun wsocket_listen_proto/2, [], ?WAIT_TIMEOUT};
wsocket_listen_proto({recv, Packet}, _) ->
  {done, Packet}.

make_client_proto(Packets, SendTimeout) ->
  Self = self(),
  fun (start, _) ->
      {continue, fun tac_client_proto/2, {Packets, SendTimeout, Self}};
      (Error, _) ->
      Error
  end.

tac_client_proto(continue, {[], _, Pid}) ->
  {error, null_packet};
tac_client_proto(sended, State={[], _, Pid}) ->
  {recv, fun tac_client_proto/2, Pid, ?RECV_TIMEOUT}; 
tac_client_proto(continue, {[P|Packets], ST, Pid}) ->
  {send, #pkg{body=P}, fun tac_client_proto/2, {Packets, ST, Pid}};
tac_client_proto(sended, {[P|Packets], ST, Pid}) ->
  timer:sleep(ST),
  {send, #pkg{body=P}, fun tac_client_proto/2, {Packets, ST, Pid}};
tac_client_proto({recv, P=#pkg{header=Head, body=Body}}, Pid) ->
  Pid ! {recv, self(), Head++Body},
  done;
tac_client_proto(Error, _) ->
  Error.

wsocket_listen_mclient(Arg, Num) when integer(Num);
                                  Num > 1 ->
  multi_run([fun () -> wsocket_listen_client(Arg) end ||
              E <- lists:seq(1, Num)]).

wsocket_listen_client({Tr, Data}) ->
  wsocket_listen_client({Tr, Data, 1});
wsocket_listen_client({Tr, Data, NumPs}) ->
  wsocket_listen_client({Tr, Data, NumPs, 0});
wsocket_listen_client({Tr, Data, NumPs, Delay}) ->
  PktSz = length(Data) div NumPs,
  Lens = if PktSz == length(Data) ->
             [PktSz];
            true ->
             LastPktSz = PktSz + length(Data) rem NumPs,
             [PktSz || A <- lists:seq(1, NumPs - 1)] ++ [LastPktSz]
         end,
  Pkts =  listn:split_on_positions(Data, Lens),

  {ok, H} = inet:gethostname(),
  {ok, Host} = inet:getaddr(H, inet),
  {TrMod, CnArgs} = 
    case Tr of
      tcp -> {wsocket_tcp_transport,
	      [{transp_opts, [{reuseaddr, false}]}, {timeout,?WAIT_TIMEOUT}]
	     };
      udp -> {wsocket_udp_transport,
	      [{transp_opts, [{reuseaddr, false}]}, {timeout,?WAIT_TIMEOUT}]}
    end,
  process_flag(trap_exit,true),
  case catch
    begin
      ProtoFun = make_client_proto(Pkts, Delay),
      SockPid =
	spawn_link(wsocket_connect, start,
		   [Host, ?LISTEN_PORT, TrMod,
		    fun packer_test/2, ProtoFun, CnArgs]),
      clear_mbox(),
      Ret = 
	receive
	  {recv, SockPid, Packet} ->
	    {ok, Packet};
	  {'EXIT', SockPid, Reason} ->
	    {error, Reason};
	  Other ->
	    {error, {unexpected, Other}}
	end,
      Ret
    end of
    {ok, Result} ->
      Result;
    Error ->
      process_flag(trap_exit,true),
      misc:better_error(Error)
  end.

clear_mbox() ->
  receive
    _Any ->
      clear_mbox()
  after 0 ->
      ok
  end.
%%
multi_run(Funs) ->
  Self = self(),
  process_flag(trap_exit, true),
  WrkPids = lists:map(fun (F) ->
                          WF = fun () -> multi_run_worker(F) end,
                          spawn_link(WF)
                      end, Funs),
  [WrkPid ! start || WrkPid <- WrkPids],
  Results = multi_run_collect_results(WrkPids),
  multi_run_sort(WrkPids, Results).

multi_run_sort(WrkPids, Results) ->
  multi_run_sort(WrkPids, Results, []).

multi_run_sort([], _, Ret) ->
  lists:reverse(Ret);
multi_run_sort([Pid|Ps], Res, Acc) ->
  PidRes = (catch [throw(Data) || {P, Data} <- Res, P == Pid]),
  multi_run_sort(Ps, lists:keydelete(Pid, 1, Res), [PidRes|Acc]).

multi_run_collect_results(WrkPids) ->
  multi_run_collect_results(WrkPids, []).

multi_run_collect_results([], Ret) ->
  lists:reverse(Ret);
multi_run_collect_results(WPs, Acc) ->
  receive
    {'EXIT', Pid, Result} ->
      multi_run_collect_results(lists:delete(Pid, WPs), [{Pid, Result}|Acc])
  end.

multi_run_worker(Fun) ->
  receive start ->
      exit(Fun())
  end.

%%%
%%
%% Test tacacs send/recv packets
%%
%%%
tacacs_packet_test(start) ->
  Port = ?LISTEN_PORT,
  Max = ?MAX_CONN,
  (catch wsocket_listen:stop(Port)),
  {TrMod, TrOpts} =
    {wsocket_tcp_transport,
     [{transp_opts, [{packet, 0}, {reuseaddr,true}]}]
    },
  PackFun = tac_packet:make_packer(?SKEY),
  ProtoFun = make_recv_tacacs_proto(authen_start, ?RECV_TIMEOUT),
  Tests =
    [
     {fun wsocket_listen_start/5, 
      [{"tacacs_packet_test server, start",
        [Port, TrMod, PackFun, ProtoFun, [{max, Max}|TrOpts]], 
        ?match({ok, _})}
      ]}
    ],
  process_tests(Tests);
tacacs_packet_test(stop) ->
  Tests =
    [{{wsocket_listen, stop},
      [{"stopping tacacs_packet_test server",
	[?LISTEN_PORT],
	ok}
      ]}
    ],
  process_tests(Tests);
tacacs_packet_test(tests) ->
  Tests =
    [
     {fun send_tacacs_packet/2,
      [{"tacacs_packet_test client_request login",
        [?SKEY,
	 #authen_start{
	    action=login,
	    user=#userid{user="TestUSER", peer={"5.6.7.8","4321"}},
	    authen=#authen{priv_lvl=12, type=pap,service=ppp}, 
	    data="Password"
	   }
	],
        ok}
      ]}
    ],
  process_tests(Tests).

send_tacacs_packet(SKey, Packet) ->
  {ok, H} = inet:gethostname(),
  {ok, Host} = inet:getaddr(H, inet),
  {TrMod, CnArgs} =
    {wsocket_tcp_transport,
     [{transp_opts, [{reuseaddr, false}]}, {timeout,?WAIT_TIMEOUT}]},
  ProtoFun = make_send_tacacs_proto(Packet, ?RECV_TIMEOUT),
  PackFun = tac_packet:make_packer(SKey),
  wsocket_connect:start(Host, ?LISTEN_PORT, TrMod, PackFun, ProtoFun, CnArgs).

make_send_tacacs_proto(Packet, Timeout) ->

  SessNo = tac_lib:int_to_bytes(16#1234),
  SeqNo = 1,
  SingleConnect = 16#0,
  Unenc = 16#0,

  fun (start, {Socket, TrMod}) ->
      {ok, {IP, Port}} = TrMod:peername(Socket),

      Sess = #session{unencrypt=Unenc,
		      single_conn=SingleConnect, sess_no=SessNo,
		      seq_no=SeqNo,
		      peer={IP,Port}
		      },
      {Header, Data} = tac_body:pack(Sess, Packet),
      {done, {Header, Data}};
      (Error, _) ->
      io:format("~p~n", [Error]),
      Error
  end.

make_recv_tacacs_proto(Type, Timeout) ->
  RecvTacacsProto = 
    fun ({recv, Data}, Peer) ->
	Packet = tac_body:parse(Type, Data),
	io:format("peer ~p packet ~p~n", [Peer, Packet]),
	done;
	(Error, _) ->
	Error
    end,
  fun (start, {Socket, TrMod}) ->
      {ok, Peer} = TrMod:peername(Socket),
      {recv, RecvTacacsProto, Peer, Timeout};
      (Error, _) ->
      io:format("Error: ~p~n", [Error]),
      Error
  end.

%%%

dbg_info(F, A) ->
  ok.

%%   {Connect, Send, Recv, Close} =
%%     case Tr of
%%       tcp ->
%% 	{fun (Addr, Port, Opts) ->
%% 	     gen_tcp:connect(Addr, Port, [{packet, 0}|Opts], ?WAIT_TIMEOUT)
%% 	 end,
%% 	 fun (Sock, Packet) ->
%% 	     gen_tcp:send(Sock, Packet)
%% 	 end,
%% 	 fun (Sock) ->
%% 	     gen_tcp:recv(Sock, 0, ?RECV_TIMEOUT)
%% 	 end,
%% 	 fun (Sock) ->
%% 	     gen_tcp:close(Sock)
%% 	 end};
%%       udp ->
%% 	{fun (Addr, Port, Opts) ->
%% 	     {ok, Sock} = gen_udp:open(0, Opts),
%% 	     {ok, {Sock, Addr, Port}}
%% 	 end,
%% 	 fun ({Sock, Addr, Port}, Packet) ->
%% 	     gen_udp:send(Sock, Addr, Port, Packet)
%% 	 end,
%% 	 fun ({Sock, Addr, Port}) ->
%% 	     case gen_udp:recv(Sock, 0, ?RECV_TIMEOUT) of
%% 	       {ok, {Addr,  Port,  Packet}} ->
%% 		 {ok, Packet};
%% 	       Error ->
%% 		 Error
%% 	     end
%% 	 end,
%% 	 fun ({Sock, _, _}) ->
%% 	     gen_udp:close(Sock)
%% 	 end}
%%     end,

%%do_recv(S, Recv, Close, Bin) ->
%%  case Recv(S) of
%%    {ok, Data} ->
%%      do_recv(S, Recv, Close, erlang:concat_binary([Bin, Data]));
%%    {error, closed} ->
%%      Close(S),
%%      %%?dbg("recv ~p", [binary_to_list(Bin)]),
%%      {ok, binary_to_list(Bin)};
%%    {error, timeout} ->
%%      %% log error here
%%      Close(S),
%%      {ok, binary_to_list(Bin)};
%%    {error, Reason} ->
%%      %% log error here
%%      Close(S),
%%      exit(Reason)
%%  end.

%%do_recv(S, Bin) ->
%%  receive
%%    {tcp, _P, Data} ->
%%      do_recv(S, erlang:concat_binary([Bin, Data]));
%%    {tcp_closed, P} ->
%%      gen_tcp:close(P),
%%      %%?dbg("recv ~p", [binary_to_list(Bin)]),
%%      {ok, Bin};
%%    {tcp_error, P, Reason} ->
%%      gen_tcp:close(P),
%%      exit(Reason);
%%    Other ->
%%      gen_tcp:close(S),
%%      exit({error, Other})
%%  after ?WAIT_TIMEOUT ->
%%      exit(timeout)
%%  end.

