%%%----------------------------------------------------------------------
%%% File    : taclient.erl
%%% Author  :  <svg@surnet.ru>
%%% Purpose : 
%%% Created : 21 Dec 1999 by  <svg@surnet.ru>
%%%----------------------------------------------------------------------

-module(taclient).
-author('svg@surnet.ru').


-include("tacacs.hrl").

-export([login/5, enable/5, chpass/5, chpass/6, account/6, authorize/5]).

-import(wslib.props).

-define(dbg(_F, _A), tac_lib:dbg(get(dbg), _F, _A)).

login(Server, User, A=#authen{type=chap}, {Id, Chal, Pass}, ConnArgs) ->
  Cipher = [Id|Chal ++ binary_to_list(erlang:md5([Id|Pass++Chal]))],
  login(Server, User, A, Cipher, ConnArgs);
login(Server, User, Authen, Pass, ConnArgs)
  when is_record(User, userid),
       is_record(Authen, authen) ->
  Data = #client_request{action=login,
			 user=User,
			 authen=Authen,
			 pass=Pass
			},
  connection(Server, Data, ConnArgs).

enable(Server, User, Authen, Pass, ConnArgs)
  when is_record(User, userid),
       is_record(Authen, authen) ->
  Data = #client_request{action=enable,
			 user=User,
			 pass=Pass,
			 authen=Authen
			},
  connection(Server, Data, ConnArgs).

chpass(Server, User, Authen, {NewPass, OldPass}, ConnArgs) ->
  chpass(Server, User, Authen, NewPass, OldPass, ConnArgs).

chpass(Server, User, Authen, NewPass, OldPass, ConnArgs)
  when is_record(User, userid),
       is_record(Authen, authen) ->
  Data = #client_request{action=chpass,
			 user=User,
			 authen=Authen,
			 pass=NewPass, data=OldPass},
  connection(Server, Data, ConnArgs).
  
authorize(Server, User, Authen, Args, ConnArgs)
  when is_record(User, userid),
       is_record(Authen, authen) ->
  Data = #client_request{action=authorize,
			 user=User,
			 authen=Authen,
			 args=Args},
  connection(Server, Data, ConnArgs).

account(Server, User, Authen, Action, Args, ConnArgs)
  when is_record(User, userid),
       is_record(Authen, authen) ->
  Args1 = lists:map(fun (A) when is_record(A, msg_arg) ->
			A;
			({K, V}) ->
			#msg_arg{name=K, value=V}
		    end, Args),
  Data = #client_request{action=accounting,
			 user=User,
			 authen=Authen,
			 flags=Action,
			 args=Args1
			},
  connection(Server, Data, ConnArgs).

connection(_S=#serverid{peer={Host,Port}, skey=SKey}, Data, Args) ->
  [Timeout, Dbg] =
    props:gv_map([timeout, {debug, []}], check_props(Args)),
  Dbg1 = if (Dbg == true) or (Dbg == all) ->
	     [{protocol, true}, {packet, true}, {transport, true}];
	    true ->
	     Dbg
	 end,
  {ok, Ip} = inet:getaddr(Host, inet),
  Self = self(),
  TrMod = wsocket_tcp_transport,
  PctFun = tac_packet:mk_packer(client, SKey, []),
  PrtFun = tac_protocol:mk_fsm(client, []),
  TrArgs = [{transp_opts,
	     [{packet, 0}, {reuseaddr,false}]},
	    {timeout,Timeout}],
  SockPid =
    wsocket_connect:start_link(
      Ip, Port, TrMod, PctFun, PrtFun, [{debug, Dbg1}|TrArgs]),
  Ret =
    receive
      {SockPid, {protocol_pid, Worker}} ->
	link(Worker),
	Worker ! {Self, {request, Data}},
	receive
	  {Worker, {answer, Result}} ->
	    Worker ! {Self, stop},
	    Result
	end
    end,
  Ret.

check_props(Args) ->
  {ok, Props} = props:check_required([{timeout, ?TCP_TIMEOUT},
				      {debug, []}
				     ],
				     Args),
  Props.
