%%%-------------------------------------------------------------------
%%% File    : wsaccount_transaction.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%%
%%% Created : 18 Oct 2004 by  <svg@surnet.ru>
%%%-------------------------------------------------------------------
%%**
%% 
%% @doc
%% @end
%%*
-module(wsaccount_transaction).

-behaviour(wsdcomp).

-export([init/2, access_info/3, handle_event/6]).

-import(wsform.search_query).

-include("wstl.hrl").

%% component
access_info(_, _, _) ->
  exit(must_never_reach).

handle_event(Event, From,  WA, Srv, State, StateData) ->
  exit(must_never_reach).

init(Arg={AccountId, TrTypeId, WA}, Props) ->
  PkNames = odbcdb_schema:primary_key(account_transaction),
  TrTypeIdT = {transaction_type_id, TrTypeId},
  AccountIdT = {account_id, AccountId},
  Dbh = wsdb:dbh(WA),
  FormType = wslib:gv(form_type, Props, table),
  Props1 = Props ++ [{template, "lib/wsform"}],

  TrTypeStr = wslanguage:msg(WA, TrTypeId, []),
  Label = wslanguage:msg(WA, 'ACCOUNT_TRANSACTION_OP', 
			 [{transaction_type_id, TrTypeStr}]),
  Form = create_form(Label, WA),

  Row =
    if (FormType =/= rowset) ->
	RowHlr = mk_row_handler(AccountId, TrTypeId),
	wsdcomp:create(wsform.row, {PkNames, Form, RowHlr, WA}, WA, Props1);
       true ->
	undefined
    end,
  Rowset =
    if (FormType =/= row) ->
	MkQuery =
	  fun (Where) ->
	      Q = wsdb_odbc:mk_select_query(
		    account_transaction,
		    [account_id, transaction_type_id|Where]
		   ),
	      fun (P, Cont) -> Q([AccountIdT, TrTypeIdT|P], Cont) end
	  end,
	Query = search_query:mk_query(
		  MkQuery, Form, WA,
		  [{order_by_fields, [{entry_date, desc}]}]
		 ),
	Pg = {wsform.odbcdb_pager, wsdb:handler(Dbh), []},
  	wsdcomp:create(wsform.rowset, {PkNames, Form, Pg, Query}, WA, Props1);
       true ->
	undefined
    end,
  case FormType of
    row ->
      {component, Row};
    rowset ->
      {component, Rowset};
    table ->
      RowsetHook = wsform.expand_rowset_hook:create_editrow_hook(Row),
      Comp =
	wscomp.expand:create_comp(Rowset, RowsetHook, Props1),
      {component, Comp}
  end.

mk_row_handler(AccountId, TrTypeId) ->
  RowHlr = wsform.row:create_db_handler(account_transaction),
  fun ({Op, Row}, Dbh) ->
      Row1 =  [{account_id, AccountId}, {transaction_type_id, TrTypeId}|Row],
      RowHlr({Op, Row1}, Dbh)
  end.

%% Form
-define(form_name, account_transaction).

create_form(Label, WA) ->
  Fields =
    [id,
     entry_date,
     source_type_id,
     source_id,
     tariff_id,
     quantity,
     amount,
     discount
    ],
  wsform:create(
    ?form_name,
    [{label, fun (_, _) -> Label end}
    ],
    Fields,
    [
     {data_type,   fun data_type/2},
     {out_attrs,   fun out_attrs/2},
     {default,     fun default/2},
     {label,       {wsform_lib, language_label}},
     {ss,          [wsite.form.field:ss(tariff_id, WA),
		    wsform.ss:form_field(wsxmlt:m_name(Fields), WA),
		    wsxmlt:skip()
		   ]}
    ],
    WA).

data_type(id                 , _) -> integer;
data_type(entry_date         , _) -> datetime;
data_type(source_type_id     , _) -> atom;
data_type(source_id          , _) -> integer;
data_type(tariff_id          , _) -> integer;
data_type(quantity           , _) -> integer;
data_type(amount             , _) -> integer;
data_type(discount           , _) -> integer.

default(id, WA) ->
  Dbh = wsdb:dbh(WA),
  wsdb:next_rowid(Dbh, account_transaction);
default(entry_date         , _) -> erlang:localtime();
default(source_type_id     , _) -> undefined;
default(source_id          , _) -> undefined;
default(tariff_id          , _) -> undefined;
default(quantity           , _) -> 0;
default(amount             , _) -> 0;
default(discount           , _) -> 0.

out_attrs(entry_date, _) -> [{maxlength, "20"}, {size, "20"}];
out_attrs(post_date, _) -> [{maxlength, "20"}, {size, "20"}];
out_attrs(tariff_id, _) -> [];
out_attrs(_, _) -> [{maxlength, "128"}, {size, "64"}].

