%%% File    : wscontact_means.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created : 24 Jul 2003 by  <svg@surnet.ru>

%%
%% $Id: wscontact_means.erl,v 1.2 2003/09/04 17:11:36 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wscontact_means).

-behaviour(wsdcomp).

-export([init/2, access_info/3, handle_event/6]).
-export([call_id_selector/4, id_selector_return/4]).

-import(wsform.search_query).
-import(wsform.ss).
-import(wsform.field_type).

-include("wstl.hrl").

-record(state, {means_type_id, props=[]}).

access_info(_, _, _) -> #access_info{events=[], default_event=refresh}.

handle_event(Event, security_alert, WA, Srv, State, StateData) ->
  {stop, ok};
handle_event(Event, From,  WA, Srv, State, StateData) ->
  {next_state, ok, State, StateData}.

init(WA, Props)  when record(WA, ws_arg) ->
  {ok, call_id_selector, #state{props=Props}};
init({TypeId, WA}, Props) ->
  Props1 = Props ++ [{template, "wscontact/means"}],
  PkNames = [id],
  Dbh = wsdb:dbh(WA),
  Form = create_form(TypeId, WA),
  FormType = wslib:gv(form_type, Props, table),
  Row = 
    if (FormType =/= rowset) ->
	RowHlr = wsite.db.contact_means,
	wsdcomp:create(
	  wsform.row,
	  {PkNames, Form, RowHlr, WA},
	  WA,
	  [{readonly_fields, [type_id]}|Props1]
	 );
       true ->
	undefined
    end,
  Rowset =
    if (FormType =/= row) ->
	MkQuery =
	  fun (Where) ->
	      Q = wsdb_odbc:mk_select_query(
		    contact_means,[type_id|Where]),
	      fun (P, Cont) ->
		  Q([{type_id, TypeId}|P], Cont)
	      end
	  end,
	Query = search_query:mk_query(MkQuery, Form, WA,
				      [{order_by_fields, [id]}]),
	Pg = {wsform.odbcdb_pager, wsdb:handler(Dbh), []},
	wsdcomp:create(wsform.rowset, {PkNames, Form, Pg, Query}, WA, Props1);
       true ->
	undefined
    end,
  Comp =
    case FormType of
      row ->
	Row;
      rowset ->
	Rowset;
      table ->
	RowsetHook = wsform.expand_rowset_hook:create_editrow_hook(Row),
	wscomp.expand:create_comp(Rowset, RowsetHook, [])
    end,
  {component, Comp}.

call_id_selector(_, WA, Srv, State) ->
  SelProps = [{template, "lib/button_selector"},
	      {selector_attrs, [{orientation, horizontal}]}
	     ],
  SelId = wsite.contact_means_type:create_id_selector(id, WA, SelProps),
  {call_exec, enter, SelId(component, WA), id_selector_return, State}.

id_selector_return({ok, [TypeId]}, WA, Srv, S=#state{props=Props}) ->
  Comp = wsdcomp:create(?MODULE, {TypeId, WA}, WA,
			[{template, "wscontact/means"}|Props]),
  {exec, enter, Comp};
id_selector_return(_, _, _, _) ->
  {stop, ok}.

%% Wsform stuff

create_form(TypeId, WA) ->
  Fields =
    case TypeId of
      "TELECOM" ->
	 [id,
	  type_id,
	  type_description,
	  country_code,
	  area_code,
	  contact_number
	 ];
      "EMAIL" ->
	 [id,
	  type_id,
	  type_description,
	  email_address
	 ]
    end,
  TypeSel = wsite.contact_means_type:create_id_selector(type_id, WA, []),
  DescSel = wsite.contact_means_type:
    create_description_selector(type_description, TypeId, WA, []),
  Label = wslanguage:
    msg(WA, 'CONTACT_MEANS_FORM_LABEL',
	[{description, wslanguage:msg(WA, list_to_atom(TypeId), [])}]),
  wsform:create(
    ?MODULE,
    [{label, fun (_, _) -> Label end},
     {ss,    {wsform.ss, form}}],
    Fields,
    [{data_filter, fun data_filter/3},
     {validate,    fun validate/4},
     {data_type,   fun data_type/2},
     {out_attrs,   fun out_attrs/2},
     {default,     mk_default(TypeId)},
     {label,       {wsform_lib, language_label}},
     {ss,          [TypeSel(wsform_ss, WA),
		    DescSel(wsform_ss, WA),
		    ss:form_field(wsxmlt:m_name(contact_number),
				  [ss:phone_number()],
				  WA),
		    ss:form_field(wsxmlt:m_name(Fields--[contact_number]), WA),
		    wsxmlt:skip()]}
    ],
    WA).

data_filter(N, V, WA) when N == contact_number;
			   N == country_code;
			   N == area_code ->
  {ok, Ret, _} = regexp:gsub(V, "[^0-9]", ""),
  Ret;
data_filter(N, V, WA) ->
  wsform_lib:def_data_filter(N, V, WA).

data_type(id, _)      -> integer;
data_type(_, _)       -> string.

validate(N, V, KVs, WA) when N == contact_number;
			     N == country_code;
			     N == area_code ->
  field_type:validate(phone, V, KVs, WA);
validate(email_address, V, KVs, WA) ->
  field_type:validate(email_address, V, KVs, WA);
validate(_, _, _, _) ->
  true.

mk_default(TypeId) ->
  fun (N, WA)  when N == country_code;
		    N == area_code ->
      wsform_lib:def_value(N, WA);
      (id, _) ->
      fun (WA) ->
	  Dbh = wsdb:dbh(WA),
	  wsdb:next_rowid(Dbh, contact_means)
      end;
      (type_id, _) ->
      TypeId;
      (_, _) ->
      required
  end.

out_attrs(id, _)               -> [{maxlength, "16"}, {size, "8"}];
out_attrs(type_id, _)          -> [{maxlength, "64"}, {size, "32"}];
out_attrs(email_address, _)    -> [{maxlength, "128"}, {size, "64"}];
out_attrs(type_description, _) -> [{maxlength, "64"}, {size, "32"}];
out_attrs(country_code, _)     -> [{maxlength, "4"}, {size, "2"}];
out_attrs(area_code, _)        -> [{maxlength, "8"}, {size, "8"}];
out_attrs(contact_number, _)   -> [{maxlength, "16"}, {size, "8"}].

