%%% File    : wsecurity_group.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created :  4 Dec 2002 by  <svg@surnet.ru>

%%
%% $Id: wsecurity_group.erl,v 1.5 2003/09/05 20:33:18 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wsecurity_group).

-behaviour(wsdcomp).

-export([init/2, access_info/3, handle_event/6]).

-import(wsform.search_query).

-include("wstl.hrl").

-define(group_permissions, "group_permissions").

access_info(_, _, _) ->
  exit(must_never_reach).

handle_event(Event, From,  WA, Srv, State, StateData) ->
  exit(must_never_reach).

init(WA, Props) ->
  PkNames = [id],
  Dbh = wsdb:dbh(WA),
  Form = create_form(WA),
  MkQuery = fun (Where) ->
		wsdb_odbc:mk_select_query(security_group, Where)
	    end,
  Query = search_query:mk_query(MkQuery, Form, WA, [{order_by_fields, [id]}]),
  Pg = {wsform.odbcdb_pager, wsdb:handler(Dbh), []},
  Rowset = wsdcomp:create(wsform.rowset, {PkNames, Form, Pg, Query}, WA, Props),
  RowHlr = wsform.row:create_db_handler(security_group),
  Row = wsdcomp:create(wsform.row, {PkNames, Form, RowHlr, WA}, WA, []),
  Row1 = wscomp.expand:create_comp(
		  Row,
		  wsform.expand_row_hook,
		  {fun row_hook/2, [], ?group_permissions},
		  []),
  RowsetHook = wsform.expand_rowset_hook:create_editrow_hook(Row1),
  Comp = wscomp.expand:create_comp(Rowset, RowsetHook, []),
  {component, Comp}.

row_hook(init, Arg) -> {ok, []};
row_hook({is_active, _WA, _Srv}, _HD) ->
  true;
row_hook({access_info, WA, Srv}, _HD) ->
  #access_info{events=[do_group_permissions], default_event=refresh};
row_hook({call, do_group_permissions, WA, Srv}, HD) ->
  case wsdcomp:super_any_state(Srv, current_rowid) of
    [{_, GroupId}] ->
      PermComp = 
	wsdcomp:create(wsecurity_group_permission,
		       {GroupId, WA}, WA, [{place_at, ?group_permissions}]),
      ok = wsdcomp:spawn_comp(Srv, ?group_permissions, PermComp),
      wsdcomp:call(Srv, ?group_permissions, enter), 
      {ok, ok, HD};
    _ ->
      {ok, ok, HD}
  end;
row_hook({call, Event, WA, Srv}, HD) ->
  {ok, undefined, HD};
row_hook({event, Msg, From, WA, Srv}, HD) ->
  {ok, undefined, HD}.

%% Wsform stuff

create_form(WA) ->
  wsform:create(
    security_group,
    [{label, {wsform_lib, language_label}}],
    [id, comment],
    [{data_filter, fun data_filter/3},
     {data_type, fun data_type/2},
     {out_attrs, fun out_attrs/2},
     {default,   fun default/2},
     {label,     {wsform_lib, language_label}}
    ],
    WA).

data_filter(K=id, V, WA) ->
  wslib.str:to_upper(wsform_lib:def_data_filter(K, V, WA));
data_filter(K, V, WA) -> 
      wsform_lib:def_data_filter(K, V, WA).

data_type(_, _)      -> string.
  
default(id, _) -> required;
default(N, WA) -> wsform_lib:def_value(N, WA).

out_attrs(id, _)      -> [{maxlength, "64"}, {size, "32"}];
out_attrs(comment, _) -> [{maxlength, "128"}, {size, "64"}].
