%%% File    : wsecurity_group_permission.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created :  4 Dec 2002 by  <svg@surnet.ru>

%%
%% $Id: wsecurity_group_permission.erl,v 1.1 2003/09/01 15:18:40 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wsecurity_group_permission).

-behaviour(wsdcomp).

-export([init/2, access_info/3, handle_event/6]).

-include("wstl.hrl").

access_info(_, _, _) ->
  exit(must_never_reach).

handle_event(Event, From,  WA, Srv, State, StateData) ->
  exit(must_never_reach).
  
init(Arg={Id, WA}, Props) ->
  GroupId = {group_id, Id},
  Comp = wsdcomp:create(
	   wscomp.add_delete_selector,
	   {fun add_permissions/4,
	    fun select_add_permissions/2,
	    fun delete_permissions/4,
	    fun select_del_permissions/2,
	    [GroupId],
	    WA
	   },
	   WA,
	   Props ++ [{label, security_permission}]
	  ),
  {component, Comp}.

add_permissions(_, Ps, WA, Params) ->
  Dbh = wsdb:dbh(WA),
  lists:foreach(fun (P) ->
		    {ok, _} = add_permission(Dbh, P, Params)
		end, Ps).

add_permission(Dbh, PermissionId, Params) ->
  wsdb:insert_row(Dbh, security_group_permission,
		  [{permission_id, PermissionId}|Params]).

select_add_permissions(WA, Params) ->
  Dbh = wsdb:dbh(WA),
  Rows =
    [selector_kv(Row)
     || Row <- wsdb:do(Dbh,select,['SECURITY_PERMISSION_NOT_IN_GROUP',Params])],
  {ok, Rows}.

delete_permissions(_, Ps, WA, Params) ->
  Dbh = wsdb:dbh(WA),
  lists:foreach(fun (P) ->
		    {ok, _} = delete_permission(Dbh, P, Params)
		end, Ps).

delete_permission(Dbh, PermissionId, Params) ->
  wsdb:delete_row(Dbh, security_group_permission,
		  [{permission_id, PermissionId}|Params]).

select_del_permissions(WA, Params) ->
  Dbh = wsdb:dbh(WA),
  Rows =
    [selector_kv(Row)
     ||Row <- wsdb:do(Dbh,select,['SECURITY_PERMISSION_IN_GROUP',Params])],
  {ok, Rows}.

selector_kv(Row) ->
  Id = wslib:gv(id, Row),
  Desc =  wslib:gv(comment, Row),
  {Id, io_lib:format("~s - ~s", [Id, Desc])}.
