%%%-------------------------------------------------------------------
%%% File    : wsite.erl
%%% Author  :  <svg@surnet.ru>
%%% Purpose : 
%%% Created : 19 Nov 2002 by  <svg@surnet.ru>
%%%-------------------------------------------------------------------

%%
%% $Id: wsite.erl,v 1.1 2003/09/01 15:18:40 vsekissov Exp $
%% 
%%**
%% 
%% .* wsite
%% 
%%*
-module(wsite).
-author('svg@surnet.ru').

-export([start/0, stop/0, out/1, init_run_env/2, install/0]).

-import(wslib.listn).

-include("wstl.hrl").

out(Arg) ->
  Res = (catch wsrun:out(Arg, ?MODULE)),
  Res.

start() ->
  mnesia:start(),
  odbcdb:start(),
  case application:start(?MODULE) of
    {error,{not_started,wstl}} ->
      application:start(wstl),
      ok = application:start(?MODULE);
    ok ->
      ok;
    Error ->
      exit(Error)
  end,
  {ok, StlEnv} = application:get_env(?MODULE, stl_yaws),
  wstl:start_application(
    ?MODULE,
    [{wsacl, [{module, wsite_acl}, {init_param, wsdb:dbh(init_db_env())}]},
     {stl_yaws, StlEnv},
     {init_run_env, []},
     {module, ?MODULE}
    ]).

stop() ->
  wstl:stop_application(?MODULE),
  application:stop(?MODULE).

init_run_env(init, _) ->
  {ok, []};
init_run_env(WA, Props) when is_record(WA, ws_arg) ->
  DbEnv = init_db_env(),
  Party = listn:mk_list(add_party_to_env(WA, wsdb:dbh(DbEnv), Props)),
  {ok, DbEnv++Party, Party}.

init_db_env() ->
  wsdb:init_run_env(wsdb_odbc:wsdb_handler(odbcdb), []).

add_party_to_env(WA=#ws_arg{user=undefined}, Dbh, Props) ->
  [];
add_party_to_env(WA=#ws_arg{user=U}, Dbh, Props) ->
  case proplists:lookup(party, Props) of
    none ->
      case catch
	begin
	  {ok, Login} = wsdb:read_row(Dbh, user_login, [{login_id, U}]),
	  {ok, Party} = wsdb:read_row(Dbh, party, 
				      [{id, wslib:gv(party_id, Login)}])
	end
	of
	{ok, P} ->
	  {party, P};
	_ ->
	  []
      end;
    P ->
      P
  end.
  
install() ->
  tacacs:install().
