%%% File    : entity_type.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created :  4 Dec 2002 by  <svg@surnet.ru>

%%
%% $Id: entity_type.erl,v 1.5 2003/10/31 17:51:02 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wsite.entity_type).

-behaviour(wsdcomp).

-export([init/2, access_info/3, handle_event/6]).

-import(.wsdb).
-import(.wsdcomp).
-import(.stl_yaws).
-import(.wsdb_odbc).
-import(.lists).
-import(.wslib).
-import(wsform.search_query).

-include("wstl.hrl").

-define(view_entities, "view_entities").
-define(entity_attrs, "entity_attrs").

%% component
access_info(_, _, _) ->
  exit(must_never_reach).

handle_event(Event, From,  WA, Srv, State, StateData) ->
  exit(must_never_reach).
  
init({WA, TableName, PkName, AttrTableName, AttrFk}, Props) ->
  PkNames = [PkName],
  Dbh = wsdb:dbh(WA),
  Form = wsite.form.entity_type:create(TableName, PkName, WA, Props),
  MkQuery = fun (Where) ->
		wsdb_odbc:mk_select_query(TableName, Where)
	    end,
  Query = search_query:mk_query(MkQuery, Form, WA,
				[{order_by_fields, PkNames}]),
  Pg = {wsform.odbcdb_pager, wsdb:handler(Dbh), []},
  RowHlr = wsform.row:create_db_handler(TableName),
  ViewEntitiesHook = {wsform.expand_row_hook,
		      {fun view_entities_hook/2, PkName, ?view_entities}
		     },
  AttrsHook = {wsform.expand_row_hook,
	       {fun type_attr_hook/2, {AttrTableName, AttrFk}, ?entity_attrs}
	      },
  RowHooks = [ViewEntitiesHook, AttrsHook],
  Rowset = wsdcomp:create(wsform.rowset, {PkNames, Form, Pg, Query}, WA, Props),
  Row = wsdcomp:create(wsform.row, {PkNames, Form, RowHlr, WA}, WA, Props),
  Row1 = wscomp.expand:create_comp(Row, RowHooks, []),
  RowsetHook = wsform.expand_rowset_hook:create_editrow_hook(Row1),
  Comp = wscomp.expand:create_comp(Rowset, RowsetHook, []),
  {component, Comp}.

%% Hooks
view_entities_hook(init, PkName) ->
  {ok, PkName};
view_entities_hook({is_active, _, _}, _) ->
  true;
view_entities_hook({access_info,  WA, Srv}, _) ->
  #access_info{events=[do_view_entities], default_event=refresh};
view_entities_hook({call, do_view_entities,  WA, Srv}, PkName) ->
  Row = wsdcomp:super_any_state(Srv, row),
  FormName = entity_form_name(Row),
  TypeId = wslib:gv(PkName, Row),
  call_entity_table_form(WA, Srv, FormName, TypeId),
  {ok, ok, PkName};
view_entities_hook({call, Event, WA, Srv}, HD) ->
  {ok, undefined, HD};
view_entities_hook({event, Msg, From, WA, Srv}, HD) ->
  {ok, undefined, HD}.

call_entity_table_form(WA, Srv, FormModule, TypeId) ->
  Comp = wsdcomp:create(FormModule, WA, WA, 
			[{place_at, ?view_entities},
			 {entity_type_id, TypeId}]), 
  ok = wsdcomp:spawn_comp(Srv, ?view_entities, Comp),
  wsdcomp:call(Srv, ?view_entities, enter),
  ok.

type_attr_hook(init, HD={TableName, ForeignKeyName}) ->
  {ok, HD};
type_attr_hook({is_active, _, _}, _) ->
  true;
type_attr_hook({access_info, WA, Srv}, HD) ->
  Row = wsdcomp:super_any_state(Srv, row),
  EI =
    case wslib:gv(has_table, Row) of
      false ->
	#access_info{events=[do_type_attrs], default_event=refresh};
      _ ->
	#access_info{events=[], default_event=refresh}
    end,
  EI;
type_attr_hook({call, do_type_attrs,  WA, Srv}, HD={TName, FkName}) ->
  [{_, FkVal}]  = wsdcomp:super_any_state(Srv, current_rowid),
  Comp = wsdcomp:create(wsite.entity_type_attr, {WA, TName, FkName, FkVal}, WA,
			[{template, "lib/wsform"}]),
  {call_exec, enter, Comp, HD};
type_attr_hook({call, Event, WA, Srv}, HD) ->
  {ok, undefined, HD};
type_attr_hook({event, Msg, From, WA, Srv}, HD) ->
  {ok, undefined, HD}.

entity_form_name(Row) ->
  wslib:gv(form, Row).
