%%% File    : operator.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created :  4 Dec 2002 by  <svg@surnet.ru>

%%
%% $Id: operator.erl,v 1.1 2003/10/31 17:53:48 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*

-module(wsite.operator).

-behaviour(wsdcomp).

-export([init/2, access_info/3, handle_event/6]).
-export([enter/4, go/4, logout_return/4, walk_org_tree/4, walk_org_tree_return/4]).

-import(lists).
-import(wsdcomp).
-import(wsdb).
-import(wslanguage).
-import(wslib).
-import(wslib.str).
-import(wscomp.expand_rowset_hook).
-import(wscomp.expand).
-import(wsform.field_type).
-import(wsite_acl).
-import(wscomp.switchwm).

-include("wstl.hrl").

-define(switchwm, "switchwm").

-record(state, {org_path=[],relationship_categories=[]}).

%%access_own_organization

access_info(_, _, S=#state{relationship_categories=RCs}) ->
  Events1 = [do_next_org_lvl,
	     do_set_current_org,
	     do_relationship_category,
	     do_logout],
  Events2 =
    case lists:any(fun (RC) -> wslib:gv(id, RC) == "CUSTOMER" end,
		   RCs) of
      true ->
	[do_party_relationship_service|Events1];
      false ->
	Events1
    end,
  #access_info{events=Events2, default_event=refresh}.

handle_event(Event, security_alert, WA, Srv, State, S) ->
  {stop, {security_alert, Event}};
handle_event(_, old_request, WA, Srv, State, StateData) ->
  {next_state, ok, State, StateData};
handle_event(_, _, WA, Srv, State, StateData) ->
  {next_state, ok, State, StateData}.
  
init(WA, Props) ->
  {ok, enter, #state{}, Props}.

enter(_, WA, Srv, S) ->
  Comp = wsdcomp:create(wscomp.switchwm,
			WA, WA, [{place_at,'SwitchwmOut'}, {show_tab, true}]),
  ok = wsdcomp:spawn_comp(Srv, ?switchwm, Comp),
  wsdcomp:call(Srv, ?switchwm, enter),
  case find_permitted_orgs(WA) of
    Perm when Perm == own;
	      Perm == all->
      walk_org_tree({init, Perm}, WA, Srv, S);
    _ ->
      wslib:show_lerror(not_found,
			[{thing, wslanguage:msg(WA, organization)}], WA),
      {stop, {error, security_alert}}
  end.

go(do_logout, WA, Srv, S) ->
  Comp = wsdcomp:create(wslogout, WA, WA, [{render_as, page}]),
  {call_exec, enter, Comp, logout_return, S};
go(E, WA, Srv, S) when E == do_set_current_org;
		       E == do_next_org_lvl ->
  walk_org_tree(E, WA, Srv, S);
go(organization_changed, WA, Srv, S) ->
  Org = current_org(S),
  Dbh = wsdb:dbh(WA),
  PartyId = wslib:gv(id, Org),
  RelCategories =
    wsite.party_relationship_category:valid_categories(Dbh, PartyId, to),
  switchwm:delete_children(Srv, ?switchwm, all),
  next_state(ok, go, S#state{relationship_categories=RelCategories});
go(do_relationship_category, WA, Srv, S=#state{relationship_categories=RCs}) ->
  CatId = wslib:gv(relationship_category_id, wslib:arg_input(WA)),
  [RelCategory] = [RC || RC <- RCs, wslib:gv(id, RC) == CatId],
  CatDesc = wslib:gv(description,RelCategory),
  Org = current_org(S),
  Comp =
    wsdcomp:create(wsparty_relationship, {Org, to, RelCategory, WA}, WA,
		   [{detail_props, [{show_categories, false}]}]),
  switchwm:add_child(Srv, ?switchwm, CatId, Comp, CatDesc),
  {next_state, ok, go, S};
go(do_party_relationship_service, WA, Srv, S) ->
  Org = current_org(S),
  Comp =
    wsdcomp:create(wsparty_relationship_service,
		   {Org, to, WA}, WA,
		   []),
  switchwm:add_child(Srv, ?switchwm, "party_relationship_service", Comp,
		     wslanguage:msg(WA, {service, plural})),
  {next_state, ok, go, S};

go(_, WA, Srv, S) ->
  next_state(ok, go, S).

logout_return(ok, _, _, _) ->
  {stop, ok};
logout_return(_, _, _, S) ->
  {next_state, ok, go, S}.

%% Path=[{Party, ChildrenSel}, ...]
walk_org_tree({init, Arg=own}, WA, Srv, S) ->
  {ok, Comp} = party_employer_selector(create_selector, Arg, WA),
  Node = {Arg, fun party_employer_selector/3},
  Path = [Node],
  {call_exec, enter, Comp, walk_org_tree_return, S#state{org_path=Path}};
walk_org_tree({init, Arg=all}, WA, Srv, S) ->
  {ok, Comp} = party_subordinate_selector(create_selector, Arg, WA),
  Node = {Arg, fun party_subordinate_selector/3},
  Path = [Node],
  {call_exec, enter, Comp, walk_org_tree_return, S#state{org_path=Path}};
walk_org_tree(do_set_current_org, WA, Srv, S=#state{org_path=Path}) ->
  In = wslib:arg_input(WA),
  PathLen = length(Path),
  NxtLvl =
    field_type:integer(in, wslib:gv(do_set_current_org, In, PathLen), WA),
  if (NxtLvl >= PathLen) ->
      {next_state, ok, go, S};
     true ->
      NxtPath = lists:sublist(Path, PathLen-NxtLvl+1, PathLen),
      case NxtPath of
	[_] ->
	  walk_org_tree(do_next_org_lvl, WA, Srv, S#state{org_path=NxtPath});
	_ ->
	  go(organization_changed, WA, Srv, S#state{org_path=NxtPath})
      end
  end;
walk_org_tree(do_next_org_lvl, WA, Srv, S=#state{org_path=Path}) ->
  {Party, Hlr} = hd(Path),
  {ok, Comp} = Hlr(create_selector, Party, WA),
  {call_exec, enter, Comp, walk_org_tree_return, S};
walk_org_tree(_, WA, Srv, S) ->
  {next_state, ok, go, S}.

walk_org_tree_return({ok, Rel}, WA, Srv, S=#state{org_path=Path}) ->
  {_, NodeHlr} = hd(Path),
  {ok, Node} = NodeHlr(select_party, Rel, WA),
  NxtPath = [Node|Path],
  go(organization_changed, WA, Srv, S#state{org_path=NxtPath});
walk_org_tree_return(_, WA, Srv, S=#state{org_path=[Root]}) ->
  walk_org_tree(do_next_org_lvl, WA, Srv, S);
walk_org_tree_return(_, WA, Srv, S) ->
  {next_state, ok, go, S}.


%% internal defs
current_org(S=#state{org_path=[{Org, _}|_]}) ->
  Org;
current_org(_) ->
  undefined.

next_state(Ret, StateName,
	   S=#state{org_path=OP,
		    relationship_categories=RC}) ->
  Out = [{'OrgPath',
	  [[]|[wslib:gv(name, Org) || {Org, _} <- tl(lists:reverse(OP))]]},
	 {'RelationshipCategories', RC}
	],
  {next_state, Ret, Out, StateName, S}.

party_employer_selector(create_selector, own, WA) ->
  Party = wslib:gv(party, WA#ws_arg.app_env),
  {ok, RelCategory} =
    wsdb:read_row(wsdb:dbh(WA), party_relationship_category,
		  [{id, 'EMPLOYMENT'}]),
  Rowset = wsdcomp:create(wsparty_relationship,
			  {Party, from, RelCategory, WA},
			  WA,
			  [{form_type, rowset},
			   {render_as, page},
			   {page_template, "screen_only"}
			  ]),
  RowsetHook = wsform.expand_rowset_hook:create_select_rowid_hook(),
  Comp = wscomp.expand:create_comp(Rowset, RowsetHook, []),
  {ok, Comp};
party_employer_selector(select_party, Rel, WA) ->
  PartyId = wslib:gv(party_id_to, Rel),
  {ok, Party} = wsdb:read_row(wsdb:dbh(WA), party, [{id, PartyId}]),
  {ok, {Party, fun party_subordinate_selector/3}}.

party_subordinate_selector(create_selector, all, WA) ->
  Rowset = wsdcomp:create(wsparty,
			  WA, WA,
			  [{form_type, rowset},
			   {party_type_id, "ORGANIZATION"},
			   {render_as, page},
			   {page_template, "screen_only"}
			  ]),
  RowsetHook = wsform.expand_rowset_hook:create_select_rowid_hook(),
  Comp = wscomp.expand:create_comp(Rowset, RowsetHook, []),
  {ok, Comp};
party_subordinate_selector(create_selector, Party, WA) ->
  {ok, RelCategory} =
    wsdb:read_row(wsdb:dbh(WA), party_relationship_category,
		  [{id, 'ORGANIZATION_DIVISION'}]),
  Rowset = wsdcomp:create(wsparty_relationship,
			  {Party, to, RelCategory, WA},
			  WA,
			  [{form_type, rowset},
			   {render_as, page},
			   {page_template, "screen_only"}
			  ]),
  RowsetHook = wsform.expand_rowset_hook:create_select_rowid_hook(),
  Comp = wscomp.expand:create_comp(Rowset, RowsetHook, []),
  {ok, Comp};
party_subordinate_selector(select_party, KVs, WA) ->
  PartyId =
    case wslib:gv(party_id_from, KVs) of
      undefined ->
	wslib:gv(id, KVs);
      Id ->
	Id
    end,
  {ok, Party} = wsdb:read_row(wsdb:dbh(WA), party, [{id, PartyId}]),
  {ok, {Party, fun party_subordinate_selector/3}}.

find_permitted_orgs(WA) ->
  OrgAllPerm =
    wsite_acl:check_permission(WA, {db, 'ACCESS_ANY_ORGANIZATION'}, []),
  case OrgAllPerm of
    permit ->
      all;
    deny ->
      OrgPerm =
	wsite_acl:check_permission(WA, {db, 'ACCESS_OWN_ORGANIZATION'}, []),
      case OrgPerm of
	permit ->
	  own;
	deny ->
	  deny
      end
  end.
