%%%-------------------------------------------------------------------
%%% File    : party.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%%
%%% Created :  3 Nov 2003 by  <svg@surnet.ru>
%%%-------------------------------------------------------------------
%%
%% $Id$
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wsite.party).

-export([party_menu_row_hook/2, party_category_menu_row_hook/2]).

-import(wslib).
-import(wsdcomp).
-import(wsrun).
-import(wsdb).

-include("wstl.hrl").

party_menu_row_hook(init, Data) ->
  {ok, Data};
party_menu_row_hook({is_active, WA, Srv}, _) ->
  true;
party_menu_row_hook({access_info, WA, Srv}, _) ->
  #access_info{events=[do_party_menu]};
party_menu_row_hook({call, do_party_menu, WA, Srv}, HD) ->
  In = wslib:arg_input(WA),
  FormName = wslib:gv(form_name, In),
  RowId = wsdcomp:super_any_state(Srv, current_rowid),
  PartyId = wslib:gv(id, RowId, wslib:gv(party_id, RowId)),
  Dbh = wsdb:dbh(WA),
  {ok, Party} = wsdb:read_row(Dbh, party, [{id, PartyId}]),
  {Module, _} = wsrun:comp_from_path(WA, FormName),
  InitArg = {Party, WA},
  Comp = wsdcomp:create(Module, InitArg, WA, [{template, FormName}]),
  {call_exec, enter, Comp, HD};
party_menu_row_hook(_, HD) ->
  {ok, undefined, HD}.

-record(category_hstate, {direction, categories}).

party_category_menu_row_hook(init, Direction) ->
  {ok, #category_hstate{direction=Direction}};
party_category_menu_row_hook({is_active, WA, Srv}, _) ->
  true;
party_category_menu_row_hook({access_info, WA, Srv}, _) ->
  #access_info{events=[do_relationship_category]};
party_category_menu_row_hook({call, do_relationship_category, WA, Srv},
			     S=#category_hstate{direction=D,
						categories=Cs}) ->
  In = wslib:arg_input(WA),
  CatId = wslib:gv(relationship_category_id, In),
  case catch [throw(C) || C <- Cs, wslib:gv(id, C) == CatId] of
    [] ->
      {ok, undefined, S};
    Category ->
      RowId = wsdcomp:super_any_state(Srv, current_rowid),
      PartyId = wslib:gv(id, RowId, wslib:gv(party_id, RowId)),
      Dbh = wsdb:dbh(WA),
      {ok, Party} = wsdb:read_row(Dbh, party, [{id, PartyId}]),
      Comp = wsdcomp:create(wsparty_relationship,
			    {Party, D, Category, WA}, WA, []),
      {call_exec, enter, Comp, S}
  end;
party_category_menu_row_hook({event, _, super, WA, Srv},
			     S=#category_hstate{direction=Dir,
						categories=undefined}) ->
  Mode = wsdcomp:super_any_state(Srv, mode),
  if (Mode == view) ->
      RowId = wsdcomp:super_any_state(Srv, current_rowid),
      if RowId =/= [] ->
	  PartyId = wslib:gv(id, RowId, wslib:gv(party_id, RowId)),
	  Dbh = wsdb:dbh(WA),
	  Cs = wsite.party_relationship_category:valid_categories(Dbh, PartyId, Dir),
	  {ok, undefined, [{'RelationshipCategories', Cs}],
	   S#category_hstate{categories=Cs}};
	 true ->
	  {ok, undefined, S}
      end;
     true ->
      {ok, undefined, S}
  end;
party_category_menu_row_hook(_, S=#category_hstate{categories=undefined}) ->
  {ok, undefined, S};
party_category_menu_row_hook(_, S=#category_hstate{categories=Cs}) ->
  {ok, undefined, [{'RelationshipCategories', Cs}], S}.
