%%% File    : wsite_acl.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created :  4 Apr 2003 by  <svg@surnet.ru>

%%
%% $Id: wsite_acl.erl,v 1.1 2003/09/01 15:18:40 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wsite_acl).

-behaviour(wsacl).

-export([init/1, terminate/1, authorize/6, check_permission/4, check_permission/3]).
-export([get_login/3, is_login_enabled/3, login_successful/3, validate_password/4, logout/3]).
-export([get_password/2, set_password/3]).

-import(wslib.passwd).

-include("wstl.hrl").

-record(state, {db}).

init(Db) ->
  {ok, #state{db=Db}}.

get_login(LoginId, _, S=#state{db=Db}) ->
  {get_login(Db, [{login_id, LoginId}]), S}.

validate_password(Login, Passwd, _, S) ->
  {check_password(wslib:kv(password, Login), Passwd), S}.

is_login_enabled(Login, _, S) ->
  {wslib:kv(enabled, Login), S}.

login_successful(Login, _, S=#state{db=Db}) ->
  ok = inc_login_counter(Db, Login),
  {{ok, wslib:kv(login_id, Login)}, S}.

logout(User, _, S) ->
  {ok, S}.

authorize(User=undefined, Event, Path, AccessId, Attrs, S) ->
  Ret = 
    case catch
      permission('ANONYMOUS', component, 'COMPONENT_ACCESS',
		 {Event, Path, AccessId, Attrs}) of
      permit ->
	permit;
      _ ->
	deny
    end,
  {Ret, S};
authorize(User, Event, Path, AccessId, Attrs, S) ->
  check_permission({user, User}, {component, 'COMPONENT_ACCESS'},
		   {Event, Path, AccessId, Attrs}, S).

check_permission(WA, Perm, Attrs) when is_record(WA, ws_arg) ->
  wsacl:check_permission(WA, {user, WA#ws_arg.user}, Perm, Attrs).

check_permission({UserCatefory, User}, {PermCategory, Perm}, Attrs,
		 S=#state{db=Db}) ->
  Groups = wsite.security_group:permission_groups(User, Perm, Db),
  Ret=check_permission(UserCatefory, Groups, PermCategory, Perm, Attrs),
  ?dbg("~p", [{UserCatefory, Groups, PermCategory, Perm, Ret}]),
  {Ret, S}.

check_permission(user, Groups, PermCat, Perm, Attrs) ->
  find_permission(fun (Group) ->
		      permission(Group, PermCat, Perm, Attrs)
		  end, Groups++['ANONYMOUS']).

terminate(S) ->
  ok.

check_password({Type, Crypt}, Key) ->
  wslib.passwd:crypt(Key, Crypt, Type) == Crypt.

find_permission(Test, []) ->
  deny;
find_permission(Test, [G|Groups]) ->
  case catch Test(G) of
    permit ->
      permit;
    _ ->
      find_permission(Test, Groups)
  end.

permission('OPERATOR', db, 'ACCESS_OWN_ORGANIZATION', _) ->
  permit;
permission('SUPER_OPERATOR', db, 'ACCESS_ANY_ORGANIZATION', _) ->
  permit;
%%
permission(_, component,'COMPONENT_ACCESS', {_, _, logout, _}) ->
  permit;
permission(_, component,'COMPONENT_ACCESS', {_, ["approot"], _, _}) ->
  permit;
permission(_, component,'COMPONENT_ACCESS', {_, ["index"], _, _}) ->
  permit;
permission('ANONYMOUS', component, 'COMPONENT_ACCESS', _) ->
  deny;
permission('OPERATOR',  component, 'COMPONENT_ACCESS', _) ->
  permit;
permission('SUPER_OPERATOR',  component, 'COMPONENT_ACCESS', _) ->
  permit;
permission(_, Group, Perm, {Event, Path, AccessId, Attrs}) ->
  deny.

%%
get_login(Dbh, LoginQ) ->
  case wsdb:read_row(Dbh, user_login, LoginQ) of
    {ok, Login} ->
      {ok, lists:map(fun ({enabled, V}) ->
			 {enabled, odbcdb.attr_type:to_boolean(V)};
			 ({password, V}) ->
			 {password, password_from_string(V)};
			 (V) ->
			 V
		     end, Login)};
    Other ->
      Other
  end.

get_password(Dbh, LoginQ) ->
  case get_login(Dbh,  LoginQ) of
    {ok, KVs} ->
      case proplists:get_value(password, KVs) of
	Str when is_list(Str) ->
	  {ok, password_from_string(Str)};
	Other ->
	  Other
      end;
    Other ->
      Other
  end.

set_password(Dbh, LoginQ, Passwd) ->
  Str = password_to_string(Passwd),
  wsdb:do(Dbh, do_query, ['SET_USER_LOGIN_PASSWORD', [{password, Str}|LoginQ]]).
inc_login_counter(Dbh, Login) ->
  {ok, _} = wsdb:do(Dbh, do_query, ['INC_USER_LOGIN_COUNTER', Login]),
  ok.

password_from_string(Str) ->
  passwd:from_string(Str).

password_to_string(Passwd) ->
  passwd:to_string(Passwd).
