%%% File    : wsite_schema.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created : 11 Sep 2003 by  <svg@surnet.ru>

%%
%% $Id: wsite_schema.erl,v 1.1 2003/10/31 17:34:10 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wsite_schema).

-export([install/1]).

-define(table_attrs(T, As), {{table, T, attrs}, As}).
-define(pkey(T, PKs), {{table, T, primary_key}, PKs}).
-define(seq(T, SN),   {{table, T, keygenerator}, {sequence, SN}}).

install(odbcdb) ->
  odbcdb_schema:add_properties(
    [?table_attrs(role_type, [id, has_table, description, {form, [{type, atom}]}]),
     ?pkey(role_type, [id]),
     
     ?table_attrs(role_type_attr, [role_type_id, {attr_name, [{type, atom}]}]),
     ?pkey(role_type_attr, [role_type_id, attr_name]),

     ?table_attrs(location_role_type, [id, has_table, description, {form, [{type, atom}]}]),
     ?pkey(location_role_type, [id]),
     
     ?table_attrs(location_role_type_attr, [location_role_type_id, {attr_name, [{type, atom}]}]),
     ?pkey(location_role_type_attr, [location_role_type_id, attr_name]),

     ?table_attrs(contact_mech_role_type, [id, has_table, description, {form, [{type, atom}]}]),
     ?pkey(contact_mech_role_type, [id]),
     
     ?table_attrs(contact_mech_role_type_attr, [contact_mech_role_type_id, {attr_name, [{type, atom}]}]),
     ?pkey(contact_mech_role_type_attr, [contact_mech_role_type_id, attr_name]),

     ?table_attrs(party_type, [id, has_table, description, {form, [{type, atom}]}]),
     ?pkey(party_type, [id]),

     ?table_attrs(party_type_attr, [party_type_id, {attr_name, [{type, atom}]}]),
     ?pkey(party_type_attr, [party_type_id, attr_name]),

     ?table_attrs(party, [{id, [{type, integer}]}, party_type_id, name]),
     ?pkey(party, [id]),
     ?seq(party, party_id_seq),

     ?table_attrs(party_role, [party_id, role_type_id]),
     ?pkey(party_role, [party_id, role_type_id]),

     ?table_attrs(party_relationship_category, [id, description]),
     ?pkey(party_relationship_category, [id]),

     ?table_attrs(party_relationship_type, [id, role_type_id_from_valid, role_type_id_to_valid, description, category_id, {form, [{type, atom}]}]),
     ?pkey(party_relationship_type, [id]),

     ?table_attrs(party_relationship, [{id, [{type, integer}]}, party_id_from, role_type_id_from, party_id_to, role_type_id_to,  from_date, thru_date, party_relationship_type_id]),
     ?pkey(party_relationship, [party_id_from, role_type_id_from, party_id_to, role_type_id_to, from_date]),
     ?seq(party_relationship, party_relationship_id_seq),

     ?table_attrs(party_location, [party_id, postal_address_id, from_date, thru_date, location_role_type_id]),
     ?pkey(party_location, [party_id, location_role_type_id, from_date]),

     ?table_attrs(party_contact_mech, [party_id, contact_means_id, from_date, thru_date, contact_mech_role_type_id]),
     ?pkey(party_contact_mech, [party_id, contact_mech_role_type_id, from_date]),

     ?table_attrs(person, [party_id, last_name, first_name, middle_name, personal_title, gender, comments]),
     ?pkey(person, [party_id]),

     ?table_attrs(security_permission, [id, comment]),
     ?pkey(security_permission, [id]),

     ?table_attrs(security_group, [id, comment]),
     ?pkey(security_group, [id]),

     ?table_attrs(security_group_permission, []),
     ?pkey(security_group_permission, [group_id, permission_id]),

     ?table_attrs(postal_address, [{id, [{type, integer}]}, city, address1, address2, postal_code]),
     ?pkey(postal_address, [id]),
     ?seq(postal_address, postal_address_id_seq),

     ?table_attrs(contact_means_type, [id, description, comment]),
     ?pkey(contact_means_type, [id, description]),

     ?table_attrs(contact_means, [{id, [{type, integer}]}, type_id, name, type_description, country_code, area_code, contact_number, email_address]),
     ?pkey(contact_means, [id]),
     ?seq(contact_means, contact_means_id_seq),

     ?table_attrs(user_login,
            [login_id, password, password_hint, party_id,
             {disabled_datetime, [{type, datetime}]},
             {enabled,       [{type, boolean}]},
             login_attempts
            ]),
     ?pkey(user_login, [login_id]),

     ?table_attrs(user_login_security_group, [login_id, group_id]),
     ?pkey(user_login_security_group, [login_id, group_id]),

     ?table_attrs(service_type, [id, has_table, description, {form, [{type, atom}]}]),
     ?pkey(service_type, [id]),

     ?table_attrs(service_type_attr, [service_type_id, {attr_name, [{type, atom}]}]),
     ?pkey(service_type_attr, [service_type_id, attr_name]),

     ?table_attrs(product_type, [id, has_table, description, {form, [{type, atom}]}]),
     ?pkey(product_type, [id]),

     ?table_attrs(product_type_attr, [product_type_id, {attr_name, [{type, atom}]}]),
     ?pkey(product_type_attr, [product_type_id, attr_name]),

     ?table_attrs(party_relationship_service,
     [{id, [{type, integer}]}, service_type_id,
      party_id_from, role_type_id_from, party_id_to, role_type_id_to, from_date,
      created_date, status_date, status, comments, name,
      user_party_id, tariff_id,
      login_id, caller_phone, email_address,
      restday_mask,
      {restday_disabled_time_from, [{type, time}]},
      {restday_disabled_time_to,   [{type, time}]},
      {workday_disabled_time_from, [{type, time}]},
      {workday_disabled_time_to,   [{type, time}]},
      {day_time_limit, [{type, integer}]},
      {month_time_limit, [{type, integer}]}, 
      {year_time_limit, [{type, integer}]}, 
      {total_time_limit, [{type, integer}]},
      portname,
      network,
      source_informer_address,
      aggregate_rule_name_src_net,
      aggregate_rule_name_dst_net,
      traffic_direction 
    ]),
     ?pkey(party_relationship_service, [id]),
     ?seq(party_relationship_service, party_relationship_service_id_seq),
     
     ?table_attrs(tariff,
     [{id, [{type, integer}]},
     description, charge_type_id, service_type_id
    ]),
     ?pkey(tariff, [id]),
     ?seq(tariff, tariff_id_seq),

     ?table_attrs(party_relationship_account,
     [{id, [{type, integer}]}, account_type_id,
     party_id_from, role_type_id_from, party_id_to, role_type_id_to, from_date,
     created_date, status_date, status, amount, comments
    ]),
     ?pkey(party_relationship_account, [id]),
     ?seq(party_relationship_account, party_relationship_account_id_seq),
     
     ?table_attrs(leasedline_attr,
		  [source_informer_address,
		   {last_stat_date, [{type, datetime}]},
		   {period, [{type, time}]},
		   community,
		   device_ident,
		   {checkpoint, [{type, integer}]},
		   {device_counter_max, [{type, integer}]}
		  ]),
     ?pkey(leasedline_attr, [source_informer_address])

    ]).
