%%% File    : wsparty.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created :  4 Dec 2002 by  <svg@surnet.ru>

%%
%% $Id: wsparty.erl,v 1.3 2003/09/05 20:33:18 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wsparty).

-behaviour(wsdcomp).

-export([init/2, access_info/3, handle_event/6]).

-import(wsdb, [dbh/1]).
-import(wsform.search_query).

-include("wstl.hrl").

-define(party_roles, "party_roles").

access_info(_, _, _) ->
  exit(must_never_reach).

handle_event(Event, From,  WA, Srv, State, StateData) ->
  exit(must_never_reach).


init(WA, Props) ->
  PartyType = wslib:gv(party_type_id, Props,
		       wslib:gv(entity_type_id, Props, all)),
  init(PartyType, WA, Props).

%% 
init(TypeId, WA, Props) ->
  Dbh = dbh(WA),
  PkNames = [id],
  TypeAttrName = party_type_id,
  AttrTable = party_type_attr,
  %% get attributes
  {PartyQueryKeys, PartyQueryKVs, PartyTypeFields, FormLabel} =
    case TypeId of
      all ->
	FL = wslib:gv(form_label, Props, ""),
	{[], [], [TypeAttrName, name], FL};
      _ ->
	AttrFk = [{TypeAttrName, TypeId}],
	EntityFields = 
	  [wslib:gv(attr_name, Row)
	   || Row <- wsdb:select_rows(Dbh, AttrTable, AttrFk, [attr_name])],
	{ok, Type} = wsdb:read_row(Dbh, party_type, [{id, TypeId}]),
	FL = wslib:gv(form_label, Props, wslib:gv(description, Type)),
	{[TypeAttrName], AttrFk, EntityFields, FL}
    end,
  %% get description
  Form = wsite.form.party:create(FormLabel, PartyTypeFields, WA),
  FormType = wslib:gv(form_type, Props, table),
  Props1 = Props ++ [{template, "lib/wsform"}],
  Props2 = if TypeId == all ->
	       [{readonly_fields, [party_type_id]}|Props1];
	      true ->
	       Props1
	   end,
  Row = 
    if (FormType =/= rowset) ->
	RowHlr = mk_row_handler(PartyQueryKVs),
	RolesHook = {wsform.expand_row_hook,
		     {fun party_roles_hook/2, [], ?party_roles}
		    },
	Row1 = wsdcomp:create(wsform.row, {PkNames, Form, RowHlr, WA}, WA, Props2),
	wscomp.expand:create_comp(Row1, RolesHook, []);
       true -> undefined
    end,
  Rowset =
    if (FormType =/= row) ->
	MkQuery =
	  case wslib:gv(mk_query, Props) of
	    undefined ->
	      fun (Where) ->
		  QW = PartyQueryKeys++Where,
		  Q=wsdb_odbc:mk_select_query(party, QW),
		  fun (P, Cont) ->
		      Q(PartyQueryKVs++P, Cont)
		  end
	      end;
	    MQ ->
	      MQ
	  end,
	Query = search_query:mk_query(MkQuery, Form, WA,
				      [{order_by_fields, [id]}]),
	Pg = {wsform.odbcdb_pager, wsdb:handler(Dbh), []},
	wsdcomp:create(wsform.rowset, {PkNames, Form, Pg, Query}, WA, Props2);
       true ->
	undefined
    end,
  case FormType of
    row ->
      {component, Row};
    rowset ->
      {component, Rowset};
    table ->
      RowsetHook = wsform.expand_rowset_hook:create_editrow_hook(Row),
      Comp = wscomp.expand:create_comp(Rowset, RowsetHook, []),
      {component, Comp}
  end.

%% Row stuff
party_roles_hook(init, Data) ->
  {ok, Data};
party_roles_hook({is_active, WA, Srv}, _) ->
  true;
party_roles_hook({access_info, WA, Srv}, _) ->
  #access_info{events=[do_party_roles], default_event=refresh};
party_roles_hook({call, do_party_roles,  WA, Srv}, HD) ->
  PartyId = wslib:gv(id, wsdcomp:super_any_state(Srv, current_rowid)),
  Comp = wsdcomp:create(wsparty_role, {PartyId, WA}, WA,
			[{template, "wsparty/role"},
			 {place_at, ?party_roles}
			]),
  ok = wsdcomp:spawn_comp(Srv, ?party_roles, Comp),
  wsdcomp:call(Srv, ?party_roles, enter),
  {ok, ok, HD};
party_roles_hook({call, Event, WA, Srv}, HD) ->
  {ok, undefined, HD};
party_roles_hook({event, Msg, From, WA, Srv}, HD) ->
  {ok, undefined, HD}.

mk_row_handler(PartyType) ->
  RowHlr = wsform.row:create_db_handler(party),
  fun ({Op, Row}, Dbh) ->
      RowHlr({Op, PartyType++Row}, Dbh)
  end.
