%%% File    : wsparty_contact_mech.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created :  4 Dec 2002 by  <svg@surnet.ru>

%%
%% $Id: wsparty_contact_mech.erl,v 1.1 2003/10/31 17:34:10 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wsparty_contact_mech).

-behaviour(wsdcomp).

-export([init/2, access_info/3, handle_event/6]).

-import(wsform.search_query).

-include("wstl.hrl").

%% component
access_info(_, _, _) ->
  exit(must_never_reach).

handle_event(Event, From,  WA, Srv, State, StateData) ->
  exit(must_never_reach).

init(Arg={Party, WA}, Props) ->
  PkNames = [contact_mech_role_type_id, contact_means_id, from_date],
  Dbh = wsdb:dbh(WA),
  PartyId = {party_id, wslib:gv(id, Party)},
  MkRoleTypeSel =
    wsite.form.entity_type:create_selector(contact_mech_role_type_id,
					   contact_mech_role_type, id, description,
					   WA, []),
  Label = wslanguage:msg(WA, 'PARTY_CONTACT_MECH_FORM_LABEL', Party),
  Form = create_form(MkRoleTypeSel, Label, WA),
  ReadOnlyFields = [contact_means_id,
		    contact_means_type_comment,
		    contact_means_name],
  MkQuery =
    fun (Where) ->
	Q = wsdb_odbc:mk_select_query(party_contact_mech,[party_id|Where]),
	fun (P, Cont) -> Q([PartyId|P], Cont) end
    end,
  Query = search_query:mk_query(
	    MkQuery, Form, WA,
	    [{order_by_fields, [{from_date, desc}]},
	     {query_fields,
	      wsform:field_names(Form) -- ReadOnlyFields}
	    ]),
  Pg = {wsform.odbcdb_pager, wsdb:handler(Dbh), []},
  RowHlr = mk_row_handler(PartyId),
  RowProps = [{readonly_fields, ReadOnlyFields}],

  Row = wsdcomp:create(wsform.row, {PkNames, Form, RowHlr, WA}, WA, RowProps),
  RowsetHook = mk_rowset_hook(PartyId, Row, Label, WA),
  Rowset = wsdcomp:create(wsform.rowset, {PkNames, Form, Pg, Query}, WA, Props),
  Comp =
    wscomp.expand:create_comp(Rowset, RowsetHook, []),
  {component, Comp}.

mk_row_handler(PartyId) ->
  RowHlr = wsform.row:create_db_handler(party_contact_mech),
  fun ({Op, Row}, Dbh) ->
      RowHlr({Op, [PartyId|Row]}, Dbh)
  end.

mk_rowset_hook(_PartyId, Row, SelectorLabel, WA) ->
  FormName = "wscontact/means",
  SelProps = [{actions, [do_select, do_create, do_exit]},
	      {label, SelectorLabel}
	     ],
  Selector =
    (wsite.contact_means_type:create_id_selector(id, WA, SelProps))
      (component,WA),
  MkDetailComp =
    fun (Mode, Data, WsArg) ->
	{MeansType, FormType} =
	  case Mode of
	    select_row ->
	      {hd(Data), rowset};
	    create_row ->
	      {hd(Data), row};
	    view_row ->
	      Dbh = wsdb:dbh(WA),
	      {ok, CM} =
		wsdb:read_row(Dbh, contact_means, Data),
	      {wslib:gv(type_id, CM), row}
	  end,
	Comp = 
	  wsdcomp:create(wscontact_means,
			 {MeansType, WsArg}, WsArg,
			 [{form_type, FormType}]),
	case Mode of
	  create_row ->
	    {ok, Comp, {do_add_row, []}};
	  _ ->
	    {ok, Comp}
	end
    end,
  MkRowComp =
    fun (Op=create_row, RowId, _, WsArg) ->
	{ok, Row, {do_add_row, RowId}};
	(Op, OpData, _, _) ->
	{ok, Row, {Op, OpData}}
    end,
  Relationship = [{contact_means_id, id}],
  wsform.expand_rowset_hook:create_editrel_hook(
	   MkDetailComp, MkRowComp, Relationship, WA,
	   [{selector, Selector}]).

%% Form
-define(form_name, party_contact_mech).

create_form(MkRoleTypeSel, Label, WA) ->
  Fields = [contact_mech_role_type_id,
	    contact_means_id,
	    contact_means_type_comment,
	    contact_means_name,
	    from_date, thru_date],
  wsform:create(
    ?form_name,
    [{label, fun (_, _) -> Label end},
    {calculate, fun form_calculate/3}
    ],
    Fields,
    [
     {data_type,   fun data_type/2},
     {out_attrs,   fun out_attrs/2},
     {default,     fun default/2},
     {calculate,   fun calculate/4},
     {label,       {wsform_lib, language_label}},
     {ss,          [MkRoleTypeSel(wsform_ss, WA),
		    wsform.ss:form_field(wsxmlt:m_name(Fields), WA),
		    wsxmlt:skip()]}
    ],
    WA).

data_type(from_date, _) -> date;
data_type(thru_date, _) -> date;
data_type(contact_means_id, _) -> integer;
data_type(_, _)    -> string.

default(from_date, _) -> 
  calendar:local_time();
default(contact_mech_role_type_id, _) ->
  required;
default(contact_means_id, _) -> 
  required;
default(_, _) -> undefined.

form_calculate(Form, KVs, WA) ->
  Dbh = wsdb:dbh(WA),
  MeansId  = wslib:gv(contact_means_id, KVs),
  case wsdb:read_row(Dbh, contact_means, [{id, MeansId}]) of
    {ok, Means} ->
      [TypeId, TypeDesc, TypeName] =
	wslib:gv_map([type_id, type_description, name], Means),
      {ok, MeansType} =
	wsdb:read_row(Dbh, contact_means_type,
		      [{id, TypeId}, {description, TypeDesc}]),
      Ret = [{contact_means_name, TypeName},
	     {contact_means_type_comment, wslib:gv(comment, MeansType)}],
      Ret;
    _ ->
      []
  end.

calculate(F=contact_means_name, _, _, Means) ->
  {ok, wslib:gv(F, Means)};
calculate(F=contact_means_type_comment, _, _, Means) ->
  {ok, wslib:gv(F, Means)};
calculate(_, V, _, _) ->
  {ok, V}.

out_attrs(from_date, _) -> [{maxlength, "20"}, {size, "10"}];
out_attrs(thru_date, _) -> [{maxlength, "20"}, {size, "10"}];
out_attrs(_, _) -> [{maxlength, "128"}, {size, "64"}].
