%%% File    : wsparty_location.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created :  4 Dec 2002 by  <svg@surnet.ru>

%%
%% $Id: wsparty_location.erl,v 1.2 2003/09/05 20:33:18 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wsparty_location).

-behaviour(wsdcomp).

-export([init/2, access_info/3, handle_event/6]).

-import(wsform.search_query).

-include("wstl.hrl").

%% component
access_info(_, _, _) ->
  exit(must_never_reach).

handle_event(Event, From,  WA, Srv, State, StateData) ->
  exit(must_never_reach).

init(Arg={Party, WA}, Props) ->
  PkNames = [location_role_type_id, postal_address_id, from_date],
  Dbh = wsdb:dbh(WA),
  PartyId = {party_id, wslib:gv(id, Party)},
  MkRoleTypeSel =
    wsite.form.entity_type:create_selector(location_role_type_id,
					   location_role_type, id, description,
					   WA, []),
  Label = wslanguage:msg(WA, 'PARTY_LOCATION_FORM_LABEL', Party),
  Form = create_form(MkRoleTypeSel, Label, WA),
  MkQuery =
    fun (Where) ->
	Q = wsdb_odbc:mk_select_query(party_location,[party_id|Where]),
	fun (P, Cont) -> Q([PartyId|P], Cont) end
    end,
  Query = search_query:mk_query(
	    MkQuery, Form, WA,
	    [{order_by_fields, [{from_date, desc}]},
	     {query_fields,
	      wsform:field_names(Form) -- [postal_address_name]}
	    ]),
  Pg = {wsform.odbcdb_pager, wsdb:handler(Dbh), []},
  ReadOnlyFields = [postal_address_id, postal_address_name],
  RowHlr = mk_row_handler(PartyId),
  RowProps = [{readonly_fields, ReadOnlyFields}],

  Row = wsdcomp:create(wsform.row, {PkNames, Form, RowHlr, WA}, WA, RowProps),
  RowsetHook = mk_rowset_hook(Row, Label, WA),
  Rowset = wsdcomp:create(wsform.rowset, {PkNames, Form, Pg, Query}, WA, Props),
  Comp =
    wscomp.expand:create_comp(Rowset, RowsetHook, []),
  {component, Comp}.

mk_row_handler(PartyId) ->
  RowHlr = wsform.row:create_db_handler(party_location),
  fun ({Op, Row}, Dbh) ->
      RowHlr({Op, [PartyId|Row]}, Dbh)
  end.

mk_rowset_hook(Row, SelectorLabel, WA) ->
  FormName = "wspostal_address",
  MkDetailComp =
    fun (Mode, _, WsArg) ->
	FormType =
	  case Mode of
	    select_row ->
	      rowset;
	    _ ->
	      row
	  end,
	Comp = 
	  wsdcomp:create(wspostal_address, WsArg, WsArg,
			 [{template, FormName},
			  {form_type, FormType}]),
	case Mode of
	  create_row ->
	    {ok, Comp, {do_add_row, []}};
	  _ ->
	    {ok, Comp}
	end
    end,
  MkRowComp =
    fun (Op=create_row, RowId, _, WsArg) ->
	{ok, Row, {do_add_row, RowId}};
	(Op, OpData, _, _) ->
	{ok, Row, {Op, OpData}}
    end,
  Relationship = [{postal_address_id, id}],
  wsform.expand_rowset_hook:create_editrel_hook(
	   MkDetailComp, MkRowComp, Relationship, WA,
	   [{selector_label, SelectorLabel}]).

%% Form
-define(form_name, party_location).

create_form(MkRoleTypeSel, Label, WA) ->
  Fields = [location_role_type_id,
	    postal_address_id,
	    postal_address_name,
	    from_date, thru_date],
  wsform:create(
    ?form_name,
    [{label, fun (_, _) -> Label end}],
    Fields,
    [
     {data_type,   fun data_type/2},
     {out_attrs,   fun out_attrs/2},
     {default,     fun default/2},
     {calculate,   fun calculate/4},
     {label,       {wsform_lib, language_label}},
     {ss,          [MkRoleTypeSel(wsform_ss, WA),
		    wsform.ss:form_field(wsxmlt:m_name(Fields), WA),
		    wsxmlt:skip()]}
    ],
    WA).

data_type(from_date, _) -> date;
data_type(thru_date, _) -> date;
data_type(postal_address_id, _) -> integer;
data_type(_, _)    -> string.

default(from_date, _) -> 
  calendar:local_time();
default(location_role_type_id, _) ->
  required;
default(postal_address_id, _) -> 
  required;
default(_, _) -> undefined.

calculate(postal_address_name, _, KVs, WA) ->
  Dbh = wsdb:dbh(WA),
  AddrId = wslib:gv(postal_address_id, KVs, undefined),
  case wsdb:read_row(Dbh, postal_address, [{id, AddrId}]) of
    {ok, Addr} ->
      {ok, wsite.postal_address:name(Addr)};
    _ ->
      {ok, ""}
  end;
calculate(_, V, _, _) ->
  {ok, V}.

out_attrs(from_date, _) -> [{maxlength, "20"}, {size, "10"}];
out_attrs(thru_date, _) -> [{maxlength, "20"}, {size, "10"}];
out_attrs(_, _) -> [{maxlength, "128"}, {size, "64"}].

