%%% File    : wsparty_organization.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created :  9 Jul 2003 by  <svg@surnet.ru>

%%
%% $Id: wsparty_organization.erl,v 1.2 2003/09/05 19:08:37 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wsparty_organization).

-define(default_role, "PARENT_ORGANIZATION").
-define(party_role, "ORGANIZATION_UNIT").
-define(party_type, "ORGANIZATION").

-behaviour(wsdcomp).

-export([init/2, access_info/3, handle_event/6]).

-import(wsform.search_query).

-include("wstl.hrl").

access_info(_, _, _) ->
  exit(must_never_reach).

handle_event(Event, From,  WA, Srv, State, StateData) ->
  exit(must_never_reach).

%% as default form shows only parent organizations
%% (role == "PARENT_ORGANIZATION")
%% 
init(WA, Props) ->
  OrgRole = wslib:gv(role_type_id, Props, ?default_role),
  OrgRoles = if OrgRole == ?party_role ->
		 [OrgRole];
		true ->
		 [?party_role, OrgRole]
	     end,
  PartyType = {party_type_id, ?party_type},
  QueryRoleType = {role_type_id, OrgRole},
  PartyRoleTypes = [{role_type_id, R} || R <- OrgRoles],
  Dbh = wsdb:dbh(WA),
  PkNames = [id],
  EntityFields = [id, name],
  %% get description
  {ok, Type} = wsdb:read_row(Dbh, party_type, [{id, ?party_type}]),
  TypeDesc = wslib:gv(form_label, Props, wslib:gv(description, Type)),
  FormType = wslib:gv(form_type, Props, table),
  Form = wsite.form.party:create(TypeDesc, EntityFields, WA),
  Props1 = Props ++ [{template, "wsparty/organization"}],
  Row = 
    if (FormType =/= rowset) ->
	create_row_comp(PkNames, Form, PartyType, PartyRoleTypes, WA, Props1);
       true ->
	undefined
    end,
  Rowset =
    if (FormType =/= row) ->
	MkQuery =
	  fun (Where) ->
	      Q = wsdb_odbc:mk_query('PARTY_BY_TYPE_AND_ROLE', Where),
	      fun (P, Cont) ->
		  Q([QueryRoleType, PartyType|P], Cont)
	      end
	  end,
	Query = search_query:mk_query(MkQuery, Form, WA,
				      [{order_by_fields, [id]}]),
	Pg = {wsform.odbcdb_pager, wsdb:handler(Dbh), []},
	wsdcomp:create(wsform.rowset, {PkNames, Form, Pg, Query}, WA, Props1);
       true ->
	undefined
    end,
  case FormType of
    row ->
      {component, Row};
    rowset ->
      {component, Rowset};
    table ->
      RowsetHook = wsform.expand_rowset_hook:create_editrow_hook(Row),
      Comp = wscomp.expand:create_comp(Rowset, RowsetHook, []),
      {component, Comp}
  end.

create_row_comp(PkNames, Form, PartType, PartyRoleTypes, WA, Props) ->
  RowHlr = mk_row_handler(PartType, PartyRoleTypes),
  MenuHook = {wsform.expand_row_hook,
	      {{wsite.party, party_menu_row_hook}, [], undefined}
	     },
  CategoryHook = {wsform.expand_row_hook,
		  {{wsite.party, party_category_menu_row_hook}, to, undefined}
		 },
  RowHooks = case wslib:gv(show_categories, Props, true) of
	       true ->
		 [MenuHook, CategoryHook];
	       false ->
		 MenuHook
	     end,
  Row = wsdcomp:create(wsform.row, {PkNames, Form, RowHlr, WA}, WA,
		       [{template, "wsparty/organization"}]),
  wscomp.expand:create_comp(Row, RowHooks, []).

mk_row_handler(PartyType, PartyRoles) ->
  RowHlr = wsform.row:create_db_handler(party),
  fun ({Op, Row}, Dbh) ->
      TrFun =
	fun (Db) ->
	    case Op of
	      insert_row ->
		Ret = {ok, _} = RowHlr({Op, [PartyType|Row]}, Db),
		ok = insert_party_roles(Db, Row, PartyRoles),
		Ret;
	      delete_row ->
		ok = delete_party_roles(Db, Row, PartyRoles),
		RowHlr({Op, [PartyType|Row]}, Db);
	      _ ->
		RowHlr({Op, [PartyType|Row]}, Db)
	    end
	end,
      wsdb:transaction(Dbh, TrFun)
  end.

%% Db

insert_party_roles(Dbh, Row, RoleTypes) ->
  PartyId = wslib:gv(id, Row),
  lists:foreach(
    fun (RT) ->
	R = [{party_id, PartyId}, RT],
	{ok, _} = wsdb:insert_row(Dbh, party_role, R)
    end, RoleTypes),
  ok.

delete_party_roles(Dbh, Row, RoleTypes) ->
  PartyId = wslib:gv(id, Row),
  lists:foreach(
    fun (RT) ->
	R = [{party_id, PartyId}, RT],
	{ok, _} = wsdb:delete_row(Dbh, party_role, R)
    end, RoleTypes),
  ok.
