%%% File    : wsparty_person.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created :  4 Dec 2002 by  <svg@surnet.ru>

%%
%% $Id: wsparty_person.erl,v 1.2 2003/09/04 17:11:37 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wsparty_person).

-behaviour(wsdcomp).

-export([init/2, access_info/3, handle_event/6]).
-export([create_form/1]).

-import(wsdb, [dbh/1]).
-import(wscomp.selector).
-import(wsform.search_query).

-include("wstl.hrl").

-define(party_type, "PERSON").

access_info(_, _, _) ->
  exit(must_never_reach).

handle_event(Event, From,  WA, Srv, State, StateData) ->
  exit(must_never_reach).


init(WA, Props) ->
  TypeId = ?party_type,
  PkNames = [party_id],
  Dbh = dbh(WA),
  Form = create_form(WA),
  FormType = wslib:gv(form_type, Props, table),
  Props1 = Props ++ [{template, "wsparty/person"}],

  Row = 
    if (FormType =/= rowset) ->
	RowHlr = wsite.party_person,
	MenuHook = {wsform.expand_row_hook,
		    {{wsite.party, party_menu_row_hook}, [], undefined}
		   },
	Row1 = wsdcomp:create(wsform.row, {PkNames, Form, RowHlr, WA}, WA,
			      Props1),
	wscomp.expand:create_comp(Row1, [MenuHook], []);
       true ->
	undefined
    end,
  Rowset =
    if (FormType =/= row) ->
	MkQuery = fun (Where) ->
		      wsdb_odbc:mk_select_query(person, Where)
		  end,
	Query = search_query:mk_query(MkQuery, Form, WA,
				      [{order_by_fields, [party_id]}]),
	Pg = {wsform.odbcdb_pager, wsdb:handler(Dbh), []},
	wsdcomp:create(wsform.rowset, {PkNames, Form, Pg, Query}, WA, Props1);
       true ->
	undefined
    end,
  case FormType of
    row ->
      {component, Row};
    rowset ->
      {component, Rowset};
    table ->
      RowsetHook = wsform.expand_rowset_hook:create_editrow_hook(Row),
      Comp = wscomp.expand:create_comp(Rowset, RowsetHook, []),
      {component, Comp}
  end.

%% Wsform stuff

create_form(WA) ->
  Fields =
    [party_id,
     last_name,
     first_name,
     middle_name,
     personal_title,
     gender,
     comments
    ],
  wsform:create(
    person,
    [{label, {wsform_lib, language_label}}],
    Fields,
    [
     {data_type,   fun data_type/2},
     {out_attrs,   fun out_attrs/2},
     {default,     fun default/2},
     {label,       {wsform_lib, language_label}},
     {ss,          [(gender_selector(WA))(wsform_ss, WA),
		    wsform.ss:form_field(wsxmlt:m_name(Fields), WA),
		    wsxmlt:skip()]}
    ],
    WA).

data_type(party_id, _)       -> integer;
data_type(last_name, _)      -> string;
data_type(first_name, _)     -> string;
data_type(middle_name, _)    -> string;
data_type(personal_title, _) -> string;
data_type(gender, _)         -> string;
data_type(comments, _)       -> string.
  
default(party_id, _) ->
  fun (WA) ->
      Dbh = wsdb:dbh(WA),
      wsdb:next_rowid(Dbh, party)
  end;
default(gender, _) -> "male";
default(N, WA) -> wsform_lib:def_value(N, WA).

out_attrs(party_id, _)      -> [{maxlength, "16"}, {size, "8"}];
out_attrs(last_name, _) -> [{maxlength, "64"}, {size, "32"}];
out_attrs(first_name, _) -> [{maxlength, "64"}, {size, "32"}];
out_attrs(middle_name, _) -> [{maxlength, "64"}, {size, "16"}];
out_attrs(personal_title, _) -> [{maxlength, "16"}, {size, "8"}];
out_attrs(comments, _) -> [{maxlength, "128"}, {size, "64"}];
out_attrs(_, _) -> [].

gender_selector(WA) ->
  SelList = [{K, wslanguage:msg(WA, V, [])}
	     || {K, V} <- [{"male", 'MALE'}, {"female", 'FEMALE'}]],
  GetValues = fun (_, _) -> {ok, SelList} end,
  OnSelect = fun (Event, [G], _, _) -> {stop, {Event, G}} end,
  Data = [],
  Render = stl_yaws:create_screen1(WA, "lib/selector"),
  Props = [],
  MkSelector = selector:create(gender, GetValues, OnSelect, Data, Props).

