%%% File    : wsparty_relationship.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created :  4 Dec 2002 by  <svg@surnet.ru>

%%
%% $Id: wsparty_relationship.erl,v 1.2 2003/09/05 20:33:18 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wsparty_relationship).

-behaviour(wsdcomp).

-export([init/2, access_info/3, handle_event/6]).

-import(wslib.str).
-import(wsform.search_query).

-include("wstl.hrl").

%% component
access_info(_, _, _) ->
  exit(must_never_reach).

handle_event(Event, From,  WA, Srv, State, StateData) ->
  exit(must_never_reach).

init(Arg={Party, Direction, RelCategory, WA}, Props) ->
  {PartyIdMasterField, PartyIdDetField, RoleTypeIdDetField} =
    case Direction of
      to ->
	{party_id_to, party_id_from, role_type_id_from};
      from ->
	{party_id_from, party_id_to, role_type_id_to}
    end,
  PartyId = wslib:gv(id, Party),
  PkNames = [party_id_from, role_type_id_from, party_id_to, role_type_id_to, from_date],
  Dbh = wsdb:dbh(WA),
  CatId = wslib:gv(id, RelCategory),
  CatDesc = wslib:gv(description, RelCategory),
  Label = wslanguage:msg(WA, 'PARTY_RELATIONSHIP_FORM_LABEL',
			 [proplists:lookup(description, RelCategory)|Party]),
  MkPartyTypeSel = wsite.party_type:
    create_selector(party_type_id, WA,
		    [{label, Label},
		     {actions, [do_select, do_create, do_exit]}]),
  MkRelTypeSel = wsite.party_relationship_type:
    create_selector(Direction, party_relationship_type_id,
		    {party_id, PartyId},
		    fun (R) -> lists:member({category_id, CatId}, R) end,
		    WA, [{label, Label}]),
  Form = 
    create_form(PartyIdDetField, RelCategory, MkPartyTypeSel, MkRelTypeSel, Label, WA),
  FormType = wslib:gv(form_type, Props, table),
  ReadOnlyFields = [party_relationship_type_id,
		    PartyIdDetField,
		    party_name,
		    party_type_id
		   ],
  NonqueryFields = [party_name, party_type_id],
  Props1 = Props
    ++ [{readonly_fields, ReadOnlyFields},
	{nonquery_fields, NonqueryFields},
	{template, "lib/wsform"}
       ],
  Row = 
    if (FormType =/= rowset) ->
	RowHlr = mk_row_handler({PartyIdMasterField, PartyId}, PartyIdDetField, RoleTypeIdDetField),
	wsdcomp:create(wsform.row,
		       {PkNames, Form, RowHlr, WA},
		       WA,
		       Props1);
       true ->
	undefined
    end,
  Rowset =
    if (FormType =/= row) ->
	RelTypeIds =
	  [wslib:gv(id, R)
	   || R <- wsdb:select_rows(
		     Dbh, party_relationship_type,
		     [{category_id, wslib:gv(id, RelCategory)}])],
	RelTypeIdStr = "('"++ str:join(RelTypeIds, "','") ++ "')",
	QW = [PartyIdMasterField,
	      {in, party_relationship_type_id}
	     ],
	QP = [{PartyIdMasterField, PartyId},
	      {party_relationship_type_id, {noquote, RelTypeIdStr}}
	     ],
	MkQuery =
	  fun (Where) ->
	      Q =
		wsdb_odbc:mk_select_query(party_relationship,
					  QW++Where),
	      fun (P, Cont) -> Q(QP++P, Cont) end
	  
	  end,
	Query = search_query:mk_query(
		  MkQuery, Form, WA,
		  [{order_by_fields, [{from_date, desc},
				      RoleTypeIdDetField,
				      PartyIdDetField
				     ]}
		   |Props1]
		 ),
	Pg = {wsform.odbcdb_pager, wsdb:handler(Dbh), []},
	wsdcomp:create(wsform.rowset, {PkNames, Form, Pg, Query}, WA, Props1);
       true ->
	undefined
    end,
  case FormType of
    row ->
      {component, Row};
    rowset ->
      {component, Rowset};
    table ->
      RowsetHook = mk_rowset_hook(PartyIdDetField,
				  MkRelTypeSel(component, WA),
				  MkPartyTypeSel(component, WA),
				  Row, WA, Props),

      Comp =
	wscomp.expand:create_comp(Rowset, RowsetHook, []),
      {component, Comp}
  end.

mk_row_handler(PartyId, PartyIdDetField, RoleTypeIdDetField) ->
  RowHlr = wsform.row:create_db_handler(party_relationship),
  fun ({Op=insert_row, Row}, Dbh) ->
      PartyIdDir = wslib:gv(PartyIdDetField, Row),  
      RoleDir = wslib:gv(RoleTypeIdDetField, Row),
      wsdb:transaction(Dbh,
		       fun (Db) ->
			   ok=create_from_role(Db, [{party_id, PartyIdDir},
						    {role_type_id, RoleDir}]),
			   RowHlr({Op, [PartyId|Row]}, Db)
		       end);
      ({Op, Row}, Dbh) ->
      RowHlr({Op, [PartyId|Row]}, Dbh)
  end.

mk_rowset_hook(PartyIdDetField, RelTypeSel, PartyTypeSel, Row, WA, Props) ->
  Selector = select_relationship(RelTypeSel, PartyTypeSel, WA),
  Relationship = [{PartyIdDetField, id}],
  MkDetailComp =
    fun (Mode, Data, WsArg) ->
	{PartyTypeId, CompModule, FormType} =
	  case Mode of
	    select_row ->
	      {_RelType, PartyType} = Data,
	      {wslib:gv(id, PartyType), wslib:gv(form, PartyType), rowset};
	    create_row ->
	      {_RelType, PartyType} = Data,
	      {wslib:gv(id, PartyType), wslib:gv(form, PartyType), row};
	    view_row ->
	      Dbh = wsdb:dbh(WA),
	      {ok, Party} = wsdb:read_row(Dbh, party, Data),
	      Id = wslib:gv(party_type_id, Party),
	      {ok, PartyType} = wsdb:read_row(Dbh, party_type, [{id, Id}]),
	      {wslib:gv(id, PartyType), wslib:gv(form, PartyType), row}
	  end,
	Comp = wsdcomp:create(CompModule,
			      WsArg,
			      WsArg,
			      [{party_type_id, PartyTypeId},
			       {form_type, FormType}
			       |wslib:gv(detail_props, Props, [])
			      ]
			     ),
	case Mode of
	  create_row ->
	    {ok, Comp, {do_add_row, []}};
	  _ ->
	    {ok, Comp}
	end
    end,
  MkRowComp =
    fun (Op=create_row, RowId, {RelType, PartyType}, WsArg) ->
	FromRole = wslib:gv(role_type_id_from_valid, RelType),
	ToRole = wslib:gv(role_type_id_to_valid, RelType),
	RelTypeId =  wslib:gv(id, RelType),
	Rel =
	  [{role_type_id_from, FromRole},
	   {role_type_id_to, ToRole},
	   {party_relationship_type_id, RelTypeId}
	   |RowId
	  ],
	{ok, Row, {do_add_row, Rel}};
	(Op, OpData, _, _) ->
	{ok, Row, {Op, OpData}}
    end,
  wsform.expand_rowset_hook:
    create_editrel_hook(MkDetailComp, MkRowComp, Relationship, WA,
			[{selector, Selector}]).

-record(srh_state, {rel_type_sel, party_type_sel, rel_type}).

select_relationship(RelTypeSel, PartyTypeSel, WA) ->
  Comp = wsdcomp:create(wscomp.chain_dialog,
			{fun select_relationship_hlr/2,
			 {RelTypeSel, PartyTypeSel}
			},
			WA,
			[]),
  Comp.

select_relationship_hlr(init, {RelTypeSel, PartyTypeSel}) ->
  {ok,
   [select_reltype, select_partytype],
   #srh_state{rel_type_sel=RelTypeSel, party_type_sel=PartyTypeSel}
  };
select_relationship_hlr({start_dialog, _, _, _}, State) ->
  {ok, enter, State};
select_relationship_hlr({select_reltype, _, _ ,_},
			S=#srh_state{rel_type_sel=RTS}) ->
  {ok, RTS, enter, S};
select_relationship_hlr({select_partytype, {do_select, [RT]}, _, _},
			S=#srh_state{party_type_sel=PTS}) ->
  {ok, PTS, enter, S#srh_state{rel_type=RT}};
select_relationship_hlr({select_partytype, next_step, _, _},
			S=#srh_state{party_type_sel=PTS, rel_type=RT}) ->
  {ok, PTS, enter, S};
select_relationship_hlr({stop_dialog, {Event, [PT]}, _, _},
			S=#srh_state{rel_type=RT}) ->
  {ok, {Event, {RT, PT}}};
select_relationship_hlr(Msg, _) ->
  {stop, ok}.

%% Form
-define(form_name, party_relationship).

create_form(PartyIdDetField,
	    RelCategory, MkPartyTypeSel, MkRelTypeSel, Label, WA) ->
  Fields = [id,
	    party_relationship_type_id,
	    PartyIdDetField,
	    party_name,
	    party_type_id,
	    from_date, thru_date],
  wsform:create(
    ?form_name,
    [{label, fun (_, _) -> Label end},
    {calculate, mk_form_calculate(PartyIdDetField)}
    ],
    Fields,
    [
     {data_type,   fun data_type/2},
     {out_attrs,   fun out_attrs/2},
     {default,     fun default/2},
     {calculate,   fun calculate/4},
     {label,       {wsform_lib, language_label}},
     {ss,          [MkRelTypeSel(wsform_ss, WA),
		    MkPartyTypeSel(wsform_ss, WA),
		    wsform.ss:form_field(wsxmlt:m_name(Fields), WA),
		    wsxmlt:skip()]}
    ],
    WA).

mk_form_calculate(PartyIdDetField) ->
  fun (Form, KVs, WA) ->
      Dbh = wsdb:dbh(WA),
      PartyId = wslib:gv(PartyIdDetField, KVs),
      case wsdb:read_row(Dbh, party, [{id, PartyId}]) of
	{ok, Row} ->
	  Row;
	_ ->
	  []
      end
  end.
data_type(id, _) ->integer;
data_type(from_date, _) -> date;
data_type(thru_date, _) -> date;
data_type(party_id_from, _) -> integer;
data_type(party_id_to, _) -> integer;
data_type(_, _)    -> string.

calculate(party_type_id, _, KVs, Party) ->
  {ok, wslib:gv(party_type_id, Party, "")};
calculate(party_name, _, KVs, Party) ->
  {ok, wslib:gv(name, Party, "")};
calculate(_, V, _, _) ->
  {ok, V}.

default(id, _) ->
  fun (WA) ->
      Dbh = wsdb:dbh(WA),
      wsdb:next_rowid(Dbh, party_relationship)
  end;
default(from_date, _) -> 
  calendar:local_time();
default(party_id_to, WA) ->
  required;
default(party_id_from, WA) ->
  required;
default(F=party_relationship_type_id, WA) ->
  required;
default(F=party_type_id, WA) ->
  "";
default(party_id_to, _) ->
  required;
default(_, _) -> undefined.

out_attrs(party_id_to, _) -> [{maxlength, "16"}, {size, "8"}];
out_attrs(party_id_from, _) -> [{maxlength, "16"}, {size, "8"}];
out_attrs(from_date, _) -> [{maxlength, "10"}, {size, "10"}];
out_attrs(thru_date, _) -> [{maxlength, "10"}, {size, "10"}];
out_attrs(_, _) -> [{maxlength, "128"}, {size, "64"}].

%% Database

create_from_role(Dbh, Params) ->
  case wsdb:read_row(Dbh, party_role, Params) of
    {error, no_data_found} ->
      {ok, _} = wsdb:insert_row(Dbh, party_role, Params),
      ok;
    _ ->
      ok
  end.
