%%% File    : wsparty_relationship_account.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created :  4 Dec 2002 by  <svg@surnet.ru>
%%**
%% 
%% @doc
%% @end
%%*
-module(wsparty_relationship_account).

-behaviour(wsdcomp).

-export([init/2, access_info/3, handle_event/6]).

-import(wslib.str).
-import(wslib.listn).
-import(wsform.ss).
-import(wsform.row).
-import(wscomp.choose).
-import(wscomp.selector).
-import(wsform.search_query).
-import(wsform.ss).
-import(wsform.field_type).

-include("wstl.hrl").

-define(account_relationship_category, 'CUSTOMER').
-define(directions, [{to, "SUPPLIER"}, {from,"CUSTOMER"}]).
-define(form_name, party_relationship_account).

-define(account_transaction(_TYPE),
	"account_transaction_"++ atom_to_list(_TYPE)).

%% component
access_info(_, _, _) ->
  exit(must_never_reach).

handle_event(Event, From,  WA, Srv, State, StateData) ->
  exit(must_never_reach).

init(WA, Props) when is_record(WA, ws_arg) ->
  Comp = choose:create(fun choose_party_direction_hlr/2, {choose, Props}),
  {component, Comp};
init(Arg={Party, Direction, WA}, Props) ->
  {PartyIdMasterField, PartyIdDetField} =
    case Direction of
      to ->
	{party_id_to, party_id_from};
      from ->
	{party_id_from, party_id_to}
    end,
  PartyId = wslib:gv(id, Party),
  PkNames = odbcdb_schema:primary_key(party_relationship_account),
  Dbh = wsdb:dbh(WA),
  FormType = wslib:gv(form_type, Props, table),
  Props1 = Props ++ [{template, "lib/wsform"}],
  DirectionDesc =
    get_role_description(wslib:gv(Direction, ?directions), Dbh),
  FormLabel =
    wslanguage:msg(WA, party_relationship_account,
		   [{party_role_description, DirectionDesc},
		    {party_id, PartyId},
		    {party_name, wslib:gv(name, Party)}
		   ]),

  Form = create_form(WA, Props1++[{form_label, fun (_, _) -> FormLabel end}]),
  Row =
    if (FormType =/= rowset) ->
	Row1 = create_row(WA, Form, PkNames, Props),
	choose:create(fun rowset_row_hlr/2, {Party, Direction, Row1, Props1});
       true ->
	undefined
    end,
  Rowset =
    if (FormType =/= row) ->
	QW = [PartyIdMasterField],
	QP = [{PartyIdMasterField, PartyId}],
	MkQuery =
	  fun (Where) ->
	      Q =
		wsdb_odbc:mk_select_query(party_relationship_account, QW++Where),
	      fun (P, Cont) -> Q(QP++P, Cont) end
	  end,
	Query = search_query:mk_query(MkQuery, Form, WA,
				      [{order_by_fields, [PartyIdDetField]}]
				     ),
	Pg = {wsform.odbcdb_pager, wsdb:handler(Dbh), []},
	wsdcomp:create(wsform.rowset, {PkNames, Form, Pg, Query},
		       WA, Props1);
       true ->
	undefined
    end,
  case FormType of
    row ->
      {component, Row};
    rowset ->
      {component, Rowset};
    table ->
      RowsetHook = wsform.expand_rowset_hook:create_editrow_hook(Row),
      Comp =
	wscomp.expand:create_comp(Rowset, RowsetHook, []),
      {component, Comp}
  end.

rowset_row_hlr(init, Data) ->
  {ok, Data};
rowset_row_hlr({call, E={do_add_row, _}, WA, _},
	       {Party, Direction, RowComp, Props}) ->
  Comp = create_row_add_dialog(Party, Direction, WA, RowComp, Props),
  {exec, E, Comp};
rowset_row_hlr({call, Event, WA, _}, {_, _, RowComp, _}) ->
  {exec, Event, RowComp};
rowset_row_hlr(_, _) ->
  {stop, ok}.

%% choose party and direction(customer->from/supplier->to) dialog
-record(pdd_state, {party, direction, direction_sel}).

choose_party_direction_hlr(init, Data) ->
  {ok, Data};
choose_party_direction_hlr({call, enter, WA, _}, {choose, Props}) ->
  Comp = create_party_direction_dialog(WA, []),
  {call_exec, enter, Comp, {result, Props}};
choose_party_direction_hlr({call, {ok, {Party, Direction}}, WA, _},
			   {result, Props}) ->
  Comp = wsdcomp:create(?MODULE, {Party, Direction, WA}, WA, Props),
  {exec, enter, Comp};
choose_party_direction_hlr(_, _) ->
  {stop, ok}.

create_party_direction_dialog(WA, Props) ->
  Dbh = wsdb:dbh(WA),
  Directions =
    [{D, get_role_description(R, Dbh)} || {D, R} <- ?directions], 
  DirectionSelector =
    selector:create(id,
		    fun (_, _) -> {ok, Directions} end,
		    fun (_, Vs, _, _) ->
			{stop, {ok, [list_to_atom(V) || V <- Vs]}}
		    end,
		    [], []),
  Comp = wsdcomp:create(wscomp.chain_dialog,
			{fun party_direction_dialog_hlr/2,
			 {DirectionSelector(component, WA)}
			},
			WA,
			[]),
  Comp.

party_direction_dialog_hlr(init, {DirectionSelector}) ->
  {ok,
   [select_direction, select_party],
   #pdd_state{direction_sel=DirectionSelector}
  };
party_direction_dialog_hlr({start_dialog, _, _, _}, State) ->
  {ok, enter, State};
party_direction_dialog_hlr({select_direction, _, _, _},
		   S=#pdd_state{direction_sel=DS}) ->
  {ok, DS, enter, S};
party_direction_dialog_hlr({select_party, {ok, [D]}, WA, Srv},
		       S) ->
  party_direction_dialog_hlr({select_party, next_step, WA, Srv},
			 S#pdd_state{direction=D});
party_direction_dialog_hlr({select_party, Repeat, WA, _},
		       S=#pdd_state{direction=D})
  when Repeat == next_step;
       Repeat == previous_step ->
  {ok, direction_party_selector(D, WA), enter, S};
party_direction_dialog_hlr({stop_dialog, {ok, PartyPk}, WA, _},
		       S=#pdd_state{direction=D}) ->
  Dbh = wsdb:dbh(WA),
  {ok, Party} = wsdb:read_row(Dbh, party, PartyPk),
  {stop, {ok, {Party, D}}};
party_direction_dialog_hlr(Msg, _) ->
  {stop, ok}.

%% row creation dialog
-record(rad_state, {party_sel, relationship, row_comp, props}).

create_row_add_dialog(Party, Direction, WA, RowComp, Props) ->
  PartySelector = party_selector(Party, Direction, WA),
  Comp = wsdcomp:create(wscomp.chain_dialog,
			{fun row_add_dialog_hlr/2,
			 {PartySelector, RowComp, Props}
			},
			WA,
			[]),
  Comp.

row_add_dialog_hlr(init,
		   {PartySelector, RowComp, Props}) ->
  {ok,
   [select_party, create_account],
   #rad_state{party_sel=PartySelector,
	      row_comp=RowComp,
	      props=Props}
  };
row_add_dialog_hlr({start_dialog, _, _, _}, State) ->
  {ok, enter, State};
row_add_dialog_hlr({select_party, _, _, _},
		   S=#rad_state{party_sel=PS}) ->
  {ok, PS, enter, S};
row_add_dialog_hlr({create_account, {ok, Rel}, WA, Srv}, S) ->
  row_add_dialog_hlr({create_account, next_step, WA, Srv},
		     S#rad_state{relationship=Rel});
row_add_dialog_hlr({create_account, Repeat=next_step, WA, Srv},
		   S=#rad_state{relationship=Rel, row_comp=RC, props=P}) ->
  {ok, RC, {do_add_row, Rel}, S};
row_add_dialog_hlr({stop_dialog, Return, _, _}, _) ->
  {ok, Return};
row_add_dialog_hlr(Msg, _) ->
  {stop, ok}.


row_account_transaction_hook(init, TrType) ->
  {ok, TrType};
row_account_transaction_hook({is_active, _WA, _Srv}, _HD) ->
  true;
row_account_transaction_hook({access_info, WA, Srv}, TrType) ->
  EventName = list_to_atom(str:fmt("do_view_account_~s", [TrType])),
  #access_info{events=[EventName], default_event=refresh};
row_account_transaction_hook({call, Event, WA, Srv}, TrType) ->
  TrEventName = list_to_atom(str:fmt("do_view_account_~s", [TrType])),
  case Event of
    TrEventName ->
      AccountId = wslib:gv(id, wsdcomp:super_any_state(Srv, row)),
      CompName = ?account_transaction(TrType),
      Comp = wsdcomp:create(wsaccount_transaction,
			    {AccountId, TrType, WA}, WA,
			    [{place_at, CompName}]
			   ),
      ok = wsdcomp:spawn_comp(Srv, CompName, Comp),
      wsdcomp:call(Srv, CompName, enter),
      {ok, ok, [{'ChildrenPlacement', vertical}], TrType};
    _ ->
      {ok, undefined, TrType}
  end;
row_account_transaction_hook({event, Msg, From, WA, Srv}, HD) ->
  {ok, undefined, HD}.

create_row(WA, Form, PkNames, Props) ->
  RowHlr = row:create_db_handler(party_relationship_account),
  ReadOnlyFields = [party_id_from,
		    party_id_to,
 		    role_type_id_to,
 		    role_type_id_from,
 		    from_date
 		   ],
  CreateOnlyFields = [id, created_date, credit, debit],
  Row1 = wsdcomp:create(wsform.row,
			{PkNames, Form, RowHlr, WA},
			WA,
			[{createonly_fields, CreateOnlyFields},
			 {readonly_fields, ReadOnlyFields}
			 |Props
			]),
  TransactionHooks =
    [{wsform.expand_row_hook,
      {fun row_account_transaction_hook/2, charge, ?account_transaction(charge)}
     },
     {wsform.expand_row_hook,
      {fun row_account_transaction_hook/2, payment, ?account_transaction(payment)}
     }
    ],
  wscomp.expand:create_comp(Row1, TransactionHooks, []).

%% Form

create_form(WA, Props) ->
  Fields = 
    [id,
     currency_type_id,
     party_id_to,
     party_name_to,
     party_id_from,
     party_name_from,
     credit,
     debit,
     credit_limit,
     status,
     status_date,
     created_date,
     comments
    ],
  wsform:create(
    ?form_name,
    [{label,     wslib:gv(form_label, Props, {wsform_lib, language_label})},
     {calculate, fun form_calculate/3}
    ],
    Fields,
    [{data,        fun data/3},
     {data_filter, fun data_filter/3},
     {validate,    fun validate/4},
     {data_type,   fun data_type/2},
     {out_attrs,   fun out_attrs/2},
     {default,     fun default/2},
     {calculate,   fun calculate/4},
     {label,       wsform_lib:mk_language_label(?form_name)},
     {ss,          
      [ss:readonly(ss:form_field(
		     wsxmlt:m_name([party_name_to, party_name_from]),
		     [ss:string()],
		     WA)),
       wsite.form.field:ss(status, WA),
       ss:form_field(wsxmlt:m_name(Fields), WA),
       wsxmlt:skip()
      ]}
    ],
    WA).

data(N, KVs, WA) ->
  wsform_lib:def_data(N, KVs, WA).

data_filter(N, V, WA) ->
  wsform_lib:def_data_filter(N, V, WA).

validate(_, undefined, _, _) ->
  true;
validate(F, V, _, _) when F == credit;
			  F == debit;
			  F == credit_limit ->
  (V >= 0);
validate(status,  V, KVs, WA) ->
  wsite.form.field:validate(status,  V, KVs, WA);
validate(_, _, _, _) ->
  true.

form_calculate(Form, KVs, WA) ->
  Dbh = wsdb:dbh(WA),
  [PartyIdTo, PartyIdFrom] = wslib:gv_map([party_id_to, party_id_from], KVs),
  {ok, PartyTo} = wsdb:read_row(Dbh, party, [{id, PartyIdTo}]),
  {ok, PartyFrom} = wsdb:read_row(Dbh, party, [{id, PartyIdFrom}]),
  {PartyTo, PartyFrom}.

data_type(id, _) ->integer;
data_type(currency_type_id, _) -> atom;
data_type(credit, _) -> integer;
data_type(credit_limit, _) -> integer;
data_type(debit, _) -> integer;
data_type(party_id_from, _) -> integer;
data_type(party_id_to, _) -> integer;
data_type(created_date, _) -> date;
data_type(status_date, _)  -> date;
data_type(status, WA)  ->
  wsite.form.field:data_type(status, WA);
data_type(_, _)    -> string.

calculate(party_name_to, _, _, {PartyTo, _}) ->
  {ok, wslib:gv(name, PartyTo)};
calculate(party_name_from, _, _, {_, PartyFrom}) ->
  {ok, wslib:gv(name, PartyFrom)};
calculate(_, V, _, _) ->
  {ok, V}.

default(id, WA) ->
  Dbh = wsdb:dbh(WA),
  wsdb:next_rowid(Dbh, party_relationship_account);
default(credit, _) ->
  0;
default(credit_limit, _) ->
  0;
default(debit, _) ->
  0;
default(created_date, _) -> 
  calendar:local_time();
default(status_date, _) -> 
  calendar:local_time();
default(status, WA) ->
  wsite.form.field:default(status, WA);
default(party_id_to, _) ->
  required;
default(party_id_from, _) ->
  required;
default(_, _) -> undefined.

out_attrs(id, _) -> [{maxlength, "16"}, {size, "8"}];
out_attrs(amount, _) -> [{maxlength, "32"}, {size, "16"}];
out_attrs(party_id_to, _) -> [{maxlength, "16"}, {size, "8"}];
out_attrs(party_id_from, _) -> [{maxlength, "16"}, {size, "8"}];
out_attrs(status_date, _) -> [{maxlength, "10"}, {size, "10"}];
out_attrs(created_date, _) -> [{maxlength, "10"}, {size, "10"}];
out_attrs(_, _) -> [{maxlength, "128"}, {size, "64"}].

%%
party_selector(Party, Direction, WA) ->
  Dbh = wsdb:dbh(WA),
  RelCategoryId = ?account_relationship_category,
  {ok, RelCategory} =
    wsdb:read_row(Dbh, party_relationship_category, [{id, RelCategoryId}]),
  Rowset = wsdcomp:create(wsparty_relationship,
			  {Party, Direction, RelCategory, WA},
			  WA,
			  [{form_type, rowset}]),
  RowsetHook = wsform.expand_rowset_hook:create_select_rowid_hook(),
  Comp = wscomp.expand:create_comp(Rowset, RowsetHook, []),
  Comp.

direction_party_selector(Direction, WA) ->
  RelCategoryId = ?account_relationship_category,
  QP= [{direction, {noquote, Direction}}, {category_id, RelCategoryId}],
  MkQuery = fun (Where) ->
		Q=wsdb_odbc:mk_query('PARTY_BY_RELATIONSHIP_CATEGORY', Where),
		fun (P, Cont) ->
		    Q(QP++P, Cont)
		end
	    end,
  Rowset = wsdcomp:create(wsparty, WA, WA,
			  [{form_type, rowset}, {mk_query, MkQuery}]),
  RowsetHook = wsform.expand_rowset_hook:create_select_rowid_hook(),
  Comp = wscomp.expand:create_comp(Rowset, RowsetHook, []),
  Comp.

get_role_description(RoleTypeId, Dbh) ->
  {ok, Role} = wsdb:read_row(Dbh, role_type, [{id, RoleTypeId}]),
  wslib:gv(description, Role).
