%%% File    : wsparty_relationship_service.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created :  4 Dec 2002 by  <svg@surnet.ru>

%%
%% $Id: wsparty_relationship_service.erl,v 1.2 2003/09/05 20:33:18 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wsparty_relationship_service).

-behaviour(wsdcomp).

-export([init/2, access_info/3, handle_event/6]).

-import(wslib.str).
-import(wslib.listn).
-import(wsform.ss).
-import(wsform.weekday).
-import(wscomp.choose).
-import(wscomp.selector).
-import(wsform.search_query).
-import(wsform.ss).
-import(wsform.field_type).

-include("wstl.hrl").

-define(service_relationship_category, 'CUSTOMER').
-define(directions, [{to, "SUPPLIER"}, {from,"CUSTOMER"}]).
-define(form_name, party_relationship_service).

%% component
access_info(_, _, _) ->
  exit(must_never_reach).

handle_event(Event, From,  WA, Srv, State, StateData) ->
  exit(must_never_reach).

init(WA, Props) when is_record(WA, ws_arg) ->
  Comp = choose:create(fun choose_party_direction_hlr/2, {choose, Props}),
  {component, Comp};
init(Arg={Party, Direction, WA}, Props) ->
  {PartyIdMasterField, PartyIdDetField} =
    case Direction of
      to ->
	{party_id_to, party_id_from};
      from ->
	{party_id_from, party_id_to}
    end,
  PartyId = wslib:gv(id, Party),
  PkNames = [id],
  Dbh = wsdb:dbh(WA),
  FormType = wslib:gv(form_type, Props, table),
  Props1 = Props ++ [{template, "lib/wsform"}],
  Row =
    if (FormType =/= rowset) ->
	choose:create(fun rowset_row_hlr/2, {Party, Direction, Props1});
       true ->
	undefined
    end,
  Rowset =
    if (FormType =/= row) ->
	DirectionDesc =
	  get_role_description(wslib:gv(Direction, ?directions), Dbh),
	FormLabel =
	  wslanguage:msg(WA, party_relationship_service_rowset,
			 [{party_role_description, DirectionDesc},
			  {party_id, PartyId},
			  {party_name, wslib:gv(name, Party)}
			 ]),
	RowsetForm = create_form(rowset,
				 [name], [], WA,
				 [{form_label, fun (_, _) -> FormLabel end}]
				),
	QW = [PartyIdMasterField],
	QP = [{PartyIdMasterField, PartyId}],
	MkQuery =
	  fun (Where) ->
	      Q =
		wsdb_odbc:mk_select_query(party_relationship_service, QW++Where),
	      fun (P, Cont) -> Q(QP++P, Cont) end
	  end,
	Query = search_query:mk_query(
		  MkQuery, RowsetForm, WA,
		  [{order_by_fields,
		    [PartyIdDetField,
		     service_type_id,
		     name
		    ]
		   }]),
	Pg = {wsform.odbcdb_pager, wsdb:handler(Dbh), []},
	wsdcomp:create(wsform.rowset, {PkNames, RowsetForm, Pg, Query},
		       WA, Props1);
       true ->
	undefined
    end,
  case FormType of
    row ->
      {component, Row};
    rowset ->
      {component, Rowset};
    table ->
      RowsetHook = wsform.expand_rowset_hook:create_editrow_hook(Row),
      Comp =
	wscomp.expand:create_comp(Rowset, RowsetHook, []),
      {component, Comp}
  end.

rowset_row_hlr(init, Data) ->
  {ok, Data};
rowset_row_hlr({call, E={do_add_row, _}, WA, _}, {Party, Direction, Props}) ->
  Comp = create_row_add_dialog(Party, Direction, WA, Props),
  {exec, E, Comp};
rowset_row_hlr({call, Event={_, Row}, WA, _}, {_, _, Props}) ->
  Comp = create_row(Row, WA, Props),
  {exec, Event, Comp};
rowset_row_hlr(_, _) ->
  {stop, ok}.

%% choose party and direction(customer->from/supplier->to) dialog
-record(pdd_state, {party, direction, direction_sel}).

choose_party_direction_hlr(init, Data) ->
  {ok, Data};
choose_party_direction_hlr({call, enter, WA, _}, {choose, Props}) ->
  Comp = create_party_direction_dialog(WA, []),
  {call_exec, enter, Comp, {result, Props}};
choose_party_direction_hlr({call, {ok, {Party, Direction}}, WA, _},
			   {result, Props}) ->
  Comp = wsdcomp:create(?MODULE, {Party, Direction, WA}, WA, Props),
  {exec, enter, Comp};
choose_party_direction_hlr(_, _) ->
  {stop, ok}.

create_party_direction_dialog(WA, Props) ->
  Dbh = wsdb:dbh(WA),
  Directions =
    [{D, get_role_description(R, Dbh)} || {D, R} <- ?directions], 
  DirectionSelector =
    selector:create(id,
		    fun (_, _) -> {ok, Directions} end,
		    fun (_, Vs, _, _) ->
			{stop, {ok, [list_to_atom(V) || V <- Vs]}}
		    end,
		    [], []),
  Comp = wsdcomp:create(wscomp.chain_dialog,
			{fun party_direction_dialog_hlr/2,
			 {DirectionSelector(component, WA)}
			},
			WA,
			[]),
  Comp.

party_direction_dialog_hlr(init, {DirectionSelector}) ->
  {ok,
   [select_direction, select_party],
   #pdd_state{direction_sel=DirectionSelector}
  };
party_direction_dialog_hlr({start_dialog, _, _, _}, State) ->
  {ok, enter, State};
party_direction_dialog_hlr({select_direction, _, _, _},
		   S=#pdd_state{direction_sel=DS}) ->
  {ok, DS, enter, S};
party_direction_dialog_hlr({select_party, {ok, [D]}, WA, Srv},
		       S) ->
  party_direction_dialog_hlr({select_party, next_step, WA, Srv},
			 S#pdd_state{direction=D});
party_direction_dialog_hlr({select_party, Repeat, WA, _},
		       S=#pdd_state{direction=D})
  when Repeat == next_step;
       Repeat == previous_step ->
  {ok, direction_party_selector(D, WA), enter, S};
party_direction_dialog_hlr({stop_dialog, {ok, PartyPk}, WA, _},
		       S=#pdd_state{direction=D}) ->
  Dbh = wsdb:dbh(WA),
  {ok, Party} = wsdb:read_row(Dbh, party, PartyPk),
  {stop, {ok, {Party, D}}};
party_direction_dialog_hlr(Msg, _) ->
  {stop, ok}.

%% row creation dialog
-record(rad_state, {party_sel, service_type_sel,
		    relationship, service_type,
		    login_id, props}).

create_row_add_dialog(Party, Direction, WA, Props) ->
  PartySelector = party_selector(Party, Direction, WA),
  ServiceTypeSelector =
    wsite.form.entity_type
    :create_selector(service_type_id, service_type, id, description, WA, []),
  Comp = wsdcomp:create(wscomp.chain_dialog,
			{fun row_add_dialog_hlr/2,
			 {PartySelector, ServiceTypeSelector, Props}
			},
			WA,
			[]),
  Comp.

row_add_dialog_hlr(init, {PartySelector, ServiceTypeSelector, Props}) ->
  {ok,
   [select_party, select_service_type, select_login_id, create_service],
   #rad_state{service_type_sel=ServiceTypeSelector,
	      party_sel=PartySelector,
	      props=Props}
  };
row_add_dialog_hlr({start_dialog, _, _, _}, State) ->
  {ok, enter, State};
row_add_dialog_hlr({select_party, _, _, _},
		   S=#rad_state{party_sel=PS}) ->
  {ok, PS, enter, S};
row_add_dialog_hlr({select_service_type, {ok, R}, WA, Srv},
		   S=#rad_state{service_type_sel=STS}) ->
  row_add_dialog_hlr({select_service_type, next_step, WA, Srv},
		     S#rad_state{relationship=R});
row_add_dialog_hlr({select_service_type, Repeat, WA, _},
		   S=#rad_state{service_type_sel=STS})
  when Repeat == next_step;
       Repeat == previous_step ->
  {ok, STS(component, WA), enter, S};
row_add_dialog_hlr({select_login_id, {do_select, [ST]}, WA, Srv}, S) ->
  NxtS = S#rad_state{service_type=ST},
  row_add_dialog_hlr({select_login_id, next_step, WA, Srv}, NxtS);
row_add_dialog_hlr({select_login_id, next_step, WA, Srv},
		   S=#rad_state{service_type=ST, relationship=R}) ->
  case lists:member(wslib:gv(id, ST), ["dialup", "email"]) of
    true ->
      {ok, party_login_selector(wslib:gv(party_id_from, R), WA), enter, S};
    false ->
      {ok, next_step, S}
  end;
row_add_dialog_hlr({create_service, {ok, [LI]}, WA, Srv}, S) ->
  row_add_dialog_hlr({create_service, next_step, WA, Srv},
		     S#rad_state{login_id=LI});
row_add_dialog_hlr({create_service, Repeat, WA, Srv},
		   S=#rad_state{service_type=ST, login_id=LI,
				relationship=R, props=P})
  when Repeat == next_step;
       Repeat == previous_step ->
  Row = [{service_type_id, wslib:gv(id, ST)}, LI|R],
  Comp = create_row(Row, WA, P),
  {ok, Comp, {do_add_row, Row}, S};
row_add_dialog_hlr({stop_dialog, Return, _, _}, _) ->
  {ok, Return};
row_add_dialog_hlr(Msg, _) ->
  {stop, ok}.

create_row(Row, WA, Props) ->
  PkNames = [id],
  Service = case wslib:gv(id, Row) of
	      undefined ->
		Row; % service creation
	      _ ->   % update,delete,view
		Dbh = wsdb:dbh(WA),
		{ok, S} =
		  wsdb:read_row(Dbh, party_relationship_service, Row),
		S
	 end,
  [ServiceType, PartyId] =
    wslib:gv_map([service_type_id, party_id_from], Service),
  RowHlr = mk_row_hlr(),
  ReadOnlyFields = [party_id_from,
		    party_id_to,
		    role_type_id_to,
		    role_type_id_from,
		    from_date,
		    service_type_id,
		    login_id
		   ],
  CreateOnlyFields = [created_date],
  Form = create_row_form(ServiceType, PartyId, WA),
  wsdcomp:create(wsform.row,
		 {PkNames, Form, RowHlr, WA},
		 WA,
		 [{readonly_fields, ReadOnlyFields},
		  {createonly_fields, CreateOnlyFields}
		  |Props]).

mk_row_hlr() ->
  Hlr = wsform.row:create_db_handler(party_relationship_service),
  fun ({Op, Row}, Dbh) when Op == insert_row;
			    Op == update_row ->
      case service_name(wslib:gv(service_type_id, Row), Row) of
	{ok, N} ->
	  Hlr({Op, [{name, N}|Row]}, Dbh);
	undefined ->
	  Hlr({Op, Row}, Dbh)
      end;
      (Data, Dbh) ->
      Hlr(Data, Dbh)
  end.
  
service_name("dialup", Row) ->
  {ok, wslib:gv(login_id, Row)};
service_name("email", Row) ->
  {ok, wslib:gv(email_address, Row)};
service_name(_, _) ->
  undefined.

%% Form

create_row_form(ServiceType, PartyId, WA) ->
  Dbh = wsdb:dbh(WA),
  ServiceFields =
    [wslib:gv(attr_name, Row)
     || Row <- wsdb:select_rows(Dbh, service_type_attr,
				[{service_type_id, ServiceType}])],
  ServiceFieldSs =
    form_service_field_ss(ServiceType, PartyId, ServiceFields, WA),
  Form = create_form(row, ServiceFields, ServiceFieldSs, WA, []),
  Form.

create_form(FormType, ServiceFields, ServiceFieldSs, WA, Props) ->
  Fields = 
    [id,
     service_type_id,
     party_id_to,
     party_name_to,
     party_id_from,
     party_name_from
    ]
    ++ ServiceFields
    ++ [status,
	status_date,
	created_date,
	tariff_id,
	comments
       ],
  wsform:create(
    ?form_name,
    [{label,     wslib:gv(form_label, Props, {wsform_lib, language_label})},
     {calculate, fun form_calculate/3}
    ],
    Fields,
    [{data,        fun data/3},
     {data_filter, fun data_filter/3},
     {validate,    fun validate/4},
     {data_type,   fun data_type/2},
     {out_attrs,   fun out_attrs/2},
     {default,     fun default/2},
     {calculate,   fun calculate/4},
     {label,       wsform_lib:mk_language_label(?form_name)},
     {ss,          
      ServiceFieldSs
      ++ [ss:readonly(ss:form_field(
			wsxmlt:m_name([party_name_to, party_name_from]),
			[ss:string()],
			WA)),
	  tariff_selector(WA),
	  service_type_selector(WA),
	  wsite.form.field:ss(tariff_id, WA),
	  wsite.form.field:ss(status, WA),
	  ss:form_field(wsxmlt:m_name(caller_phone),
			[ss:field_value(
			   string,
			   fun (_, _, _) -> string end,
			   fun (_, _, [Str], S) ->
			       PhoneStr =
				 str:join(
				   [field_type:format(phone_number, P)
				    || P <- string:tokens(Str, ",")],
				   $,
				  ),
			       [PhoneStr];
			       (_, _, Content=[], S) ->
			       Content
			   end
			  ),
			 ss:text()
			],
			WA),
	  ss:form_field(wsxmlt:m_name(Fields), WA),
	  wsxmlt:skip()
	 ]}
    ],
    WA).

data(N=restday_mask, KVs, WA) ->
  weekday:data(N, KVs);
data(N, KVs, WA) ->
  wsform_lib:def_data(N, KVs, WA).

data_filter(caller_phone, V, _) ->
  {ok, Str, _} = regexp:gsub(V, "[^0-9,]", ""),
  Ret = str:join(string:tokens(Str, ","), $,),
  Ret;
data_filter(N, V, WA) ->
  wsform_lib:def_data_filter(N, V, WA).

validate(_, undefined, _, _) ->
  true;
validate(N, V, KVs, WA) when N == restday_disabled_time_from;
			     N == restday_disabled_time_to;
			     N == workday_disabled_time_from;
			     N == workday_disabled_time_to ->
  field_type:validate(time, V, KVs, WA);
validate(restday_mask, V, _, _) ->
  weekday:validate(V);
validate(caller_phone, Str, KVs, WA) ->
  Vs = string:tokens(Str, ","),
  lists:foreach(fun (V) ->
		    case field_type:validate(phone_number, V, KVs, WA) of
		      false ->
			throw(false);
		      Other ->
			Other
		    end
		end, Vs),
  true;
validate(email_address, V, KVs, WA) ->
  field_type:validate(email_address, V, KVs, WA);
validate(status,  V, KVs, WA) ->
  wsite.form.field:validate(status,  V, KVs, WA);
validate(_, _, _, _) ->
  true.

form_calculate(Form, KVs, WA) ->
  Dbh = wsdb:dbh(WA),
  [PartyIdTo, PartyIdFrom] = wslib:gv_map([party_id_to, party_id_from], KVs),
  {ok, PartyTo} = wsdb:read_row(Dbh, party, [{id, PartyIdTo}]),
  {ok, PartyFrom} = wsdb:read_row(Dbh, party, [{id, PartyIdFrom}]),
  {PartyTo, PartyFrom}.

data_type(id, _) ->integer;
data_type(party_id_from, _) -> integer;
data_type(party_id_to, _) -> integer;
data_type(restday_mask, _) ->
  {wsform.weekday, data_type};
data_type(restday_disabled_time_from, _) -> time;
data_type(restday_disabled_time_to, _) -> time;
data_type(workday_disabled_time_from, _) -> time;
data_type(workday_disabled_time_to, _) -> time;
data_type(created_date, _) -> date;
data_type(status_date, _)  -> date;
data_type(portname, _) -> string;
data_type(network, _) -> string;
data_type(source_informer_address, _) -> string;
data_type(aggregate_rule_name_src_net, _) -> string;
data_type(aggregate_rule_name_dst_net, _) -> string;
data_type(traffic_direction, _) -> string;
data_type(F, WA) when F == status; F == tariff_id ->
  wsite.form.field:data_type(F, WA);
data_type(_, _) -> string.

calculate(party_name_to, _, _, {PartyTo, _}) ->
  {ok, wslib:gv(name, PartyTo)};
calculate(party_name_from, _, _, {_, PartyFrom}) ->
  {ok, wslib:gv(name, PartyFrom)};
calculate(_, V, _, _) ->
  {ok, V}.

default(id, WA) ->
  Dbh = wsdb:dbh(WA),
  wsdb:next_rowid(Dbh, party_relationship_service);
default(created_date, _) -> 
  calendar:local_time();
default(status_date, _) -> 
  calendar:local_time();
default(status, WA) ->
  wsite.form.field:default(status, WA);
default(party_id_to, _) ->
  required;
default(party_id_from, _) ->
  required;
default(restday_mask, _) ->
  weekday:default();
default(restday_disabled_time_from, _) -> {0, 0, 0};
default(restday_disabled_time_to, _) -> {0, 0, 0};
default(workday_disabled_time_from, _) -> {0, 0, 0};
default(workday_disabled_time_to, _) -> {0, 0, 0};
default(_, _) -> undefined.

out_attrs(id, _) -> [{maxlength, "16"}, {size, "8"}];
out_attrs(F, WA) when F == tariff_id; F == status ->
  wsite.form.field:out_attrs(F, WA);
out_attrs(party_id_to, _) -> [{maxlength, "16"}, {size, "8"}];
out_attrs(party_id_from, _) -> [{maxlength, "16"}, {size, "8"}];
out_attrs(status_date, _) -> [{maxlength, "10"}, {size, "10"}];
out_attrs(created_date, _) -> [{maxlength, "10"}, {size, "10"}];
out_attrs(login_id, _) -> [{maxlength, "64"}, {size, "32"}];
out_attrs(restday_disabled_time_from, _) -> [{maxlength, "10"}, {size, "10"}];
out_attrs(restday_disabled_time_to, _)   -> [{maxlength, "10"}, {size, "10"}];
out_attrs(workday_disabled_time_from, _) -> [{maxlength, "10"}, {size, "10"}];
out_attrs(workday_disabled_time_to, _)   -> [{maxlength, "10"}, {size, "10"}];
out_attrs(restday_mask, _) -> [];
out_attrs(_, _) -> [{maxlength, "128"}, {size, "64"}].

form_service_field_ss(ServiceType, PartyId, ServiceFields, WA) ->
  [tariff_selector(ServiceType, WA)
   |lists:flatmap(fun (FN) ->
		      listn:mk_list(form_service_field_ss(FN, PartyId, WA))
		  end, ServiceFields)
  ].

form_service_field_ss(login_id, PartyId, WA) ->
  ss:form_field(login_id, [ss:string()], WA);
form_service_field_ss(Name=restday_mask, _, WA) ->
  weekday:ss(Name, WA);
form_service_field_ss(Name=restday_disabled_time_from, _, WA) ->
  ss:interval(restday_disabled_time_from, restday_disabled_time_to, WA);
form_service_field_ss(Name=restday_disabled_time_to, _, WA) ->
  ss:interval(workday_disabled_time_from, workday_disabled_time_to, WA);
form_service_field_ss(_, _, _) ->
  [].

%%
party_selector(Party, Direction, WA) ->
  Dbh = wsdb:dbh(WA),
  RelCategoryId = ?service_relationship_category,
  {ok, RelCategory} =
    wsdb:read_row(Dbh, party_relationship_category, [{id, RelCategoryId}]),
  Rowset = wsdcomp:create(wsparty_relationship,
			  {Party, Direction, RelCategory, WA},
			  WA,
			  [{form_type, rowset}]),
  RowsetHook = wsform.expand_rowset_hook:create_select_rowid_hook(),
  Comp = wscomp.expand:create_comp(Rowset, RowsetHook, []),
  Comp.

party_login_selector(PartyId, WA) ->
  Rowset = wsdcomp:create(wsuser_login, {PartyId, WA}, WA,
			  [{form_type, rowset}]),
  RowsetHook = wsform.expand_rowset_hook:create_select_rowid_hook(),
  Comp = wscomp.expand:create_comp(Rowset, RowsetHook, []),
  Comp.

direction_party_selector(Direction, WA) ->
  RelCategoryId = ?service_relationship_category,
  QP= [{direction, {noquote, Direction}}, {category_id, RelCategoryId}],
  MkQuery = fun (Where) ->
		Q=wsdb_odbc:mk_query('PARTY_BY_RELATIONSHIP_CATEGORY', Where),
		fun (P, Cont) ->
		    Q(QP++P, Cont)
		end
	    end,
  Rowset = wsdcomp:create(wsparty, WA, WA,
			  [{form_type, rowset}, {mk_query, MkQuery}]),
  RowsetHook = wsform.expand_rowset_hook:create_select_rowid_hook(),
  Comp = wscomp.expand:create_comp(Rowset, RowsetHook, []),
  Comp.

get_role_description(RoleTypeId, Dbh) ->
  {ok, Role} = wsdb:read_row(Dbh, role_type, [{id, RoleTypeId}]),
  wslib:gv(description, Role).

tariff_selector(ServiceType, WA) ->
  Dbh = wsdb:dbh(WA),
  Tariffs = wsdb:select_rows(Dbh, tariff, [{service_type_id, ServiceType}]),
  MkSelector = selector:create_selector(tariff_id, id, description, Tariffs, []),
  MkSelector(wsform_ss, WA).

tariff_selector(WA) ->
  MkSelector = wsite.form.entity_type
    :create_selector(tariff_id, tariff, id, description, WA, []),
  MkSelector(wsform_ss, WA).

service_type_selector(WA) ->
  MkSelector = wsite.form.entity_type
    :create_selector(service_type_id, service_type, id, description, WA, []),
  MkSelector(wsform_ss, WA).
