%%% File    : wsparty_relationship_type.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created :  4 Dec 2002 by  <svg@surnet.ru>

%%
%% $Id: wsparty_relationship_type.erl,v 1.2 2003/09/05 19:08:37 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wsparty_relationship_type).

-behaviour(wsdcomp).

-export([init/2, access_info/3, handle_event/6]).
-export([create_form/1]).

-import(wsdb, [dbh/1]).
-import(wsform.search_query).

-include("wstl.hrl").

access_info(_, _, _) ->
  exit(must_never_reach).

handle_event(Event, From,  WA, Srv, State, StateData) ->
  exit(must_never_reach).


init(WA, Props) ->
  PkNames = [id],
  Dbh = dbh(WA),
  Form = create_form(WA),
  MkQuery = fun (Where) ->
		wsdb_odbc:mk_select_query(party_relationship_type, Where)
	    end,
  Query = search_query:mk_query(MkQuery, Form, WA, [{order_by_fields, [id]}]),
  Pg = {wsform.odbcdb_pager, wsdb:handler(Dbh), []},
  RowHlr = wsform.row:create_db_handler(party_relationship_type),
  Rowset = wsdcomp:create(wsform.rowset, {PkNames, Form, Pg, Query}, WA, Props),
  Row = wsdcomp:create(wsform.row, {PkNames, Form, RowHlr, WA}, WA, []),
  RowsetHook = wsform.expand_rowset_hook:create_editrow_hook(Row),
  Comp = wscomp.expand:create_comp(Rowset, RowsetHook, []),
  {component, Comp}.
%% Wsform stuff

create_form(WA) ->
  Dbh = wsdb:dbh(WA),
  RoleList = [selector_kv(Row)
	      || Row <- wsdb:select_rows(Dbh, role_type, [], [id])],
  CategoryList =
    [selector_kv(Row)
     || Row <- wsdb:select_rows(Dbh, party_relationship_category, [], [id])],
  Fields =
    [id, role_type_id_from_valid, role_type_id_to_valid, category_id, description, form],
  wsform:create(
    party_relationship_type,
    [{label, {wsform_lib, language_label}}],
    Fields,
    [
     {data_filter,   fun data_filter/3},
     {data_type,   fun data_type/2},
     {out_attrs,   fun out_attrs/2},
     {default,     fun default/2},
     {label,       {wsform_lib, language_label}},
     {ss, [wsform.ss:form_field(wsxmlt:m_name(role_type_id_to_valid),
				[wsform.ss:selector(RoleList, [])], WA),
	   wsform.ss:form_field(wsxmlt:m_name(role_type_id_from_valid),
				[wsform.ss:selector(RoleList, [])], WA),
	   wsform.ss:form_field(wsxmlt:m_name(category_id),
				[wsform.ss:selector(CategoryList, [])], WA),
	   wsform.ss:form_field(wsxmlt:m_name(Fields), WA),
	   wsxmlt:skip()]}
    ],
    WA).

data_filter(K=id, V, WA) ->
  wslib.str:to_upper(wsform_lib:def_data_filter(K, V, WA));
data_filter(K, V, WA) -> 
      wsform_lib:def_data_filter(K, V, WA).

data_type(form, _) -> atom;
data_type(_, _) -> string.

default(form, _) -> undefined;  
default(_, _) -> required.

out_attrs(id, _)      -> [{maxlength, "64"}, {size, "32"}];
out_attrs(role_type_id_from_valid, _) -> [{maxlength, "64"}, {size, "32"}];
out_attrs(role_type_id_to_valid, _) -> [{maxlength, "64"}, {size, "32"}];
out_attrs(category_id, _)      -> [{maxlength, "64"}, {size, "32"}];
out_attrs(form, _)      -> [{maxlength, "64"}, {size, "32"}];
out_attrs(description, _) -> [{maxlength, "128"}, {size, "64"}].

selector_kv(Row) ->
  Id = wslib:gv(id, Row),
  Desc =  wslib:gv(description, Row),
  {Id, Desc}.
