%%% File    : wstariff.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created :  4 Dec 2002 by  <svg@surnet.ru>

%%
%% $Id: wstariff.erl,v 1.2 2003/09/04 17:11:37 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wstariff).

-behaviour(wsdcomp).

-export([init/2, access_info/3, handle_event/6]).
-export([create_form/1]).

-import(wsdb, [dbh/1]).
-import(wscomp.selector).
-import(wsform.search_query).
-import(wsform.row).

-include("wstl.hrl").

access_info(_, _, _) ->
  exit(must_never_reach).

handle_event(Event, From,  WA, Srv, State, StateData) ->
  exit(must_never_reach).


init(WA, Props) ->
  PkNames = [id],
  Dbh = dbh(WA),
  Form = create_form(WA),
  FormType = wslib:gv(form_type, Props, table),
  Props1 = Props ++ [{template, "wstariff"}],

  Row = 
    if (FormType =/= rowset) ->
	RowHlr = row:create_db_handler(tariff),
	MenuHook = {wsform.expand_row_hook,
		    {{wsite.party, party_menu_row_hook}, [], undefined}
		   },
	wsdcomp:create(wsform.row, {PkNames, Form, RowHlr, WA}, WA, Props1);
       true ->
	undefined
    end,
  Rowset =
    if (FormType =/= row) ->
	MkQuery = fun (Where) ->
		      wsdb_odbc:mk_select_query(tariff, Where)
		  end,
	Query = search_query:mk_query(MkQuery, Form, WA,
				      [{order_by_fields, [id]}]),
	Pg = {wsform.odbcdb_pager, wsdb:handler(Dbh), []},
	wsdcomp:create(wsform.rowset, {PkNames, Form, Pg, Query}, WA, Props1);
       true ->
	undefined
    end,
  case FormType of
    row ->
      {component, Row};
    rowset ->
      {component, Rowset};
    table ->
      RowsetHook = wsform.expand_rowset_hook:create_editrow_hook(Row),
      Comp = wscomp.expand:create_comp(Rowset, RowsetHook, []),
      {component, Comp}
  end.

%% Wsform stuff

create_form(WA) ->
  Fields =
    [id,description, charge_type_id, currency_type_id, service_type_id],
  FormName = tariff,
  LabelFun = wsform_lib:mk_language_label(FormName),
  wsform:create(
    FormName,
    [{label, LabelFun}],
    Fields,
    [
     {data_type,   fun data_type/2},
     {out_attrs,   fun out_attrs/2},
     {default,     fun default/2},
     {label,       LabelFun},
     {ss,          [wsite.form.field:ss(charge_type_id, WA),
		    wsite.form.field:ss(currency_type_id, WA),
		    wsite.form.field:ss(service_type_id, WA),
		    wsform.ss:form_field(wsxmlt:m_name(Fields), WA),
		    wsxmlt:skip()]}
    ],
    WA).

data_type(id, _) -> integer;
data_type(F, _)  when F == charge_type_id;
		      F == currency_type_id ->
  atom;
data_type(_, _)  -> string.
  
default(id, _) ->
  fun (WA) ->
      Dbh = wsdb:dbh(WA),
      wsdb:next_rowid(Dbh, tariff)
  end;
default(_, _) -> required;
default(N, WA) -> wsform_lib:def_value(N, WA).

out_attrs(id, _)      -> [{maxlength, "16"}, {size, "8"}];
out_attrs(description, _) -> [{maxlength, "128"}, {size, "64"}];
out_attrs(_, _) -> [].
