%%% File    : wsecurity_group.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created :  4 Dec 2002 by  <svg@surnet.ru>

%%
%% $Id: wsuser_login.erl,v 1.2 2003/10/31 17:51:02 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wsuser_login).

-behaviour(wsdcomp).

-export([init/2, access_info/3, handle_event/6]).
-export([create_form/1]).

-import(wsform.search_query).

-include("wstl.hrl").

-define(chpass_form, "change_password").
-define(security_groups, "security_groups").

access_info(_, _, _) ->
  exit(must_never_reach).

handle_event(Event, From,  WA, Srv, State, StateData) ->
  exit(must_never_reach).

init(WA, Props) when is_record(WA, ws_arg) ->
  init({undefined, WA}, Props);
init({PartyId, WA}, Props) ->
  PkNames = [login_id],
  Dbh = wsdb:dbh(WA),
  Form =  Form = create_form(WA),
  FormType = wslib:gv(form_type, Props, table),
  Props1 = Props ++ [{template, "lib/wsform"}],
  Row = 
    if (FormType =/= rowset) ->
	ReadOnlyFields = [password, password_hint, login_attempts],
	RowProps = [{readonly_fields, ReadOnlyFields}|Props1],
	RowHlr = wsform.row:create_db_handler(user_login),
	Row1 = wsdcomp:create(wsform.row, {PkNames, Form, RowHlr, WA}, WA,
			      RowProps),
	ChpassHook = {wsform.expand_row_hook,
		      {fun chpass_hook/2, [], ?chpass_form}
		     },
	SecGroupsHook = {wsform.expand_row_hook,
			 {fun security_group_hook/2, [], ?security_groups}
			},

	wscomp.expand:create_comp(Row1, [ChpassHook, SecGroupsHook], []);
       true ->
	undefined
    end,
  Rowset =
    if (FormType =/= row) ->
	MkQuery =
	  if PartyId == undefined ->
	      fun (Where) ->
		  wsdb_odbc:mk_select_query(user_login, Where)
	      end;
	     true ->
	      fun (Where) ->
		  Q = wsdb_odbc:mk_select_query(user_login, [party_id|Where]),
		  fun (P, Cont) ->
		      Q([{party_id, PartyId}|P], Cont)
		  end
	      end
	  end,
	Query = search_query:mk_query(MkQuery, Form, WA,
				      [{order_by_fields, PkNames}]),
	Pg = {wsform.odbcdb_pager, wsdb:handler(Dbh), []},
	wsdcomp:create(wsform.rowset, {PkNames, Form, Pg, Query}, WA, Props1);
       true ->
	undefined
    end,
  case FormType of
    row ->
      {component, Row};
    rowset ->
      {component, Rowset};
    table ->
      RowsetHook = wsform.expand_rowset_hook:create_editrow_hook(Row),
      Comp = wscomp.expand:create_comp(Rowset, RowsetHook, []),
      {component, Comp}
  end.

chpass_hook(init, Data) -> 
  {ok, Data};
chpass_hook({is_active, WA, Srv}, _) ->
  true;
chpass_hook({access_info,  WA, Srv}, _) ->
  #access_info{events=[do_change_password], default_event=refresh};
chpass_hook({call, do_change_password,  WA, Srv}, HD) ->
  Pk = wsdcomp:super_any_state(Srv, current_rowid),
  Dbh = wsdb:dbh(WA),
  Setter = fun (P) ->
	       wsite_acl:set_password(Dbh, Pk, P)
	   end,
  Comp = wsdcomp:create(wsuser_login_chpass, {Setter, WA}, WA,
			[{place_at, ?chpass_form}]), 
  ok = wsdcomp:spawn_comp(Srv, ?chpass_form, Comp),
  wsdcomp:call(Srv, ?chpass_form, {md5, undefined}),
  {ok, ok, HD};
chpass_hook(_, HD) ->
  {ok, undefined, HD}.

security_group_hook(init, Data) -> {ok, Data};
security_group_hook({is_active, _WA, _Srv}, _HD) ->
  true;
security_group_hook({access_info, WA, Srv}, _HD) ->
  #access_info{events=[do_security_groups], default_event=refresh};
security_group_hook({call, do_security_groups, WA, Srv}, HD) ->
  case wsdcomp:super_any_state(Srv, current_rowid) of
    [{_, LoginId}] ->
      GroupComp = 
	wsdcomp:create(wsuser_login_security_group,
		       {LoginId, WA}, WA, [{place_at, ?security_groups}]),
      ok = wsdcomp:spawn_comp(Srv, ?security_groups, GroupComp),
      wsdcomp:call(Srv, ?security_groups, enter), 
      {ok, ok, HD};
    _ ->
      {ok, ok, HD}
  end;
security_group_hook({call, Event, WA, Srv}, HD) ->
  {ok, undefined, HD};
security_group_hook({event, Msg, From, WA, Srv}, HD) ->
  {ok, undefined, HD}.

%% Wsform stuff

create_form(WA) ->
  Fields = [login_id, party_id, disabled_datetime, enabled, login_attempts, password],
  wsform:create(
    user_login,
    [{label, {wsform_lib, language_label}},
     {ss,    {wsform.ss, form}}],
    Fields,
    [{data_type,   fun data_type/2},
     {out_attrs,   fun out_attrs/2},
     {default,     fun default/2},
     {calculate,   fun calculate/4},
     {label,       {wsform_lib, language_label}},
     {ss, [wsform.ss:form_field(wsxmlt:m_name(Fields), WA),
	   wsxmlt:skip()]}
    ],
    WA).

data_type(party_id, _)       -> integer;
data_type(enabled, _)        -> boolean;
data_type(disabled_datetime, _)  -> datetime;
data_type(login_attempts, _) -> integer;
data_type(Name, _)           -> string.

default(login_id, _) -> required;
default(N, WA)       -> wsform_lib:def_value(N, WA).

calculate(password, _, _, _) ->
  {ok, "********"};
calculate(_, V, _, _) ->
  {ok, V}.

out_attrs(login_id, _) -> [{maxlength, "32"}, {size, "16"}];
out_attrs(party_id, _) -> [{maxlength, "16"}, {size, "16"}];
out_attrs(_, _) -> [].
