%%%-------------------------------------------------------------------
%%% File    : wsuser_login_chpass.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : User login change password form
%%%
%%% Created : 25 Mar 2003 by  <svg@surnet.ru>
%%%-------------------------------------------------------------------
%%
%% $Id: wsuser_login_chpass.erl,v 1.2 2003/10/31 17:51:02 vsekissov Exp $
%%
%%**
%% 
%% @doc User login change password form
%% @end
%%*
-module(wsuser_login_chpass).

-behaviour(wsdcomp).

-export([init/2, access_info/3, handle_event/6]).

-export([enter/4, go/4]).

-import(wsform.ss).

-include("wstl.hrl").

-record(state, {form, setter, save_as}).

-define(password_len, 8).

access_info(_, _, _) ->
  #access_info{events=[do_save, do_exit], default_event=refresh}.

init({Setter, WA}, Props) ->
  Form = form_create(WA),
  {ok, enter, #state{form=Form, setter=Setter},
   Props++[{template, "wsuser/change_password"}]
  }.

handle_event(_Msg, _From, WA, Srv, _State, S) ->
  go(refresh, WA, Srv, S).

enter({DefaultType, DefaultValue}, WA, Srv, S) ->
  Val = if DefaultValue == undefined ->
	    random_string(?password_len);
	   true ->
	    DefaultValue
	end,
  next_state({[],
	      [{password_type, DefaultType},{password_value, Val}]},
	     WA, go, S).

go(do_save, WA, Srv, S=#state{form=F, setter=Set, save_as=SA}) ->
  case wsform:input(WA, WA, F) of
    Good={[], Row} ->
      Type = kv(password_type, Row),
      Val  = wslib.passwd:crypt(kv(password_value, Row), Type),
      Pwd = {Type, Val},
      case Set(Pwd) of
	{ok, Res} ->
	  wslib:show_lmessage(password_changed, [], WA),
	  {stop, ok, []};
	Error ->
	  wsmessage:show_error(Error, WA),
	  next_state(Good, WA, go, S)
      end;
    Bad={_,Row} ->
      next_state(Bad, WA, go, S)
  end;
go(do_exit, WA, Srv, StateData) ->
  {stop, ok, []};
go(_, WA, Srv, StateData) ->
  {next_state, ok, go, StateData}.

next_state(Data, WA, go, S=#state{form=F}) ->
  Out = wsform:output(Data, [{mode, edit}], WA, F),
  {next_state, ok, [{'FormOut', Out}], go, S}.

form_create(WA) ->
  Fields = [password_type, password_value],
  wsform:create(
    change_password,
    [{label, fun (_, _) -> "" end}],
    Fields,
    [{data_type,   fun data_type/2},
     {out_attrs,   fun out_attrs/2},
     {default,     fun default/2},
     {validate,    fun validate/4},
     {label,       {wsform_lib, language_label}},
     {ss, [ss:form_field(
	     wsxmlt:m_name(password_type),
	     [ss:selector([{"md5", "md5"},
			   {"des", "des"},
			   {"cleartext","cleartext"}],
			  [])
	     ],
	     WA),
	   ss:form_field(wsxmlt:m_name(Fields), WA),
	   wsxmlt:skip()]}
    ],
    WA).

default(password_value, _) -> undefined;
default(password_type, _)  -> md5.

data_type(password_value, _) -> string;
data_type(password_type, _)  -> atom.

out_attrs(password_value, _) -> [{maxlength, "32"}, {size, "16"}];
out_attrs(_, _) -> [].

validate(password_value, undefined, _, _) ->
  exit(required);
validate(password_value, V, _, _) when length(V) < ?password_len ->
  exit(weak_password);
validate(_, _, _, _) ->
  true.
  
kv(K, L) ->
  proplists:get_value(K, L).

random_string(Len) ->
  randomx:alnum_string(Len).
