%%
%% $Id: html.erl,v 1.1 2003/11/01 12:39:21 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wslib.html).

-export([url_format/2]).
-export([checkbox/4, selector/4, radio_button/3, submit_button/3, button/2, button/3]).

-import(lists).
-import(wslib).

url_format(Path, Args) ->
  lists:flatten([Path,[$?|url_format_args(Args)]]).

url_format_args(Args) ->
  str:join([[str:to_str(K),"=",str:to_str(V)] || {K, V} <- Args], $&).

checkbox(Name, Value, Checked, Attrs) ->
  Attrs1=  [{type, "checkbox"},
	    attr(name, Name),
	    attr(value, Value)
	    |Attrs
	   ],
  {input,
   case Checked of
     true ->
       [{checked, ""}|Attrs1];
     false ->
       Attrs1
   end
  }.

submit_button(Name, Value, Attrs) ->
  button({Name, Value}, Attrs).

radio_button(Name, Value, Attrs) ->
  button({Name, Value}, [{type, "radio"}|Attrs]).

button(Action, Attrs) ->
  button(Action, Attrs, fun (V) when is_list(V) -> V;
			    (V) -> atom_to_list(V)
			end).

button({N, V}, Attrs, LabelFun) ->
  Attrs1 = [attr(name, N), {value, LabelFun(V)}
	    |wslib:v_set_undef(type, "submit", Attrs)],
  {input, Attrs1};
button(N, Attrs, LabelFun) ->
  button({N, N}, Attrs, LabelFun).

selector(Name, KVs, SelectedKeys, Attrs) ->
  Type = wslib:gv(type, Attrs, selector),
  Attrs1 = wslib:k_delete(type, Attrs),
  MkEntry = mk_selector_entry(Type, Name, Attrs1),
  Entries = [MkEntry(K, V, lists:member(K, SelectedKeys))
	     || {K, V} <- KVs],
  selector_element(Type, Name, Entries, Attrs1).

mk_selector_entry(selector, _Name, _Attrs) ->
  fun selector_option/3;
mk_selector_entry(checkbox,  Name, Attrs) ->
  Orientation = wslib:gv(orientation, Attrs, vertical),
  Attrs1 = wslib:k_delete(orientation, Attrs),
  fun (Key, Value, Selected) ->
      V = str:to_str(Value),
      K = checkbox(Name, Key, Selected, []),
      case Orientation of
	vertical ->
	  [V, K, {br, []}];
	horizontal ->
	  [V, K]
      end
  end.

selector_element(selector, Name, Entries, Attrs) ->
  Attrs1 = wslib:v_set_undef(size, "1", Attrs),
  {select, [attr(name, Name)|Attrs1], Entries};
selector_element(checkbox, _Name, Entries, Attrs) ->
  Attrs1 = wslib:k_delete(orientation, Attrs),
  {'div', Attrs1, Entries}.

selector_option(K, V, true) ->
  {option, [{selected, ""}, {value, K}], [V]};
selector_option(K, V, false) ->
  {option, [{value, K}], [V]}.

attr(N, K) when is_atom(N) ->
  {N, str:to_str(K)}.
