%%% File    : http.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : Various http related functions
%%% Created : 12 May 2003 by  <svg@surnet.ru>

%%
%% $Id: http.erl,v 1.2 2003/08/29 13:52:54 vsekissov Exp $
%%
%%**
%% 
%% @doc Various http related functions
%% @end
%%*
-module(wslib.http).

-export([header_value/2]).

-import(wslib).
-import(lists).
-import(string).

-include("yaws_api.hrl").

-define(header_sep, " ,").
-define(attr_sep, ";").
-define(default_quality, 1.0).

%% 'Accept-Language'
%% 
%% [LangDesc], LangDesc = Lang | {Lang, Country}
%%
%% Lang = lower_case_atom()
%% Country = lower_case_atom()
%%
%% 'Accept-Charset' = [Charset]
%%
%% Charset = lower_case_atom()
%% 
%% 'Accept-Encoding' = [Method]
%%
%% Method = lower_case_atom()
%%
%% Returned lists are sorted by quality with higher
%% quality values first
%%
header_value(Name, A=#arg{headers=#headers{other=O}}) ->
  header_value(Name, O);
header_value(Name, Headers) ->
  format_header_val(Name, get_header_value(Name, Headers)).

format_header_val(undefined, _) ->
  undefined;
format_header_val(_, undefined) ->
  undefined;
format_header_val('Accept-Language', Str) ->
  Parser = parse_lang(),
  lists:reverse(
    [V || {V, _} <- lists:keysort(2, [Parser(A) ||  A <- header_attrs(Str)])]);
format_header_val(Name, Str) when Name == 'Accept-Charset';
				  Name == 'Accept-Encoding' ->
  Parser = parse_attr_quality(fun lower_atom/1),
  lists:reverse(
    [V || {V, _} <- lists:keysort(2, [Parser(A) ||  A <- header_attrs(Str)])]);
format_header_val(_, Str) ->
  Str.

parse_lang() ->
  Parser = parse_attr(fun parse_country/1, fun parse_quality/1),
  fun (A) -> set_quality(Parser(A)) end.

parse_attr_quality(Left) ->
  Parser = parse_attr(Left, fun parse_quality/1),
  fun (A) -> set_quality(Parser(A)) end.

parse_attr(Left, Right) ->
  parse_attr(?attr_sep, Left, Right).

parse_attr(Sep, Left, Right) ->
  mk_parse(Sep, Left, Right).

parse_country(Str) ->
  case string:tokens(Str, "-") of
    [L] ->
      lower_atom(L);
    [L, C] ->
      {lower_atom(L), lower_atom(C)}
  end.

parse_quality("q="++Str) ->
  case catch list_to_float(Str) of
    {'EXIT', _} ->
      ?default_quality;
    Res ->
      Res
  end;
parse_quality(_) ->
  ?default_quality.

set_quality(V={_, Q}) when is_float(Q) ->
      V;
set_quality(V) ->
  {V, ?default_quality}.

mk_parse(Sep, ToLeft, ToRight) ->
  fun (Str) ->
      case string:tokens(Str, Sep) of
	[L] ->
	  ToLeft(L);
	[L, R] ->
	  {ToLeft(L), ToRight(R)}
      end
  end.

get_header_value(Name, Headers) ->
  case lists:keysearch(Name, 3, Headers) of
    {value, {http_header,_,_,_,V}} ->
      V;
    false ->
      undefined
  end.

header_attrs(Head) ->
  string:tokens(Head, ?header_sep).

lower_atom(Str) ->
  list_to_atom(str:to_lower(Str)).

