%%% File    : misc.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created : 10 Dec 2002 by  <svg@surnet.ru>

%%
%% $Id: misc.erl,v 1.4 2003/10/27 16:45:59 vsekissov Exp $
%%
%%
%%**
%% 
%% @doc Miscellaneous functions
%% @end
%%*
-module(wslib.misc).

-export([nvl/2, better_error/1, call_stack/0, call_node/4]).
-export([application_nodes/1]).

-import(mnesia).
-import(application).
-import(rpc).
-import(lists).

%% @spec nvl(Value::term(), Default::term()) -> Return::term()
%%
%% @doc Return _Default_ if _Value == undefined_ or _Value_ otherwise.
%% 
nvl(undefined, Default) -> Default;
nvl(Value, _) -> Value.
  
%%**
%% @type error() = {error, Error}
%%      Error = term().
%% 
%% @spec better_error(Error::term()) -> Return::error()
%%
%% @doc Try to convert catched result to error return format
%%      user expected
%%
%% @end
%%*
better_error({'EXIT', {{badmatch, Error}, _}}) ->
  better_error(Error);
better_error({aborted, Reason}) ->
  {error, mnesia:error_description(Reason)};
better_error({'EXIT', Error}) ->
  better_error(Error);
better_error(Error) when is_tuple(Error),
			 element(1,Error) == error ->
  Error;
better_error(Error) ->
  {error, Error}.

%% Stolen from jungerl/lib/ce/src/ce_lib.erl
%% @spec call_stack() -> [{Module, Function, Arity}]
%% @doc Returns the current Erlang call stack (not including
%% <code>wslib.misc:call_stack/0</code>).

call_stack() ->
  {'EXIT', {Error, CallStack}} = (catch 1 = 2),
  tl(CallStack).

%% @spec call_node(node(), Mod::atom(), Fun::atom(), Args::list()) ->
%%           Result::term() | exit
%% @doc Call function with <code>apply</code> on local node or on remote node
%% with <code>rpc</code>. Return result of the call or exit if function or
%% <code>rpc</code> failed.
%% 
call_node(Node, Mod, Fun, Args) ->
  MyNode = node(),
  if Node =/= MyNode -> % remote node
      case rpc:call(Node, Mod, Fun, Args) of
	{badrpc, Reason} ->
	  exit(Reason);
	Other ->
	  Other
      end;
     true -> % local node
      apply(Mod, Fun, Args)
  end.

%%
%% @spec application_nodes(AppName::atom()) -> [node()] | undefined
%% @doc Nodes where distributed application can start
%%
application_nodes(AppName) ->
  case catch
    begin
      {ok, D} = application:get_env(kernel, distributed),
      {value, AppSpec} = lists:keysearch(AppName, 1, D),
      [First, Other] = element(size(AppSpec), AppSpec),
      {ok, [First|tuple_to_list(Other)]}
    end of
    {ok, Ret} ->
      Ret;
    _ ->
      undefined
  end.
