%%%-------------------------------------------------------------------
%%% File    : props.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : Functions for properties I use frequently
%%%
%%% Created :  6 Jan 2004 by  <svg@surnet.ru>
%%%-------------------------------------------------------------------
%%
%% $Id$
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wslib.props).

-export([gv/2, gv/3, gv_map/2, set/3, set_undef/3, update/3, delete/2, delete_map/2, lookup/2, lookup_map/2]).
-export([check_required/2]).

-import(proplists).
-import(lists).

gv(K, L) ->
  proplists:get_value(K, L).

gv(K, L, D) ->
  case gv(K, L) of
    undefined ->
      D;
    V ->
      V
  end.

gv_map(Ks, L) ->
  lists:map(fun ({K, D}) ->
		gv(K, L, D);
		(K) ->
		gv(K, L)
	    end, Ks).

lookup(K, L) ->
  case proplists:lookup(K, L) of
    none ->
      undefined;
    KV ->
      KV
  end.

lookup_map(Ks, L) ->
  lists:map(fun (KD={K, _D}) ->
		case proplists:lookup(K, L) of
		  none ->
		    KD;
		  KV ->
		    KV
		end;
		(K) ->
		lookup(K, L)
	    end, Ks).

set(K, V, L) ->
  [{K, V}|proplists:delete(K, L)].

set_undef(K, V, L) ->
  case gv(K, L) of
    undefined ->
      set(K, V, L);
    _ ->
      L
  end.

delete(K, L) ->
  proplists:delete(K, L).

delete_map(Ks, L) ->
  Test = fun ({K, _}) -> lists:member(K, Ks);
	     (K) -> lists:member(K, Ks)
	 end,
  [E || E <- L, Test(E) == false].

update(Name, List, UpdFun) ->
  lists:map(fun ({N, V}) when N == Name ->
		{N, UpdFun(V)};
		(KV) ->
		KV
	    end, List).

%% @spec check_required(ReqDefs, Props) -> {ok, NewProps}|{error, Error}
%% ReqDefs = [ReqDef]
%% Props = [{Key, Value}]
%% NewProps = [{Key, Value}]
%% ReqDef = {KeyOptional, Default}
%% ReqDef = {KeyRequired}
%% KeyRequired = term()
%% KeyOptional = term()
%% Error = term()
%% 
%% @doc Add <code>{KeyOptional, Default}</code> to <code>Props</code> if
%% <code>KeyOptional</code> property doesn't exist.
%% <p>
%% Check that <code>KeyRequired</code> exists in <code>Props</code>.
%% @end
check_required([], Props) ->
  {ok, Props};
check_required([KV={Key, _Default}|Rest], Props) ->
  case proplists:lookup(Key, Props) of
    {_, _} ->
      check_required(Rest, Props);
    none ->
      check_required(Rest, [KV|Props])
  end;
check_required([{Key}|Rest], Props) -> %required, without default
  case proplists:lookup(Key, Props) of
    {_, _} ->
      check_required(Rest, Props);
    none ->
      {error, {property_required, Key}}
  end.
