%%%-------------------------------------------------------------------
%%% File    : tuple.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%%
%%% Created : 30 Sep 2004 by  <svg@surnet.ru>
%%%-------------------------------------------------------------------
%%
%% $Id$
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wslib.tuple).

-export([merge/3, merge/2]).

%% @spec merge(Tuple1::tuple(), Tuple2::tuple(), MergeFun)
%%            -> Tuple3::tuple()
%% MergeFun = fun (Element1, Element2) -> Element3
%% @doc Merge two tuples of the same size. _MergeFun_ returns element of
%% resulting tuple.
%% 
merge(Tuple1, Tuple2, MergeFun) ->
  Size = size(Tuple1),
  hof:while(fun ({0, Ret}) ->
		{false, list_to_tuple(Ret)};
		({Pos, Acc}) ->
		PosE = MergeFun(element(Pos, Tuple1), element(Pos, Tuple2)),
		NxtAcc = [PosE|Acc],
		{true, {Pos-1, NxtAcc}}
	    end, {Size, []}).

%% @spec merge(Tuples, MergeFun) -> Tuple::tuple()
%% Tuples = [tuple()]
%% MergeFun = fun ([Element]) -> NewElement
%% @doc Merge list of tuples of the same size. _MergeFun_ returns element of
%% resulting tuple. It accepts the list of elements of tuples on equal
%% positions.
%%
merge(Tuples, MergeFun) when Tuples =/= [] ->
  Size = size(hd(Tuples)),
  hof:while(fun ({0, Ret}) ->
		{false, list_to_tuple(Ret)};
		({Pos, Acc}) ->
		PosE = MergeFun([element(Pos, T) || T <- Tuples]),
		NxtAcc = [PosE|Acc],
		{true, {Pos-1, NxtAcc}}
	    end, {Size, []}).
