%%%-------------------------------------------------------------------
%%% File    : wsocket_connect.erl
%%% Author  :  <svg@disney.surnet.local>
%%% Description : TCP/IP/UDP client
%%%
%%% Created : 12 Jun 2002 by  <svg@surnet.ru>
%%%-------------------------------------------------------------------
%%
%% 
%%*

-module(wsocket_connect).

-export([start_link/6]).

-import(wslib.props).

start_link(Addr, Port, TrMod, PctFun, PrtFun, Props) ->
  {ConnectProps, MediatorProps} =
    proplists:split(Props, [timeout, transp_opts]),
  [Timeout, TrProps] =
    props:gv_map([timeout, transp_opts],
		 check_props(lists:append(ConnectProps))),
  ConnectParams = [Addr, Port, TrProps, Timeout],
  wsocket_mdr:spawn_client(ConnectParams,TrMod, PctFun, PrtFun, MediatorProps).

check_props(Props) ->
  {ok, Props1} = props:check_required(
		   [{timeout, infinity},
		    {transp_opts, []}
		   ], Props),
  [check_prop(P) || P <- Props1].

check_prop(P={timeout, infinity}) -> P;
check_prop(P={timeout, N}) when is_integer(N), N > 0 -> P;
check_prop(P={transp_opts, _}) -> P;
check_prop(P) ->
  exit({badopt, P}).
