%%%-------------------------------------------------------------------
%%% File    : stl_yaws.erl
%%% Author  :  <svg@surnet.ru>
%%% Purpose : STL helper for Yaws
%%% Created : 20 Nov 2002 by  <svg@surnet.ru>
%%%-------------------------------------------------------------------

%%
%% $Id: stl_yaws.erl,v 1.4 2003/10/31 17:57:31 vsekissov Exp $
%%
%%**
%% 
%% .* stl_yaws
%% 
%%*
-module(stl_yaws).
-author('svg@surnet.ru').

-export([start_link/2, screen_place_at/0]).
-export([create_screen/1, create_screen/2, create_screen1/2, create_screen/3]).
-export([create_page/1, create_page/2, create_page/3, create_page/4]).
-export([include/3, include/4, ws_arg/1, enc_path/2, enc_path/3, service_spec/3]).
-export([create/2, render_props/1, render_props/2]).

-import(wslib.str).
-import(wslib.listn).
-import(wslib.html).

-include("wstl.hrl").
-include_lib("kernel/include/file.hrl").

-define(arg, '!!$_stl_arg_$!!').
-define(template_input, template).
-define(screen_var, 'ScreenOutput').

-define(layout_defaults, [{pages_top, "pages"},
			  {navigations_top, "navigations"},
			  {screens_top, "screens"}]).

-record(stl_arg, {server, 
		  pages_top,
		  navigations_top,
		  screens_top}).

-record(screen, {place_at, template}).
-record(page, {screen, place_at, template}).

start_link(Name, Options) ->		  
  stl:start_link(Name, Options).

service_spec(AppName, Proc, Props) ->
  [PageTop, NaviTop, ScreenTop] =
    [proplists:get_value(K, Props, D) || {K, D} <- ?layout_defaults],
  DefDir = case code:priv_dir(AppName) of
	     {error, _} ->
	       ".";
	     Dir ->
	       DirStl = filename:join([Dir, "stl"]),
	       case file:read_file_info(DirStl) of
		 {ok, #file_info{type=directory}} ->
		   DirStl;
		 _ ->
		   Dir
	       end
	   end,
  StlDefaults = [{dir, DefDir}, {mod_prefix, atom_to_list(AppName)}],	      
  StlProps =
    [{K, proplists:get_value(K, Props, D)} || {K, D} <- StlDefaults]
    ++
    lists:foldl({proplists, delete}, Props,
		[dir, mod_prefix|proplists:get_keys(?layout_defaults)]),
  SA = #stl_arg{server=Proc,
		pages_top=PageTop,
		navigations_top=NaviTop,
		screens_top=ScreenTop},
  InitEnv = [mk_init_env(SA)],
  {ok, StlProps, InitEnv}.

render_props(Render) ->
  Render(state).

render_props(KVs, Render) ->
  Render({inherit, KVs}).

screen_place_at() ->
  ?screen_var.

include(SubDir, Ctxt, Tmpl) ->
  include(SubDir, Ctxt, Tmpl, []).

include(SubDir, Ctxt, Tmpl, AddToCtxt) ->
  stl:include(Ctxt, template(SubDir, get_env(ws_arg(Ctxt)), Tmpl), AddToCtxt).

create(WA, Props) ->
  Tmpl = 
    case wslib:gv(template, Props) of
      undefined ->
	template(WA#ws_arg.yaws_arg);
      T ->
	T
    end,
  HolderVar = wslib:gv(place_at, Props),
  case wslib:gv(render_as, Props) of
    page ->
      PageTmpl = wslib:gv(page_template, Props, Tmpl),
      Screen = create_screen(WA, Tmpl, ?screen_var),
      Page = create_page(WA, PageTmpl, Screen, HolderVar);
    _ ->
      create_screen(WA, Tmpl, HolderVar)
  end.

create_screen(WA) ->
  create_screen(WA, ?screen_var).

create_screen(WA=#ws_arg{yaws_arg=YA}, HolderVar) ->
  Tmpl = template(YA),
  create_screen(WA, Tmpl, HolderVar).

create_screen1(WA, Tmpl) ->
  create_screen(WA, Tmpl, undefined).

create_screen(WA, Tmpl, HolderVar) ->
  State = #screen{place_at=HolderVar, template=Tmpl},
  mk_screen(State).

create_page(WA) ->
  create_page(WA, create_screen(WA)).

create_page(WA, Screen) ->
  S = Screen(internal_state),
  create_page(WA, S#screen.template, Screen).

create_page(WA, Tmpl, Screen) ->
  create_page(WA, Tmpl, Screen, undefined).

create_page(WA, Tmpl, Screen, HolderVar) ->
  SA = get_env(WA),
  {ok, PageTmpl} = search_template(SA, template(page, SA, Tmpl)),
  mk_page(#page{template=PageTmpl, screen=Screen, place_at=HolderVar}).

mk_screen(State) ->
  fun (state) ->
      [{place_at, State#screen.place_at},
       {template, State#screen.template}];
      (internal_state) ->
      State;
      ({display, WA, Out}) ->
      process(WA, Out, State); 
      ({inherit, RS}) ->
      case wslib:gv(place_at, RS) of
	undefined ->
	  mk_screen(State);
	HV ->
	  NxtS = State#screen{place_at=HV},
	  mk_screen(NxtS)
      end
  end.

mk_page(State) ->
  fun (state) ->
      [{place_at, State#page.place_at},
       {template, State#page.template}];
      (internal_state) ->
      State;
      ({display, WA, Out}) ->
      process(WA, Out, State); 
      ({inherit, RS}) ->
      case wslib:gv(place_at, RS) of
	undefined ->
	  mk_page(State);
	HV ->
	  NxtS = State#page{place_at=HV},
	  mk_page(NxtS)
      end
  end.

process(WA, Out, S=#page{screen=Scr, place_at=HV, template=T}) ->
  ?dbg("process ~p", [S]),
  SA = get_env(WA),
  Ctxt = wslib:c_set([{?arg, WA}|Scr({display, WA, Out})] ++ Out,
		     wslib:c_new()),
  PageOut = process(SA, T, [], Ctxt),
  
  Ret = return_holder(HV, PageOut);
process(WA, Out, S=#screen{place_at=HV, template=T}) ->
  ?dbg("process ~p~ninput data:~n~p", [S, Out]),
  SA = get_env(WA),
  ScreenTmpl = template(screen, SA, T),
  Ctxt = wslib:c_set([{?arg, WA}|Out], wslib:c_new()),
  Ret = return_holder(HV, process(SA, ScreenTmpl, [], Ctxt)).

process(SA=#stl_arg{server=Srv}, Template, Actions, Ctxt) ->
  stl:process(Srv, Template, Actions, Ctxt,
	      fun process_format/1, ?PROCESS_TIMEOUT).

return_holder(undefined, Data) ->
  Data;
return_holder(HolderVar, Data) ->
  case Data of
    {ok, {_, Out}} ->
      [{HolderVar, Out}];
    Error ->
      [{HolderVar, Error}]
  end.

template(screen, SA=#stl_arg{screens_top=Dir}, Tmpl) ->
  template_name_join(Dir, Tmpl);
template(page, SA=#stl_arg{pages_top=Dir}, Tmpl) ->
  template_name_join(Dir, Tmpl);
template(navigation, SA=#stl_arg{navigations_top=Dir}, Tmpl) ->
  template_name_join(Dir, Tmpl).

template(Arg) ->
  Input = wslib:arg_input(Arg),
  Path =
    case kv(?template_input, Input) of
      undefined ->
	Arg#arg.appmoddata;
      T ->
	T
    end,
  filename:absname(Path, "").

template_name_join(Dir, Tmpl) ->
  filename:join(Dir, string:strip(Tmpl, left, $/)).

%% We try to find template for page recursively
search_template(SA=#stl_arg{server=Srv}, Path) ->
  search_template0(Srv, filename:absname(Path, "")).

search_template0(Srv, Path) ->
  case stl:template(Srv, Path) of
    Good={ok, _} ->
      {ok, Path};
    Error ->
      case filename:dirname(Path) of
	Path ->
	  Error;
	DirName ->
	  search_template0(Srv, DirName)
      end
  end.

enc_path(Path, Ctxt) ->
  wsrun:enc_path(Path, ws_arg(Ctxt)).

enc_path(Path, Args, Ctxt) ->
  html:url_format(wsrun:enc_path(Path, ws_arg(Ctxt)), Args).

ws_arg(Ctxt) ->
  wslib:c_val(?arg, Ctxt).

process_format({error, Error}) when list(Error) ->
  {html, "<pre>"++Error++"</pre>"};
process_format({error, Error}) when list(Error) ->  
  {html, io_lib:format("<pre>~p</pre>", [Error])};
process_format({'(*', Str}) ->
  {html, Str};
process_format(V={html, _}) ->
  V;
process_format(V={ehtml, _}) ->
  V;
process_format(V={content, _, _}) ->
  V;
process_format(V={streamcontent, _, _}) ->
  V;
process_format(V={header, _}) ->
  V;
process_format(V={allheaders, _}) ->
  V;
process_format(V={status, _}) ->
  V;
process_format(V={redirect, _}) ->
  V;
process_format(V={redirect_local, _}) ->
  V;
process_format(V={get_more, _, _}) ->
  V;
process_format(L) when is_list(L) ->
  fmt_list(L, [], [], fun (L1) -> lists:reverse(L1) end);
process_format(V) ->
  {html, listn:mk_list(stl:to_list(V))}.

fmt_list([], Cs, Acc, Cont) ->
  Cont(chars_to_acc(Cs, Acc));
fmt_list([T|Rest], Cs, Acc, Cont) when is_tuple(T) ->
  fmt_list(Rest, [], [process_format(T)|chars_to_acc(Cs, Acc)], Cont);
fmt_list([L|Rest], Cs, Acc, Cont) when is_list(L) ->
  fmt_list(L, Cs, Acc, fun (Acc1) -> fmt_list(Rest, [], Acc1, Cont) end);
fmt_list([C|Rest], Cs, Acc, Cont) ->
  fmt_list(Rest, [term_to_chars(C)|Cs], Acc, Cont).

%% fmt_list([], Cs, Acc) ->
%%   list_to_acc(Cs, Acc);
%% fmt_list([T|Rest], Cs, Acc) when tuple(T) ->
%%   fmt_list(Rest, [], [process_format(T)|list_to_acc(Cs,Acc)]);
%% fmt_list([L|Rest], Cs, Acc) when is_list(L) ->
%%   fmt_list(Rest, [],
%% 		      fmt_list(L, [],list_to_acc(Cs, Acc)));
%% fmt_list([C|Rest], Cs, Acc) ->
%%   fmt_list(Rest, [term_to_chars(C)|Cs], Acc).

chars_to_acc([], Acc) ->
  Acc;
chars_to_acc(Cs, Acc) ->
  [{html, lists:reverse(Cs)}|Acc].

term_to_chars(V) when is_integer(V), V =< 256 ->
  V;
term_to_chars(V) ->
  io_lib:write(V).

mk_init_env(Env) ->
  fun (WA=#ws_arg{ws_env=WE}) ->
      WA#ws_arg{ws_env=[{?MODULE, Env}|WE]}
  end.

get_env(#ws_arg{ws_env=WE}) ->
  case proplists:get_value(?MODULE, WE) of
    undefined ->
      exit({service_not_started, ?MODULE});
    V ->
      V
  end.

kv(K,L) ->
  case lists:keysearch(K,1,L) of
    {value, {K, V}} ->
      V;
    _ ->
      undefined
  end.
