%%% File    : add_delete_selector.erl.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created :  4 Dec 2002 by  <svg@surnet.ru>

%%
%% $Id: add_delete_selector.erl,v 1.3 2003/10/27 16:56:19 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wscomp.add_delete_selector).

-behaviour(wsdcomp).

-export([init/2, access_info/3, handle_event/6]).

-import(.wslib).
-import(.wsmessage).
-import(.stl_yaws).
-import(.wsdcomp).

-include("wstl.hrl").

access_info(_, _, _) ->
  exit(must_never_reach).

handle_event(Event, From,  WA, Srv, State, StateData) ->
  exit(must_never_reach).

%% Props:
%%    [{size, 10}, {label, ""}, place_at]
%%    
%% add_items, delete_items =
%%     function([Id::string()], WA::ws_arg(), Data) -> ok | {ok, NextData}
%%     Data = term()
%%     NextData = term()
%% 
%% select_add_items, select_delete_items =
%% function(WA::ws_arg(), Data::term()) -> {ok, [KV]} | {ok, [KV], NextData}
%% KV = {Id, Description}
%% Id = string()
%% Description = string()
%% NextData = term()
%% 
init({AddItems, SelectAddItems, DelItems, SelectDelItems, Arg, WA}, Props) ->
  Size = wslib:gv(size, Props, "10"),
  Label = wslib:gv(label, Props, ""),
  RenderProps = Props++[{template, "lib/add_delete_selector"}],
  SelAttrs = {selector_attrs, [{size, Size}, multiple]},
  MkAddComp = selector:create(
		id,
		SelectAddItems,
		AddItems,
		Arg,
		[{access_id, add_selector},
		 {place_at, 'AddItems'},
		 {actions, [insert]},
		 SelAttrs
		]),
  MkDelComp = selector:create(
		id,
		SelectDelItems,
		DelItems,
		Arg,
		[{access_id, delete_selector},
		 {place_at, 'DelItems'},
		 {actions, [delete]},
		 SelAttrs
		]),
  Comps =
    [multiplex:create_subcomp("add_items", MkAddComp(component, WA), enter),
     multiplex:create_subcomp("delete_items", MkDelComp(component, WA), enter)],
  MplexHlr = {fun show_selector_result/2, [{'Label', Label}]},
  Comp = wsdcomp:create(wscomp.multiplex, {Comps, MplexHlr}, WA, RenderProps),
  {component, Comp}.

show_selector_result(init, Out) ->
  {ok, Out};
show_selector_result({call, _, _, _}, Out) ->
  {ok, Out};
show_selector_result({event, {ok, {selected, Num}},  WA, Srv, Name}, Out) ->
      Op = case Name of
	     "add_items" ->
	       'RECORDS_INSERTED';
	     "delete_items" ->
	       'RECORDS_DELETED'
	   end,
  wslib:show_lmessage(Op, [{num, Num}], WA),
  {ok, Out};
show_selector_result({event, {ok, Msg}, WA, _, _}, Out) ->
  wsmessage:show_message(Msg, WA),
  {ok, Out};
show_selector_result({event, ok, WA, _, _}, Out) ->
  {ok, Out};
show_selector_result({event, Error, WA, _, _}, Out) ->
  wsmessage:show_error(Error, WA),
  {ok, Out}.
