%%% File    : wscomp.chain_dialog.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created :  4 Dec 2002 by  <svg@surnet.ru>

%%
%% $Id: chain_dialog.erl,v 1.1 2003/10/27 16:45:04 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wscomp.chain_dialog).

-behaviour(wsdcomp).

-export([init/2, access_info/3, handle_event/6]).
-export([enter/4, go/4]).

-import(.wsdcomp).
-import(.wslib).
-import(.wsmessage).
-import(.stl_yaws).
-import(.lists).

-include("wstl.hrl").

-define(step_comp, "step").
-define(current_template_var, 'StepOut').
-define(template_var, 'Dialog').

-record(state, {current,
		prev=[],
		next=[],
		processed=[],
		handler,
		hdata,
		template_kvs}).

%%
%% Handler(init, Data) -> {ok, StepNames, State};
%%        ({start_dialog, UserEvent, WA, Srv}, State) -> {ok, InitEvent, NextState};
%%        ({stop_dialog, Result, WA, Srv}, State)     -> {ok, Return};
%%        ({StepName, next_step, WA, Srv}, State)            -> Result;
%%        ({StepName, previous_step, WA, Srv}, State)        -> Result;
%%        ({StepName, PreviousStepResult, WA, Srv}, State)   -> Result
%%        Result = {ok, NextComp, NextStartEvent, NextState}
%%                 | {stop, Return}
%%                 | {ok, NextStartEvent, NextState}
%%        Data = term()
%%        State = term()
%%        StepNames = [StepName]
%%        StepName = atom()
%%        NextComp = wsdcomp(),
%%        TemplateName = string()
%%        NextStartEvent = term()
%%        PreviousStepResult = term()
%%        InitEvent = term()
%%        WA = ws_arg()
%%        
access_info(_, _, S=#state{next=[]}) ->
  #access_info{events=[do_previous, do_exit], default_event=refresh};
access_info(_, _, S=#state{current=N, processed=Ps, prev=P}) ->
  case lists:member(N, Ps) of
    true when P == [] ->
      #access_info{events=[do_next, do_exit], default_event=refresh};
    true ->
      #access_info{events=[do_previous, do_next, do_exit], default_event=refresh};
    false when P == [] ->
      #access_info{events=[do_exit], default_event=refresh};
    false ->
      #access_info{events=[do_previous, do_exit], default_event=refresh}
  end;
access_info(_, _, _) ->
  #access_info{events=[do_exit], default_event=refresh}.

handle_event(Event, security_alert, WA, Srv, State, StateData) ->
  {stop, ok};
%% Old request, no data in output
handle_event(Msg, old_request, WA, Srv, enter, StateData) ->
  enter(enter, WA, Srv, StateData);
handle_event(Event, ?step_comp,  WA, Srv, State, StateData) ->
  handle_current_return(Event, WA, Srv, StateData);
handle_event(_Event, From,  WA, Srv, State, StateData) ->
  {next_state, ok, State, StateData}.
%%
%% init({Handler, HandlerArg, StepDefs, WA}, Props) ->
%%    {ok, StartStateName, StateData}
%%
%% Handler = function()
%% HandlerArg = term()
%% StepDefs = [StepDefinition]
%% StepDefinition = StepName
%% WA = ws_arg()
%% Props = [{template_kvs, [{K::atom(), V::term()}]}]
%% K = 'Label'
%%
init({Handler, HandlerArg}, Props) ->
  TmplKVs = wslib:gv(template_kvs, Props, []),
  H = mk_handler(Handler),
  {ok, StepDefs, HD} = H(init, HandlerArg),
  {ok, enter, #state{next=StepDefs, handler=H, hdata=HD, template_kvs=TmplKVs},
   Props++[{template, "lib/chain_dialog"}]}.

enter(Event, WA, Srv, S=#state{handler=H, hdata=HD}) ->
  {ok, StartEvent, NxtHD} = H({start_dialog, Event, WA, Srv}, HD),
  call_next(StartEvent, WA, Srv, S#state{hdata=NxtHD}).

go(do_exit, WA, Srv, S) ->
  {stop, ok};
go(do_previous, WA, Srv, S=#state{current=Cur, prev=[H|Rest], next=Nxt}) ->
  wsdcomp:kill(Srv, ?step_comp),
  call_next(previous_step, WA, Srv, S#state{next=[H, Cur|Nxt], prev=Rest});
go(do_next, WA, Srv, S=#state{current=Cur, prev=Prev}) ->
  wsdcomp:kill(Srv, ?step_comp),
  call_next(next_step, WA, Srv, S#state{prev=[Cur|Prev]}).

call_next(PrevRes, WA, Srv, S=#state{next=[], handler=H, hdata=HD}) ->
  case H({stop_dialog, PrevRes, WA, Srv}, HD) of
    {Good, Ret} when Good == ok;
		     Good == stop ->
      {stop, Ret};
    Error ->
      exit(Error)
  end;
call_next(PrevRes, WA, Srv,
	  S=#state{next=[N|Rest], handler=H, hdata=HD}) ->
  case H({N, PrevRes, WA, Srv}, HD) of
    {ok, NextComp, NextStartEvent, NxtHD} ->
      NextComp1 =
	wsdcomp:render_props([{place_at, ?current_template_var}], NextComp),
      ok = wsdcomp:spawn_comp(Srv, ?step_comp, NextComp1),
      Ret = wsdcomp:call(Srv, ?step_comp, NextStartEvent),
      NxtS = S#state{current=N, next=Rest, hdata=NxtHD},
      handle_current_return(Ret, WA, Srv, NxtS);
    {ok, NextStartEvent, NxtHD} ->
      NxtS = S#state{current=N, next=Rest, hdata=NxtHD},
      call_next(NextStartEvent, WA, Srv, NxtS);
    Return={stop, _} ->
      Return
  end.

handle_current_return(Res, WA, Srv,
		      S=#state{current=N,
			       prev=P, processed=Ps,
			       template_kvs=TKVs}) ->
  case wsdcomp:is_alive(Srv, ?step_comp) of
    true ->
      #access_info{events=Actions} = access_info(WA, go, S),
      {next_state, ok, [{?template_var, [{actions, Actions}|TKVs]}], go, S};
    false ->
      NxtPs =
	case lists:member(N, Ps) of
	  true ->
	    Ps;
	  false ->
	    [N|Ps]
	end,
      call_next(Res, WA, Srv, S#state{prev=[N|P], processed=NxtPs})
  end.

mk_handler(H) ->
  H.
