%%% File    : expand.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created : 22 Aug 2003 by  <svg@surnet.ru>

%%
%% $Id: expand.erl,v 1.6 2003/11/05 13:51:37 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wscomp.expand).

-behaviour(wsdcomp).

-export([behaviour_info/1]).
-export([init/2, access_info/3, handle_event/6]).
-export([enter/4, call_exec_return/4]).
-export([create_comp/3, create_comp/4]).
%%
-export([call_hook/3, mk_hook/1]).

-import(.wsdcomp).
-import(.wslib).
-import(.lists).

-include("wstl.hrl").

-define(template_var, 'ExpOut').

-record(state, {hook,
		hdata,
		template_var,
		access_info=#access_info{}
	       }).

-record(dstate, {hooks, active_hook}).

%% Props = [{template_var, 'ExpOut'}]
%%
%% Hook = Module | HookFunction
%% Module must implement
%%
%% Module:is_active(WA, Srv, HD) -> true | false
%% Module:access_info(WA, Srv, HD) -> access_info()
%% Module:init(Arg) -> {ok, HookData}
%% Module:handle_call(Event, WA, Srv, HookData) -> HookReturn
%% Module:handle_event(Msg, From, WA, Srv, HookData) -> HookReturn
%%
%% HookFunction =
%% 
%%      fun ({is_active, WA, Srv}, HD) -> true | false
%%          ({access_info, WA, Srv}, HD) -> access_info()
%%          (init, Arg) -> {ok, HookData}
%%          ({call, Event, WA, Srv}, HD) -> HookReturn
%%          ({event, Msg, From, WA, Srv}, HD) -> HookReturn
%%      end.
%% HookReturn = {call_exec, Msg, Comp, HookData} |
%%              {call_exec, Msg, Comp, HookData}   |
%%              {exec, Msg, Comp}                  |
%%              {exec, Msg, Comp}}		   |
%%              {stop, Ret}				   |
%%              {stop, Ret, Out}			   |
%%              {ok, Ret, Out, HookData}		   |
%%              {ok ,Ret, HookData}                        
%%
%% HookData = term()
%% Ret = term()
%% Out = [KV]
%% KV = {Key::atom(), Value::term()}
%% Event = atom()
%% Comp = wsdcomp()
%%
%% If hook returns `undefined' as answer to call super is called,
%% only events from parent are delegated to super, `Event' is returned
%% as is if hook return `undefined' on other events

%% Grouping hooks [Hook]
%%
%% - hooks react only to calls on events returned by hook's `access_info' call;
%% - hooks are called on received events, calling order is undetermined,
%%   processing stops on first hook which returns result distinct from
%%   `undefined';
%% - component returns exec and stop if any hook returns them;
%% - if hook returns call_exec it handles all following events and calls
%%   before it returns result distinct from call_exec, no other hooks are
%%   called on this event;
%% - in other case component calls super with event and returns hooks output
%%   and result of super
%% 
behaviour_info(callbacks) ->
  [{init, 1}, {is_active, 3}, {access_info, 3},
   {handle_event, 5}, {handle_call, 4}].

create_comp(Comp, Hook, HookArg, Props) ->
  create_comp(Comp, {Hook, HookArg}, Props).

create_comp(C=#comp{render=R}, Hooks, Props) ->% Hooks = [Hook]|{HookFun, HookArg}
  SuperRender=wslib:mk_dummy_render([]),
  Super = C#comp{render=SuperRender},
  wsdcomp:create(?MODULE, Hooks, R, [{super, Super}|Props]).

access_info(_, _, S=#state{access_info=AI}) ->
  AI.

handle_event(Event, security_alert, WA, Srv, State, StateData) ->
  {stop, ok};
handle_event(Msg, old_request, WA, Srv, State, StateData) ->
  {next_state, ok, State, StateData};
handle_event(Msg, From, WA, Srv, State, StateData) ->
  do_hook_event(Msg, From, WA, Srv, StateData).

init(Hooks, Props) when is_list(Hooks) ->
  {Hook, HookArg} = mk_group_hook(Hooks),
  init({Hook, HookArg}, Props);
init({Hook, HookArg}, Props) ->
  Hook1 = mk_hook(Hook),
  TmplVar = wslib:gv(template_var, Props, ?template_var),
  {ok, HookData} = Hook1(init, HookArg),
  {ok,
   enter,
   #state{hook=Hook1, hdata=HookData, template_var=TmplVar},
   Props
  }.

enter(Event, WA, Srv, S) ->
  do_call_hook(Event, WA, Srv, S).

call_exec_return(Msg, WA, Srv, State) ->
  do_call_hook(Msg, WA, Srv, State).

do_call_hook(Event, WA, Srv, S=#state{hook=H, hdata=HD}) ->
  next_state(WA, Srv, return_call_hook({call, Event, WA, Srv}, H, HD, S)).

do_hook_event(Msg, From, WA, Srv, S=#state{hook=H, hdata=HD}) ->
  next_state(WA, Srv, return_call_hook({event, Msg, From, WA, Srv}, H, HD, S)).

return_call_hook(Msg, Hook, HookData, State) ->
  return_call_hook(Msg, Hook, HookData, [], State).

return_call_hook(Msg, Hook, HookData, PrevOut, State) ->
  {HookRet, NxtHookData} = call_hook(Msg, Hook, HookData),
  case HookRet of
    {ok, undefined, Out} ->
      NxtOut = Out++PrevOut,
      case Msg of
	{call, Event, WA, Srv} ->
	  SR = wsdcomp:super(Srv, Event),
	  return_call_hook({event, SR, super, WA, Srv},
			   Hook, NxtHookData, NxtOut, State);
	{event, Event, parent, WA, Srv} ->
	  SR = wsdcomp:super_any_state(Srv, Event),
	  return_call_hook({event, SR, super, WA, Srv},
			   Hook, NxtHookData, NxtOut, State);
	{event, Event, _, _, _} ->
	  fmt_hook_return({ok, Event, NxtOut}, NxtHookData, [], State)
      end;
    _ ->
      fmt_hook_return(HookRet, NxtHookData, PrevOut, State)
  end.

call_hook(Msg={event, _, _, WA, Srv}, Hook, HookData) ->
  call_hook(Msg, Hook, HookData, Hook({is_active, WA, Srv}, HookData));
call_hook(Msg={call, _, WA, Srv}, Hook, HookData) ->
  call_hook(Msg, Hook, HookData, Hook({is_active, WA, Srv}, HookData)).

call_hook(_Msg, _Hook, HookData, Active=false) ->
  ?dbg("call hook, hook inactive~nstate:~p", [HookData]),
  {{ok, undefined, []}, HookData};
call_hook(Msg, Hook, HookData, Active=true) ->
  ?dbg("call hook, hook active", []),
  Ret =
    case Hook(Msg, HookData) of
      {call_exec, ExecMsg, Comp, NxtHD} ->
	{{call_exec, ExecMsg, Comp}, NxtHD};
      {ok, A, NxtHD} ->
	{{ok, A, []}, NxtHD};
      {ok, A, Out, NxtHD} ->
	{{ok, A, Out}, NxtHD};
      {stop, A} ->
	{{stop, A, []}, HookData};
      Other when element(1, Other) == stop;
		 element(1, Other) == exec ->
	{Other, HookData};
      Other ->
	Other
    end,
  ?dbg("hook {return, state} ~p", [Ret]),
  Ret.

fmt_hook_return({call_exec, Msg, Comp}, NxtHD, _, S) ->
  {call_exec, Msg, Comp, call_exec_return, S#state{hdata=NxtHD}};
fmt_hook_return({ok, Ret, Out}, NxtHD, PrevOut, S) ->
  {next_state, Ret, Out++PrevOut, enter, S#state{hdata=NxtHD}};
fmt_hook_return({stop, Ret, Out},  _, PrevOut, _) ->
  {stop, Ret, Out++PrevOut};
fmt_hook_return(Exec, _, _, _) when element(1, Exec) == exec ->
  Exec.

next_state(WA, Srv,
	   {next_state, Ret, Out, State,
	    S=#state{template_var=TV, hook=H, hdata=HD}
	   }) ->
  SAI = wsdcomp:access_info(Srv, super),
  AI1 =
    case H({is_active, WA, Srv}, HD) of
      true ->
	H({access_info, WA, Srv}, HD);
      false ->
	#access_info{}
    end,
  AI = case AI1#access_info.default_event of
	 undefined ->
	   AI1#access_info{default_event=SAI#access_info.default_event};
	 _ ->
	   AI1
       end,
  Children = wsdcomp:children(Srv),
  {next_state, Ret,
   [{TV,
     [{actions, AI#access_info.events++SAI#access_info.events},
      {children, Children}
     ]}
    |Out],
   State,
   S#state{access_info=AI}};
next_state(_, _, Other) ->
  Other.

mk_hook(Module) when atom(Module) ->
  fun ({is_active, WA, Srv}, HD) ->
      Module:is_active(WA, Srv, HD);
      ({access_info, WA, Srv}, HD) ->
      Module:access_info(WA, Srv, HD);
      (init, Arg) ->
      Module:init(Arg);
      ({call, Event, WA, Srv}, HD) ->
      Module:handle_call(Event, WA, Srv, HD);
      ({event, Msg, From, WA, Srv}, HD) ->
      Module:handle_event(Msg, From, WA, Srv, HD)
  end;
mk_hook(Hook) ->
  Hook.

%% Hooks

%% Group hook
mk_group_hook(Hooks) ->
  Hooks1 = lists:map(fun ({H, HA}) ->
			 Hook = mk_hook(H),
			 {ok, HookData} = Hook(init, HA),
			 {Hook, HookData}
		     end, Hooks),
  {fun hook_dispatch/2, #dstate{hooks=Hooks1}}.

hook_dispatch(init, State) ->
  {ok, State};
hook_dispatch({is_active, _, _}, _) ->
  true;
hook_dispatch(E={access_info, WA, Srv}, S=#dstate{hooks=Hs}) ->
  Events =
    lists:flatmap(fun ({H, HD}) ->
		      case H({is_active, WA, Srv}, HD) of
			true ->
			  (H(E, HD))#access_info.events;
			false ->
			  []
		      end
		  end, Hs),
  #access_info{events=Events};
hook_dispatch(Event, S=#dstate{hooks=Hs, active_hook={H, HD}}) ->
  hook_dispatch_next_state(Event, H, HD, S);
hook_dispatch(Event={call, Msg, WA, Srv}, S=#dstate{hooks=Hooks}) ->
  ?dbg(
    "group hook:"
    "~ncall:~p"
    "~nstate:~p~n", [Msg, S]),
  hook_dispatch(Event, Hooks, [], []);
hook_dispatch(Event={event, Msg, From, WA, Srv}, S=#dstate{hooks=Hooks}) ->
  ?dbg(
    "group hook:"
    "~nevent:~p"
    "~nfrom:~p"
    "~nstate:~p~n", [Msg, From, S]),
  hook_dispatch(Event, Hooks, [], []).

hook_dispatch(Event, [], Hooks, Out) ->
  {ok,
   undefined,
   lists:append(lists:reverse(Out)),
   #dstate{hooks=lists:reverse(Hooks)}
  };
hook_dispatch(Event, R=[{H, HD}|Rest], Hooks, Out) ->
  {Res, NxtHD} = call_hook(Event, H, HD),
  case element(1, Res) of
    T when T == exec; T == stop ->
      Res;
    call_exec ->
      {Res, #dstate{hooks=lists:reverse(Hooks)++Rest, active_hook={H, NxtHD}}};
    ok when element(2, Res) =/= undefined ->
      {Res, #dstate{hooks=lists:reverse(Hooks)++R, active_hook=undefined}};
    _ ->
      case Res of
	{_, _, HookOut} ->
	  hook_dispatch(Event, Rest, [{H, NxtHD}|Hooks], [HookOut|Out]);
	_ ->
	  hook_dispatch(Event, Rest, [{H, NxtHD}|Hooks], Out)
      end
  end.

hook_dispatch_next_state(E, H, HD, S=#dstate{hooks=Hs}) ->
  {Ret, NxtHD} = call_hook(E, H, HD),
  case element(1, Ret) of
    T when T == exec; T == stop ->
      Ret;
    call_exec ->
      {Ret, S#dstate{active_hook={H, NxtHD}}};
    _ ->
      {Ret, S#dstate{hooks=[{H, NxtHD}|Hs], active_hook=undefined}}
  end.

