%%% File    : multiplex.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created :  4 Dec 2002 by  <svg@surnet.ru>

%%
%% $Id: multiplex.erl,v 1.2 2003/10/27 16:56:19 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wscomp.multiplex).

-behaviour(wsdcomp).

-export([init/2, access_info/3, handle_event/6]).
-export([go/4, enter/4]).
-export([create_subcomp/3]).

-import(.lists).
-import(.wsdcomp).
-import(.wslib).

-include("wstl.hrl").

%% -define(children_var, 'Children').

-record(state, {handler = fun dummy_hander/1,
		hdata,
		exit_mode=for_all, % for_all | for_one
		components=[]}).

create_subcomp(Name, Comp, EnterMsg) ->
  {Name, Comp, EnterMsg}.

access_info(_, _, _) ->
  #access_info{events=[do_exit], default_event=refresh}.

handle_event(Event, security_alert, WA, Srv, State, StateData) ->
  {stop, ok};
%% Old request, no data in output
handle_event(Msg, old_request, WA, Srv, enter, StateData) ->
  enter(enter, WA, Srv, StateData);
handle_event(Msg, old_request, WA, Srv, State, StateData) ->
  {next_state, ok, go, StateData};
handle_event(Event, From, WA, Srv, go,
	     S=#state{exit_mode=ExitMode, handler=H, hdata=HD}) ->
  case wsdcomp:children(Srv) of
    [] ->
      {stop, ok};
    Children ->
      case lists:member(From, Children) of
	true ->
	  {Ret, Out} = H({event, Event, WA, Srv, From}, HD),
	  IsAlive = wsdcomp:is_alive(Srv, From),
	  if IsAlive or (ExitMode == for_all) ->
	      lists:foreach(fun (Name) when Name == From ->
				ok;
				(Name) ->
				case wsdcomp:call(Srv, Name, refresh) of
				  {fatal, Error} ->
				    exit(Error);
				  _ ->
				    ok
				end
			    end, Children),
	      {next_state, Ret, Out, go, S};
	     true ->
	      {stop, Ret, Out}
	  end;
	false ->
	  {next_state, ok, go, S}
      end
  end.

init({Components, {Hlr, HlrArg}}, Props) ->
  ExitMode = wslib:gv(exit_mode, Props, for_all),
  {ok, HlrData} = Hlr(init, HlrArg),
  {ok, enter, #state{handler=Hlr, hdata=HlrData,
		     components=Components, exit_mode=ExitMode}};
init(Components, Props) ->
  init({Components, {fun dummy_handler/2, []}}, Props).

enter(Event, WA, Srv, S=#state{components=Cs,  handler=H, hdata=HD}) ->
  {Ret, Out} = H({call, Event, WA, Srv}, HD),
  lists:foreach(fun ({Name, Comp, EnterMsg}) ->
		    ok = wsdcomp:spawn_comp(Srv, Name, Comp),
		    case wsdcomp:call(Srv, Name, EnterMsg) of
		      {fatal, Error} ->
			exit(Error);
		      _ ->
			Name
		    end
		end, Cs),
  {next_state, Ret, Out, go, S#state{components=[]}}.

go(do_exit, WA, Srv, _) ->
  {stop, ok};
go(_, WA, Srv, State) ->
  {next_state, ok, go, State}.

dummy_handler(_, _) -> {ok, []}.
