%%% File    : wscomp.selector.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created :  4 Dec 2002 by  <svg@surnet.ru>

%%
%% $Id: selector.erl,v 1.2 2003/10/27 16:56:19 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wscomp.selector).

-behaviour(wsdcomp).

-export([init/2, access_info/3, handle_event/6]).
-export([enter/4, user_input/4]).
-export([create/5, create_selector/5]).

-import(wslib).
-import(wslib.str).
-import(wslib.html).
-import(wsform.ss).
-import(wsxmlt).
-import(wsdcomp).
-import(wsmessage).
-import(wsrun).
-import(wsform).
-import(stl_yaws).
-import(proplists).
-import(lists).
-import(error_logger).

-include("wstl.hrl").

-record(state, {value_field,
		get_values_hlr,
		on_select_hlr,
		hdata,
		label="",
		actions=[do_select, do_exit],
		selector_attrs=[]}).

create_selector(FieldName, KeyName, ValName, Rowset, Props) ->
  SelVals =
    [{str:to_str(wslib:gv(KeyName, Row)), str:to_str(wslib:gv(ValName, Row))}
     || Row <- Rowset],
  create(
    FieldName,
    fun (_, _) -> {ok, SelVals} end,
    mk_selector_return(KeyName, Rowset),
    [],
    Props).

mk_selector_return(KeyName, Rowset) ->
  fun (Event, Keys, _, _) ->
      KVs = 
	[R || R <- Rowset, lists:member(wslib:gv(KeyName, R), Keys) == true],
      {stop, {Event, KVs}}
  end.

%%
%% On success component selector returns Result
%% 
%% Props = [{actions, [do_select, do_exit]},
%%          {label, ""},
%%          {selector_attrs , []}]
%%
%% ValField = atom()
%% SelectValues(WA, State) -> {ok, Rows} | {ok, Rows, NxtState}
%% OnSelect(Event, Vs, WA, State) -> ok | {ok, NxtState} | {stop, Result}
%% Vs = [string()]
%% Event = atom()
%% SelectValues - returns [{K, V}] data for displaying selector
%% OnSelect     - returns result to return or next state to continue
%% 
create(ValField, SelectValues, OnSelect, State, Props) ->
  fun (component, WA) ->
      Comp = wsdcomp:create(?MODULE,
			    {ValField, SelectValues, OnSelect, State},
			    WA,
			    Props++[{template, "lib/selector"}]
			   ),
      Comp;
      (ehtml_element, WA) ->
      [Attrs] = wslib:gv_map([{selector_attrs, []}], Props),
      {ok, Opts, _} = select_values(SelectValues, State, WA),
      fun (Selected) ->
	  html:selector(ValField, Opts, Selected, Attrs)
      end;
      (wsform_ss, WA) ->
      {ok, Opts, _} = select_values(SelectValues, State, WA),
      wsform.ss:form_field(wsxmlt:m_name(ValField),
			   [wsform.ss:selector(Opts, Props)], WA)
  end.

%% component
access_info(_, _, S=#state{actions=As}) ->
  #access_info{events=As, default_event=refresh}.

handle_event(Event, security_alert, WA, Srv, State, StateData) ->
  {stop, ok};
handle_event(Event, From,  WA, Srv, State, StateData) ->
  {next_state, ok, State, StateData}.

init({ValField, SelectValues, OnSelect, State}, Props) ->
  [Actions, Label, SelAttrs] =
    wslib:gv_map([{actions        , [do_select, do_exit]},
                  {label         , ""},
                  {selector_attrs , []}
		 ],
		  Props),
  {ok, enter,
   #state{value_field    = ValField,   
	  get_values_hlr = SelectValues,  
	  on_select_hlr  = OnSelect,   
	  hdata          = State,
	  actions        = Actions, 
	  label         = Label,
	  selector_attrs  = SelAttrs
	 },
   Props
  }.

enter(_, WA, Srv, S) ->
  next_state(ok, WA, Srv, user_input, S).

user_input(Ev, WA, Srv,
	   S=#state{actions=As, value_field=VF, on_select_hlr=OS, hdata=HD}) ->
  case lists:member(Ev, As) of
    false ->
      next_state(ok, WA, Srv, user_input, S);
    true ->
      if Ev == do_exit ->
	  {stop, ok};
	 true ->
	  Vs =
	    [V || KV={K, V} <- wslib:arg_input(WA), K == VF, V =/= undefined],
	  {Next, Ret, NxtHD} =
	    case Vs of
	      [] ->
		{next_state, {error, no_record_selected}, HD};
	      _ ->
		case catch on_select(OS, Ev, Vs, HD, WA) of
		  {ok, HD1} ->
		    {next_state, {ok, {selected, length(Vs)}}, HD1};
		  {stop, Res} ->
		    {stop, Res, HD};
		  Error ->
		    error_logger:error_report([{module, ?MODULE}, Error]),
		    {next_state, Error, HD}
		end
	    end,
	  if Next == stop ->
	      {stop, Ret};
	     true ->
	      next_state(Ret, WA, Srv, user_input, S#state{hdata=NxtHD})
	  end
      end
  end.

next_state(Ret, WA, Srv, NxtState,
	   S=#state{get_values_hlr=HgVs, hdata=HD}) ->
  {ok, Rows, NxtHD} = select_values(HgVs, HD, WA),
  Out = out(Rows, S),
  {next_state, Ret, Out, NxtState, S#state{hdata=NxtHD}}.

select_values(SelectValues, HlrState, WA) ->
  case SelectValues(WA, HlrState) of
    {ok, Rows} ->
      {ok, Rows, HlrState};
    Ret={ok, Rows, NxtHlrState} ->
      Ret
  end.

on_select(OnSelect, Event, Vs, HlrState, WA) ->
  case OnSelect(Event, Vs, WA, HlrState) of
    ok ->
      {ok, HlrState};
    Ret={ok, NxtHlrState} ->
      Ret;
    Other ->
      Other
  end.
  
out(SelList, S=#state{value_field    = VF,
		      actions        = Actions, 
		      label         = Label,
		      selector_attrs  = SelAttrs
		     }) ->
  [{'ValueField', VF},
   {'SelList', SelList}, {'SelAttrs', SelAttrs},
   {'Label', Label}, {'Actions', Actions}].
