%%% File    : wscomp.switchwm.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created :  4 Dec 2002 by  <svg@surnet.ru>

%%
%% $Id: switchwm.erl,v 1.2 2003/10/27 16:56:19 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wscomp.switchwm).

-behaviour(wsdcomp).

-export([add_child/5, delete_children/3]).
%%
-export([init/2, access_info/3, handle_event/6]).
-export([enter/4, go/4]).

-import(.wsdcomp).
-import(.wslib).
-import(.wslib.str).
-import(.wsmessage).
-import(.stl_yaws).
-import(.proplists).
-import(.lists).
-import(.string).

-include("wstl.hrl").

-define(default_template, "lib/switchwm").
-define(empty_comp_str, atom_to_list(?empty_comp)).
-define(current_template_var, 'CurrentOut').
-define(state_template_var, 'SwitchwmState').

-record(state, {current_comp, show_tab=true, comps=[], max_comps=10}).

add_child(Srv, SwitchName, ChildName, ChildComp, ChildLabel) ->
  wsdcomp:call_any_state(Srv, SwitchName,
			 {add_child, ChildComp, ChildName, ChildLabel}).

delete_children(Srv, SwitchName, ChildrenNames) ->
  wsdcomp:call_any_state(Srv, SwitchName, {delete_children, ChildrenNames}).

access_info(_, _, _) ->
  #access_info{events=[do_switch, do_run, do_kill, do_exit],
	       default_event=refresh}.

handle_event(Event, security_alert, WA, Srv, State, StateData) ->
  {stop, ok};
handle_event({add_child, Comp, CompName, CompLabel},
	     parent,  WA, Srv, State, S=#state{comps=Cs}) ->
  Children =  wsdcomp:children(Srv),
  case is_max_children(length(Children), WA, S) of
    true ->
      next_state({error, max_children_limit}, State, S);
    false ->
      case lists:member(CompName, Children) of
	true ->
	  do_switch(CompName, Srv, S);
	false ->
	  Comp1 = wsdcomp:render_props([{place_at, 'CurrentOut'}], Comp),
	  ok =wsdcomp:spawn_comp(Srv, CompName, Comp1),
	  wsdcomp:call(Srv, CompName, enter),
	  NxtS = set_current(CompName, Srv, S),
	  next_state(ok, go, NxtS#state{comps=[{CompName, CompLabel}|Cs]})
      end
  end;
handle_event({delete_children, ChildrenNames}, parent, WA, Srv, State, S) ->
  CNs = case ChildrenNames of
	  all ->
	    wsdcomp:children(Srv);
	  current ->
	    [S#state.current_comp];
	  _ ->
	    ChildrenNames
	end,
  lists:foreach(fun (CN) -> wsdcomp:kill(Srv, CN) end, CNs),
  NxtS = set_current(Srv, S),
  next_state(ok, go, NxtS);
handle_event(_Event, From,  WA, Srv, State, S) ->
  NxtS = set_current(From, Srv, S),
  next_state(ok, State, NxtS).

init(WA, Props) ->
  [MaxComps, ShowTab] =
    wslib:gv_map([{max_comps, 10}, {show_tab, true}], Props),
  {ok, enter, #state{max_comps=MaxComps, show_tab=ShowTab},
   Props++[{template, ?default_template}]}.

enter(_, WA, Srv, S) ->
  next_state(ok, go, S).
			    
go(do_run, WA=#ws_arg{app_env=AE}, Srv, S=#state{comps=Cs}) ->
  Children =  wsdcomp:children(Srv),
  case is_max_children(length(Children), WA, S) of
    true ->
      next_state(ok, go, S);
    false ->
      Inp = wslib:arg_input(WA),
      CompModuleStr = wslib:gv(do_run, Inp, ?empty_comp_str),
      CompModule =
	list_to_atom(wslib.str:join(string:tokens(CompModuleStr, "/"), "_")),
      CompTemplate = wslib:gv(comp_template, Inp, CompModuleStr),
      CompName = wslib:gv(comp_name, Inp, CompModuleStr),
      CompLabel = wslib:gv(comp_label, Inp, CompModuleStr),
      case lists:member(CompName, Children) of
	true ->
	  do_switch(CompName, Srv, S);
	false ->
	  CompProps = [{render_as, screen},
		       {place_at, ?current_template_var},
		       {template, CompTemplate}
		      ],
	  Comp = wsdcomp:create(CompModule, WA, WA, CompProps),
	  ok = wsdcomp:spawn_comp(Srv, CompName, Comp),
	  wsdcomp:call(Srv, CompName, enter),
	  NxtS = set_current(CompName, Srv, S),
	  next_state(ok, go, NxtS#state{comps=[{CompName, CompLabel}|Cs]})
      end
  end;
go(do_switch, WA, Srv, S) ->
  Inp = wslib:arg_input(WA),
  CompName = wslib:gv(do_switch, Inp),
  do_switch(CompName, Srv, S);
go(do_kill, WA, Srv, S=#state{current_comp=C}) ->
  Inp = wslib:arg_input(WA),
  CompName = wslib:gv(comp_name, Inp, C),
  wsdcomp:kill(Srv, CompName),
  NxtS = set_current(Srv, S),
  next_state(ok, go, NxtS);
go(do_exit, WA, Srv, S) ->
  {stop, ok};
go(_, WA, Srv, S) ->
  next_state(ok, go, S).

do_switch(undefined, Srv, State) ->
  {next_state, ok, go, State};
do_switch(Name, Srv, S) ->
  NxtS = set_current(Name, Srv, S),
  next_state(ok, go, NxtS).

next_state(Ret, State, S=#state{current_comp=CC, comps=Cs, show_tab=ST}) ->
  StateOut = [{?state_template_var,
	       [{current_comp, CC}, {comps, Cs}, {show_tab, ST}]}],
  {next_state, Ret, StateOut, State, S}.

set_current(Srv, S) ->      
  case children(Srv) of
    Cs=[Top|Rest] ->
      set_current(Top, Srv, S, Cs);
    [] ->
      set_current(S#state.current_comp, Srv, S, [])
  end.
  
set_current(Name, Srv, S) ->
  set_current(Name, Srv, S, children(Srv)).

set_current(Name, Srv, S=#state{current_comp=CC, comps=Cs}, Comps) ->
  IsDead = not lists:member(Name, Comps),
  NxtCs = [KV || KV={K, _} <- Cs,  lists:member(K, Comps) == true],
  NxtCC =
    if IsDead ->
	wsdcomp:display(Srv, [Name], false),
	case Comps of
	  [Top|_] ->
	    Top;
	  [] ->
	    undefined
	end;
       true ->
	Name
    end,
  case {NxtCC, CC} of
    {undefined, _} ->
      ok;
    {_, undefined} ->
      wsdcomp:display(Srv, [NxtCC], true);
    {CC, _} ->
      ok;
    _ ->
      wsdcomp:display(Srv, [NxtCC], true),
      wsdcomp:display(Srv, [CC], false)
  end,
  S#state{current_comp=NxtCC, comps=NxtCs}.

children(Srv) ->
  [C || C <- wsdcomp:children(Srv), wsdcomp:is_alive(Srv, C) == true].

is_max_children(ChildrenLen, WA,  S=#state{max_comps=MC})
  when ChildrenLen >= MC ->
  wslib:show_lerror('MAX_NUMBER_EXCEEDED', [{num, MC}], WA),
  true;
is_max_children(_, _, _) ->
  false.
