%%% File    : wsdb.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created :  7 May 2003 by  <svg@surnet.ru>

%%
%% $Id: wsdb.erl,v 1.4 2003/10/31 17:57:31 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wsdb).



-export([dbh/1, handler/1, init_run_env/2, transaction/2, transaction/3, do/3]).
-export([select_rows/3, select_rows/4, update_row/3, insert_row/3, delete_row/3, read_row/3]).
-export([next_rowid/2, current_rowid/2]).

-include("wstl.hrl").

-define(wsdb_key, wsdb_handler).

%% if
init_run_env(Dbh, _) ->
  [{wsdb_handler, Dbh}].

dbh(WA=#ws_arg{app_env=AE}) ->
  dbh(AE);
dbh(Props) ->
  wslib:gv(wsdb_handler, Props).

handler(H) ->
  Dbh = get_dbh(H),
  Dbh(handler, []).

transaction(H, Fun) ->
  transaction(H, Fun, infinity).

transaction(H, Fun, Timeout) ->
  Dbh=get_dbh(H),
  put_key(Dbh),
  Res = (catch {ok, Dbh(transaction, [Fun, Timeout])}),
  erase_key(),
  case Res of
    {ok, Ret} ->
      Ret;
    {'EXIT', Error} ->
      exit(Error)
  end.

do(H, Fun, Args) ->
  Dbh = get_dbh(H),
  Dbh(do, [Fun, Args]).

select_rows(H, Table, Where) -> 
  Dbh = get_dbh(H),
  Dbh(select_rows, [Table, Where]).

select_rows(H, Table, Where, Rest) ->
  Dbh = get_dbh(H),
  Dbh(select_rows, [Table, Where, Rest]).

read_row(H, Table, Row) ->
  Dbh = get_dbh(H),
  Dbh(read_row, [Table, Row]).

insert_row(H, Table, Row) ->
  Dbh = get_dbh(H),
  Dbh(insert_row, [Table, Row]).

update_row(H, Table, Row) ->
  Dbh = get_dbh(H),
  Dbh(update_row, [Table, Row]).

delete_row(H, Table, Row) ->
  Dbh = get_dbh(H),
  Dbh(delete_row, [Table, Row]).

next_rowid(H, Table) ->
  Dbh = get_dbh(H),
  Dbh(next_rowid, [Table]).

current_rowid(H, Table) ->
  Dbh = get_dbh(H),
  Dbh(current_rowid, [Table]).

get_dbh(H) ->
  case get_key() of
    undefined ->
      H;
    {Dbh, _} ->
      Dbh
  end.

get_key() ->
  get(?wsdb_key).

put_key(H) ->
  case get_key() of
    undefined ->
      put(?wsdb_key, {H, 1});
    {Dbh, C} ->
      put(?wsdb_key, {Dbh, C+1})
  end.

erase_key() ->
  case get_key() of
    undefined ->
      ok;
    {Dbh, 1} ->
      erase(?wsdb_key);
    {Dbh, C} ->
      put(?wsdb_key, {Dbh, C-1})
  end,
  ok.
