%%% File    : wsdb_odbc.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created :  7 May 2003 by  <svg@surnet.ru>

%%
%% $Id: wsdb_odbc.erl,v 1.6 2003/10/31 17:57:31 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*

-module(wsdb_odbc).

-export([wsdb_handler/1, dbh/1, do/2, transaction/2, transaction/3]).
-export([update_row/3, insert_row/3, delete_row/3, read_row/3]).
-export([select_rows/3, select_rows/4, fetch_row/3, select/3, select/5]).
-export([do_query/3, do_query/4, mk_query/2, mk_select_query/2, group_by/2]).
-export([current_rowid/2, next_rowid/2]).

wsdb_handler(Arg) ->
  Dbh = dbh(Arg),
  fun (handler, _) ->
      Dbh;
      (do, [FunName, Args]) when is_atom(FunName) ->
      apply(?MODULE, FunName, [Dbh|Args]); 
      (do, [Fun, Args]) ->
      apply(Fun, [Dbh|Args]); 
      (FunName, Args) ->
      apply(?MODULE, FunName, [Dbh|Args])
  end.

dbh(_) ->
  Dbh =
    fun (Fun) ->
	{ok, Conn} = odbcdb:connect(),
	Res = (catch {ok, Fun(Conn)}),
	odbcdb:disconnect(Conn),
	case Res of
	  {ok, Ret} ->
	    Ret;
	  {'EXIT', Error} ->
	    exit(Error)
	end
    end.

select(Dbh, Query, Params) ->
  select(Dbh, Query, Params, tuple, infinity).

select(Dbh, Query, Params, ResultAs,  Timeout) ->
  Fun = fun (Db) -> odbcdb:do(Db, Query, Params, ResultAs,  Timeout) end,
  case do(Dbh, Fun) of
    Rows when is_list(Rows) ->
      Rows;
    Error ->
      exit(Error)
  end.

group_by(Query, GroupBy) ->
  odbcdb:group_by(Query, GroupBy).

mk_select_query(Table, Where) ->
  odbcdb:mk_select_query(Table, Where).

mk_query(QueryName, Where) ->
  odbcdb:mk_query(QueryName, Where).

do_query(Dbh, Query, Params) ->
  do_query(Dbh, Query, Params, infinity).

do_query(Dbh, Query, Params, Timeout) ->
  Fun = fun (Db) -> odbcdb:do(Db, Query, Params) end,
  transaction(Dbh, Fun, Timeout).

current_rowid(Dbh, Table) ->
  do(Dbh, fun (Db) -> odbcdb:current_rowid(Db, Table) end).

next_rowid(Dbh, Table) ->
  do(Dbh, fun (Db) -> odbcdb:next_rowid(Db, Table) end).

select_rows(Db, Table, Where) ->
  select_rows(Db, Table, Where, []).

select_rows(Dbh, Table, Where, GroupBy) ->
  Fun = fun (Db) -> odbcdb:select_rows(Db, Table, Where, GroupBy) end,
  case do(Dbh, Fun) of
    Rows when is_list(Rows) ->
      Rows;
    Error ->
      exit(Error)
  end.

read_row(Dbh, Table, Row) ->
  Fun = fun (Db) -> odbcdb:read_row(Db, Table, Row) end,
  do(Dbh, Fun).

insert_row(Dbh, Table, Row) ->
  Fun = fun (Db) -> odbcdb:insert_row(Db, Table, Row) end,
  transaction(Dbh, Fun).

update_row(Dbh, Table, Row) ->
  Fun = fun (Db) -> odbcdb:update_row(Db, Table, Row) end,
  transaction(Dbh, Fun).

delete_row(Dbh, Table, Row) ->
  Fun = fun (Db) -> odbcdb:delete_row(Db, Table, Row) end,
  transaction(Dbh, Fun).

fetch_row(Dbh, Query, QueryParams) ->
  Fun = fun (Db) -> odbcdb:fetch_row(Db, Query, QueryParams) end,
  do(Dbh, Fun).

transaction(Dbh, Fun) ->
  transaction(Dbh, Fun, infinity).

transaction(Dbh, Fun, Timeout) ->
  Fun1 =
    %% avoid nested transactions, only top level do real commit/rollback
    case odbcdb:is_connection(Dbh) of
      true ->
	Fun;
      false ->
	fun (Db) -> odbcdb:transaction(Db, Fun, [Db], Timeout) end
    end,
  do(Dbh, Fun1).

do(Dbh, Fun) ->
  Res =
    case odbcdb:is_connection(Dbh) of
      true ->
	Fun(Dbh);
      false ->
	Dbh(Fun)
    end,
  case Res of
    {updated, Num} ->
      {ok, Num};
    {selected, Rows} ->
      Rows;
    {selected, _, Rows} ->
      Rows;
    _ ->
      Res
  end.

