%%
%% $Id: wsdb_phrases.erl,v 1.7 2003/10/31 17:57:31 vsekissov Exp $
%%
%%**
%% 
%% @doc SQL queries for <code>phrase</code> server.
%% @end
%%*
-module(wsdb_phrases).

-export([all/0]).

all() ->
  {[%% party
    {'PARTY_BY_TYPE_AND_ROLE',
     "select * from party as P"
     " where party_type_id=~s and"
     " exists (select 'x' from party_role"
     "        where role_type_id=~s and party_id=P.id and ~s)",
     [party_type_id, role_type_id, sql_where]},
    {'PARTY_BY_TYPE_RELATIONSHIP',
     "select * from party as P"
     " where party_type_id=~s and"
     " exists (select 'x' from party_relationship"
     "        where party_relationship_type_id=~s and"
     "              party_id_from=P.id and party_id_to=~s)"
     " order by id",
     [party_type_id, party_relationship_type_id, party_id_to]},
    {'PARTY_BY_RELATIONSHIP_CATEGORY',
     "select * from party as P"
     " where"
     " exists"
     " (select 'x'"
     " from party_relationship"
     " where"
     " party_id_~s=P.id"
     " and party_relationship_type_id"
     "       in (select party_relationship_type_id"
     "           from party_relationship_type"
     "           where category_id=~s)"
     " )"
     " and ~s",
     [direction, category_id, sql_where],
     [{A, odbcdb_schema:attribute_property({party_relationship, A}, type)}
      || A <- odbcdb_schema:attributes(party_relationship)]
    },
    %% person
    %% security_group
    %% security_permission
    %% security_group_permission
    {'SECURITY_PERMISSION_NOT_IN_GROUP',
"select * from security_permission
where id not in (select permission_id from security_group_permission
                 where group_id = ~s)
order by id", [group_id]},
    {'SECURITY_PERMISSION_IN_GROUP',
"select * from security_permission
where id in (select permission_id from security_group_permission
              where group_id = ~s)
order by id", [group_id]},
    %% user_login
    {'INC_USER_LOGIN_COUNTER',
     "update user_login set login_attempts = login_attempts+1 where login_id=~s",
     [login_id]},
    {'SET_USER_LOGIN_PASSWORD',
     "update user_login set password=~s where login_id=~s",
     [password, login_id]},
     {'UPDATE_USER_LOGIN',
     "update user_login
set party_id=~s, disabled_datetime=~s, enabled=~s
where login_id=~s",
[party_id, disabled_datetime, enabled, login_id]},
    %% user_login_security_group
    {'SECURITY_GROUP_NOT_IN_USER_LOGIN',
"select * from security_group
where id not in (select group_id from user_login_security_group
                 where login_id = ~s)
order by id", [login_id]},
    {'SECURITY_GROUP_IN_USER_LOGIN',
"select * from security_group
where id in (select group_id from user_login_security_group
              where login_id = ~s)
order by id", [login_id]},
    %% party relationship type
    {'PARTY_RELATIONSHIP_TYPE_BY_PARTY',
"select * from party_relationship_type"
" where role_type_id_to_valid in"
" (select role_type_id from party_role where party_id=~s) "
" order by id", [party_id]},
    {'PARTY_VALID_RELATIONSHIP_FROM_CATEGORY_IDS',
"select distinct category_id from party_relationship_type"
" where role_type_id_from_valid in"
"       (select role_type_id from party_role where party_id=~s)"
" order by category_id",
     [party_id]},
    %% party relationship
    %% party role
    {'PARTY_ROLE',
"select RT.*
from party_role as PR, role_type as RT
where PR.party_id=~s and PR.role_type_id = RT.id order by RT.id",
[party_id]},
    {'GET_PARTY_ROLE',
"select * from party_role where party_id=~s and role_type_id=~s",
[party_id, role_type_id]},
    {'DELETE_PARTY_ROLE',
"delete from party_role where party_id=~s and role_type_id=~s",
[party_id, role_type_id]},
    {'INSERT_PARTY_ROLE',
"insert into party_role(party_id, role_type_id) values(~s, ~s)",
[party_id, role_type_id]},
    {'PARTY_ROLE_NOT_IN_PARTY',
"select * from role_type
where id not in (select role_type_id from party_role where party_id=~s)
order by id", [party_id]},
%% party_relationship_category
%% direction = {noquote, from|to}
{'PARTY_RELATIONSHIP_CATEGORY_BY_PARTY',
"select * from party_relationship_category"
" where id in"
" (select category_id from party_relationship_type"
"  where role_type_id_~s_valid in"
"  (select role_type_id from party_role where party_id=~s))"
" order by id", [direction, party_id]},
%% postal address
%% party location
{'PARTY_LOCATION_BY_PARTY',
"select PL.*,"
" (PA.city || ',' || PA.address1 || ',' || PA.address2) as postal_address_name"
" from party_location as PL, postal_address as PA"
" where PL.party_id=~s"
" and PA.id = PL.postal_address_id"
" order by PL.location_role_type_id, PL.thru_date",
[party_id]},
{'GET_PARTY_LOCATION',
"select PL.*,"
" (PA.city || ',' || PA.address1 || ',' || PA.address2) as postal_address_name"
" from party_location as PL, postal_address as PA"
" where PL.party_id=~s"
" and PL.location_role_type_id=~s"
" and PL.from_date=~s"
" and PA.id = PL.postal_address_id",
[party_id, location_role_type_id, from_date]},
{'INSERT_PARTY_LOCATION',
"insert into party_location(party_id, postal_address_id, from_date, thru_date, location_role_type_id)"
" values(~s, ~s, ~s, ~s, ~s)",
[party_id, postal_address_id, from_date, thru_date, location_role_type_id]},
{'UPDATE_PARTY_LOCATION',
"update party_location set thru_date=~s, postal_address_id=~s"
" where party_id=~s"
" and location_role_type_id=~s"
" and from_date=~s",
[thru_date, postal_address_id, party_id, location_role_type_id, from_date]},
{'DELETE_PARTY_LOCATION',
"delete from party_location"
" where party_id=~s"
" and location_role_type_id=~s"
" and from_date=~s",
[party_id, location_role_type_id, from_date]},
{'AAA_SERVICE',
"select
party_relationship_service.*,party_relationship.id as party_relationship_id
from party_relationship_service, party_relationship
where (~s)
and party_relationship.party_id_from=party_relationship_service.party_id_from
and party_relationship.party_id_to=party_relationship_service.party_id_to
and party_relationship.role_type_id_from=party_relationship_service.role_type_id_from
and party_relationship.role_type_id_to=party_relationship_service.role_type_id_to
and party_relationship.from_date=party_relationship_service.from_date",
[sql_where],
[{A, odbcdb_schema:attribute_property({party_relationship_service, A}, type)}
      || A <- odbcdb_schema:attributes(party_relationship_service)]
}
],
%% contact means type
%% contact means
   fun () -> end_of_input end
  }.
