%%% File    : wsdb_schema.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created :  7 May 2003 by  <svg@surnet.ru>

%%
%% $Id: wsdb_schema.erl,v 1.1 2003/05/08 22:44:29 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wsdb_schema).

-export([install/0, install/1]).

-include("wsdb.hrl").

-import(wslib.mnesia_dbi).

install () ->
  io:format("~p", [(catch install([node()]))]).

install(Nodes) ->
  mnesia_dbi:schema_create(Nodes, database_schema()),
  mnesia_dbi:init_sequences(Nodes, sequences()),
  mnesia_dbi:create_constraints(database_constraints()),
  ok.

database_schema() ->
  [?table_def(party, [{type, set}]),
   ?table_def(person, [{type, set}]),
   ?table_def(security_permission, [{type, set}]),
   ?table_def(security_group, [{type, set}]),
   ?table_def(user_login, [{type, set}])
  ].

sequences() ->
  [{party, 0}].

database_constraints() ->
  %% party
  [
   {attr, {party, id}, [{type, integer}, {required, true}]},
   {attr, {party, party_type_id}, [{type, text}, {default, ""}]},
   %% person
   {attr, {person, party_id}, [{type, integer} , {required, true}]},
   {attr, {person, last_name}, [{type, text}, {default, ""}]},
   {attr, {person, first_name}, [{type, text}, {default, ""}]},
   {attr, {person, middle_name}, [{type, text}, {default, ""}]},
   {attr, {person, personal_title}, [{type, text}, {default, ""}]},
   {attr, {person, gender}, [{type, text}, {default, ""}]},
   {attr, {person, comments}, [{type, text}, {default, ""}]},
   %% security_permission
   {attr, {security_permission, id}, [{type, text}, {required, true}]},
   {attr, {security_permission, comment}, [{type, text}, {default, ""}]},
   %% security_group
   {attr, {security_group, id}, [{type, text}, {required, true}]},
   {attr, {security_group, comment}, [{type, text}, {default, ""}]},
   {attr, {security_group, permissions}, [{type, list}, {default, []}]},
   %% user_login
   {attr, {user_login, login_id}, [{type, text}, {required, true}]},
   {attr, {user_login, password}, [{type, text}, {required, true}]},
   {attr, {user_login, password_hint}, [{type, text}, {default, ""}]},
   {attr, {user_login, party_id}, [{type, integer}]},
   {attr, {user_login, disabled_datetime}, [timestamp]},
   {attr, {user_login, enabled}, [{type, fun is_bool/1}, {default, false}]},
   {attr, {user_login, login_attempts}, [{type, integer}, {default, 0}]},
   {attr, {user_login, sec_groups}, [{type, list}, {default, []}]},

   %% integrity constraints

   %% party
   {attr, {party, id},
    [{references,
      [{person, party_id,
	[{delete, cascade}, {update, cascade}]}],
      [{user_login, party_id,
	[{delete, cascade}, {update, cascade}]}]
     }]}
  ].

is_bool(true) -> true;
is_bool(false) -> true;
is_bool(_) -> false.

