%%%-------------------------------------------------------------------
%%% File    : wsdcomp.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%%
%%% Created : 26 Mar 2003 by  <svg@surnet.ru>
%%%-------------------------------------------------------------------
%%
%% $Id: wsdcomp.erl,v 1.6 2003/10/31 17:57:31 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wsdcomp).

-export([behaviour_info/1]).
-export([create/4, call/3, call_any_state/3, spawn_comp/3, children/1, name/1, path/1, is_alive/2, display/3, access_info/2, kill/2, super/2, super_any_state/2]).
-export([render_props/2, create_dcomp/3]).
-export([security_alert/1]).

-include("wstl.hrl").

behaviour_info(callbacks) ->
  [{init, 2}, {access_info, 3}, {handle_event, 6}].

-define(access_info_msg(WA), {call, '_$access_info$_', WA}).
-define(super, "__super__").
-define(kill, '_$kill$_').
-define(access_info, '_$access_info$_').
-define(killed, '_$killed$_').

-record(state, {handler,
		hstate_name,
		hdata,
		hpid,
		access_id,
		secure=true,
		super
	       }).

create(Module, DcompArg, RenderArg, Props) ->
  ?dbg("create ~p", [Module]),
  case create_dcomp1(Module, DcompArg, Props) of
    Comp when is_record(Comp, comp) ->
      render_props(Props, Comp);
    Other ->
      {Dcomp, RenderProps} =
	case Other of
	  {ok, D, RP} ->
	    {D, RP};
	  D ->
	    {D, Props}
	end,
      Render = create_render(Module, RenderArg, RenderProps),
      Comp=#comp{dcomp=Dcomp, render=Render}
  end.

render_props(Props, C=#comp{render=R}) ->
  C#comp{render=stl_yaws:render_props(Props, R)}.

create_render(_Module, Render, Props) when is_function(Render) ->
  Render;
create_render(_Module, RenderArg, Props) ->
  Render = stl_yaws:create(RenderArg, Props).

create_dcomp(Module, HlrArg, Props) ->
  case create_dcomp1(Module, HlrArg, Props) of
    {ok, Dcomp, _} ->
      Dcomp;
    Comp when is_record(Comp, comp) ->
      Comp#comp.dcomp;
    Other ->
      Other
  end.

spawn_comp(Srv, CompName, Comp) ->
  Self = self(),
  Srv ! {Self, {spawn, CompName, Comp}},
  receive
    {Srv, {answer, CompName, Ret}} ->
      Ret
  end.

children(Srv) ->
  Self = self(),
  Srv ! {Self, children},
  receive
    {Srv, {answer, children, Names}} ->
      Names
  end.

name(Srv) ->
  Self = self(),
  Srv ! {Self, name},
  receive
    {Srv, {answer, name, Name}} ->
      Name
  end.

path(Srv) ->
  Self = self(),
  Srv ! {Self, path},
  receive
    {Srv, {answer, path, Path}} ->
      Path
  end.

is_alive(Srv, super) ->
  is_alive(Srv, ?super);
is_alive(Srv, Name) ->
  Self = self(),
  Srv ! {Self, {is_alive, Name}},
  receive
    {Srv, {answer, is_alive, Ret}} ->
      Ret
  end.

display(Srv, Names, NeedDisplay) ->
  Names1 = lists:map(fun (super) -> ?super;
			 (N) -> N
		     end, Names),
  Self = self(),
  Srv ! {Self, {display, Names1, NeedDisplay}},
  receive
    {Srv, {answer, display, ok}} ->
      ok
  end.

call(Srv, super, Msg) ->
  call(Srv, ?super, Msg);
call(Srv, CompName, Msg) ->
  Self = self(),
  Srv ! {Self, {call, CompName, Msg}},
  receive
    {Srv, {answer, CompName, Answer}} ->
      Answer;
    {Srv, Error={error, _}} ->
      Error
  end.

call_any_state(Srv, CompName, Msg) ->
  call(Srv, CompName, {any_state_event, Msg}).

super(Srv, Msg) ->
  call(Srv, ?super, Msg).

super_any_state(Srv, Msg) ->
  call_any_state(Srv, ?super, Msg).

access_info(Srv, super) ->
  access_info(Srv, ?super);
access_info(Srv, CompName) ->
  call(Srv, CompName, ?access_info).

kill(Srv, CompName) ->
  call(Srv, CompName, ?kill),
  ok.

create_dcomp1(Module, HlrArg, Props) ->
  InitRes =
    case Module:init(HlrArg, Props) of
      {ok, HS, HD} ->
	{ok, HS, HD, Props};
      {ok, HS, HD, CPs} ->
	{ok, HS, HD, Props++CPs};
      {component, CompModule, CompArg} ->
	create_dcomp1(CompModule, CompArg, Props);
      {component, Comp} ->
	Comp
    end,
  case InitRes of
    {ok, HlrState, HlrData, CompProps} ->
      [Secure, Super, AccessId] =
	wslib:gv_map([{secure, true}, super, access_id], CompProps),
      Super1 = if Super == undefined ->
		   Super;
		  true ->
		   render_props([{place_at, 'Super'}], Super)
	       end,
      State = #state{handler=Module,
		     hstate_name=HlrState,
		     hdata=HlrData,
		     secure=Secure,
		     access_id=AccessId,
		     super=Super1
		    },
      {ok, mk_dcomp({fun start/3, State}), CompProps};
    Other ->
      Other
  end.

mk_dcomp({State, StateData}) ->
  ?dbg("instrument dcomp~n~p", [StateData]),
  fun (Msg) ->
      fun (Cont) ->
	  State(Msg,
		fun (Res) ->
		    Cont(fmt_dcomp_result(Res))
		end,
		StateData)
      end
  end.

start(Msg, Cont, S=#state{super=Super}) when is_record(Super, comp) ->
  Nxt = next_state(fun spawn_super/3, {Msg, S}),
  Cont({spawn, ?super, Super, Nxt});
start(?access_info_msg(WA), Cont, State) ->
  {{AI=#access_info{events=Es}, SAI=#access_info{events=SEs}},NxtCont} =
    self_access_info(WA, Cont, State),
  Es1 = Es++[E || E <- SEs, lists:member(E, Es) == false],
  Nxt = next_state(fun start/3, State),
  NxtCont({reply, AI#access_info{events=Es1}, Nxt});
start({call, ?kill, WA}, Cont, S) ->
  Cont({stop, ?killed});
start({answer, From, ?killed, WA}, Cont, S) ->
  start({answer, From, killed, WA}, Cont, S);
start(Msg, Cont, State) ->
  call_comp(Msg, Cont, State).

call_comp({call, UR={user_req, Path}, WA}, Cont, S) ->
  {AI, NxtCont} = self_access_info(WA, Cont, S),
  case select_event(WA, AI) of
    {self, Event} ->
      call_comp_authorize_req(Event, Path, WA, NxtCont, S);
    {super, Event} ->
      NxtCont({{call, ?super, UR}, next_state(fun super_return/3, S)})
  end;
call_comp({call, {unknown_call, {user_req, _}, Path}, WA}, Cont, S) ->
  {{#access_info{default_event=D}, _}, NxtCont} =
    self_access_info(WA, Cont, S),
  call_comp_authorize_req(D, Path, WA, NxtCont, S);
call_comp(Msg, Cont, S) ->
  call_comp0(Msg, Cont, S).

call_comp_authorize_req(Event, Path, WA, Cont,
			S=#state{secure=Secure, access_id=AId}) ->
  if Secure ->
      case authorize_req(WA, Event, Path, AId) of
	permit ->
	  call_check_old_req(Event, WA, Cont, S);
	deny ->
	  call_comp0({call, security_alert(Event), WA}, Cont, S)
      end;
     true ->
      call_check_old_req(Event, WA, Cont, S)
  end.

call_check_old_req(Event, WA=#ws_arg{cont_num=CN, url_cont_num=UCN}, Cont, S)
  when  UCN < CN -> % Old request
  call_comp0({call, {old_request, Event}, WA}, Cont, S);
call_check_old_req(Event, WA, Cont, S) ->
  call_comp0({call, Event, WA}, Cont, S).

call_comp0(Msg, Cont,
	   S=#state{hpid=undefined, handler=H, hstate_name=SN, hdata=HD}) ->
  Self = self(),
  Pid = spawn_link(fun () -> hlr_loop(Self, H, SN, HD) end),
  call_comp0(Msg, Cont, S#state{hpid=Pid});
call_comp0(Msg, Cont, S=#state{hpid=Pid, handler=H, hstate_name=HS}) ->
  Self = self(),
  Pid ! {Self, Msg},
  receive
    {Pid, Answer} ->
      Ret1 =
	case Answer of
	  {reply, Reply, Out, NxtState, NxtData} ->
	    NxtS = S#state{hstate_name=NxtState, hdata=NxtData, hpid=undefined},
	    Nxt = next_state(fun start/3, NxtS),
	    {reply, Reply, Out, Nxt};
	  {reply, Reply, NxtState, NxtData} ->
	    NxtS = S#state{hstate_name=NxtState, hdata=NxtData, hpid=undefined},
	    Nxt = next_state(fun start/3, NxtS),
	    {reply, Reply, Nxt};
	  Stop={stop, _, _} ->
	    Stop;
	  Stop={stop, _} ->
	    Stop;
	  {spawn, CompName, DataComp} ->
	    Nxt = next_state(fun spawn_result/3, S),
	    {spawn, CompName, DataComp, Nxt};
	  {call_exec, Exec, ExecMsg, NxtState, NxtData} ->
	    NxtS = S#state{hstate_name=NxtState, hdata=NxtData, hpid=undefined},
	    Nxt = next_state(fun call_exec_answer/3, NxtS),
	    {call_exec, Exec, ExecMsg, Nxt};
	  Exec when element(1, Exec) == exec ->
	    Exec;
	  %% Other =
	  %% {call, CompName, CompMsg}
	  %% {is_alive, CompName}}
	  %% {display, Names, NeedDisplay}
	  Other ->
	    Nxt = next_state(fun call_answer/3, S),
	    {Other, Nxt}
	end,
      {IsAlive, NxtCont} = self_is_alive(Cont, S),
      Ret =
	if IsAlive ->
	    Ret1;
	 true ->
	    case Ret1 of
	      {reply, R, O, _} ->
		{stop, R, O};
	      {reply, R, _} ->
		{stop, R};
	      {call_exec, Exec1, ExecMsg1, _} ->
		{exec, Exec1, ExecMsg1};
	      Other1 ->
		Other1
	    end
	end,
      ?dbg("return~nmodule:~s~nstate:~s~nreturn:~p", [H, HS, Ret]),
      NxtCont(Ret);
    {'EXIT', Pid, Reason} ->
      exit(Reason)
  end.

self_access_info(WA, 
	      Cont,
	      S=#state{handler=H, hstate_name=SN, hdata=HD, super=Super}) ->
  AI = H:access_info(WA, SN, HD),
  if Super == undefined ->
      {{AI, #access_info{}}, Cont};
     true ->
      Cont({{call, ?super, ?access_info},
	    next_state(fun super_access_info/3, AI)
	   })
  end.

super_access_info({answer, ?super, SAI, _}, Cont, AI) ->
  {{AI, SAI}, Cont}.

self_is_alive(Cont, S=#state{super=undefined}) ->
  {true, Cont};
self_is_alive(Cont, State) ->
  Cont({{is_alive, ?super}, next_state(fun super_is_alive/3, [])}).

super_is_alive({answer, is_alive, Ret, _}, Cont, _) ->
  {Ret, Cont}.

spawn_super({answer, ?super, ok, _}, Cont, {Msg, S}) ->
  start(Msg, Cont, S#state{super=?super}).

super_return(Msg={answer, ?super, Answer, _}, Cont, State) ->
  
  {IsAlive, NxtCont} = self_is_alive(Cont, State),
  if IsAlive ->
      call_comp0(Msg, NxtCont, State);
     true ->
      NxtCont({stop, Answer})
  end;
super_return({error, Error, WA}, Cont, State) ->
  super_return({answer, {error, Error}, ?super, WA}, Cont, State).

spawn_result({answer, From, Answer, _}, Cont, State) ->
  call_comp({answer, From, Answer}, Cont, State).

call_answer({answer, From, Answer, _}, Cont, State) ->
  call_comp({answer, From, Answer}, Cont, State);
call_answer({error, Error, _}, Cont, State) ->
  call_comp({error, Error}, Cont, State).

call_exec_answer({answer, exec, ?killed, WA}, Cont, State) ->
  Cont({stop, ?killed});
call_exec_answer({answer, exec, Answer, WA}, Cont, State) ->
  start({call, Answer, WA}, Cont, State).

hlr_loop(Srv, Module, StateName, StateData) ->
  Self = self(),
  case catch
    receive
      {Srv, {call, {security_alert, Event}, WA}} ->
	{ok, hlr_any_state(Event, security_alert, WA, Srv, Module, StateName, StateData)};
      {Srv, {call, {old_request, Event}, WA}} ->
	{ok, hlr_any_state(Event, old_request, WA, Srv, Module, StateName, StateData)};
      {Srv, {call, {any_state_event, Event}, WA}} ->
	{ok, hlr_any_state(Event, parent, WA, Srv, Module, StateName, StateData)};
      {Srv, {call, Msg, WA}} ->
	{ok, hlr_call(Msg, WA, Srv, Module, StateName, StateData)};
      {Srv, {answer, From, Msg, WA}} ->
	From1 =
	  case From of
	    ?super ->
	      super;
	    _ ->
	      From
	  end,
	{ok, hlr_any_state(Msg, From1, WA, Srv, Module, StateName, StateData)};
      {Srv, stop} ->
	Srv ! {Self, {state, {StateName, StateData}}},
	stop
    end of
    {'EXIT', Error}  ->
      error_logger:error_report(
	[{module, Module}, {state, StateName}, Error]),
      Srv ! {Self, {stop, {fatal, Error}, []}};
    {ok, Res} ->
      Ret =
	case Res of
	  stop ->
	    exit(normal);
	  {next_state, Reply, Out, NxtState, NxtData} ->
	    {reply, Reply, Out, NxtState, NxtData};
	  {next_state, Reply, NxtState, NxtData} ->
	    {reply, Reply, NxtState, NxtData};
	  StopReply={stop, _} ->
		StopReply;
	  StopReply={stop, _, _} ->
	    StopReply;
	  {exec, ExecMsg, Exec, ExecName} ->
	    {exec, Exec, ExecName, ExecMsg};
	  {exec, ExecMsg, Exec} ->
	    {exec, Exec, ExecMsg};
	  {call_exec, ExecMsg, Exec, NxtState, NxtData} ->
	    {call_exec, Exec, ExecMsg, NxtState, NxtData}
	end,
      Srv ! {Self, Ret}
  end.


hlr_call(Event, WA, Srv, Module, StateName, StateData) ->
  ?dbg("call~nmodule: ~s~nstate: ~s~nevent: ~p~n", [Module, StateName, Event]),
  Module:StateName(Event, WA, Srv, StateData).

hlr_any_state(Msg, From, WA, Srv, Module, StateName, StateData) ->
  ?dbg("all state call~nmodule: ~s~nstate: ~s~nmessage: ~p~n", [Module, StateName, Msg]),
  Module:handle_event(Msg, From, WA, Srv, StateName, StateData).


security_alert(Event) ->
  {security_alert, Event}.

select_event(WA, {AI=#access_info{events=Es, default_event=D},
		  SAI=#access_info{events=SEs}}) ->
  In = wslib:arg_input(WA),
  case lists:dropwhile(
	 fun (N) -> proplists:get_value(N, In) == undefined end,
	 Es) of
    [] ->
      case lists:dropwhile(
	     fun (N) -> proplists:get_value(N, In) == undefined end,
	     SEs) of
	[] ->
	  {self, D};
	[Name|_] ->
	  {super, Name}
      end;
    [Name|_] ->
      {self, Name}
  end.

authorize_req(WA=#ws_arg{user=U, app_env=AE}, Event, Path, AccessId) ->
  case wsacl:authorize(WA, U, Event, Path, AccessId, AE) of
    deny ->
      if U == undefined ->
	  throw({authenticate, Event, Path});
	 true ->
	  deny
      end;
    permit ->
      permit
  end.

next_state(StateFun, StateData) ->
  {StateFun, StateData}.

fmt_dcomp_result(Stop) when element(1, Stop) == stop ->
  Stop;
fmt_dcomp_result({exec, C=#comp{dcomp=DC, render=R}, Msg}) ->
  {exec, DC, R, Msg};
fmt_dcomp_result({call_exec, C=#comp{dcomp=DC, render=R}, Msg, Dcomp}) ->
  {call_exec, DC, R, Msg, mk_dcomp(Dcomp)};
fmt_dcomp_result({spawn, CompName, C=#comp{dcomp=DC, render=R}, Dcomp}) ->
  {spawn, CompName, DC, R, mk_dcomp(Dcomp)};
fmt_dcomp_result(T) ->
  Sz = size(T),
  setelement(Sz, T, mk_dcomp(element(Sz, T))).
