%%%-------------------------------------------------------------------
%%% File    : wsession.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%%
%%% Created : 26 Mar 2003 by  <svg@surnet.ru>
%%%-------------------------------------------------------------------
%%
%% $Id: wsession.erl,v 1.4 2003/10/31 17:57:31 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*

-module(wsession).
-author('svg@surnet.ru').

-export([init_env/1, create/3, update/2, delete/1, check/1]).

-include("wstl.hrl").

init_env(WA=#ws_arg{user=User, app_env=AE}) ->
  NewUser =
    case check(WA) of
      {ok, #wsession{user=U}} ->
	U;
      _ ->
	undefined
    end,
  case {User, NewUser} of
    {_, undefined} when User =/= undefined ->
      wsacl:logout(WA, User, AE);
    _ ->
      ok
  end,
  WA#ws_arg{user=NewUser}.

create(WA, User, Data) when record(WA, ws_arg) ->
  create(get_sid(WA), User, Data);
create(CookieName, User, Data) ->
  SessData = create_session_data(User, Data),
  Cookie = yaws_api:new_cookie_session(SessData),
  yaws_api:setcookie(CookieName, Cookie, "/").

delete(WA) ->
  delete(get_sid(WA), WA).

delete(CookieName, WA=#ws_arg{yaws_arg=A}) ->
  case get_cookie_val(CookieName, A) of
    []  ->
      {error, no_session};
    Cookie ->
      yaws_api:delete_cookie_session(Cookie),
      yaws_api:setcookie(CookieName, [], "/")
  end.

update(Data, WA) ->
  update(get_sid(WA), Data, WA).

update(CookieName, Data, WA=#ws_arg{yaws_arg=A, user=U}) ->
  case get_cookie_val(CookieName, A) of
    []  ->
      {error, no_session};
    Cookie ->
      yaws_api:replace_cookie_session(Cookie, #wsession{user=U, data=Data})
  end.

check(WA) ->
  check(get_sid(WA), WA).

check(CookieName, WA=#ws_arg{yaws_arg=A}) ->
  case get_cookie_val(CookieName, A) of
    []  ->
      {error, no_session};
    Cookie ->
      yaws_api:cookieval_to_opaque(Cookie)
  end.

get_cookie_val(CookieName, Arg) ->
  H = Arg#arg.headers,
  yaws_api:find_cookie_val(CookieName, H#headers.cookie).

create_session_data(User, Data) ->
  #wsession{user=User, data=Data}.

get_sid(WA) ->
  wsacl:session_id(WA).
