%%%-------------------------------------------------------------------
%%% File    : wsform.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : HTML forms handling
%%%
%%% Created :  8 Jan 2003 by  <svg@surnet.ru>
%%%-------------------------------------------------------------------
%%
%% $Id: wsform.erl,v 1.8 2003/10/31 17:57:31 vsekissov Exp $
%%
%%**
%% 
%% @doc HTML forms handling
%% @end
%%*
-module(wsform).

-export([create/5, empty/2, data/3, data_format/3, out/3, validate/3, calculate/3]).
-export([input/3, input/4, output/4, render/4, field_names/1, ss/1, name/1]).
-export([coerce_types/4, label/1, out_attrs/2, field_labels/1]).
-export([test/0]).

-include("wstl.hrl").

create(FormName, FormProps, FieldNames, FieldProps, CommonArg) ->
  FieldProps1 = FieldProps ++
    [{data        , {wsform_lib, def_data}},
     {label       , {wsform_lib, def_label}},
     {data_filter , {wsform_lib, def_data_filter}},
     {data_type   , {wsform_lib, def_data_type}},
     {validate    , {wsform_lib, def_validate}},
     {calculate   , {wsform_lib, def_calculate}},
     {out         , {wsform_lib, def_field_out}},
     {out_attrs   , {wsform_lib, def_field_out_attrs}},
     {default     , {wsform_lib, def_value}},
     {ss          , wsform.ss:form_field(wsxmlt:m_name(FieldNames), CommonArg)}
    ],
  [FieldData, DataFilter, Default, Validate, FieldCalc, FieldOut, FieldSs,
   FieldLabel, DataType, OutAttrs] =
    [normalize_field_prop(K, kv(K, FieldProps1))
     || K <- [data, data_filter, default, validate, calculate, out, ss,
	      label, data_type, out_attrs]],

  FieldLabels = [{FN, FieldLabel(FN, CommonArg)} || FN <- FieldNames],
  FormProps1 = FormProps ++
    [{data   ,   {wsform_lib, def_form_data}},
     {label  ,   {wsform_lib, def_form_label}},
     {input  ,   {wsform_lib, def_form_input}},
     {output ,   {wsform_lib, def_form_output}},
     {calculate, {wsform_lib, def_form_calculate}},
     {out    ,   {wsform_lib, def_form_out}},
     {out_attrs, {wsform_lib, def_form_out_attrs}},
     {ss     ,   {wsform.ss,  form}},
     {render ,   {wsform_lib, def_form_render}}
		
    ],
  [FormData, FormInput, FormOutput, FormOut, FormSs, FormRender,
   FormLabel, FormOutAttrs, FormCalc] =
    [normalize_form_prop(K, kv(K, FormProps1))
     || K <- [data, input, output, out, ss, render, label, out_attrs, calculate]],

  FormLabel1 = FormLabel(FormName, CommonArg),
  Data = mk_data(FieldData, DataFilter, FieldNames),
  Out = mk_out(DataType, FieldOut, OutAttrs, FieldLabel, FieldNames),

  FormEmpty = mk_empty(Default, FieldNames),
  FormDataFormat = mk_data_format(DataType, Default),
  FormDataType = mk_data_type(DataType),
  FormValidate = mk_validate(Validate),
  Calc = mk_calculate(FieldCalc),
  
  FormData1 = mk_form_data(FormData, Data),
  FormInput1 = mk_form_input(FormInput),
  FormOut1 = mk_form_out(FormOut, Out),  
  Ss = mk_ss(FormName, FormSs, FieldSs, CommonArg),
  Render1 = mk_form_render(FormRender),
  FormOutput1 = mk_form_output(FormOutput),
  FormCalc1 = mk_form_calculate(FormCalc, Calc),

  Form =
    fun (_F, name) ->
	FormName;
	(_, label) ->
	FormLabel1;
	(_, out_attrs) ->
	FormOutAttrs;
	(_, field_labels) ->
	FieldLabels;
	(_F, field_names) ->
	FieldNames;
	(_F, empty) ->
	FormEmpty;
	(F, data) ->
	FormData1(mk_dispatch(F));
	(F, data_format) ->
	FormDataFormat;
	(F, input) ->
	FormInput1(mk_dispatch(F));
	(_F, data_type) ->
	FormDataType;
	(F, validate) ->
	FormValidate;
	(F, calculate) ->
	FormCalc1(mk_dispatch(F));
	(F, out) ->
	FormOut1(mk_dispatch(F));
	(F, output) ->
	FormOutput1(mk_dispatch(F));
	(F, ss) ->
	Ss;
	(F, render) ->
	Render1(mk_dispatch(F))
    end,
  mk_dispatch(Form).
  
mk_dispatch(Form) ->
  fun (Sel) ->
      Form(Form, Sel)
  end.

name(FD) ->
  FD(name).

label(FD) ->
  FD(label).

field_labels(FD) ->
  FD(field_labels).

out_attrs(Arg, FD) ->
  (FD(out_attrs))(FD, Arg).

field_names(FD) ->
  FD(field_names).

empty(Arg, FD) ->
  (FD(empty))(Arg).

input(Data, Arg, FD) ->
  (FD(input))(Data, [data_format, validate, calculate], Arg).

input(Data, Arg, FunStream, FD) ->
  (FD(input))(Data, FunStream, Arg).

output(In, RenderArgs, Arg, FD) ->
  (FD(output))(In, RenderArgs, Arg).

data(Data, Arg, FD) ->
  (FD(data))(Data, Arg).

data_format(KVs, Arg, FD) ->
  (FD(data_format))(KVs, Arg).

coerce_types(KVs, Direction, Arg, FD) ->
  (FD(data_type))(KVs, Direction, Arg).

validate(KVs, Arg, FD) ->
  (FD(validate))(KVs, Arg).

calculate(KVs, Arg, FD) ->
  (FD(calculate))(KVs, Arg).

out(Out, Arg, FD) ->
  (FD(out))(Out, Arg).

ss(FD) ->
  FD(ss).

render(Tree, Ps, Arg, FD) ->
  (FD(render))(Tree, Ps, Arg).

mk_empty(DefaultFun, FNs) ->
  KVs = [{N, N} || N <- FNs],
  EmptyFun =
  fun (Arg) ->
      {[], Obj} =
	form_map(fun({N, _}) ->
		     case DefaultFun(N, Arg) of
		       Fun when function(Fun) ->
			 Fun(Arg);
		       required ->
			 undefined;
		       V ->
			 V
		     end
		 end, KVs),
      Obj
  end.

mk_form_output(FormOutput) ->
  fun (FD) ->
      fun (In, RenderArgs, Arg) ->
	  FormOutput(FD, In, RenderArgs, Arg)
      end
  end.

mk_form_out(FormOutFun, FieldMap) ->
  fun (FD) ->
      FNs = field_names(FD),
      fun (KVs, Arg) ->
	  KVs1 = [KV || KV={K,_} <- KVs, lists:member(K, FNs) == true],
	  {[], FieldOuts} = FieldMap(KVs1, Arg),
	  FormOutFun(FD, FieldOuts, Arg)
      end
  end.
	  
mk_form_data(FormDataFun, DataFun) ->
  fun (FD) ->
      fun (Data, Arg) ->
	  DataFun(FormDataFun(FD, Data, Arg), Arg)
      end
  end.

mk_form_input(FormInput) ->
  fun (FD) ->
      FormData = FD(data),
      FieldNames = FD(field_names),
      fun (Data, FunStream, Arg) ->
	  FieldInput = mk_input(FD, FunStream),
	  case FormData(Data, Arg) of
	    {[], In} ->
	      FormInput(In, FieldNames, FieldInput, Arg);
	    %% we can't process if input data is invalid
	    {Errors, _} when list(Errors) ->
	      exit({form_input, Errors});
	    Other ->
	      Other
	  end
      end
  end.

%% Field funs
mk_out(TypeFun, OutFun, OutAttrs, Label, FNs) ->
  fun (KVs, Arg) ->
      Out =
	form_map(fun ({K, V}) ->
		     OutV =
		       case V of
			 %%undefined ->
			 %%  V;
			 {badval, _} ->
			   V;
			 _ ->
			   TypeFun(K, V, out, Arg)
		       end,
		     OutFun(K, OutV, Label(K, Arg), OutAttrs(K, Arg), Arg)
		 end, KVs),
      Out
  end.

mk_data(DataFun, FilterFun, FNs) ->
  KVs = [{N, N} || N <- FNs],
  fun (Data, Arg) ->
      form_map(fun ({N, _}) ->
		   case DataFun(N, Data, Arg) of
		     undefined ->
		       undefined;
		     V ->
		       FilterFun(N, V, Arg)
		   end
	       end, KVs)
  end.
  
mk_input(FD, FunNames) ->
  Funs = lists:map(fun (FN=data_type) ->
		       F = FD(FN),
		       fun (D, A) -> F(D, in, A) end;
		       (FN) ->
		       FD(FN)
		   end, FunNames),
  Fun =
    fun([], Ret, _, _) ->
	Ret;
       ([Fun|Rest], {[], Prev}, Arg, Cont) ->
	Cont(Rest, Fun(Prev, Arg), Arg, Cont);
       (_, Ret, _, _) -> %previous Fun returned errors
	Ret
    end,
  fun (Data, Arg) ->
      Fun(Funs, {[], Data}, Arg, Fun)
  end.

mk_data_format(TypeFun, DefaultFun) ->
  Fun =
    fun (N, V, Arg) ->
	case {V, DefaultFun(N, Arg)} of
	  {undefined, required} ->
	    {error, missing};
	  {undefined, undefined} ->
	    undefined;
	  {undefined, Def} when function(Def) ->
	    Def(Arg);
	  {undefined, Def} ->
	    Def;
	  _ ->
	    TypeFun(N, V, in, Arg)
	end
    end,
  fun (KVs, Arg) ->
      form_map(fun ({N, V}) -> Fun(N, V, Arg) end, KVs)
  end.

mk_data_type(DataTypeFun) ->
  fun (KVs, Dir, Arg) ->
      form_map(fun %({N, undefined}) ->
		   %undefined;
		   ({N, V}) ->
		   DataTypeFun(N, V, Dir, Arg)
	       end, KVs)
  end.

mk_validate(ValidateFun) ->
  fun (KVs, Arg) ->
      form_map(fun ({N, V}) ->
		   case ValidateFun(N, V, KVs, Arg) of
		     true ->
		       V;
		     false ->
		       throw({error, validate})
		   end
	       end, KVs)
  end.

mk_form_calculate(Calc, FieldMap) ->
  fun (FD) ->
      fun (KVs, Arg) ->
	  FieldMap(KVs, Calc(FD, KVs, Arg))
      end
  end.

mk_calculate(CalculateFun) ->
  fun (KVs, Arg) ->
      form_map(fun ({N, V}) ->
		   case CalculateFun(N, V, KVs, Arg) of
		     {ok, CV} ->
		       CV;
		     Error ->
		       throw({error, Error})
		   end
	       end, KVs)
  end.

mk_ss(FormName, FormFun, FieldSs, Arg) ->
  %%  FieldRules = [FieldFun(N, Arg) || {N, _} <- FieldLabels],
  FormFun(FormName, mk_list(FieldSs), Arg).

mk_form_render(FormRender) ->
  fun (FD) ->
      fun(Tree, Ps, Arg) ->
	  Ss = FD(ss),
	  FormRender(Ss, Tree, Ps, Arg)
      end
  end.


kv(K,L) ->
  kv(K,L, undefined).

kv(K,L, D) ->
  proplists:get_value(K, L, D).

form_map(Fun, KVs) ->
  {Errors, Values} =
    lists:foldl(fun (KV={K, _}, {Es, Vs}) ->
		    case catch Fun(KV) of
		      {EK, Error} when EK == error;
				       EK == 'EXIT' ->
			{[{K, Error}|Es], Vs};
		      V ->
			{Es, [{K, V}|Vs]}
		    end
		end, {[], []}, KVs),
  {lists:reverse(Errors), lists:reverse(Values)}.

normalize_field_prop(data_type, F) when function(F) ->
  fun (Name, Value, Direction, Arg) ->
      case F(Name, Arg) of
	Type when atom(Type) ->
	  wsform.field_type:Type(Direction, Value, Arg);
	TypeFun ->
	  TypeFun(Direction, Value, Arg)
      end
  end;
normalize_field_prop(data_type, {M, F}) ->
  normalize_field_prop(data_type,
		       fun (Name, Arg) ->
			   M:F(Name, Arg)
		       end);
normalize_field_prop(_, F) when function(F) ->
  F;
normalize_field_prop(data, {M, F}) ->
  fun (Name, KVs, Arg) ->
      M:F(Name, KVs, Arg)
  end;
normalize_field_prop(label, {M, F}) ->
  fun (Name, Arg) ->
      M:F(Name, Arg)
  end;
normalize_field_prop(data_filter, {M, F}) ->
  fun (Name, Value, Arg) ->
      M:F(Name, Value, Arg)
  end;
normalize_field_prop(validate, {M, F}) ->
  fun (Name, Value, KVs, Arg) ->
      M:F(Name, Value, KVs, Arg)
  end;
normalize_field_prop(calculate, {M, F}) ->
  fun (Name, Value, KVs, Arg) ->
      M:F(Name, Value, KVs, Arg)
  end;
normalize_field_prop(out, {M, F}) ->
  fun (Name, Value, Label, Attrs, Arg) ->
      M:F(Name, Value, Label, Attrs, Arg)
  end;
normalize_field_prop(out_attrs, {M, F}) ->
  fun (Name, Arg) ->
      M:F(Name, Arg)
  end;
normalize_field_prop(default, {M, F}) ->
  fun (Name, Arg) ->
      M:F(Name, Arg)
  end;
normalize_field_prop(ss, Ss) ->
  Ss.

normalize_form_prop(_, F) when function(F)->
  F;
normalize_form_prop(data, {M, F}) ->
  fun (Form, Data, Arg) ->
      M:F(Form, Data, Arg)
  end;
normalize_form_prop(label, {M, F}) ->
  fun (Form, Arg) ->
      M:F(Form, Arg)
  end;
normalize_form_prop(input,  {M, F}) ->
  fun (In, FieldNs, FieldMap, Arg) ->	 
      M:F(In, FieldNs, FieldMap, Arg)
  end;
normalize_form_prop(output,  {M, F}) ->
  fun (Form, Data, RenderArgs, Arg) ->	 
      M:F(Form, Data, RenderArgs, Arg)
  end;
normalize_form_prop(calculate,  {M, F}) ->
  fun (Form, KVs, Arg) ->	 
      M:F(Form, KVs, Arg)
  end;
normalize_form_prop(out_attrs, {M, F}) ->
  fun (Form, Arg) ->
      M:F(Form, Arg)
  end;
normalize_form_prop(out, {M, F}) ->
  fun (Form, Data, Arg) ->	 
      M:F(Form, Data, Arg)
  end;
normalize_form_prop(ss, {M, F}) ->
  fun (FormN, FieldRs, Arg) ->	 
      M:F(FormN, FieldRs, Arg)
  end;
normalize_form_prop(render, {M, F}) ->
  fun (Ss, Tree, RenderArgs, Arg) ->
      M:F(Ss, Tree, RenderArgs, Arg)
  end.
     
mk_list(L) when list(L) ->
  L;
mk_list(V) ->
  [V].

test() ->
  create(test, [], [f1, f2], [], []).
