%%% File    : expand_rowset_hook.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created : 30 Aug 2003 by  <svg@surnet.ru>

%%
%% $Id: expand_rowset_hook.erl,v 1.3 2003/11/01 12:11:36 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wsform.expand_rowset_hook).

-behaviour('wscomp.expand').

-export([create_select_rowid_hook/0, create_editrow_hook/1, create_editrow_hook/2]).
-export([create_editrel_hook/5]).
-export([init/1, is_active/3, access_info/3, handle_event/5, handle_call/4]).

-import(lists).
-import(wslib).
-import(wsdcomp).
-import(wscomp.expand).
-import(wscomp.selector).
-import(wslanguage).
-import(stl_yaws).
-import(wsmessage).

-include("wstl.hrl").

-record(state, {hook,
		hdata,
		comp_name,
		empty_rowset_events=[],
		rowset_events=[],
		default_event=refresh
	       }).

-define(editrow_events, [do_add_row, do_edit_row, do_view_row, do_delete_row]).
-define(editrow_empty_events, [do_add_row]).

-record(editrow_state, {sname=init, row_comp}).
-record(editrel_state, {sname=init,
			create_edit_rel,
			mk_row_comp,
			action_selector}).

create_select_rowid_hook() ->
  {?MODULE,
   {fun select_rowid_hook/2,
    [],
    undefined,
    [{rowset_events, [do_select]}]
   }
  }.

create_editrow_hook(Row) ->
  create_editrow_hook(Row, []).

create_editrow_hook(Row, Props) ->
  {?MODULE,
   {fun editrow_hook/2,
    Row,
    undefined,
    Props++ [{rowset_events, ?editrow_events},
	     {empty_rowset_events, ?editrow_empty_events}]
   }
  }.

%%
%% MkDetailComp(rowset|row, Data::term(), WA::ws_arg()) ->
%%   Comp::wsdcomp()
%%   
%% MkRowComp(Data::term(), WA::ws_arg()) ->
%%   Comp::wsdcomp()
%%   
%% Props = [{selector, edit_rel_action_selector()::selector()},
%%          selector_label
%%         |CommonProps]
%% Relationship = [{PrimaryKeyName::atom(), ForeignKeyName::atom()}]
%%
%% Selector = component() -> {Event, Data}
%% Event = do_create|do_select|do_exit
%% 

create_editrel_hook(MkDetailComp, MkRowComp, Relationship, WA, Props) ->
  Selector = case wslib:gv(selector, Props) of
	       undefined ->
		 Label = wslib:gv(selector_label, Props, ""),
		 edit_rel_action_selector(Label, WA);
	       V ->
		 V
	     end,
  {?MODULE,
   {fun editrel_hook/2,
    {MkDetailComp, MkRowComp, Relationship, Selector},
    undefined,
    Props++ [{rowset_events, ?editrow_events++[do_view_relationship_details]},
	     {empty_rowset_events, ?editrow_empty_events}]
   }
  }.

  
select_rowid_hook({is_active, WA, Srv}, HD) -> true;
select_rowid_hook(init, HD) -> {ok, HD};
select_rowid_hook({call, do_select, WA, Srv}, HD) ->
  case wsdcomp:super_any_state(Srv, current_rowid) of
    [] ->
      wsmessage:show_error({error, record_not_selected}, WA),
      {ok, ok, HD};
    Rowid ->
      {stop, {ok, Rowid}}
  end;
select_rowid_hook({call, _, WA, Srv}, HD) ->
  {ok, undefined, HD};
select_rowid_hook({event, Msg, From, WA, Srv}, HD) ->
  {ok, undefined, HD}.

editrow_hook({is_active, WA, Srv}, S) -> true;
editrow_hook(init, Row) -> {ok, #editrow_state{sname=enter, row_comp=Row}};
editrow_hook({call, Event, WA, Srv},
	     S=#editrow_state{sname=enter, row_comp=Row}) ->
  case lists:member(Event, ?editrow_events) of
    true ->
      Rowid = wsdcomp:super_any_state(Srv, current_rowid),
      {call_exec, {Event, Rowid}, Row, S#editrow_state{sname=call_exec}};
    false ->
      {ok, undefined, S#editrow_state{sname=enter}}
  end;
editrow_hook({call, _, WA, Srv}, S) ->
  {ok, undefined, S#editrow_state{sname=enter}};
editrow_hook({event, Msg, From, WA, Srv}, S) ->
  {ok, undefined, S#editrow_state{sname=enter}}.

%%
editrel_hook({is_active, WA, Srv}, S) -> true;
editrel_hook(init, {MkDetComp, MkRowComp, Rel, Selector}) ->
  CreateEditRel =
    fun (Event, Data, WA) ->
	wsdcomp:create(wsform.edit_relationship,
		       {Event, MkDetComp, MkRowComp, Rel, Data, WA},
		       wslib:mk_dummy_render([]),
		       [])
    end,
  {ok,
   #editrel_state{sname=enter,
		  create_edit_rel=CreateEditRel,
		  mk_row_comp=MkRowComp,
		  action_selector=Selector
		 }
  };
editrel_hook({call, do_add_row, WA, Srv},
	    S=#editrel_state{sname=enter, action_selector=AS}) ->
  {call_exec, enter, AS, S#editrel_state{sname=choose_action}};
editrel_hook({call, do_view_relationship_details, WA, Srv},
	     S=#editrel_state{sname=enter, create_edit_rel=CER}) ->
  Rowid = wsdcomp:super_any_state(Srv, current_rowid),
  if Rowid == [] ->
      wsmessage:show_error({error, record_not_selected}, WA),
      {ok, ok, S};
     true ->
      Comp = CER(view_details, Rowid, WA),
      {call_exec, enter, Comp, S#editrel_state{sname=call_exec}}
  end;
editrel_hook({call, Event, WA, Srv},
	     S=#editrel_state{sname=enter, mk_row_comp=MkRowComp}) ->
  case lists:member(Event, ?editrow_events) of
    true ->
      Rowid = wsdcomp:super_any_state(Srv, current_rowid),
      {Row, RowEvent} =
	fmt_mk_ret(MkRowComp(Event, Rowid, [], WA), {Event, Rowid}),
      {call_exec, RowEvent, Row, S#editrel_state{sname=call_exec}};
    false ->
      {ok, undefined, S}
  end;
editrel_hook({call, E={Event, EventData}, WA, Srv},
	     S=#editrel_state{sname=choose_action,
			      create_edit_rel=CER
			     })
  when Event == do_select;
       Event == do_create ->
  {Action, ActionData} =
    if (EventData == do_select) or (EventData == do_create) ->
	%% button selector returns {do_select, Action}
	{EventData, []};
       true ->
	E
    end,
  {call_exec, enter, CER(Action, ActionData, WA), S};
editrel_hook(_, S=#editrel_state{sname=call_exec}) ->
  {ok, ok, S#editrel_state{sname=enter}};
editrel_hook(_, S) ->
  {ok, undefined, S#editrel_state{sname=enter}}.

edit_rel_action_selector(Label, WA) ->
  SelList = [{K, wslanguage:msg(WA, K, [])}
	     || K <- [do_select, do_create, do_exit]],
  GetValues = fun (_, _) -> {ok, SelList} end,
  OnSelect = fun (Event, [K], _, _) -> {stop, {Event, list_to_atom(K)}} end,
  Data = [],
  Props = [{template, "lib/button_selector"},
	   {label, Label},
	   {selector_attrs,
	    [{orientation, horizontal}]}
	  ],
  (selector:create(id,GetValues,OnSelect,Data,Props))(component, WA).

%%
init({Hook, HookData, CompName, Props}) ->
  [EmptyEvents, RowsetEvents, DefaultEvent] =
    wslib:gv_map([{empty_rowset_events, []},
		  {rowset_events, []},
		  {default_event, refresh}
		 ],
		 Props),
  {ok, HD} = Hook(init, HookData),
  {ok, #state{hook=wscomp.expand:mk_hook(Hook),
	      hdata=HD,
	      comp_name=CompName,
	      empty_rowset_events=EmptyEvents,
	      rowset_events=RowsetEvents,
	      default_event=DefaultEvent}
  }.

is_active(WA, Srv, S=#state{hook=H, hdata=HD, comp_name=undefined}) ->
  (wsdcomp:super_any_state(Srv, state) == rowset)
    andalso H({is_active, WA, Srv}, HD);
is_active(WA, Srv, S=#state{hook=H, hdata=HD, comp_name=CN}) ->
  (not wsdcomp:is_alive(Srv, CN))
    andalso (wsdcomp:super_any_state(Srv, state) == rowset)
    andalso H({is_active, WA, Srv}, HD).

access_info(WA, Srv,
	   S=#state{empty_rowset_events=EEs,
		    rowset_events=TEs,
		    default_event=DE}) ->
  CEs=
    case wsdcomp:super_any_state(Srv, pager_state) of
      %%{IsFirst, IsLast, Sz}
      {_, _, 0} ->
	EEs;
      _ ->
	TEs
    end,
  #access_info{events=CEs, default_event=DE}.

handle_call(Event, WA, Srv, S=#state{hook=H, hdata=HD}) ->
  {Ret, NxtHD} = wscomp.expand:call_hook({call, Event, WA, Srv}, H, HD),
  {Ret, S#state{hdata=NxtHD}}.

handle_event(Msg, CN, WA, Srv, S=#state{hook=H, hdata=HD, comp_name=CN}) ->
  wsdcomp:display(Srv, [CN], false),
  {Ret, NxtHD} =
    wscomp.expand:call_hook({event, Msg, CN, WA, Srv}, H, HD),
  {Ret,  S#state{hook=H, hdata=NxtHD}};
handle_event(Msg, From, WA, Srv,
	     S=#state{hook=H, hdata=HD}) ->
  {Ret, NxtHD} = wscomp.expand:call_hook({event, Msg, From, WA, Srv}, H, HD),
  {Ret,  S#state{hdata=NxtHD}}.

fmt_mk_ret({ok, Comp, Event}, _) ->
  {Comp, Event};
fmt_mk_ret({ok, Comp}, DefaultEvent) ->
  {Comp, DefaultEvent}.
