%%
%% $Id: field_type.erl,v 1.3 2003/11/01 12:11:36 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wsform.field_type).

-export([integer/3, atom/3, string/3, boolean/3, datetime/3, date/3, time/3]).
-export([validate/4, format/2]).

-import(regexp).

-import(wslib.str).
-import(wslib.listn).

-define(email_regexp,
	"^[_A-Za-z0-9-]+(\\.[_A-Za-z0-9-]+)*"
	"@"
	"[A-Za-z0-9-]+(\\.[A-Za-z0-9-]+)*\$"
       ).

-define(phone_regexp, "^[0-9]+").

validate(_, undefined, _, _) ->
  true;
validate(email_address, V, _, _) ->
  validate_regexp(V, ?email_regexp);
validate(phone_number, V, _, _) ->
  validate_regexp(V, ?phone_regexp);
validate(time, {HH24, MI, S}, _, _) when HH24 >= 0,
					 HH24 =< 23,
					 MI >= 0,
					 MI =< 59,
					 S >= 0,
					 S =< 59 ->
  true;
validate(time, _, _, _) ->
  false;
validate(_, _, _, _) ->
  true.

format(phone_number, "") ->
  "";
format(phone_number, Str) ->
  str:join(listn:split(3, Str), $-);
format(_, V) ->
  V.

integer(in, Int, A) when is_integer(Int) ->
  Int;
integer(in, Float, A) when is_float(Float) ->
  trunc(Float);
integer(in, undefined, A) ->
  undefined;
integer(in, Str, A) ->
  list_to_integer(Str);
integer(out, undefined, A) ->
  [{integer, [], ""}];
integer(out, Int, A) ->
  [{integer, [], integer_to_list(Int)}].

atom(in, T, A) when is_atom(T) ->
  T;
atom(in, undefined, A) ->
  undefined;
atom(in, Str, A) ->
  list_to_atom(Str);
atom(out, undefined, A) ->
  [{string, [], ""}];
atom(out, T, A) ->
  [{string, [], atom_to_list(T)}].

string(in, Str, A) ->
  Str;
string(out, undefined, A) ->
  [{string, [], ""}];
string(out, Str, A) ->
  [{string, [], Str}].

boolean(in, undefined, A) ->
  undefined;
boolean(in, Bool, A) when Bool == true;
			  Bool == false ->
  Bool;
boolean(in, Bool, A) ->
  odbcdb.attr_type:to_boolean(Bool);
boolean(out, undefined, A) ->
  [{boolean, [], ""}];
boolean(out, Bool, A) ->
  [{boolean, [], atom_to_list(Bool)}].

%% 2003-06-10
date(in, undefined, A) ->
  undefined;
date(in, Date, A) ->
  odbcdb.attr_type:to_date(Date);
date(out, T, A) ->
  [{date, [{value, T}], []}].

%% 2003-06-10 02:13:57
datetime(in, undefined, A) ->
  undefined;
datetime(in, T, A) ->
  odbcdb.attr_type:to_datetime(T);
datetime(out, T, A) ->
  [{datetime, [{value, T}], []}].

%% HH24:MI:S
time(in, undefined, A) ->
  undefined;
time(in, Str, A) when is_list(Str) ->
  {ok, Vs} = regexp:split(Str, "[^0-9]"),
  to_time([list_to_integer(V) || V <- Vs]);
time(in, Time, A) ->
  odbcdb.attr_type:to_time(Time);
time(out, T, A) ->
  [{time, [{value, T}], []}].

to_time([HH24, MI, S]) ->
  {HH24, MI, S};
to_time([HH24, MI]) ->
  {HH24, MI, 0};
to_time([HH24]) ->
  {HH24, 0, 0}.

%%
validate_regexp(Str, Re) ->
  case regexp:first_match(Str, Re) of
    {match, _, _} ->
      true;
    nomatch ->
      false
  end.
