%%% File    : mnemosyne_pager.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : Form pager
%%% Created : 22 Feb 2003 by  <svg@surnet.ru>

%%
%% $Id: mnemosyne_pager.erl,v 1.1 2003/05/11 22:26:35 vsekissov Exp $
%%
%%**
%% 
%% @doc Form pager
%% @end
%%*
-module(wsform.mnemosyne_pager).

-export([init/1, next/2, prev/2, current/2, pos/1]).

-import(.mnemosyne).

-record(cursor, {pos=0, db, db_query, chunk_format}).

init({Db, Query, QueryParams, ChunkFormat}) ->
  create_cursor(Db, Query, QueryParams, ChunkFormat).

create_cursor(Db, Query, QueryParams, ChunkFormat) ->
  DbQuery = Query(QueryParams),
  #cursor{db=Db, db_query=DbQuery, chunk_format=ChunkFormat}.

next(C=#cursor{pos=P, db=Db, db_query=Q, chunk_format=CF}, ChunkSize) ->
  QuerySetup = mnemosyne:setup_query(Q),
  Fun =
    fun () ->
	case do_next(P, ChunkSize, mnemosyne:init_query(QuerySetup)) of
	  {ok, NP, HasMore, Rows} ->
	    {ok, {CF(Rows),  HasMore, C#cursor{pos=NP}}};
	  Error ->
	    Error
	end
    end,
  Ret = Db(Fun),
  mnemosyne:delete_query(QuerySetup),
  Ret.

prev(C=#cursor{pos=P}, Sz) when P =< 1 ->
  current(C, Sz);
prev(C=#cursor{pos=P}, Sz) ->
  next(C#cursor{pos=P-2}, Sz).

current(C=#cursor{pos=0}, Sz) ->
  next(C, Sz);
current(C=#cursor{pos=P}, Sz) ->
  next(C#cursor{pos=P-1}, Sz).

pos(C=#cursor{pos=P}) ->
  P.

do_next(Cur, Size, Cursor) ->
  select_chunk(Cursor, Size, Cur+1).

select_chunk(Cursor, Size, Num) ->
  select_chunk(Cursor, Size, 0, Num,  []).

select_chunk(Cursor, Size, Cur, Cur,  Rows) ->
  {ok, Cur, true, Rows};
select_chunk(Cursor, Size, Cur, End,  PrevChunlk) ->
  case mnemosyne:next_answers(Cursor, 1, Size) of
    Rows when length(Rows) == Size ->
      select_chunk(Cursor, Size, Cur+1, End, Rows);
    Rows -> % no more records
      {ok, Cur+1, false, Rows}
  end.

