%%% File    : odbcdb_pager.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : Form pager
%%% Created : 22 Feb 2003 by  <svg@surnet.ru>

%%
%% $Id: odbcdb_pager.erl,v 1.2 2003/10/27 16:56:19 vsekissov Exp $
%%
%%**
%% 
%% @doc Form pager
%% @end
%%*
-module(wsform.odbcdb_pager).

-export([init/1, new_query/4, next/2, prev/2, current/2, pos/1]).

-import(odbcdb).

-record(cursor, {offset=0, db, db_query, last_size=0}).

init(Db) ->
  create_cursor(Db).

create_cursor(Db) ->
  #cursor{db=Db}.

new_query(Query, QueryParams, OrderBy, C=#cursor{db=Db}) ->
  Query1 = odbcdb:group_by(Query, OrderBy),
  DbQuery =
    fun (Dbh, Offset, ChunkSize) ->
	Query2 =
	  odbcdb:group_by(Query1,
			  {[], [{limit, ChunkSize}, {offset, Offset}]}),
	odbcdb:select_count(Dbh, Query2, QueryParams, tuple, infinity)
    end,
  C#cursor{db_query=DbQuery, offset=0}.

next(C=#cursor{offset=O, db=Db, db_query=Q}, ChunkSize) ->
  Fun =
    fun (Dbh) ->
	case do_next(Dbh, Q, O, ChunkSize) of
	  {ok, Rows, RowsLen, Offset, HasMore} ->
	    {ok, {Rows, HasMore,
		  C#cursor{offset=Offset+RowsLen, last_size=RowsLen}}};
	  Error ->
	    Error
	end
    end,
  Db(Fun).

prev(C=#cursor{offset=0}, ChunkSize) ->
  current(C, ChunkSize);
prev(C=#cursor{offset=O, last_size=LS}, ChunkSize) ->
  next(C#cursor{offset=max(O-LS-ChunkSize, 0)}, ChunkSize).

current(C=#cursor{offset=O, last_size=LS}, ChunkSize) ->
  next(C#cursor{offset=max(O-LS, 0), last_size=LS}, ChunkSize).

pos(C=#cursor{offset=O, last_size=LS}) ->
  O-LS.

do_next(Dbh, DbQuery, Offset, Limit) ->
  do_next(Dbh, DbQuery, Offset, Limit, true).

do_next(Dbh, DbQuery, Offset, Limit, HasMore) ->
  case DbQuery(Dbh,  Offset, Limit) of
    {ok, _} ->
      Rows = select_chunk(Dbh, Limit),
      RowsLen = length(Rows),
      case {RowsLen, Offset} of
	{0, 0} ->
	  {ok, Rows, RowsLen, Offset, false};
	{0, _} -> %try to fetch previous chunk
	  do_next(Dbh, DbQuery, max(0,Offset-Limit), Limit, false);
	%% no more rows, stay here
	_ when RowsLen < Limit -> % no more rows, stay here
	  {ok, Rows, RowsLen, Offset, false};
	_ ->
	  {ok, Rows, RowsLen, Offset, HasMore}
      end;
    Error ->
      Error
  end.

select_chunk(Dbh, Size) ->
  {selected, Rows} = odbcdb:next(Dbh, Size, infinity),
  Rows.

max(A, B) when A >= B ->
  A;
max(A, B) ->
  B.
  
