%%% File    : wsform.pager.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : Form pager
%%% Created : 22 Feb 2003 by  <svg@surnet.ru>

%%
%% $Id: pager.erl,v 1.2 2003/10/27 16:56:19 vsekissov Exp $
%%
%%**
%% 
%% @doc Form pager
%% @end
%%*
-module(wsform.pager).

-export([create/3, new_query/4, next/1, prev/1, current/1, pos/1, has_more/1]).
-export([chunk_size/1, chunk_size/2]).

-import(.proplists).

-record(state, {cursor, chunk_size=10, has_more=true}).

create(CursorMod, CursorParams, PagerParams) ->
  UserParams = PagerParams ++ [{chunk_size, 10}],
  [ChunkSize] = check_params(UserParams),
  CursorData = CursorMod:init(CursorParams),
  Cursor = {CursorMod, CursorData},
  S = #state{cursor=Cursor, chunk_size=ChunkSize},
  mk_pager(S).

new_query(Query, QueryParams, OrderBy, S=#state{cursor={M, Data}}) ->
  NxtData = M:new_query(Query, QueryParams, OrderBy, Data),
  S#state{cursor={M, NxtData}}.

mk_pager(State) ->
  State.

chunk_size(NewSize, S) when is_integer(NewSize),
			    NewSize >= 1 ->
  S#state{chunk_size=NewSize}.

chunk_size(S=#state{chunk_size=Size}) ->
  Size.

next(S) ->
  cursor_do(next, S).

prev(S) ->
  cursor_do(prev, S).

current(S) ->
  cursor_do(current, S).

pos(S=#state{cursor={M, Data}}) ->
  M:pos(Data).

has_more(S=#state{has_more=HM}) ->
  HM.

cursor_do(Op, S=#state{cursor={M, Data}, chunk_size=Sz}) ->
  case M:Op(Data, Sz) of
    {ok, {Rows, HasMore, NxtData}} ->
      {ok, {Rows, S#state{cursor={M, NxtData}, has_more=HasMore}}};
    Error ->
      Error
  end.

check_params(UserProps) ->
  [check_param(K, proplists:get_value(K, UserProps)) || K <- [chunk_size]].

check_param(chunk_size, Sz) when integer(Sz), Sz > 0 ->
  Sz;
check_param(Name, Val) ->
  exit({bad_param_value, Name, Val}).
