%%% File    : wsform.row.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created : 27 Mar 2003 by  <svg@surnet.ru>

%%
%% $Id: row.erl,v 1.5 2003/10/27 19:51:44 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wsform.row).

-behaviour(wsdcomp).

-export([init/2, access_info/3, handle_event/6]).
-export([enter/4,go/4]).
-export([create_display/1, create_comp/5, create_db_handler/1]).

-import(.lists).
-import(.proplists).
-import(.wsdcomp).
-import(.wsmessage).
-import(.proplists).
-import(.wsform).
-import(.wslib).
-import(.stl_yaws).
-import(.wsdb).

-include("wstl.hrl").

-define(row_events, [do_save, do_exit]).
-define(template,   "lib/editrow").	   
-define(template_var, 'RowOut').

-record(state, {pk_fields=[],
		readonly_fields=[],
		createonly_fields=[],
		form,
		display,
		mode,
		old_row=[],
		handler
	       }).

create_db_handler(Table) ->
  fun ({Op, Row}, Dbh) ->
      wsdb:Op(Dbh, Table, Row)
  end.

%%
%% Properties = [
%%               {template_var, 'RowOut'},
%%               {readonly_fields, []},
%%               {createonly_fields, []}
%%               add_row_handler*
%%               ]
create_comp(PkNames, Form, RowHlr, WA, Props) ->
  wsdcomp:create(?MODULE, {PkNames, Form, RowHlr, WA, Props}).

create_display(TmplVar) ->
  fun (Data, Form, WA, Props) ->
      Out = [{TmplVar, form_output(Form, Data, WA, Props)}]
  end.
%%
access_info(_, _, S=#state{mode=view}) ->
  #access_info{events=[do_exit, do_edit], default_event=refresh};
access_info(_, _, _) ->
  #access_info{events=?row_events, default_event=refresh}.

init(State, Props) when record(State, state) ->
  {ok, enter, State, Props};
init({PKs, Form, Handler, WA}, Props) ->
  TmplVar = wslib:gv(template_var, Props, ?template_var),
  RenderProps = [{place_at, ?template_var}, {template, ?template}],
  Display =
    case wslib:gv(display, Props) of
      undefined ->
	create_display(TmplVar);
      D ->
	D
    end,
  Handler1 = mk_handler(Handler, Props),
  State = #state{pk_fields     = PKs,
		 readonly_fields = wslib:gv(readonly_fields, Props, []),
		 createonly_fields = wslib:gv(createonly_fields, Props, []),
		 form         = Form,
		 display      = Display,
		 handler      = Handler1},
  Comp = wsdcomp:create(?MODULE, State, WA, Props++RenderProps),
  {component, Comp}.

handle_event(Event, security_alert, WA, Srv, State, StateData) ->
  {stop, ok};
handle_event(mode, _From, WA, Srv, State, S=#state{mode=M}) ->
  {next_state, M, State, S};
handle_event(row, _From, WA, Srv, State, S=#state{old_row=OR}) ->
  {next_state, OR, State, S};
handle_event(current_rowid,  _From, WA, Srv, State,
	     S=#state{pk_fields=PkNs, old_row=OR}) ->
  Pks = [KV || KV <- OR, lists:member(element(1, KV), PkNs) == true],
  {next_state, Pks, State, S};
handle_event(Msg, From, WA, Srv, State, StateData) ->
  {next_state, ok, State, StateData}.

enter({Event, EvArg}, WA, Srv, State) ->
  do_enter(event_to_mode(Event), EvArg, WA, Srv, State);
enter(refresh, WA, Srv, State) ->
  do_enter(insert, [], WA, Srv, State).

go(Event, WA, Srv, State) ->
  do(Event, WA, Srv, State).

%%
do(do_exit, WA, _Srv, S=#state{mode=view, old_row=Row}) ->
  {stop, {ok, Row}};
do(do_edit, WA, _Srv, S=#state{mode=view, old_row=Row}) ->
  next_state({[], Row}, WA,go, S#state{mode=edit});
do(do_exit, WA, _Srv, _S) ->
  {stop, ok};
do(do_save, WA, Srv, S) ->
  case form_input(WA, S) of
    Good={[], Row} ->
      Row1 = merge_changed_row(Row, S),
      case save_row(Row1, WA, Srv, S) of
	{ok, Res} ->
	  show_save_result_message(Res, WA, S#state.mode),
	  do_after_save(Row1, WA, Srv, S);
	ok ->
	  do_after_save(Row1, WA, Srv, S);
	Error ->
	  wsmessage:show_error(Error, WA),
	  next_state(Good, WA,go, S)
      end;
    Bad={_,Row} ->
      next_state(Bad, WA, go, S)
  end;
do(_, WA, _Srv, S) -> %refresh
  {next_state, ok, go, S}.

do_enter(Mode, HA, WA, Srv, S) ->
  do_select(HA, WA, Srv, S#state{mode=Mode}).

do_after_save(Row, WA, Srv, S=#state{pk_fields=PKs, mode=insert}) ->
  HA = [KV || KV={K, _} <- Row, PK <- PKs, K == PK],
  do_select(HA, WA, Srv, S#state{mode=view});
do_after_save(_R, WA, Srv,
	      S=#state{pk_fields=PKs, old_row=OR, mode=edit}) ->
  HA = [KV || KV={K, _} <- OR, PK <- PKs, K == PK],
  do_select(HA, WA, Srv, S#state{mode=view});
do_after_save(_R, _WA, _Srv, S) ->
  {stop, ok}.

do_select(HA, WA, Srv, S=#state{form=F}) ->
  case read_row(HA, WA, Srv, S) of
    {ok, Data, MissingData} ->
      Row1 = [{N, wslib:gv(N, Data)} || N <- wsform:field_names(F)],
      OutData={[], Row2} = wsform:coerce_types(Row1, in, WA, F),
      next_state(OutData, WA, enter, S#state{old_row=Row2++MissingData});
    Error ->
      wsmessage:show_error(Error, WA),
      {stop, Error}
  end.

read_row(HA, WA, Srv,
	 S=#state{form=F, readonly_fields=RO, pk_fields=PKs, mode=insert}) ->
  EmptyRow = wsform:empty(WA, F),
  FN = wsform:field_names(F),
  MissingKeys =
    [{K, undefined} || K <- PKs++RO, lists:member(K, FN) == false], 
  Data = [{K, wslib:gv(K, HA, V)} || {K, V} <- EmptyRow],
  MissingData = [{K, wslib:gv(K, HA, V)} || {K, V} <- MissingKeys],
  {ok, Data, MissingData};
read_row(HA, WA, Srv, S=#state{pk_fields=PkNs})
  when length(HA) < length(PkNs) ->
  {error, record_not_selected};
read_row(HA, WA, Srv,
	 S=#state{form=F, handler=H,  readonly_fields=RO, pk_fields=PKs}) ->
  FN = wsform:field_names(F),
  MissingKeys =
    [{K, undefined} || K <- PKs++RO, lists:member(K, FN) == false], 
  case call_handler(H, WA, {read_row, HA}) of
    {ok, Data} ->
      MissingData = [{K, wslib:gv(K, Data, V)} || {K, V} <- MissingKeys],
      {ok, Data, MissingData};
    Error ->
      Error
  end.

save_row(Row, WA, Srv, S=#state{mode=insert, handler=H}) ->
  call_handler(H, WA, {insert_row, Row});
save_row(Row, WA, Srv, S=#state{mode=edit, old_row=OR, handler=H}) ->
  call_handler(H, WA, {update_row, Row});
save_row(_Row, WA, Srv, S=#state{mode=delete, old_row=OR, handler=H}) ->
  call_handler(H, WA, {delete_row, OR});
save_row(_Row, WA, Srv, S=#state{mode=view, handler=H}) ->
  ok.

next_state({Errors, Data}, WA, State,
	   S=#state{mode=M, form=F, display=D, old_row=OR,
		    pk_fields=PkNs,
		    readonly_fields=RNs,
		    createonly_fields=CNs}) ->
  Props = case M of
	    edit ->
	      [{mode, M}, {readonly_fields, PkNs++RNs++CNs}];
	    _ ->
	      [{mode, M}, {readonly_fields, RNs}]
	  end,
  {[], OutData} = wsform:calculate(Data, WA, F),
  {next_state, ok,
   D({Errors, OutData}, F, WA, Props) ++ [{'FormType', row}, {'RowData', OR}],
   choose_next_state(State), S}.

choose_next_state(enter) ->
 go;
choose_next_state(go) ->
 go.

event_to_mode(do_add_row)    -> insert;
event_to_mode(do_edit_row)   -> edit;
event_to_mode(do_view_row)   -> view;
event_to_mode(do_delete_row) -> delete;
event_to_mode(_)             -> undefined.

form_input(WA, S=#state{mode=edit, old_row=Old, form=F,
			pk_fields=PKs,
			readonly_fields=RNs,
			createonly_fields=CNs}) ->
  Row = merge_changed_row(wslib:arg_input(WA), Old, PKs++RNs++CNs),
  wsform:input(Row, WA, F);
form_input(WA, S=#state{mode=insert, old_row=Old, form=F,
			readonly_fields=RNs}) ->
  Row = merge_changed_row(wslib:arg_input(WA), Old, RNs),
  wsform:input(Row, WA, F);
form_input(WA, S=#state{mode=delete, old_row=Old}) ->
  {[], Old};
form_input(WA, S=#state{form=F}) ->
  wsform:input(WA, WA, F).

form_output(Form, Data, WA, Props) ->
  Mode = wslib:gv(mode, Props, view),
  FormOut = wsform:output(Data, Props, WA, Form),
  Actions =
    if Mode == view ->
	[do_exit, do_edit];
       true ->
	?row_events
    end,
  [{out, FormOut}, {mode, Mode}, {actions, Actions}].

merge_changed_row(New,
		  S=#state{mode=insert, old_row=OR, readonly_fields=RNs}) ->
  merge_changed_row(New, OR, RNs);
merge_changed_row(New, S=#state{mode=edit, old_row=OR,
				pk_fields=PKs, readonly_fields=RNs}) ->
  merge_changed_row(New, OR, PKs++RNs);
merge_changed_row(_, S=#state{old_row=OR}) ->
  OR.

merge_changed_row(New, Old, ReadOnlyFields) ->
  Missing =
    lists:filter(fun (F={K,_}) ->
		     (lists:member(K, ReadOnlyFields) == true)
		       orelse
			 (lists:keymember(K, 1, New) == false)
		 end, Old),
  Uniq =
    lists:filter(fun (F={K, _}) ->
		     lists:member(K, ReadOnlyFields) == false
		 end, New),
  Missing ++ Uniq.

call_handler(Handler, WA, Msg) ->
  Dbh = wsdb:dbh(WA),
  (catch Handler(Msg, Dbh)).


mk_handler(Hlr, Props) ->
  Hlr1 = mk_handler(Hlr),
  AddHlrs = [mk_handler(H)
	    || H <- proplists:append_values(add_row_handler, Props)],
  add_row_handlers(Hlr1, lists:reverse(AddHlrs)).

mk_handler(Module) when is_atom(Module) ->
  HlrFun =
    fun (Msg, Dbh) ->
	case Msg of
	  {read_row, Pk} ->
	    Module:read_row(Dbh, Pk);
	  {insert_row, Row} ->
	    Module:insert_row(Dbh, Row);
	  {update_row, New} ->
	    Module:update_row(Dbh, New);
	  {delete_row, Row} ->
	    Module:delete_row(Dbh, Row)
	end
    end,
  HlrFun;
mk_handler(Handler) ->
  Handler.

add_row_handlers(Hlr, AddHlrs) ->
  fun (Msg, Dbh) ->
      Fun =
	fun (Db) ->
	    case element(1, Msg) of
	      read_row ->
		{ok, Row} = Hlr(Msg, Db),
		RetRow =
		lists:foldl(fun (H, R) ->
				case H({read_row, R}, Db) of
				  {ok, NR} ->
				    NR;
				  ok ->
				    R
				end
			    end, Row, AddHlrs),
		{ok, RetRow};
	      delete_row ->
		lists:foreach(fun (H) ->
				  R = H(Msg, Db),
				  check_row_hlr_return(R)
			      end,
			      lists:reverse(AddHlrs)),
		Hlr(Msg, Db);
	      _ ->
		Ret = {ok, _} = Hlr(Msg, Db),
		lists:foreach(fun (H) ->
				  R = H(Msg, Db),
				  check_row_hlr_return(R)
			      end,
			      AddHlrs),
		Ret
	    end
	end,
      wsdb:transaction(Dbh, Fun)
  end.

check_row_hlr_return(ok) -> ok;
check_row_hlr_return(R={ok, _}) -> R;
check_row_hlr_return(Error) ->
  exit(Error).
  
show_save_result_message(Num, WA, Mode) ->
  MsgId =
    case Mode of
      insert ->
	'RECORDS_INSERTED';
      edit ->
	'RECORDS_UPDATED';
      delete ->
	'RECORDS_DELETED'
    end,
  wslib:show_lmessage(MsgId, [{num, Num}], WA).
       
