%%% File    : wsform.rowset.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created : 27 Mar 2003 by  <svg@surnet.ru>

%%
%% $Id: rowset.erl,v 1.2 2003/10/27 16:56:19 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wsform.rowset).

-behaviour(wsdcomp).

-export([init/2, access_info/3, handle_event/6]).
-export([enter/4, go/4, new_search/4]).

-import(.lists).
-import(.wsdcomp).
-import(.wsmessage).
-import(.proplists).
-import(.wsform.pager).
-import(.wsform).
-import(.wslib).

-include("wstl.hrl").

-define(events, [do_next_page, do_prev_page, do_refresh, do_search, do_exit]).
-define(rownum_param, rownum).
-define(chunk_param,  rowset_chunk_size).
-define(template_var, 'RowsetOut').

-record(state, {pg,
		search_query,
		pk_fields,
		pks,
		display
	       }).

%%
%% Properties = [{template_var, 'RowsetOut'},
%%               {display, display(data(), [action()], ws_arg())}
%%               ]
%%               

access_info(_WA, _, S=#state{pg=Pg}) ->
  #access_info{events=current_actions(Pg), default_event=do_refresh}.

init({PKs, Form, Pg, Query}, Props) ->
  Display =
    case wslib:gv(display, Props) of
      undefined ->
	TemplateVar = wslib:gv(template_var, Props, ?template_var),
	create_display(Form, TemplateVar);
      D ->
	D
    end,
  Pg1 = create_pager(Pg),
  {ok, enter,
   #state{pg       = Pg1,
	  search_query = Query,
	  pk_fields = PKs,
	  pks      = [],
          display  = Display
	 }}.

handle_event(Event, security_alert, WA, Srv, State, S) ->
  {stop, ok};
%% Old request, no data in output
handle_event(Msg, old_request, WA, Srv, enter, StateData) ->
  enter(enter, WA, Srv, StateData);
handle_event(current_rowid, _, WA, Srv, State, S=#state{pks=PKs}) ->
  In = wslib:arg_input(WA),
  PK =
    case wslib:gv(?rownum_param, In, undefined) of
      undefined ->
	[];
      Num ->
	case catch lists:nth(list_to_integer(Num), PKs) of
	  {'EXIT', _} ->
	    [];
	  V ->
	    V
	end
    end,
  {next_state, PK, State, S};
handle_event(pager_state, _, WA, Srv, State, S=#state{pg=Pg, pks=PKs}) ->
  IsFirst = (pager:pos(Pg) =< 1),
  IsLast  = (not pager:has_more(Pg)),
  Sz = length(PKs),
  {next_state, {IsFirst, IsLast, Sz}, State, S};
handle_event(pk_fields, _, WA, Srv, State, S=#state{pk_fields=PkNs}) ->
  {next_state, PkNs, State, S};
handle_event(state, From, WA, Srv, State, StateData) ->
  {next_state, rowset, State, StateData};
handle_event(Msg, From, WA, Srv, State, StateData) ->
  {next_state, ok, State, StateData}.

enter(_Event, WA, Srv, S=#state{search_query=SQ, pg=Pg}) ->
  {ok, {Query, QueryParams, OrderBy}} = SQ([], WA, []),
  go(do_refresh, WA, Srv,
     S#state{pg=pager:new_query(Query, QueryParams, OrderBy, Pg)}).

go(Event, WA, Srv, S) ->
  do(Event, WA, S).

new_search({ok, {Query, QueryParams, OrderBy}}, WA, Srv, S=#state{pg=Pg}) ->
  go(do_refresh, WA, Srv,
     S#state{pg=pager:new_query(Query, QueryParams, OrderBy, Pg)});
new_search(Ret, WA, Srv, S) ->
  go(do_refresh, WA, Srv, S).

%%
do(do_exit, WA, _S) ->
  {stop, ok};
do(do_search, WA, S=#state{search_query=SQ}) ->
  Comp = wsdcomp:create(wsform.search_query, {SQ, WA}, WA, []),
  {call_exec, enter, Comp, new_search, S};
do(Event, WA,
   S=#state{pg=Pg, display=Display, pk_fields=PkNs}) ->
  In = wslib:arg_input(WA),
  Pg1 = case wslib:gv(?chunk_param, In) of
	  undefined ->
	    Pg;
	  Str ->
	    case catch pager:chunk_size(list_to_integer(Str), Pg) of
	      {'EXIT', _} ->
		Pg;
	      NPg ->
		NPg
	    end
	end,
  Op = event_to_pager(Event),
  {ok, {Rows, NxtPg}} = pager:Op(Pg1),
  PKs = [select_pk(PkNs, Row) || Row <- Rows],
  Out = Display(Rows, current_actions(NxtPg), WA),
  {next_state, ok, Out, go, S#state{pg=NxtPg, pks=PKs}}.

event_to_pager(do_next_page) -> next;
event_to_pager(do_prev_page) -> prev;
event_to_pager(_)   -> current.

create_pager({CursorMod, CursorParams, PgParams}) ->
  Pg = wsform.pager:create(CursorMod, CursorParams, PgParams).

create_display(Form, TmplVar) ->
  fun (Data, Actions, WA) ->
      FormOut = form_output(Form, Data, Actions, WA),
      [{TmplVar, FormOut}, {'FormType', rowset}]
  end.

form_output(Form, Data, Actions, WA) ->
  Name = wsform:name(Form),
  Width = length(wsform:field_names(Form)),
  FieldLabels = wsform:field_labels(Form),
  FormOut =
    {rowset,
     [{label, wsform:label(Form)},
      {field_labels, FieldLabels},
      {attrs, wsform:out_attrs(WA, Form)}
     ],
     [form_row_out(Row, WA, Form) || Row <- Data]
    },
  Out = wsform:render(FormOut,
		      [{mode, rowset},
		       {rownum_name, ?rownum_param},
		       {chunksize_name, ?chunk_param}
		      ],
		      WA, Form),
  [{out, Out}, {mode, rowset}, {form_name, Name}, {actions, Actions},
   {form_width, Width}].

form_row_out(Row, WA, Form) ->
  Row1 = [{N, wslib:gv(N, Row)} || N <- wsform:field_names(Form)],
  {[], Row2} = wsform:coerce_types(Row1, in, WA, Form),
  {[], Row3} = wsform:calculate(Row2, WA, Form),
  wsform:out(Row3, WA, Form).

select_pk(Names, KVs) ->
  lists:reverse(
    lists:foldl(fun (Name, Acc) ->
		    case lists:keysearch(Name, 1, KVs) of
		      {value, KV} ->
			[KV|Acc];
		      _ ->
			Acc
		    end
		end, [], Names)).

current_actions(Pg) ->
  IsFirst = (pager:pos(Pg) =< 1),
  IsLast  = (not pager:has_more(Pg)),
  Events =
    case {IsFirst, IsLast} of
      {true, true} ->
	[do_refresh, do_search, do_exit];
      {false, true} ->
	[do_refresh, do_prev_page, do_search, do_exit];
      {true, false} ->
	[do_refresh, do_next_page, do_search, do_exit];
      {false, false} ->
	?events
    end,
  Events.
