%%%-------------------------------------------------------------------
%%% File    : search_query.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%%
%%% Created : 19 Sep 2003 by  <svg@surnet.ru>
%%%-------------------------------------------------------------------
%%
%% $Id: search_query.erl,v 1.1 2003/10/27 16:45:04 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wsform.search_query).

-behaviour(wsdcomp).

-export([init/2, access_info/3, handle_event/6]).
-export([enter/4,go/4]).
-export([mk_query/4]).

-import(.lists).
-import(.io_lib).
-import(.wsdcomp).
-import(.wsform).
-import(.wsform_lib).
-import(.wslib).
-import(.stl_yaws).
-import(.wsdb).
-import(.wslib.listn).

-include("wstl.hrl").

-define(template,   "lib/editrow").	   
-define(template_var, 'RowOut').
-define(row_events, [do_search, do_exit]).

access_info(_, _, _) ->
  #access_info{events=?row_events, default_event=refresh}.

handle_event(Event, security_alert, WA, Srv, State, StateData) ->
  {stop, ok};
handle_event(Msg, From, WA, Srv, State, StateData) ->
  {next_state, ok, State, StateData}.

mk_query(MkQuery, Form, WA, Props) ->

  [QueryFields, NonQueryFields, OrderByFields] =
    wslib:gv_map([{query_fields, wsform:field_names(Form)},
		  {nonquery_fields, []},
		  {order_by_fields, []}],
		 Props),
  TmplVar = wslib:gv(template_var, Props, ?template_var),
  Display =
    case wslib:gv(display, Props) of
      undefined ->
	create_display(TmplVar);
      D ->
	D
    end,
  QueryFields1 = QueryFields -- NonQueryFields,
  fun (undefined, WsArg, DisplayProps) ->
      Data = wsform_lib:order_fields([{FN, undefined} || FN <- QueryFields1],
				     wsform:field_names(Form)),
      {display, Display({[], Data}, Form, WsArg, DisplayProps)};
      (Data, WsArg, DisplayProps) ->
      case form_input(Data, QueryFields1, Form, WsArg) of
	Good={[], KVs} ->
	  In = wslib:arg_input(WsArg),
	  KOVs = [{{op_param_val(K, In), K}, {K, V}} ||
		   {K, V} <- KVs,
		   V =/= undefined],
	  {QueryWhere, QueryParams} = listn:unzip(KOVs),
	  Query = MkQuery(QueryWhere),
	  {ok, {Query, QueryParams, OrderByFields}};
	Bad ->
	  Out =
	    Display(Bad, Form, WsArg, DisplayProps),
	  {display, Out}
      end
  end.

init(SearchQuery, _) when is_function(SearchQuery) ->
  {ok, enter, SearchQuery};
init({SearchQuery, WA}, Props) ->
  RenderProps = [{template, ?template}, {place_at,  ?template_var}|Props], 
  Comp = wsdcomp:create(?MODULE, SearchQuery, WA, RenderProps),
  {component, Comp};
init({MkQuery, Form, WA}, Props) ->
  SearchQuery = mk_query(MkQuery, Form, WA, Props),
  init({SearchQuery, WA}, Props).

enter(_, WA, Srv, SQ) ->
  {display, Out} = SQ(undefined, WA, []),
  {next_state, ok, Out, go, SQ}.

go(do_exit, _, _, _) ->
  {stop, ok};
go(do_search, WA, Srv, SQ) ->
  case SQ(WA, WA, []) of
    Ret={ok, _} ->
      {stop, Ret};
    {display, Out} ->
      {next_state, ok, Out, go, SQ}
  end;
go(_, WA, Srv, S) ->
  {next_state, ok, go, S}.

op_param_name(N) ->
  list_to_atom(lists:flatten(io_lib:format("~s.op", [N]))).

op_param_val(K, KVs) ->
  list_to_atom(wslib:gv(op_param_name(K), KVs, "=")).

create_display(TmplVar) ->
  fun (Data, Form, WA, Props) ->
      Out = [{TmplVar, form_output(Form, Data, WA, Props)}, {'FormType', row}]
  end.

form_input(Data, QueryFields, Form, WA) ->
  FN = wsform:field_names(Form),
  {Bad, Good} = wsform:input(Data, WA, [data_type], Form),
  Good1 = wsform_lib:order_fields(
	    [KV || KV={K,_} <- Good, lists:member(K, QueryFields) == true],
	    FN),
  Bad1 = [KV || KV={K,_} <- Bad, lists:member(K, QueryFields)],
  {Bad1, Good1}.

form_output(Form, Data, WA, Props) ->
  Props1 = [{mode, search}, {make_op_param_name, fun op_param_name/1}|Props],
  FormOut = wsform:output(Data, Props1, WA, Form),
  Actions = ?row_events,
  [{out, FormOut}, {mode, search}, {actions, Actions}].
