%%
%% $Id: ss.erl,v 1.3 2003/10/27 22:04:10 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wsform.ss).

-import(wsxmlt).
-import(wslib).
-import(wslib.html).
-import(wslib.str).
-import(wslib.listn).
-import(wslanguage).
-import(proplists).
-import(lists).

-export([form/3, form/4, form_field/2,form_field/3,form_field/5,readonly/1,readonly/2]).
-export([form_post_handler/1, form_row/2]).
-export([form_field_pre_handler/2, form_field_post_handler/3]).
-export([field_content_rules/1, field_value/3]).
-export([datetime/0, date/0, time/0, date/1, boolean/1, string/0, integer/0, text/0, selector/2]).
-export([interval/3, phone_number/0]).

readonly(Rule) ->
  readonly(Rule, [edit, insert, search]).

readonly(Rule={Test, {Pre, Post, ContentRules}}, Modes) ->
  {Test,
   {fun (N, S) ->
	case lists:member(wsxmlt:get_var(mode, S), Modes) of
	  false ->
	    ok;
	  true ->
	    wsxmlt:new_var(mode, view, S)
	end,
	Pre(N, S)
    end,
    Post,
    ContentRules
   }
  }.

form(Name, FieldRules, WA) ->
  form(Name,
       form_post_handler(WA),
       fun form_row/2,
       FieldRules).

form(FormName, PostHandler, MkRowRule, FieldRules) ->
  wsxmlt:r(wsxmlt:m_name([rowset, FormName]),
	   {PostHandler,
	    MkRowRule(FormName, FieldRules)
	   }).

form_post_handler(WA) ->
  ChunkSizeMsg = wslanguage:msg(WA, chunk_size, []),
  fun ({rowset, Args}, Content, Seed) ->
  TableHeader =
    {tr, [{class, row_class(0)}],
     [{th, [], "#"}|[{th, [], L}
		     || {_, L} <- wslib:kv(field_labels, Args)]]
    },
  SizeName = atom_to_list(wsxmlt:get_var(chunksize_name, Seed)),
  ColNum = integer_to_list(length(wslib:kv(field_labels, Args)) +1),
  RowsetHeader =
    {tr, [{class, "caption"}],
	   [{td, [{colspan, ColNum}],
	     [{table, [{width, "100%"}, {border, "0"}],
	       [{tr, [{class, "caption"}],
		 [{th,[{align, "left"}], [wslib:kv(label, Args)]},
		  {th, [{align, "right"}],
		   [{small, [], ChunkSizeMsg},
		    {input, 
		     [{name, SizeName}, {maxlength, "4"}, {size, "4"}, {type, "text"}]}
		   ]}
		 ]}
	       ]}
	     ]}
	   ]},
  {table, 
   [{width, "100%"}],
   [TableHeader, RowsetHeader|Content]
  };
      ({_Name, Args}, Content, Seed) ->
      {Content1, _} =
	lists:mapfoldl(fun ({tr, As, C}, Cnt) ->
			   {{tr, [{class, row_class(Cnt)}|As], C}, Cnt+1}
		       end,
		       1, Content),
      {table, 
       [],
       [{caption, [],[wslib:kv(label, Args)]}|Content1]
      }
  end.

form_row(FormName, FieldRules) ->
  FieldRules1 = listn:mk_list(FieldRules)++[wsxmlt:skip()],
  fun ({rowset, _}, _Seed) ->
      [wsxmlt:r(
	 FormName,
	 {fun (_Node, Content, Seed) ->
	      SibPos = wsxmlt:sibling_pos(Seed),
	      NumName = wsxmlt:get_var(rownum_name, Seed),
	      RowNum = html:radio_button(NumName, integer_to_list(SibPos), []),
	      Row = {tr, [{class, row_class(SibPos)}],
		     [{td, [], [RowNum]}|Content]
		    }
	  end,
	  FieldRules1
	 })];
      (_Node, _Seed) ->
      FieldRules1
  end.

form_field(Test, WA) ->
  form_field(Test, field_content_rules(WA), WA).

form_field(Test, ContentRules, WA) ->
  form_field(Test,
	     fun form_field_pre_handler/2,
	     fun form_field_post_handler/3,
	     ContentRules, WA).

form_field(Test, PreHlr, PostHlr, ContentRules, WA) ->
  wsxmlt:r(Test, {PreHlr, PostHlr, ContentRules}).

form_field_pre_handler({N, _}, S) ->
  Mode = wsxmlt:get_var(mode, S),
  if (Mode == edit) or (Mode == insert) ->
      ExcludeVar = readonly_fields,
      case wsxmlt:find_var(ExcludeVar, S) of
	{ok, RNs} ->
	  case lists:member(N, RNs) of
	    true ->
	      wsxmlt:new_var(mode, view, S);
	    false ->
	      ok
	  end;
	false ->
	  ok
      end;
     true ->
      ok
  end.

form_field_post_handler({Name, As}, C, S) ->
  Mode = wsxmlt:get_var(mode, S),
  case Mode of
    rowset ->
      {td, [], C};
    _ ->
      {tr, [],
       [{td, [], [wslib:kv(label, As)]}|
	if Mode == search ->
	    [{type, FieldType}|CRest] = C,
	    [{td, [], [search_op_selector(FieldType, Name, S)]},
	     {td, [], CRest}
	    ];
	   (Mode == edit) or (Mode == insert) ->
	    [{td, [], tl(C)}];
	   true -> % view
	    [{td, [], C}]
	end
       ]}
  end.

field_content_rules(WA) ->
  [datetime(), .wsform.ss:date(), .wsform.ss:time(),
   boolean(WA), integer(), string(), text()].

date() ->
  date(date).

datetime() ->
  date(datetime).

time() ->
  date(time).

date(SubType) ->
  field_value(SubType,
	      fun (_, _, _) -> SubType end,
	      fun (_, {_, [{value, T}]}, _, S) ->
		  case T of
		    undefined ->
		      "";
		    _ ->
		      odbcdb.attr_type:coerce_type(out, SubType, T)
		  end
	      end
	     ).

boolean(WA) ->
  TrueL = wslanguage:msg(WA, true, []),
  FalseL = wslanguage:msg(WA, false, []),
  selector(wsxmlt:m_name(boolean), [{"true", TrueL}, {"false", FalseL}], []).

string() ->
  field_value(string,
	      fun (_, {Type, _}, _) -> Type end,
	      fun (_, _, Str, _) -> Str end
	     ).

text() ->
  field_value(wsxmlt:m_text(),
	      fun (_, _, _) -> string end,
	      fun (_, Str, _, _) -> Str end
	     ).

integer() ->
  field_value(integer,
	      fun (_, {Type, _}, _) -> Type end,
	      fun (_, _, Str, _) -> Str end
	     ).

selector(SelList, SelOpts) ->
  MatchText = wsxmlt:m_text(),
  MatchType = wsxmlt:m_name([string, integer]),
  Match =
    fun (N, S) ->  MatchType(N, S) orelse MatchText(N, S) end,
  selector(Match, SelList, SelOpts).

selector(Match, SelList, SelOpts) ->
  SelAttrs = proplists:get_value(selector_attrs, SelOpts, []),
  wsxmlt:r(
    Match,
    fun (N, Str, S) ->
	Str1 = if is_tuple(N) -> Str;
		  true -> N %text
	       end,
	%% content is always wrapped to list
	Str2 = case Str1 of
		 [St] ->
		   St;
		 [] ->
		   ""
	       end,
	case wsxmlt:get_var(mode, S) of
	  Mode when Mode == insert;
		    Mode == edit;
		    Mode == search ->
	    {Selected, SelList1} =
	      if Mode == search ->
		  {"", [{"", ""}|SelList]};
		 true ->
		  {Str2, SelList}
	      end,
	    [{FieldName, FieldAttrs}|_] = wsxmlt:ancestors(S),
	    [{type, selector},
	     html:selector(FieldName, SelList1, [Selected], SelAttrs)
	    ];
	  _ ->
	    wslib:gv(Str2, SelList, Str2)
	end
    end
   ).

phone_number() ->
  field_value(
    string,
    fun (_, _, _) -> string end,
    fun (_, _, [Str], S) ->
	PhoneStr = field_type:format(phone_number, Str),
	[PhoneStr];
	(_, _, Content=[], S) ->
	Content
    end
   ).

interval(FromName, ToName, WA) ->
  interval(FromName, ToName, FromName, field_content_rules(WA), WA).

interval(FromName, ToName, TempVar, ValueRules, WA) ->
  form_field(
    wsxmlt:m_name([FromName, ToName]),
    fun (Node, Seed) ->
	case wsxmlt:find_var(TempVar, Seed) of
	  false ->
	    wsxmlt:new_upvar(TempVar, undefined, 1, Seed);
	  _ ->
	    ok
	end,
	form_field_pre_handler(Node, Seed)
    end,
    fun ({Name, Args}, Content, Seed) ->
	case wsxmlt:get_var(TempVar, Seed) of
	  undefined ->
	    wsxmlt:set_var(TempVar, {Name, Args, Content}, Seed),
	    [];
	  Node ->
	    interval_fmt(FromName, {Name, Args, Content}, Node, Seed)
	end
    end,
    ValueRules,
    WA
   ).

interval_fmt(FromName, Node1, Node2, Seed)
  when element(1, Node1) == FromName ->
  interval_fmt(Node1, Node2, Seed);
interval_fmt(_, Node1, Node2, Seed) ->
  interval_fmt(Node2, Node1, Seed).

interval_fmt({FromName, FromAs, FromContent}, {ToName, _, ToContent}, Seed) ->
  ToContent1 =
    begin
      Mode = wsxmlt:get_var(mode, Seed),
      case Mode of
	rowset ->
	  "-"++ToContent;
	_ ->
	  ["-",
	   if Mode == search ->
	       [{type, FieldType}|Rest] = ToContent,
	       [search_op_selector(FieldType, ToName, Seed)|Rest];
	      (Mode == edit) or (Mode == insert) ->
	       tl(ToContent);
	      true -> % view
	       ToContent
	   end
	  ]
      end
    end,
  form_field_post_handler({FromName, FromAs}, FromContent++ToContent1, Seed).

row_class(Row) ->
  "rowset"++ integer_to_list(Row band 16#1).

search_op_selector(selector, _, _) ->
  "";
search_op_selector(boolean, _, _) ->
  "";
search_op_selector(FieldType, FieldName, Seed) ->
  MkName = wsxmlt:get_var(make_op_param_name, Seed),
  {Default, SelList} = search_op_sel_list(FieldType),
  html:selector(MkName(FieldName), SelList, [Default], []).

search_op_sel_list(Type) when Type == date;
			      Type == datetime;
			      Type == integer;
			      Type == float ->
  {"=", [{"=", "="}, {"!=", "!="}, {">", ">"}, {"<", "<"}]};
search_op_sel_list(_Type) -> % string|text
  {"=", [{"=", "="}, {"like", "~"}, {"!=", "!="}]}.

field_value(Match, GetType, GetValue) ->
  wsxmlt:r(
    Match,
    fun (Node, Content, Seed) ->
	Mode = wsxmlt:get_var(mode, Seed),
	Val = GetValue(Mode, Node, Content, Seed),
	case Mode of
	  _ when Mode == insert;
		 Mode == edit;
		 Mode == search ->
	    [{FieldName, FieldAttrs}|_] = wsxmlt:ancestors(Seed),
	    [{type, GetType(Mode, Node, Seed)},
	     {input, [{name, atom_to_list(FieldName)},
		      {value, Val}
		      |wslib:kv(attrs, FieldAttrs)]}
	    ];
	  _ ->
	    Val
	end
    end).
