%%%-------------------------------------------------------------------
%%% File    : weekday.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%%
%%% Created :  8 Jan 2004 by  <svg@surnet.ru>
%%%-------------------------------------------------------------------
%%
%% $Id$
%%
%%**
%% 
%% @doc
%% week is represented as integer,
%% workday mask = 0
%% weekend mask = 1
%% @end
%%*
-module(wsform.weekday).

-export([data/2, data_type/3, default/0, ss/2, validate/1]).
-export([is_restday/2, is_weekday/2]).

-import(calendar).
-import(lists).
-import(wslib.cal).
-import(wslib.str).
-import(wslib.props).
-import(wslib.html).
-import(wsxmlt).
-import(wslanguage).

is_restday(Day, WeekMask) ->
  cal:week_is_restday(Day, WeekMask).

is_weekday(Day, WeekMask) ->
  cal:week_is_weekday(Day, WeekMask).

data(Name, KVs) ->
  WeekMask = 0,
  lists:foldl(fun ({N, DayName}, Acc) when N == Name,
					   is_list(DayName) ->
		  case catch {ok, cal:day_num(DayName)} of
		    {ok, DayNum} ->
		      cal:week_set_restday(DayNum, Acc);
		    _ ->
		      Acc
		  end;
		  (_, Acc) ->
		  Acc
	      end, WeekMask, KVs).

data_type(in, WeekMask, A) ->
  field_type:integer(in, WeekMask, A);
data_type(out, undefined, _) ->
  [{weekday, [], []}];
data_type(out, WeekMask, _) ->
  RestDays =
    lists:foldl(fun(DayNum, Acc) ->
		    case is_weekday(DayNum, WeekMask) of
		      true ->
			Acc;
		      false ->
			[cal:day_name(DayNum)|Acc]
		    end
		end, [], lists:seq(1, 7)),
  [{weekday, [], lists:reverse(RestDays)}].
      

validate(V) when is_integer(V),
		 V >= 0,
		 V =< 127 ->
  true;
validate(_V) ->
  false.

default() ->
  0.

ss(Name, WA) ->
  KVs = lists:map(fun (DayNum) ->
		      DN = cal:day_name(DayNum),
		      {DN, wslanguage:msg(WA, DN)}
		  end, lists:seq(1, 7)),
  ss:form_field(
    wsxmlt:m_name(Name),
    [wsxmlt:r(
       wsxmlt:m_name(weekday),
       fun (_Node, RestDays, Seed) ->
	   Mode = wsxmlt:get_var(mode, Seed),
	   case Mode of
	     _ when Mode == insert;
		    Mode == edit;
		    Mode == search ->
	       [{_, FieldAttrs}|_] = wsxmlt:ancestors(Seed),
	       Attrs = 
		 [{type, checkbox},{orientation, horizontal}
		  |props:gv(attrs, FieldAttrs,[])],
	       [{type, selector}, html:selector(Name, KVs, RestDays, Attrs)];
	     _ ->
	       str:join([props:gv(RD, KVs) || RD <- RestDays], " ")
	   end
       end
      )
    ],
    WA
   ).
