%%% File    : wsform_lib.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : Default wsform callbacks
%%% Created : 21 Feb 2003 by  <svg@surnet.ru>

%%
%% $Id: wsform_lib.erl,v 1.8 2003/10/31 17:57:31 vsekissov Exp $
%%
%%**
%% 
%% @doc Default wsform callbacks
%% @end
%%*
-module(wsform_lib).

-export([def_value/2, def_data_type/2, def_data_filter/3, def_label/2]).
-export([def_data/3, def_field_out/5, def_field_out_attrs/2, def_validate/4]).
-export([def_calculate/4, def_form_calculate/3]).
-export([def_form_render/4, def_form_label/2, def_form_out_attrs/2]).
-export([def_form_out/3, language_label/2, mk_language_label/1]).
-export([def_form_data/3, def_form_output/4, def_form_input/4]).
-export([order_fields/2]).

-import(wslib.str).
-import(wslib.listn).

-include("wstl.hrl").

%% Form defaults
def_form_input(In, FieldNames, FieldMap, Arg) ->
  {Errors, Good} = FieldMap(In, Arg),
  Ret =
    case Errors of
      [] ->
	Good;
      _ ->
	Ret0 =
	  lists:foldl(fun (N, Acc) ->
			  [{N, {badval, kv(N, In, [])}}|Acc]
		      end,
		      Good, [N || {N, _} <- Errors]),
	order_fields(Ret0, FieldNames)
    end,
  {Errors, Ret}.

def_form_label(Name, Arg) ->
  def_label(Name, Arg).

def_form_out_attrs(_Form, _Arg) ->
  [].

def_form_output(Form, {Errors, In}, RenderArgs, Arg) ->
  if (Errors =/= []) and is_record(Arg, ws_arg) ->
      lists:foreach(fun (E) -> wsmessage:show_error(E, Arg) end, Errors);
     true ->
      ok
  end,
  Out = wsform:out(In, Arg, Form),
  wsform:render(Out, RenderArgs, Arg, Form).

def_form_data(_Form, WsArg, _Arg) when record(WsArg, ws_arg) ->
  wslib:arg_input(WsArg);
def_form_data(_Form, Data, _Arg) ->
  Data.

def_form_calculate(Form, KVs, Arg) ->
  Arg.

def_form_out(Form, FieldOuts, Arg) ->
  ?dbg("fields out ~p~n", [FieldOuts]),
  FieldOuts1 = [O || {_, O} <- FieldOuts],
  Label = wsform:label(Form),
  Attrs = wsform:out_attrs(Arg, Form),
  FieldLabels = wsform:field_labels(Form),
  Name = wsform:name(Form),
  {Name, [{label, Label}, {field_labels, FieldLabels}, {attrs, Attrs}],
   FieldOuts1}.

def_form_render(Ss, Tree, Ps, _Arg) ->
  Ehtml = wsxmlt:transform(Ss, Ps, Tree),
  {ehtml, Ehtml}.

%% Field defaults
language_label(FieldName, WA) ->
  L = wslanguage:msg(WA, FieldName, []).

mk_language_label(FormName) ->
  fun (FieldName, WA) ->
      case wslanguage:find_msg(WA, {FormName, FieldName}, []) of
	undefined ->
	  wslanguage:msg(WA, FieldName, []);
	Msg ->
	  Msg
      end
  end.

def_label(Name, _Arg) ->
  str:join([str:capitalize(S) 
	    || S <- string:tokens(atom_to_list(Name), "_")], $\s).

def_field_out_attrs(_Name, _Arg) ->
  [].

def_field_out(Name, undefined, Label, Attrs, _Arg) ->
  {Name, [{label, Label}, {attrs, Attrs}], ""};
def_field_out(Name, {badval, Value}, Label, Attrs, _Arg) ->
  IsStr = io_lib:deep_char_list(Value),
  Value1 = if  IsStr ->
	       Value;
	       Value == undefined ->
	       "";
	       true ->
	       io_lib:fromat("~w", Value)
	   end,
  {Name, [{label, Label}, {attrs, [{class, "error"}|Attrs]}], [Value1]};
def_field_out(Name, Value, Label, Attrs, _Arg) ->
  {Name, [{label, Label}, {attrs, Attrs}], Value}.

def_data(Name, KVs, _Arg) ->
  case kv(Name, KVs) of
    "" ->
      undefined;
    V ->
      V
  end.

def_data_filter(_N, V, _Arg) when is_list(V) ->
  str:trim(V, " \t\n\r\f");
def_data_filter(_N, V, _Arg) ->
  V.

def_data_type(N, _Arg) ->
  fun(Direction, V, _A) ->
      V
  end.

def_validate(_N, _V, _KVs, _Arg) ->
  true.

def_calculate(_N, V, _KVs, _Arg) ->
  {ok, V}.

def_value(_N, _Arg) ->
  undefined.

order_fields(KVs, FNs) ->
  Cmp =
    fun ({A, _}, {B, _}) ->
	cmp_fields(A, B, FNs)
    end, 
  lists:sort(Cmp, KVs).

cmp_fields(A, B, []) ->
  false;
cmp_fields(A, B, [A|_]) ->
  true;
cmp_fields(A, B, [B|_]) ->
  false;
cmp_fields(A, B, [_|FNs]) ->
  cmp_fields(A, B, FNs).

kv(K, L) ->
  proplists:get_value(K, L).

kv(K,L, D) ->
  case proplists:get_value(K, L, D) of
    undefined ->
      D;
    V ->
      V
  end.
