%%% File    : wslanguage.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : Localization support
%%% Created : 13 May 2003 by  <svg@surnet.ru>

%%
%% $Id: wslanguage.erl,v 1.2 2003/10/31 17:57:31 vsekissov Exp $
%%
%%**
%% 
%% @doc Localization support
%% @end
%%*
-module(wslanguage).

-export([start_link/2, service_spec/3]).
-export([msg/2, msg/3, find_msg/3, charset/1, language/1]).

-import(wslib.http).

-include("wstl.hrl").

-define(default_props, [{src, [{wslanguage_phrases, all}]},
			{default_lang, en},
			{default_charset, 'iso-8859-1'}
		       ]).

-record(lang_arg, {server,
		   lang,
		   charset,
		   default_lang,
		   default_charset}).

msg(WA, Name) ->
  msg(WA, Name, []).

msg(WA, Name, Params) ->
  case msg_find(get_env(WA), Name) of
    undefined when is_list(Name) ->
      io_lib:format("~s", [Name]);
    undefined ->
      io_lib:write(Name);
    Fun ->
      Fun(Params)
  end.

find_msg(WA, Name, Params) ->
  case msg_find(get_env(WA), Name) of
    undefined ->
      undefined;
    Fun ->
      Fun(Params)
  end.

charset(WA) ->  
  (get_env(WA))#lang_arg.charset.

language(WA) ->  
  (get_env(WA))#lang_arg.lang.

start_link(N={_, Name}, Options) ->
  phrase.ets_dict:start_link(N, Name, Options).

service_spec(AppName, Proc, Props) ->
  [Src, DefLang, DefCharset] = 
    [proplists:get_value(K, Props, D) || {K, D} <- ?default_props],
  PhraseProps = [{src, Fun} || Fun <- Src],
  LA = #lang_arg{server=Proc,
		 default_lang=DefLang,
		 default_charset=DefCharset},
  InitEnv = [mk_init_env(LA)],
  {ok, PhraseProps, InitEnv}.

mk_init_env(Env=#lang_arg{server=Serv,
			  default_lang=DL,
			  default_charset=DC}) ->
  fun (WA=#ws_arg{yaws_arg=YA, ws_env=WE}) ->
      UserLangs = header_value('Accept-Language', YA, [DL]),
      {Lang, CharSet} = get_current_env(Serv, UserLangs, {DL, DC}),
      NewEnv = Env#lang_arg{lang=Lang, charset=CharSet},
      WA#ws_arg{ws_env=[{?MODULE, NewEnv}|WE]}
  end.

get_env(#ws_arg{ws_env=WE}) ->
  case proplists:get_value(?MODULE, WE) of
    undefined ->
      exit({service_not_started, ?MODULE});
    V ->
      V
  end.

get_current_env(Serv, Langs, Def) ->
  case get_current_env(Serv, Langs) of
    undefined ->
      Def;
    Ret ->
      Ret
  end.

get_current_env(Srv, []) ->
  undefined;
get_current_env(Srv, [Lang|Rest]) ->
  case catch phrase:msg(Srv, {accept_language, Lang}) of
    {'EXIT', _} ->
      get_current_env(Srv, Rest);
    MsgFun ->
      {CharSet, _, _} = MsgFun([]),
      {Lang, CharSet}
  end.

header_value(Name, Arg, Def) ->
  case http:header_value(Name, Arg) of
    undefined ->
      Def;
    V ->
      V
  end.

msg_find(LA=#lang_arg{server=Srv, lang=L, default_lang=DL}, Name) ->
  msg_find(Srv, Name, msg_langs([L, DL])).

msg_find(Srv, Name, []) ->
  undefined;
msg_find(Srv, Name, [Lang|Rest]) ->
  case phrase:find_msg(Srv, {Lang, Name}) of
    undefined ->
      msg_find(Srv, Name, Rest);
    MsgFun ->
      MsgFun
  end.
      
msg_langs(Ls) ->
  lists:flatmap(fun (V={Lang, Country}) ->
		    [V, Lang];
		    (Lang) ->
		    [Lang]
		end, Ls).
