%%%-------------------------------------------------------------------
%%% File    : wslib.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : Common functions
%%%
%%% Created : 12 Feb 2003 by  <svg@surnet.ru>
%%%-------------------------------------------------------------------
%%
%% $Id: wslib.erl,v 1.8 2003/10/31 17:57:31 vsekissov Exp $

%%**
%% 
%% @doc Common functions
%% @end
%%*
-module(wslib).

-export([approot_path/1, arg_input/1, is_nil/1, undef_to_nil/1, int/1]).
-export([c_new/0, c_set/2, c_set/3, c_val/2, c_val/3, c_del/2]).
-export([kv/2, kv/3, gv/2, gv/3, gv_map/2, v_set/3, v_set_undef/3, v_update/3]).
-export([k_delete/2, k_delete_map/2]).
-export([app_env_val/2, app_env_val/3, app_env_set/3]).
-export([ws_arg_kvs/1, show_lmessage/3, show_lerror/3]).
-export([mk_dummy_render/1]).

-import(wslib.listn).

-include("wstl.hrl").

mk_dummy_render(State) ->
  fun (state) ->
      State;
      ({inherit, NxtState}) ->
      mk_dummy_render(NxtState);
     ({display, _, Out}) ->
      Out
  end.

show_lmessage(Key, Params, WA) ->
  wsmessage:show_message(wslanguage:msg(WA, Key, Params), WA).

show_lerror(Key, Params, WA) ->
  wsmessage:show_error(wslanguage:msg(WA, Key, Params), WA).

approot_path(WA=#ws_arg{application=A}) ->
  [$/|atom_to_list(A)].

arg_input(WA=#ws_arg{yaws_arg=YA}) ->
  arg_input(YA);
arg_input(Arg) ->
  Inp =
    case (Arg#arg.req)#http_request.method of
      'GET' ->
	yaws_api:parse_query(Arg);
      _ ->
	yaws_api:parse_post(Arg)
    end,
  [{list_to_atom(K), V} || {K, V} <- Inp].

ws_arg_kvs(Ctxt) ->
  listn:zip(record_info(fields, ws_arg),
	    tl(tuple_to_list(stl_yaws:ws_arg(Ctxt)))).

undef_to_nil(KVs) when list(KVs) ->
  [undef_to_nil(KV) || KV <- KVs];
undef_to_nil({K, undefined}) -> {K, []};
undef_to_nil(undefined) -> [];
undef_to_nil(V) -> V.

is_nil([]) -> true;
is_nil(_) -> false.

int(I) ->
  stl:int(I).

c_new() ->
  stl:context_new().

c_val(K, Ctxt) ->
  stl:context_value(K, Ctxt).

c_val(K, D, Ctxt) ->
  stl:context_value(K, D, Ctxt).

c_set(L, Ctxt) ->
  stl:context_set(L, Ctxt).

c_set(K, V, Ctxt) ->
  stl:context_set(K, V, Ctxt).

c_del(K, Ctxt) ->
  stl:context_delete(K, Ctxt).

app_env_val(K, WA=#ws_arg{app_env=AE}) ->
  gv(K, AE).

app_env_val(K, WA=#ws_arg{app_env=AE}, D) ->
  gv(K, AE, D).

app_env_set(K, V, WA=#ws_arg{app_env=AE}) ->
  WA#ws_arg{app_env=v_set(K, V, AE)}.

kv(K, L) ->
  proplists:get_value(K, L, []).

kv(K, L, D) ->
  case proplists:get_value(K, L, D) of
    undefined ->
      D;
    V ->
      V
  end.

gv(K, L) ->
  proplists:get_value(K, L).

gv_map(Ks, L) ->
  lists:map(fun ({K, D}) ->
		gv(K, L, D);
		(K) ->
		gv(K, L)
	    end, Ks).

gv(K, L, D) ->
  proplists:get_value(K, L, D).

v_set(K, V, L) ->
  [{K, V}|proplists:delete(K, L)].

v_set_undef(K, V, L) ->
  case proplists:is_defined(K, L) of
    false ->
      [{K, V}|L];
    true ->
      L
  end.

k_delete(Name, L) ->
  proplists:delete(Name, L).

k_delete_map(Ns, L) ->
  Test = fun ({K, _}) -> lists:member(K, Ns);
	     (K) -> lists:member(K, Ns)
	 end,
  [E || E <- L, Test(E) == false].

v_update(Name, UpdFun, List) ->
  {_, V} = proplists:lookup(Name, List),
  lists:keyreplace(Name, 1, List, {Name, UpdFun(V)}).
