%%%-------------------------------------------------------------------
%%% File    : wslogin.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : User login form
%%%
%%% Created : 25 Mar 2003 by  <svg@surnet.ru>
%%%-------------------------------------------------------------------
%%
%% $Id: wslogin.erl,v 1.5 2003/10/31 17:57:31 vsekissov Exp $
%%
%%**
%% 
%% @doc User login form
%% @end
%%*
-module(wslogin).

-behaviour(wsdcomp).

-export([init/2, access_info/3, handle_event/6]).

-export([enter/4, update_browser_cookie/4]).

-include("wstl.hrl").

-record(state, {data=[{login_id, []}, {password, []}], user}).

access_info(_, _, _) ->
  #access_info{events=[do_login, do_exit], default_event=refresh}.


init(_WA, _Props) ->
  {ok, enter, #state{}}.

handle_event(_Msg, _From, WA, Srv, _State, S) ->
  enter(refresh, WA, Srv, S).

enter(refresh, WA, Srv, S=#state{data=D}) ->
  {next_state, ok, gen_out(D), enter, S};

enter(_, WA=#ws_arg{user=U}, Srv, S) when U =/= undefined ->
  wsmessage:show_message({already_logged_in, U}, WA),
  {stop, ok};

enter(do_login, WA, Srv, S) ->
  {Errors, KVs} = input_values(WA),
  if Errors == [] ->
      LoginId = kv(login_id, KVs),
      Password = kv(password, KVs),
      NxtS = S#state{data=KVs},
      case authenticate(WA, LoginId, Password) of
	{ok, User} ->
	  create_session(User, WA, NxtS#state{user=User});
	Error ->
	  wsmessage:show_error(Error, WA),
	  {next_state, ok, gen_out(KVs), enter, NxtS}
      end;
     true ->
      lists:foreach(fun (E) -> wsmessage:show_error(E, WA) end, Errors),
      NxtS = S#state{data=KVs},
      {next_state, ok, gen_out(KVs), enter, NxtS}
  end;

enter({login, LoginId, Password}, WA, Srv, S) ->
  KVs = [{login_id, LoginId}, {password, Password}],
  enter(refresh, WA, Srv, S#state{data=KVs});

enter(do_exit, WA, Srv, StateData) ->
  {stop, {error, cancel}, []}.

update_browser_cookie(_, WA, Srv, S) ->
  {stop, ok, []}.

%%
authenticate(WA, LoginId, Password) ->
  wsacl:login(WA, LoginId, Password, WA#ws_arg.app_env).

create_session(User, WA=#ws_arg{yaws_arg=YA}, S) ->
  {abs_path, AbsPath} = (YA#arg.req)#http_request.path,
  Path =
    case wscc:dec_path(AbsPath) of
      {ok, U=#cc_url{path=P}} ->
	P;
      _ ->
	AbsPath
    end,
  Out = [{'DoRedirect', true},
         {'RedirectPath', Path},
	 {'SessionCookie', wsession:create(WA, User, [])}],
  {next_state, ok, Out, update_browser_cookie, S}.

input_values(WA) ->
  Input = wslib:arg_input(WA),
  lists:foldl(fun ({K, V}, {Es, KVs}) when V == [] ->
		  {[{error, {K, required}}|Es], [{K, []}|KVs]};
		  (KV, {Es, KVs}) ->
		  {Es, [KV|KVs]}
	      end, {[], []},
	      [{K, wslib:undef_to_nil(kv(K, Input))}
	       || K <- [login_id, password]]).

kv(K, L) ->
  proplists:get_value(K, L).

gen_out(Data) ->
  [{'NoLeft', true}|[{key_to_var(K), V} || {K, V} <- Data]].

key_to_var(login_id) -> 'LoginId';
key_to_var(password) -> 'Password'.
  
