%%%-------------------------------------------------------------------
%%% File    : wslogout.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : User logout form
%%%
%%% Created : 25 Mar 2003 by  <svg@surnet.ru>
%%%-------------------------------------------------------------------
%%
%% $Id: wslogout.erl,v 1.5 2003/10/31 17:57:31 vsekissov Exp $
%%
%%**
%% 
%% @doc User logout form
%% @end
%%*
-module(wslogout).

-behaviour(wsdcomp).

-export([init/2, access_info/3, handle_event/6]).

-export([enter/4]).
-export([update_browser_cookie/4]).

-include("wstl.hrl").

-record(state, {data=[], redirect}).

access_info(_, _, _) ->
  #access_info{events=[do_logout, do_exit], default_event=refresh}.


init(WA=#ws_arg{yaws_arg=YA}, Props) ->
  In = wslib:arg_input(WA),
  {abs_path, AbsPath} = (YA#arg.req)#http_request.path,
  RedirPath =
    case  wscc:dec_path(AbsPath) of
      {ok, U=#cc_url{path=P}} ->
	P;
      _ ->
	AbsPath
    end,
  R = wslib:gv(redirect, In, RedirPath),
  {ok, enter, #state{redirect=R},
   Props ++
   [{template, "wslogout"},
    {page_template, "wslogout"},
    {access_id, logout}
   ]
  }.

handle_event(_Msg, _From, WA, Srv, _State, S) ->
  enter(refresh, WA, Srv, S).

enter(refresh, WA, Srv, S) ->
  {next_state, ok, [], enter, S};
enter(do_exit, WA, Srv, S=#state{redirect=R}) ->
  Out = [{'DoCancel', true}, {'RedirectPath', R}],
  {stop, {error, cancel}, Out};
enter(_, WA=#ws_arg{user=undefined}, Srv, S) ->
  wsmessage:show_message(not_logged_in, WA),
  {stop, ok};
enter(do_logout, WA, Srv, S) ->
  delete_session(WA, S);
enter(_, _, _, S) ->
  {next_state, ok, enter, S}.

update_browser_cookie(_, WA, _Srv, S) ->
  {stop, ok, []}.
%%

delete_session(WA, S=#state{redirect=R}) ->
  Out = [{'DoLogout', true},
	 {'SessionCookie', wsession:delete(WA)},
	 {'RedirectPath', R}],
  {next_state, ok, Out, update_browser_cookie, S}.
