%%%-------------------------------------------------------------------
%%% File    : wsmessage.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%%
%%% Created : 19 Apr 2003 by  <svg@surnet.ru>
%%%-------------------------------------------------------------------
%%
%% $Id: wsmessage.erl,v 1.2 2003/08/29 13:46:39 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wsmessage).

-behaviour(gen_server).

-export([show_message/2, show_error/2, messages/1]).

-export([start_link/0]).

-export([init/1, handle_call/3, handle_cast/2, handle_info/2, terminate/2, code_change/3]).

-import(wslib.misc).

-include("wstl.hrl").

-define(SERVER, ?MODULE).

-record(state, {threads=gb_trees:empty()}).

start_link() ->
  gen_server:start_link({local, ?SERVER}, ?MODULE, [], []).


show_message(Msg, Arg) ->
  add_message({msg, Msg}, Arg).

show_error(Error={'EXIT', _}, Arg) ->
  show_error(misc:better_error(Error), Arg);
show_error(Error={error, _}, Arg) ->
  add_message(Error, Arg);
show_error(Error, Arg) ->
  add_message({error, Error}, Arg).

messages(WA=#ws_arg{thread_pid=Self}) ->
  gen_server:call(?SERVER, {messages, Self}).

add_message(Msg, WA=#ws_arg{thread_pid=Self}) ->
  gen_server:cast(?SERVER, {add_message, Msg, Self}).

init([]) ->
  {ok, #state{}}.

handle_call({messages, Thread}, From, S=#state{threads=Ts}) ->
  {Reply, NxtTs} =
    case gb_trees:lookup(Thread, Ts) of
      {value, Ms} ->
	{lists:reverse(Ms), gb_trees:update(Thread, [], Ts)};
      none ->
	{[], Ts}
    end,
  {reply, Reply, S#state{threads=NxtTs}}.

handle_cast({add_message, Msg, Thread}, S=#state{threads=Ts}) ->
  NxtTs =
    case gb_trees:lookup(Thread, Ts) of
      {value, Ms} ->
	gb_trees:update(Thread, [Msg|Ms], Ts);
      none ->
	erlang:monitor(process, Thread),
	gb_trees:enter(Thread, [Msg], Ts)
    end,
  {noreply, S#state{threads=NxtTs}}.

handle_info({'DOWN', _, process, Thread, _}, S=#state{threads=Ts}) ->
  NxtTs = gb_trees:delete(Thread, Ts),
  {noreply, S#state{threads=NxtTs}}.

terminate(Reason, State) ->
  ok.

code_change(OldVsn, State, Extra) ->
  {ok, State}.

%%--------------------------------------------------------------------
%%% Internal functions
%%--------------------------------------------------------------------

