%%% File    : wsrun.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created : 15 Apr 2003 by  <svg@surnet.ru>

%%
%% $Id: wsrun.erl,v 1.8 2003/10/31 17:57:31 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wsrun).

-export([out/2, enc_path/2, comp_from_path/1, comp_from_path/2]).

-include("wstl.hrl").

-define(empty_path_name, "approot").

out(YA, Application) ->
  ArgPath = YA#arg.appmoddata,
  Start = mstime(),
  Res =
    case wscc:dec_path(ArgPath) of
      {ok, U0=#cc_url{path=P, cont_key=Kid, cont_num=UCN, user_data=UD}} ->
	CompPath = dec_comp_path(UD),
	YA1 = YA#arg{appmoddata=P},
	UR = {Kid, CompPath, UCN+1, YA1},
	case wscc:disp_call(Kid, [UR]) of
	  {ok, {_, Out}} ->
	    Out;
	  {error,{nothread, _}} ->
	    {abs_path, AbsPath} = (YA#arg.req)#http_request.path,
	    {ok, U1=#cc_url{path=AP0}} = wscc:dec_path(AbsPath),
	    AP1 = AP0 ++ "?" ++ YA#arg.querydata,
	    HttpReq = (YA#arg.req)#http_request{path={abs_path, AP1}},
	    YA2 = YA#arg{appmoddata=P, req=HttpReq},
	    start_new_thread(YA2, Application)
	end;
      _ ->
	start_new_thread(YA, Application)
    end,
  End = mstime(),
  io:format("Req time ~w~n", [End - Start]),
  Res.

start_new_thread(YA, Application) ->
  InitEnv = wsapplication_mgr:init_run_env(Application),
  RootComp = create_root_comp(YA, InitEnv),
  {ok, Kid} = wscc:new(RootComp),
  case wscc:disp_call(Kid, [Kid]) of
    {ok, {_,Out}} -> 
      Out;
    {ok, Out} ->
      Out
  end.

enc_path(Path, WA=#ws_arg{application=A, yaws_arg=YA,
			  cont_key=Kid, cont_num=CN, path=P}) ->
  Path1 =
    case filename:pathtype(Path) of
      absolute ->
	Path;
      _ ->
	AppModPath = [$/|atom_to_list(A)],
	AppModPath ++ YA#arg.appmoddata ++ Path
    end,
  enc_path(Path1, Kid, CN, P).

enc_path(Path, Kid, CN, CompPath) ->
  case Kid of
    undefined ->
      Path;
    Kid ->
      wscc:enc_path(Path, Kid, CN, enc_comp_path(CompPath))
  end.

dec_comp_path(Path) ->
  string:tokens(Path, "!").

enc_comp_path(Path) ->
  lists:flatten([[P,"!"] || P <- Path]).

create_root_comp(YA, InitEnv) ->
  {ok, WA, NxtInitEnv} = InitEnv(#ws_arg{yaws_arg=YA}),
  {Module, Name} = comp_from_path(WA),
  Render = stl_yaws:create_page(WA),
  Comp = wsdcomp:create_dcomp(Module, WA, []),
  RootComp = wsvcomp:create(Name, [], Comp, Render),
  CompPath = [Name],
  CN = 0,
  fun (Kid) ->
      %%      {abs_path, Path} = (YA#arg.req)#http_request.path,
      %%      Out = [{redirect_local, enc_path(Path, Kid, 0, CompPath)}],
      %%      {continue, {ok, Out}, mk_root_cont(RootComp, 1, NxtInitEnv)}
      (mk_root_cont(RootComp, CN, NxtInitEnv))({Kid, CompPath, CN, YA}) 
  end.

mk_root_cont(Comp, CN, InitEnv) ->
  fun ({Kid, Path, UCN, YA}) ->
      {ok, WA0, NxtInitEnv} = InitEnv(#ws_arg{yaws_arg=YA}),
      WA1 = WA0#ws_arg{
	      thread_pid=self(),
	      cont_key=Kid,
	      cont_num=CN,
	      url_cont_num=UCN
	     },
      case catch wsvcomp:user_request(
		   Path, WA1, Comp, mk_return_cont(NxtInitEnv)) of
	{'EXIT', Error} ->
	  exit(Error);
	{authenticate, Event, _P} ->
	  AuthComp = wsauthen:create(Path, WA1, Comp, Event),
	  (AuthComp({call, do, WA1}))(mk_return_cont(NxtInitEnv));
	Other ->
	  Other
      end
  end.

mk_return_cont(InitEnv) ->
  fun ({reply, _AN,
	WA=#ws_arg{cont_num=CN, yaws_arg=YA}, Cont}) when CN == 0 ->
      {abs_path, Path} = (YA#arg.req)#http_request.path,
      RedirectOut = [{redirect_local, enc_path(Path, WA)}],
      RedirectCont =
	fun (_) ->
	    {reply, Out, NxtCont} =
	      wsvcomp:display(WA#ws_arg{cont_num=CN+1},
			      Cont, fun (A) -> A end
			     ),
	    {continue, Out, mk_root_cont(NxtCont, CN+2, InitEnv)}
	end,
      {continue, {ok, RedirectOut}, RedirectCont};
      ({reply, AN,  WA=#ws_arg{cont_num=CN}, Cont}) ->
      {reply, Out, NxtCont} = wsvcomp:display(WA, Cont, fun (A) -> A end),
      {continue, Out, mk_root_cont(NxtCont, CN+1, InitEnv)};
      ({stop, _, WA, Cont}) ->
      {stop, Out} =
	wsvcomp:display(WA#ws_arg{cont_key=undefined}, Cont, fun (A) -> A end),
      {die, Out}
  end.

comp_from_path(WA=#ws_arg{yaws_arg=YA}) ->
  comp_from_path(WA, YA#arg.appmoddata).

comp_from_path(WA=#ws_arg{application=A}, Path) ->
  AppName =
    if is_atom(A) ->
	atom_to_list(A);
       is_list(A) ->
	A
    end,
  comp_from_path(AppName, Path);
comp_from_path(Package, Path) ->
  Paths = comp_path_names(string:tokens(Path, "/")),
  PackagePaths = [{Package ++ "." ++ P, P} || P <- Paths],
  case comp_module(PackagePaths) of
    none ->
      case comp_module([{P, P} || P <- Paths]) of
	none ->
	  {?empty_comp, ?empty_path_name};
	Res ->
	  Res
      end;
    Res ->
      Res
  end.

comp_module([]) ->
  none;
comp_module([{MN, N}|Rest]) ->
  M = list_to_atom(MN),
  case code:ensure_loaded(M) of
    Shure when Shure == {module, M};
	       Shure == {error, embedded} ->
      IsComp = is_comp(M),
      if IsComp ->
	  {M, N};
	 true ->
	  comp_module(Rest)
      end;
    _ ->
      comp_module(Rest)
  end.

comp_path_names([]) ->
  [];
comp_path_names([P|Rest]) ->
  comp_path_names(Rest, P, [P]).

comp_path_names([], _, Ret) ->
  Ret;
comp_path_names([P|Rest], Longest, Acc) ->
  Longest0 = Longest ++ "_" ++ P,
  comp_path_names(Rest, Longest0, [Longest0|Acc]).

is_comp(M) ->
  Exports = M:module_info(exports),
  (lists:dropwhile(fun (F) -> lists:member(F, Exports) end,
		   wsdcomp:behaviour_info(callbacks)) == []).

mstime() ->
  {M, S, U} = now(),
  M*1000000000+S*1000+(U div 1000).
