%%% File    : wsvcomp.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%% Created : 12 Apr 2003 by  <svg@surnet.ru>

%%
%% $Id: wsvcomp.erl,v 1.5 2003/11/04 19:47:16 vsekissov Exp $
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wsvcomp).

-export([create/4, display/3, user_request/4]).
-export([test/0]).

-include("wstl.hrl").

-record(state, {name, sname, path, dcomp, render, children=[], out=[]}).

-record(child, {f, display=true}).

%% Render interface
%% ExecRender({inherit, Render(state)}) -> NewExecRender
%%   used with exec|call_exec to inherit properties of substituted component's
%%   render
%% Render({display, WsArg, Data}) -> OutData
%% 

create(Name, Path, Comp, Render) ->
  State = #state{name=Name,
		 path=Path++[Name],
		 dcomp=Comp,
		 render=Render,
		 sname=call},
  create(State).

create(State) ->
  fun (name) ->
      State#state.name;
      (is_alive) ->
      (State#state.dcomp =/= undefined);
      (Msg) ->
      fun (Cont) ->
	  start(Msg, Cont, State)
      end
  end.

comp_name(Comp) ->
  Comp(name).

user_request(Path, Data, Comp, Cont) ->
  ReqPath =
    case Path of
      [] ->
	comp_name(Comp);
      _ ->
	Path
    end,
  call({call, {user_req, ReqPath}, Path, Data}, Comp, Cont).

display(Data, Comp, Cont) ->
  call({display, Data}, Comp, Cont).

%%% States
  
start(M={call, Msg, Path, Data}, Cont, State=#state{name=N}) ->
  ?dbg("call ~s~npath: ~p~nmessage: ~p", [N, Path, Msg]),
  do_dcomp_call(Msg, Path, Data, Cont, State);
start(M={display, RenderData}, Cont, State=#state{name=N}) ->
  do_display(RenderData, Cont, State);
start(M={call, Msg, Data}, Cont, State=#state{name=N}) ->
  ?dbg("call ~s~nmessage: ~p", [N, Msg]),
  do_dcomp_call(Msg, Data, Cont, State).

child_answer({error_reply, Error, Data, NewCh}, Cont,
	     S=#state{children=Chs, dcomp=DC}) ->
  NxtS = S#state{children=update_child(NewCh, Chs)},
  error_dcomp(DC, Error, Data, fun dcomp_answer/4, Cont, NxtS);

child_answer({exec, {OldName, ExecMsg}, Data, ExecCh}, Cont,
	     S=#state{children=Chs}) ->
  NxtS = S#state{children=update_child(OldName, ExecCh, Chs)},
  do_child_call(ExecMsg, [ExecCh(name)], Data, Cont, NxtS);
child_answer({_, Answer, Data, NewCh}, Cont,
	     S=#state{children=Chs}) ->
  NxtS = S#state{children=update_child(NewCh, Chs)},
  do_dcomp_answer(comp_name(NewCh), Answer, Data, Cont, NxtS).

dcomp_answer({stop, Msg}, Data, Parent, S) ->
  Cont = create(S#state{dcomp=undefined}),
  return_parent(stop, Msg, Data, Cont, Parent);
dcomp_answer({stop, Msg, Out}, Data, Parent, S) ->
  Cont = create(S#state{dcomp=undefined, out=Out}),
  return_parent(stop, Msg, Data, Cont, Parent);

dcomp_answer({reply, Msg, DC}, Data, Parent, S) ->
  Cont = create(S#state{dcomp=DC}),
  return_parent(reply, Msg, Data, Cont, Parent);
dcomp_answer({reply, Msg, Out, DC}, Data, Parent, S) ->
  Cont = create(S#state{dcomp=DC, out=Out}),
  return_parent(reply, Msg, Data, Cont, Parent);

dcomp_answer({spawn, ChName, ChDC, ChRender, DC}, Data, Cont,
	     S=#state{children=Chs, path=P}) ->
  case find_child(ChName, Chs) of
    undefined ->
      Ch = create(ChName, P, ChDC, ChRender),
      NxtS = S#state{children=add_child(Ch, Chs), dcomp=DC},
      do_dcomp_answer(comp_name(Ch), ok, Data, Cont, NxtS);
    {ok, Ch} ->
      case Ch(is_alive) of
	false ->
	  NewCh = create(ChName, P, ChDC, ChRender),
	  NxtS =S#state{children=add_child(NewCh, delete_child(ChName, Chs)),
			dcomp=DC},
	  do_dcomp_answer(comp_name(Ch), ok, Data, Cont, NxtS);
	true ->
	  NxtS = S#state{dcomp=DC},
	  do_dcomp_answer(comp_name(Ch), {error, already_exists},
			  Data, Cont, NxtS)
      end
  end;
dcomp_answer({children, DC}, Data, Cont, S=#state{children=Chs}) ->
  ChNames = [comp_name(F) || C=#child{f=F} <- Chs],
  do_dcomp_answer(children, ChNames, Data, Cont, S#state{dcomp=DC});

dcomp_answer({{is_alive, Name}, DC}, Data, Cont, S=#state{children=Chs}) ->
  Ret =
    case find_child(Name, Chs) of
      {ok, Ch} ->
	Ch(is_alive);
      undefined ->
	false
    end,
  do_dcomp_answer(is_alive, Ret, Data, Cont, S#state{dcomp=DC});

dcomp_answer({name, DC}, Data, Cont, S=#state{name=N}) ->
  do_dcomp_answer(name, N, Data, Cont, S#state{dcomp=DC});

dcomp_answer({path, DC}, Data, Cont, S=#state{path=P}) ->
  do_dcomp_answer(path, P, Data, Cont, S#state{dcomp=DC});

dcomp_answer({{display, Names, NeedDisplay}, DC}, Data, Cont,
	     S=#state{children=Chs}) ->
  NxtChs =
    lists:foldl(fun (Name, Cs) ->
		    set_need_display(Name, NeedDisplay, Cs)
		end, Chs, Names),
  do_dcomp_answer(display, ok, Data, Cont, S#state{dcomp=DC, children=NxtChs});

dcomp_answer({{call, Name, Query}, DC}, Data, Cont,
	     S=#state{children=Chs}) ->
  case find_child(Name, Chs) of
    {ok, Ch} ->
      NxtS = S#state{dcomp=DC},
      call_child(Ch, Query, Data, fun child_answer/3, Cont, NxtS);
    undefined ->
      Reply = unknown_child,
      error_dcomp(DC, Reply, Data, fun dcomp_answer/4, Cont, S)
  end;

dcomp_answer({call_exec, ExecDC, ExecRender, ExecQuery, DC},
	     Data, Cont, S=#state{name=N, path=P, render=R}) ->
  ExecRender1 = ExecRender({inherit, R(state)}),
  ExecComp = create(N, lists:sublist(P, length(P)-1), ExecDC, ExecRender1),
  Exec = create_exec(ExecComp, S#state{dcomp=DC, sname=call_exec}),
  (Exec({call, ExecQuery, Data}))(Cont);

dcomp_answer({exec, ExecDC, ExecName, ExecRender, ExecMsg},
	     Data, Cont, S=#state{name=N, path=P, render=R, sname=SN}) ->
  ExecRender1 = ExecRender({inherit, R(state)}),
  ?dbg("exec ~s ~p", [ExecName, ExecMsg]),
  ExecComp = create(ExecName, lists:sublist(P, length(P)-1), ExecDC, ExecRender1),
  case SN of
    call_exec ->
      (ExecComp({call, ExecMsg, [ExecName], Data}))(Cont);
    _ ->
      return_parent(exec, {N, ExecMsg}, Data, ExecComp, Cont)
  end;
dcomp_answer({exec, ExecDC, ExecRender, ExecMsg},
	     Data, Parent, S=#state{name=N}) ->
  dcomp_answer({exec, ExecDC, N, ExecRender, ExecMsg}, Data, Parent, S);
dcomp_answer({exec, ExecVC, ExecMsg}, _D, Cont, _S) ->
  (ExecVC(ExecMsg))(Cont).

%%% Helpers

do_dcomp_call(Msg, Path, Data, Cont, S=#state{name=N}) when Path == [];
							    Path == [N] ->
  do_dcomp_call(Msg, Data, Cont, S);
do_dcomp_call(Msg, [N|Path], Data, Cont, S=#state{name=N}) ->
  do_child_call(Msg, Path, Data, Cont, S);
do_dcomp_call(Msg, Path, Data, Cont, S=#state{dcomp=C,  path=P}) ->
  Call = {unknown_call, Msg, P},
  do_dcomp_call(Call, Data, Cont, S).


do_dcomp_call(_Msg, Data, Parent, S=#state{dcomp=undefined}) ->
  Cont = create(S),
  return_parent(error_reply, terminated, Data, Cont, Parent);
do_dcomp_call(Msg, Data, Cont, S=#state{dcomp=DC}) ->
  call_dcomp(DC, Msg, Data, fun dcomp_answer/4, Cont, S).

do_child_call(Msg, FullP=[ChName|_], Data, Cont,
	      S=#state{name=N, children=Chs, path=P}) ->
  case find_child(ChName, Chs) of
    {ok, Ch} ->
      Call = {call, Msg, FullP, Data},
      call_child(Ch, Call, fun child_answer/3, Cont, S);
    undefined ->
      Call = {unknown_call, Msg, P},
      do_dcomp_call(Call, Data, Cont, S)
  end.

do_dcomp_answer(From, Msg, Data, Cont, S=#state{dcomp=DC}) ->
  answer_dcomp(DC, From, Msg, Data, fun dcomp_answer/4, Cont, S).

do_display(Data, Cont,
	   S=#state{out=O, path=P, children=Chs, dcomp=C, render=R}) ->
  {ChOut, NxtChs} = do_display_children(Data, Chs, [], []),
  Out = lists:reverse(mk_list(O))++ChOut,
  case C of
    undefined ->
      Ret = R({display, Data#ws_arg{path=P, cont_key=undefined}, Out}),
      Cont({stop, Ret});
    _ ->
      Ret = R({display, Data#ws_arg{path=P}, Out}),
      Cont({reply, Ret, create(S#state{children=NxtChs})})
  end.

do_display_children(_, [], Out, Chs) ->
  {lists:append(lists:reverse(Out)), lists:reverse(Chs)};
do_display_children(Data, [Ch=#child{f=F, display=false}|Rest], Out, Chs) ->
  NxtChs =
    case F(is_alive) of
      true ->
	[Ch|Chs];
      false ->
	Chs
    end,
  do_display_children(Data, Rest, Out, NxtChs);
do_display_children(Data, [Ch=#child{f=F}|Rest], Out, Chs) ->
  case display(Data, F, fun (A) -> A end) of
    {reply, O, C} ->
      do_display_children(Data, Rest, [O|Out], [Ch#child{f=C}|Chs]);
    {stop, O} ->
      do_display_children(Data, Rest, [O|Out], Chs)
  end.

call_child(Comp, Query, State, Cont, StateData) ->
  RetCont = mk_cont(State, Cont, StateData),
  call(Query, Comp, RetCont).

call_child(Comp, Query, Data, State, Cont, StateData) ->
  RetCont = mk_cont(State, Cont, StateData),
  call_comp(Query, Data, Comp, RetCont).

call_dcomp(Comp, Query, Data, State, Cont, StateData) ->
  RetCont = mk_cont(State, Data, Cont, StateData),
  call_comp(Query, Data, Comp, RetCont).

answer_dcomp(Comp, From, Msg, Data, State, Cont, StateData) ->
  RetCont = mk_cont(State, Data, Cont, StateData),
  call({answer, From, Msg, Data}, Comp, RetCont).

error_dcomp(Comp, Msg, Data, State, Cont, StateData) ->
  RetCont = mk_cont(State, Data, Cont, StateData),
  call({error, Msg, Data}, Comp, RetCont).

call_comp(Msg, Data, Comp, Cont) ->
  call({call, Msg, Data}, Comp, Cont).

call(Msg, Comp, Cont) ->
  (Comp(Msg))(Cont).

return_parent(Status, Answer, Data, Cont, Parent) ->
  Reply = {Status, Answer, Data, Cont},
  Parent(Reply).

mk_cont(State, Cont, StateData) ->
  fun (name) ->
      StateData#state.name;
      (is_alive) ->
      (StateData#state.dcomp =/= undefined);
      (Msg) ->
      State(Msg, Cont, StateData)
  end.

mk_cont(State, Data, Cont, StateData) ->
  fun (name) ->
      StateData#state.name;
      (is_alive) ->
      (StateData#state.dcomp =/= undefined);
      (Msg) ->
      State(Msg, Data, Cont, StateData)
  end.

add_child(Ch, Chs) ->
  Chs ++ [#child{f=Ch}].

update_child(OldName, NewF, Chs) ->
  Name = NewF(name),
  {Before, [C|After]} =
    lists:splitwith(fun (C=#child{f=F}) -> F(name) =/= OldName end, Chs),
  Before ++ [C#child{f=NewF}|After].

update_child(NewF, Chs) ->
  Name = NewF(name),
  {Before, [C|After]} =
    lists:splitwith(fun (C=#child{f=F}) -> F(name) =/= Name end, Chs),
  Before ++ [C#child{f=NewF}|After].

find_child(Name, []) ->
  undefined;
find_child(Name, [C=#child{f=F}|Rest]) ->  
  case F(name) of
    Name ->
      {ok, F};
    _ ->
      find_child(Name, Rest)
  end.

delete_child(Name, Chs) ->
  [C || C <- Chs, (C#child.f)(name) =/= Name].

set_need_display(Name, NeedDisplay, Cs) ->
  Set =
    fun(C=#child{f=F}) ->
	case comp_name(F) of
	  Name ->
	    C#child{display=NeedDisplay};
	  _ ->
	    C
	end
    end,
  [Set(C) || C <- Cs].

create_exec(Exec, State) ->
  fun (name) ->
      State#state.name;
      (is_alive) ->
      true;
      (Msg) ->
      fun (Cont) ->
	  (Exec(Msg))(exec_cont(Cont, State))
      end
  end.

exec_cont(Cont, State) ->
  fun ({stop, Answer, Data, _ExecCont}) ->
      exec_answer(Answer, Data, Cont, State);
      ({reply, Out, ExecCont}) ->
	Cont({reply, Out, create_exec(ExecCont, State)});
      ({reply, Answer, Data, ExecCont}) ->
      Cont({reply, Answer, Data, create_exec(ExecCont, State)});
      ({exec, {N, ExecMsg}, Data, ExecComp}) ->
      Exec = create_exec(ExecComp, State),
      (Exec({call, ExecMsg, [N], Data}))(Cont)
  end.

exec_answer(Answer, Data, Cont, State) ->
  do_dcomp_answer(exec, Answer, Data, Cont, State).

mk_list(L) when list(L) ->
  L;
mk_list(T) ->
  [T].

%%% Tests

test_dcomp(State) ->
  fun (Msg) ->
      fun (Cont) ->
	  test_dcomp(Msg, Cont, State)
      end
  end.

%% simple call      
test_dcomp({call, Msg, Data}, Cont, S=1) ->
  Cont({reply, {test_call, Msg}, [{test_call, Msg, Data}], test_dcomp(S+1)});
%% spawn
test_dcomp({call, Msg, Data}, Cont, S=2) ->
  Cont({spawn, "t11", test_dcomp(child_answer),
	fun test_render/2, test_dcomp(S+1)});
test_dcomp({answer, From, ok, Data}, Cont, S=3) ->
  Cont({call, From, {request, 1}, test_dcomp(S+1)});
test_dcomp({answer, From, Answer, Data}, Cont, S=4) ->
  Cont({reply,
	{test_answer, From, Answer},
	[{test_answer, From, Answer, Data}], test_dcomp(S+1)});
test_dcomp({call, Msg, Data}, Cont, S=5) ->
  Cont({call, "t11", {request, 2}, test_dcomp(S+1)});
test_dcomp({answer, From, Answer, Data}, Cont, S=6) ->
%%  Cont({reply,
%%	{test_answer, From, Answer},
%%	[{test_answer, From, Answer, Data}], test_dcomp(S+1)});
  Cont({call, "t11", {request, 3}, test_dcomp(S+1)});
test_dcomp({error, Error, Data}, Cont, S=7) ->
  Cont({reply,
	{call_error, Error},
	[{call_error, Error, Data}], test_dcomp(S+1)});
test_dcomp({call, Msg, Data}, Cont, S=8) ->
  Cont({call, "t11", {request, 3}, test_dcomp(S+1)});
test_dcomp({error, Error, Data}, Cont, S=9) ->
  Cont({reply,
	{call_error, Error},
	[{call_error, Error, Data}], test_dcomp(S+1)});
test_dcomp({call, Msg, Data}, Cont, S=10) ->
  Cont({call_exec, test_dcomp(12), fun test_render/2, do_exec, test_dcomp(S+1)});

test_dcomp({answer, exec, Answer, Data}, Cont, S=11) ->
  Cont({reply, {test_exec, Answer}, [{test_exec, Answer, Data}], test_dcomp(S+1)});
test_dcomp({call, Msg, Data}, Cont, S=12) ->
  Cont({stop, Msg, [{out, Data}]});
%%
test_dcomp({call, Msg, Data}, Cont, child_answer) ->
  Cont({reply, {child_rec, Msg}, [{child_out, Msg, Data}],
	test_dcomp(child_answer1)});
test_dcomp({call, Msg, Data}, Cont, child_answer1) ->
  Cont({stop, {child_rec, Msg}, [{child_out, Msg, Data}]}).

test_vcomp(Name) ->
  VC = create(Name, [], test_dcomp(1), fun test_render/2).

test_render(Data, Out) ->
  {render, {Data, Out}}.

test() ->
  Num=1,
  VC = test_vcomp("t1"),
  start_test(Num, VC).

start_test(Num, VC) ->
  test_loop(Num, test_call(Num, VC)).

test_loop(Num, {stop, Repl, Data, VC}) ->
  io:format("Stop:~w ~p~n", [Num, Repl]),
  {stop, Disp} = display(Data, VC, fun (A) -> A end),
  io:format("Display:~w ~p~n", [Num, Disp]);
test_loop(Num, {reply, Repl, Data, VC}) ->
  io:format("Reply:~w ~p~n", [Num, Repl]),
  {reply, Disp, VCC} = display(Data, VC, fun (A) -> A end),
  io:format("Display:~w ~p~n", [Num, Disp]),
  test_loop(Num+1, test_call(Num+1, VCC)).


test_call(Num, VC) ->
  call_comp(test_message(Num),test_data(Num), VC, fun (A) -> A end).

test_message(N) ->
  {test_msg, N}.
				     
test_data(N) ->
  {test_data, N}.
