%%%-------------------------------------------------------------------
%%% File    : var.erl
%%% Author  :  <svg@surnet.ru>
%%% Description : 
%%%
%%% Created : 20 May 2003 by  <svg@surnet.ru>
%%%-------------------------------------------------------------------
%%
%% $Id$
%%
%%**
%% 
%% @doc
%% @end
%%*
-module(wsxmlt.var).

-export([create/1, delete/1, push_frame/1, pop_frame/1, new/3, new/4, set/3, find/2]).


-import(orddict).
-import(lists).

create(Vs) ->
  Srv = start(),
  push_frame(Srv, Vs),
  Srv.

delete(Srv) ->
  Srv ! {self(), stop}.

start() ->
  Self = self(),
  spawn_link(fun () -> init(Self) end).

push_frame(Srv) ->
  push_frame(Srv, []).

push_frame(Srv, Vs) ->
  Srv ! {self(), {push_frame, Vs}},
  Srv.

pop_frame(Srv) ->
  Srv ! {self(), pop_frame},
  Srv.

new(N, V, Srv) ->
  new(N, V, 0, Srv).

new(N, V, Lvl, Srv)  when Lvl >= 0 ->
  send_recv(Srv, {new, N, V, Lvl}).

set(N, V, Srv) ->
  send_recv(Srv, {set, N, V}).
      
find(N, Srv) ->
  send_recv(Srv, {find, N}).

send_recv(Srv, Req) ->
  Srv ! {self(), Req},
  receive
    {'EXIT', Srv, Reason} ->
      exit(Reason);
    {Srv, Ret} ->
      Ret
  end.

init(From) ->
  loop(From, []).

loop(From, Frames) ->
  receive
    {From, {push_frame, Vars}} ->
      loop(From, [orddict:from_list(Vars)|Frames]);
    {From, pop_frame} ->
      loop(From, tl(Frames));
    {From, {set, N, V}} ->
      {Ret, NewFrames} = var_set(N, V, Frames),
      From ! {self(), Ret},
      loop(From, NewFrames);
    {From, {new, N, V, Lvl}} ->
      {Ret, NewFrames} = var_new(N, V, Lvl, Frames),
      From ! {self(), Ret},
      loop(From, NewFrames);
    {From, {find, N}} ->
      From ! {self(), var_find(N, Frames)},
      loop(From, Frames);
    {From, stop} ->
      exit(normal);
    {'EXIT', From, Reason} ->
      exit(Reason)
  end.

var_new(N, V, Lvl, Frames) ->
  var_new(N, V, Lvl, Frames, []).

var_new(N, V, _, [], Acc) ->
  {badlevel, lists:reverse(Acc)};
var_new(N, V, 0, Fs=[F|Rest], Acc) ->
  case orddict:is_key(N, F) of
    true ->
      {already_exist, lists:reverse(Acc)++Fs};
    false ->
      {ok, lists:reverse(Acc)++[orddict:store(N, V, F)|Rest]}
  end;
var_new(N, V, Lvl, [F|Rest], Acc) ->
  var_new(N, V, Lvl-1, Rest, [F|Acc]).

var_set(N, V, Fs) ->
  case frame_map(fun (F) ->
		     case orddict:is_key(N, F) of
		       true ->
			 {ok, ok, orddict:store(N, V, F)};
		       false ->
			 false
		     end
		 end, Fs) of
    false ->
      {undefined, Fs};
    {ok, _, NFs} ->
      {ok, NFs}
  end.
		       
var_find(N, Fs) ->
  frame_map(fun (F) -> orddict:find(N, F) end, Fs).

frame_map(Fun, Fs) ->
  frame_map(Fun, Fs, []).

frame_map(Fun, [], _) ->
  false;
frame_map(Fun, [F|Rest], A) ->
  case Fun(F) of
    Ret={ok, _} ->
      Ret;
    {ok, Res, NF} ->
      {ok, Res, lists:reverse(A) ++ [NF|Rest]};
    _ ->
      frame_map(Fun, Rest, [F|A])
  end.
